"""Copied from:  https://static.emzi0767.com/misc/discordEmojiMap.min.json
versionTimestamp:           2021-03-25T14:31:09.773+00:00
"""

emojis = {
    '💯': [':100:'],
    '🔢': [':1234:'],
    '🎱': [':8ball:'],
    '🅰️': [':a:'],
    '🆎': [':ab:'],
    '🧮': [':abacus:'],
    '🔤': [':abc:'],
    '🔡': [':abcd:'],
    '🉑': [':accept:'],
    '🪗': [':accordion:'],
    '🩹': [':adhesive_bandage:'],
    '🧑': [':adult:'],
    '🧑🏻': [':adult_tone1:', ':adult_light_skin_tone:', ':adult::skin-tone-1:'],
    '🧑🏼': [':adult_tone2:', ':adult_medium_light_skin_tone:', ':adult::skin-tone-2:'],
    '🧑🏽': [':adult_tone3:', ':adult_medium_skin_tone:', ':adult::skin-tone-3:'],
    '🧑🏾': [':adult_tone4:', ':adult_medium_dark_skin_tone:', ':adult::skin-tone-4:'],
    '🧑🏿': [':adult_tone5:', ':adult_dark_skin_tone:', ':adult::skin-tone-5:'],
    '🚡': [':aerial_tramway:'],
    '✈️': [':airplane:'],
    '🛬': [':airplane_arriving:'],
    '🛫': [':airplane_departure:'],
    '🛩️': [':airplane_small:', ':small_airplane:'],
    '⏰': [':alarm_clock:'],
    '⚗️': [':alembic:'],
    '👽': [':alien:'],
    '🚑': [':ambulance:'],
    '🏺': [':amphora:'],
    '🫀': [':anatomical_heart:'],
    '⚓': [':anchor:'],
    '👼': [':angel:'],
    '👼🏻': [':angel_tone1:', ':angel::skin-tone-1:'],
    '👼🏼': [':angel_tone2:', ':angel::skin-tone-2:'],
    '👼🏽': [':angel_tone3:', ':angel::skin-tone-3:'],
    '👼🏾': [':angel_tone4:', ':angel::skin-tone-4:'],
    '👼🏿': [':angel_tone5:', ':angel::skin-tone-5:'],
    '💢': [':anger:'],
    '🗯️': [':anger_right:', ':right_anger_bubble:'],
    '😠': [':angry:', '>:(', '>:-(', '>(', '>-('],
    '😧': [':anguished:'],
    '🐜': [':ant:'],
    '🍎': [':apple:'],
    '♒': [':aquarius:'],
    '♈': [':aries:'],
    '◀️': [':arrow_backward:'],
    '⏬': [':arrow_double_down:'],
    '⏫': [':arrow_double_up:'],
    '⬇️': [':arrow_down:'],
    '🔽': [':arrow_down_small:'],
    '▶️': [':arrow_forward:'],
    '⤵️': [':arrow_heading_down:'],
    '⤴️': [':arrow_heading_up:'],
    '⬅️': [':arrow_left:'],
    '↙️': [':arrow_lower_left:'],
    '↘️': [':arrow_lower_right:'],
    '➡️': [':arrow_right:'],
    '↪️': [':arrow_right_hook:'],
    '⬆️': [':arrow_up:'],
    '↕️': [':arrow_up_down:'],
    '🔼': [':arrow_up_small:'],
    '↖️': [':arrow_upper_left:'],
    '↗️': [':arrow_upper_right:'],
    '🔃': [':arrows_clockwise:'],
    '🔄': [':arrows_counterclockwise:'],
    '🎨': [':art:'],
    '🚛': [':articulated_lorry:'],
    '🧑‍🎨': [':artist:'],
    '🧑🏻‍🎨': [':artist_tone1:', ':artist_light_skin_tone:', ':artist::skin-tone-1:'],
    '🧑🏼‍🎨': [':artist_tone2:', ':artist_medium_light_skin_tone:', ':artist::skin-tone-2:'],
    '🧑🏽‍🎨': [':artist_tone3:', ':artist_medium_skin_tone:', ':artist::skin-tone-3:'],
    '🧑🏾‍🎨': [':artist_tone4:', ':artist_medium_dark_skin_tone:', ':artist::skin-tone-4:'],
    '🧑🏿‍🎨': [':artist_tone5:', ':artist_dark_skin_tone:', ':artist::skin-tone-5:'],
    '*️⃣': [':asterisk:', ':keycap_asterisk:'],
    '😲': [':astonished:'],
    '🧑‍🚀': [':astronaut:'],
    '🧑🏻‍🚀': [':astronaut_tone1:', ':astronaut_light_skin_tone:', ':astronaut::skin-tone-1:'],
    '🧑🏼‍🚀': [':astronaut_tone2:', ':astronaut_medium_light_skin_tone:', ':astronaut::skin-tone-2:'],
    '🧑🏽‍🚀': [':astronaut_tone3:', ':astronaut_medium_skin_tone:', ':astronaut::skin-tone-3:'],
    '🧑🏾‍🚀': [':astronaut_tone4:', ':astronaut_medium_dark_skin_tone:', ':astronaut::skin-tone-4:'],
    '🧑🏿‍🚀': [':astronaut_tone5:', ':astronaut_dark_skin_tone:', ':astronaut::skin-tone-5:'],
    '👟': [':athletic_shoe:'],
    '🏧': [':atm:'],
    '⚛️': [':atom:', ':atom_symbol:'],
    '🛺': [':auto_rickshaw:'],
    '🥑': [':avocado:'],
    '🪓': [':axe:'],
    '🅱️': [':b:'],
    '👶': [':baby:'],
    '🍼': [':baby_bottle:'],
    '🐤': [':baby_chick:'],
    '🚼': [':baby_symbol:'],
    '👶🏻': [':baby_tone1:', ':baby::skin-tone-1:'],
    '👶🏼': [':baby_tone2:', ':baby::skin-tone-2:'],
    '👶🏽': [':baby_tone3:', ':baby::skin-tone-3:'],
    '👶🏾': [':baby_tone4:', ':baby::skin-tone-4:'],
    '👶🏿': [':baby_tone5:', ':baby::skin-tone-5:'],
    '🔙': [':back:'],
    '🥓': [':bacon:'],
    '🦡': [':badger:'],
    '🏸': [':badminton:'],
    '🥯': [':bagel:'],
    '🛄': [':baggage_claim:'],
    '🩰': [':ballet_shoes:'],
    '🎈': [':balloon:'],
    '🗳️': [':ballot_box:', ':ballot_box_with_ballot:'],
    '☑️': [':ballot_box_with_check:'],
    '🎍': [':bamboo:'],
    '🍌': [':banana:'],
    '‼️': [':bangbang:'],
    '🪕': [':banjo:'],
    '🏦': [':bank:'],
    '📊': [':bar_chart:'],
    '💈': [':barber:'],
    '⚾': [':baseball:'],
    '🧺': [':basket:'],
    '🏀': [':basketball:'],
    '🦇': [':bat:'],
    '🛀': [':bath:'],
    '🛀🏻': [':bath_tone1:', ':bath::skin-tone-1:'],
    '🛀🏼': [':bath_tone2:', ':bath::skin-tone-2:'],
    '🛀🏽': [':bath_tone3:', ':bath::skin-tone-3:'],
    '🛀🏾': [':bath_tone4:', ':bath::skin-tone-4:'],
    '🛀🏿': [':bath_tone5:', ':bath::skin-tone-5:'],
    '🛁': [':bathtub:'],
    '🔋': [':battery:'],
    '🏖️': [':beach:', ':beach_with_umbrella:'],
    '⛱️': [':beach_umbrella:', ':umbrella_on_ground:'],
    '🐻': [':bear:'],
    '🧔': [':bearded_person:'],
    '🧔🏻': [':bearded_person_tone1:', ':bearded_person_light_skin_tone:', ':bearded_person::skin-tone-1:'],
    '🧔🏼': [':bearded_person_tone2:', ':bearded_person_medium_light_skin_tone:', ':bearded_person::skin-tone-2:'],
    '🧔🏽': [':bearded_person_tone3:', ':bearded_person_medium_skin_tone:', ':bearded_person::skin-tone-3:'],
    '🧔🏾': [':bearded_person_tone4:', ':bearded_person_medium_dark_skin_tone:', ':bearded_person::skin-tone-4:'],
    '🧔🏿': [':bearded_person_tone5:', ':bearded_person_dark_skin_tone:', ':bearded_person::skin-tone-5:'],
    '🦫': [':beaver:'],
    '🛏️': [':bed:'],
    '🐝': [':bee:'],
    '🍺': [':beer:'],
    '🍻': [':beers:'],
    '🪲': [':beetle:'],
    '🔰': [':beginner:'],
    '🔔': [':bell:'],
    '🫑': [':bell_pepper:'],
    '🛎️': [':bellhop:', ':bellhop_bell:'],
    '🍱': [':bento:'],
    '🧃': [':beverage_box:'],
    '🚲': [':bike:'],
    '👙': [':bikini:'],
    '🧢': [':billed_cap:'],
    '☣️': [':biohazard:', ':biohazard_sign:'],
    '🐦': [':bird:'],
    '🎂': [':birthday:'],
    '🦬': [':bison:'],
    '🐈‍⬛': [':black_cat:'],
    '⚫': [':black_circle:'],
    '🖤': [':black_heart:'],
    '🃏': [':black_joker:'],
    '⬛': [':black_large_square:'],
    '◾': [':black_medium_small_square:'],
    '◼️': [':black_medium_square:'],
    '✒️': [':black_nib:'],
    '▪️': [':black_small_square:'],
    '🔲': [':black_square_button:'],
    '👱‍♂️': [':blond_haired_man:'],
    '👱🏻‍♂️': [':blond_haired_man_tone1:', ':blond_haired_man_light_skin_tone:', ':blond_haired_man::skin-tone-1:'],
    '👱🏼‍♂️': [':blond_haired_man_tone2:', ':blond_haired_man_medium_light_skin_tone:', ':blond_haired_man::skin-tone-2:'],
    '👱🏽‍♂️': [':blond_haired_man_tone3:', ':blond_haired_man_medium_skin_tone:', ':blond_haired_man::skin-tone-3:'],
    '👱🏾‍♂️': [':blond_haired_man_tone4:', ':blond_haired_man_medium_dark_skin_tone:', ':blond_haired_man::skin-tone-4:'],
    '👱🏿‍♂️': [':blond_haired_man_tone5:', ':blond_haired_man_dark_skin_tone:', ':blond_haired_man::skin-tone-5:'],
    '👱': [':blond_haired_person:', ':person_with_blond_hair:'],
    '👱🏻': [':blond_haired_person_tone1:', ':person_with_blond_hair_tone1:', ':blond_haired_person::skin-tone-1:', ':person_with_blond_hair::skin-tone-1:'],
    '👱🏼': [':blond_haired_person_tone2:', ':person_with_blond_hair_tone2:', ':blond_haired_person::skin-tone-2:', ':person_with_blond_hair::skin-tone-2:'],
    '👱🏽': [':blond_haired_person_tone3:', ':person_with_blond_hair_tone3:', ':blond_haired_person::skin-tone-3:', ':person_with_blond_hair::skin-tone-3:'],
    '👱🏾': [':blond_haired_person_tone4:', ':person_with_blond_hair_tone4:', ':blond_haired_person::skin-tone-4:', ':person_with_blond_hair::skin-tone-4:'],
    '👱🏿': [':blond_haired_person_tone5:', ':person_with_blond_hair_tone5:', ':blond_haired_person::skin-tone-5:', ':person_with_blond_hair::skin-tone-5:'],
    '👱‍♀️': [':blond_haired_woman:'],
    '👱🏻‍♀️': [':blond_haired_woman_tone1:', ':blond_haired_woman_light_skin_tone:', ':blond_haired_woman::skin-tone-1:'],
    '👱🏼‍♀️': [':blond_haired_woman_tone2:', ':blond_haired_woman_medium_light_skin_tone:', ':blond_haired_woman::skin-tone-2:'],
    '👱🏽‍♀️': [':blond_haired_woman_tone3:', ':blond_haired_woman_medium_skin_tone:', ':blond_haired_woman::skin-tone-3:'],
    '👱🏾‍♀️': [':blond_haired_woman_tone4:', ':blond_haired_woman_medium_dark_skin_tone:', ':blond_haired_woman::skin-tone-4:'],
    '👱🏿‍♀️': [':blond_haired_woman_tone5:', ':blond_haired_woman_dark_skin_tone:', ':blond_haired_woman::skin-tone-5:'],
    '🌼': [':blossom:'],
    '🐡': [':blowfish:'],
    '📘': [':blue_book:'],
    '🚙': [':blue_car:'],
    '🔵': [':blue_circle:'],
    '💙': [':blue_heart:'],
    '🟦': [':blue_square:'],
    '🫐': [':blueberries:'],
    '😊': [':blush:', ':\\)', ':-\\)', '\\)', '-\\)'],
    '🐗': [':boar:'],
    '💣': [':bomb:'],
    '🦴': [':bone:'],
    '📖': [':book:'],
    '🔖': [':bookmark:'],
    '📑': [':bookmark_tabs:'],
    '📚': [':books:'],
    '💥': [':boom:'],
    '🪃': [':boomerang:'],
    '👢': [':boot:'],
    '💐': [':bouquet:'],
    '🏹': [':bow_and_arrow:', ':archery:'],
    '🥣': [':bowl_with_spoon:'],
    '🎳': [':bowling:'],
    '🥊': [':boxing_glove:', ':boxing_gloves:'],
    '👦': [':boy:'],
    '👦🏻': [':boy_tone1:', ':boy::skin-tone-1:'],
    '👦🏼': [':boy_tone2:', ':boy::skin-tone-2:'],
    '👦🏽': [':boy_tone3:', ':boy::skin-tone-3:'],
    '👦🏾': [':boy_tone4:', ':boy::skin-tone-4:'],
    '👦🏿': [':boy_tone5:', ':boy::skin-tone-5:'],
    '🧠': [':brain:'],
    '🍞': [':bread:'],
    '🤱': [':breast_feeding:'],
    '🤱🏻': [':breast_feeding_tone1:', ':breast_feeding_light_skin_tone:', ':breast_feeding::skin-tone-1:'],
    '🤱🏼': [':breast_feeding_tone2:', ':breast_feeding_medium_light_skin_tone:', ':breast_feeding::skin-tone-2:'],
    '🤱🏽': [':breast_feeding_tone3:', ':breast_feeding_medium_skin_tone:', ':breast_feeding::skin-tone-3:'],
    '🤱🏾': [':breast_feeding_tone4:', ':breast_feeding_medium_dark_skin_tone:', ':breast_feeding::skin-tone-4:'],
    '🤱🏿': [':breast_feeding_tone5:', ':breast_feeding_dark_skin_tone:', ':breast_feeding::skin-tone-5:'],
    '🧱': [':bricks:'],
    '🌉': [':bridge_at_night:'],
    '💼': [':briefcase:'],
    '🩲': [':briefs:'],
    '🥦': [':broccoli:'],
    '💔': [':broken_heart:', '</3', '<\\\\3'],
    '🧹': [':broom:'],
    '🟤': [':brown_circle:'],
    '🤎': [':brown_heart:'],
    '🟫': [':brown_square:'],
    '🧋': [':bubble_tea:'],
    '🪣': [':bucket:'],
    '🐛': [':bug:'],
    '💡': [':bulb:'],
    '🚅': [':bullettrain_front:'],
    '🚄': [':bullettrain_side:'],
    '🌯': [':burrito:'],
    '🚌': [':bus:'],
    '🚏': [':busstop:'],
    '👤': [':bust_in_silhouette:'],
    '👥': [':busts_in_silhouette:'],
    '🧈': [':butter:'],
    '🦋': [':butterfly:'],
    '🌵': [':cactus:'],
    '🍰': [':cake:'],
    '📆': [':calendar:'],
    '🗓️': [':calendar_spiral:', ':spiral_calendar_pad:'],
    '🤙': [':call_me:', ':call_me_hand:'],
    '🤙🏻': [':call_me_tone1:', ':call_me_hand_tone1:', ':call_me::skin-tone-1:', ':call_me_hand::skin-tone-1:'],
    '🤙🏼': [':call_me_tone2:', ':call_me_hand_tone2:', ':call_me::skin-tone-2:', ':call_me_hand::skin-tone-2:'],
    '🤙🏽': [':call_me_tone3:', ':call_me_hand_tone3:', ':call_me::skin-tone-3:', ':call_me_hand::skin-tone-3:'],
    '🤙🏾': [':call_me_tone4:', ':call_me_hand_tone4:', ':call_me::skin-tone-4:', ':call_me_hand::skin-tone-4:'],
    '🤙🏿': [':call_me_tone5:', ':call_me_hand_tone5:', ':call_me::skin-tone-5:', ':call_me_hand::skin-tone-5:'],
    '📲': [':calling:'],
    '🐫': [':camel:'],
    '📷': [':camera:'],
    '📸': [':camera_with_flash:'],
    '🏕️': [':camping:'],
    '♋': [':cancer:'],
    '🕯️': [':candle:'],
    '🍬': [':candy:'],
    '🥫': [':canned_food:'],
    '🛶': [':canoe:', ':kayak:'],
    '🔠': [':capital_abcd:'],
    '♑': [':capricorn:'],
    '🗃️': [':card_box:', ':card_file_box:'],
    '📇': [':card_index:'],
    '🎠': [':carousel_horse:'],
    '🪚': [':carpentry_saw:'],
    '🥕': [':carrot:'],
    '🐱': [':cat:'],
    '🐈': [':cat2:'],
    '💿': [':cd:'],
    '⛓️': [':chains:'],
    '🪑': [':chair:'],
    '🍾': [':champagne:', ':bottle_with_popping_cork:'],
    '🥂': [':champagne_glass:', ':clinking_glass:'],
    '💹': [':chart:'],
    '📉': [':chart_with_downwards_trend:'],
    '📈': [':chart_with_upwards_trend:'],
    '🏁': [':checkered_flag:'],
    '🧀': [':cheese:', ':cheese_wedge:'],
    '🍒': [':cherries:'],
    '🌸': [':cherry_blossom:'],
    '♟️': [':chess_pawn:'],
    '🌰': [':chestnut:'],
    '🐔': [':chicken:'],
    '🧒': [':child:'],
    '🧒🏻': [':child_tone1:', ':child_light_skin_tone:', ':child::skin-tone-1:'],
    '🧒🏼': [':child_tone2:', ':child_medium_light_skin_tone:', ':child::skin-tone-2:'],
    '🧒🏽': [':child_tone3:', ':child_medium_skin_tone:', ':child::skin-tone-3:'],
    '🧒🏾': [':child_tone4:', ':child_medium_dark_skin_tone:', ':child::skin-tone-4:'],
    '🧒🏿': [':child_tone5:', ':child_dark_skin_tone:', ':child::skin-tone-5:'],
    '🚸': [':children_crossing:'],
    '🐿️': [':chipmunk:'],
    '🍫': [':chocolate_bar:'],
    '🥢': [':chopsticks:'],
    '🎄': [':christmas_tree:'],
    '⛪': [':church:'],
    '🎦': [':cinema:'],
    '🎪': [':circus_tent:'],
    '🌆': [':city_dusk:'],
    '🌇': [':city_sunset:', ':city_sunrise:'],
    '🏙️': [':cityscape:'],
    '🆑': [':cl:'],
    '👏': [':clap:'],
    '👏🏻': [':clap_tone1:', ':clap::skin-tone-1:'],
    '👏🏼': [':clap_tone2:', ':clap::skin-tone-2:'],
    '👏🏽': [':clap_tone3:', ':clap::skin-tone-3:'],
    '👏🏾': [':clap_tone4:', ':clap::skin-tone-4:'],
    '👏🏿': [':clap_tone5:', ':clap::skin-tone-5:'],
    '🎬': [':clapper:'],
    '🏛️': [':classical_building:'],
    '📋': [':clipboard:'],
    '🕰️': [':clock:', ':mantlepiece_clock:'],
    '🕐': [':clock1:'],
    '🕙': [':clock10:'],
    '🕥': [':clock1030:'],
    '🕚': [':clock11:'],
    '🕦': [':clock1130:'],
    '🕛': [':clock12:'],
    '🕧': [':clock1230:'],
    '🕜': [':clock130:'],
    '🕑': [':clock2:'],
    '🕝': [':clock230:'],
    '🕒': [':clock3:'],
    '🕞': [':clock330:'],
    '🕓': [':clock4:'],
    '🕟': [':clock430:'],
    '🕔': [':clock5:'],
    '🕠': [':clock530:'],
    '🕕': [':clock6:'],
    '🕡': [':clock630:'],
    '🕖': [':clock7:'],
    '🕢': [':clock730:'],
    '🕗': [':clock8:'],
    '🕣': [':clock830:'],
    '🕘': [':clock9:'],
    '🕤': [':clock930:'],
    '📕': [':closed_book:'],
    '🔐': [':closed_lock_with_key:'],
    '🌂': [':closed_umbrella:'],
    '☁️': [':cloud:'],
    '🌩️': [':cloud_lightning:', ':cloud_with_lightning:'],
    '🌧️': [':cloud_rain:', ':cloud_with_rain:'],
    '🌨️': [':cloud_snow:', ':cloud_with_snow:'],
    '🌪️': [':cloud_tornado:', ':cloud_with_tornado:'],
    '🤡': [':clown:', ':clown_face:'],
    '♣️': [':clubs:'],
    '🧥': [':coat:'],
    '🪳': [':cockroach:'],
    '🍸': [':cocktail:'],
    '🥥': [':coconut:'],
    '☕': [':coffee:'],
    '⚰️': [':coffin:'],
    '🪙': [':coin:'],
    '🥶': [':cold_face:'],
    '😰': [':cold_sweat:'],
    '☄️': [':comet:'],
    '🧭': [':compass:'],
    '🗜️': [':compression:'],
    '💻': [':computer:'],
    '🎊': [':confetti_ball:'],
    '😖': [':confounded:'],
    '😕': [':confused:', ':-\\\\', ':-/', '-\\\\', '-/'],
    '㊗️': [':congratulations:'],
    '🚧': [':construction:'],
    '🏗️': [':construction_site:', ':building_construction:'],
    '👷': [':construction_worker:'],
    '👷🏻': [':construction_worker_tone1:', ':construction_worker::skin-tone-1:'],
    '👷🏼': [':construction_worker_tone2:', ':construction_worker::skin-tone-2:'],
    '👷🏽': [':construction_worker_tone3:', ':construction_worker::skin-tone-3:'],
    '👷🏾': [':construction_worker_tone4:', ':construction_worker::skin-tone-4:'],
    '👷🏿': [':construction_worker_tone5:', ':construction_worker::skin-tone-5:'],
    '🎛️': [':control_knobs:'],
    '🏪': [':convenience_store:'],
    '🧑‍🍳': [':cook:'],
    '🧑🏻‍🍳': [':cook_tone1:', ':cook_light_skin_tone:', ':cook::skin-tone-1:'],
    '🧑🏼‍🍳': [':cook_tone2:', ':cook_medium_light_skin_tone:', ':cook::skin-tone-2:'],
    '🧑🏽‍🍳': [':cook_tone3:', ':cook_medium_skin_tone:', ':cook::skin-tone-3:'],
    '🧑🏾‍🍳': [':cook_tone4:', ':cook_medium_dark_skin_tone:', ':cook::skin-tone-4:'],
    '🧑🏿‍🍳': [':cook_tone5:', ':cook_dark_skin_tone:', ':cook::skin-tone-5:'],
    '🍪': [':cookie:'],
    '🍳': [':cooking:'],
    '🆒': [':cool:'],
    '©️': [':copyright:'],
    '🌽': [':corn:'],
    '🛋️': [':couch:', ':couch_and_lamp:'],
    '👫': [':couple:', ':woman_and_man_holding_hands_tone1:', ':woman_and_man_holding_hands_light_skin_tone:', ':woman_and_man_holding_hands_tone1_tone2:', ':woman_and_man_holding_hands_light_skin_tone_medium_light_skin_tone:', ':woman_and_man_holding_hands_tone1_tone3:', ':woman_and_man_holding_hands_light_skin_tone_medium_skin_tone:', ':woman_and_man_holding_hands_tone1_tone4:', ':woman_and_man_holding_hands_light_skin_tone_medium_dark_skin_tone:', ':woman_and_man_holding_hands_tone1_tone5:', ':woman_and_man_holding_hands_light_skin_tone_dark_skin_tone:', ':woman_and_man_holding_hands_tone2:', ':woman_and_man_holding_hands_medium_light_skin_tone:', ':woman_and_man_holding_hands_tone2_tone1:', ':woman_and_man_holding_hands_medium_light_skin_tone_light_skin_tone:', ':woman_and_man_holding_hands_tone2_tone3:', ':woman_and_man_holding_hands_medium_light_skin_tone_medium_skin_tone:', ':woman_and_man_holding_hands_tone2_tone4:', ':woman_and_man_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:', ':woman_and_man_holding_hands_tone2_tone5:', ':woman_and_man_holding_hands_medium_light_skin_tone_dark_skin_tone:', ':woman_and_man_holding_hands_tone3:', ':woman_and_man_holding_hands_medium_skin_tone:', ':woman_and_man_holding_hands_tone3_tone1:', ':woman_and_man_holding_hands_medium_skin_tone_light_skin_tone:', ':woman_and_man_holding_hands_tone3_tone2:', ':woman_and_man_holding_hands_medium_skin_tone_medium_light_skin_tone:', ':woman_and_man_holding_hands_tone3_tone4:', ':woman_and_man_holding_hands_medium_skin_tone_medium_dark_skin_tone:', ':woman_and_man_holding_hands_tone3_tone5:', ':woman_and_man_holding_hands_medium_skin_tone_dark_skin_tone:', ':woman_and_man_holding_hands_tone4:', ':woman_and_man_holding_hands_medium_dark_skin_tone:', ':woman_and_man_holding_hands_tone4_tone1:', ':woman_and_man_holding_hands_medium_dark_skin_tone_light_skin_tone:', ':woman_and_man_holding_hands_tone4_tone2:', ':woman_and_man_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:', ':woman_and_man_holding_hands_tone4_tone3:', ':woman_and_man_holding_hands_medium_dark_skin_tone_medium_skin_tone:', ':woman_and_man_holding_hands_tone4_tone5:', ':woman_and_man_holding_hands_medium_dark_skin_tone_dark_skin_tone:', ':woman_and_man_holding_hands_tone5:', ':woman_and_man_holding_hands_dark_skin_tone:', ':woman_and_man_holding_hands_tone5_tone1:', ':woman_and_man_holding_hands_dark_skin_tone_light_skin_tone:', ':woman_and_man_holding_hands_tone5_tone2:', ':woman_and_man_holding_hands_dark_skin_tone_medium_light_skin_tone:', ':woman_and_man_holding_hands_tone5_tone3:', ':woman_and_man_holding_hands_dark_skin_tone_medium_skin_tone:', ':woman_and_man_holding_hands_tone5_tone4:', ':woman_and_man_holding_hands_dark_skin_tone_medium_dark_skin_tone:'],
    '👨‍❤️‍👨': [':couple_mm:', ':couple_with_heart_mm:'],
    '💑': [':couple_with_heart:'],
    '👩‍❤️‍👨': [':couple_with_heart_woman_man:'],
    '👩‍❤️‍👩': [':couple_ww:', ':couple_with_heart_ww:'],
    '💏': [':couplekiss:'],
    '🐮': [':cow:'],
    '🐄': [':cow2:'],
    '🤠': [':cowboy:', ':face_with_cowboy_hat:'],
    '🦀': [':crab:'],
    '🖍️': [':crayon:', ':lower_left_crayon:'],
    '💳': [':credit_card:'],
    '🌙': [':crescent_moon:'],
    '🦗': [':cricket:'],
    '🏏': [':cricket_game:', ':cricket_bat_ball:'],
    '🐊': [':crocodile:'],
    '🥐': [':croissant:'],
    '✝️': [':cross:', ':latin_cross:'],
    '🎌': [':crossed_flags:'],
    '⚔️': [':crossed_swords:'],
    '👑': [':crown:'],
    '🛳️': [':cruise_ship:', ':passenger_ship:'],
    '😢': [':cry:', ":'(", ":'-(", ':(', ':-(', "'(", "'-(", '(', '-('],
    '😿': [':crying_cat_face:'],
    '🔮': [':crystal_ball:'],
    '🥒': [':cucumber:'],
    '🥤': [':cup_with_straw:'],
    '🧁': [':cupcake:'],
    '💘': [':cupid:'],
    '🥌': [':curling_stone:'],
    '➰': [':curly_loop:'],
    '💱': [':currency_exchange:'],
    '🍛': [':curry:'],
    '🍮': [':custard:', ':pudding:', ':flan:'],
    '🛃': [':customs:'],
    '🥩': [':cut_of_meat:'],
    '🌀': [':cyclone:'],
    '🗡️': [':dagger:', ':dagger_knife:'],
    '💃': [':dancer:'],
    '💃🏻': [':dancer_tone1:', ':dancer::skin-tone-1:'],
    '💃🏼': [':dancer_tone2:', ':dancer::skin-tone-2:'],
    '💃🏽': [':dancer_tone3:', ':dancer::skin-tone-3:'],
    '💃🏾': [':dancer_tone4:', ':dancer::skin-tone-4:'],
    '💃🏿': [':dancer_tone5:', ':dancer::skin-tone-5:'],
    '🍡': [':dango:'],
    '🕶️': [':dark_sunglasses:'],
    '🎯': [':dart:'],
    '💨': [':dash:'],
    '📅': [':date:'],
    '🧏‍♂️': [':deaf_man:'],
    '🧏🏻‍♂️': [':deaf_man_tone1:', ':deaf_man_light_skin_tone:', ':deaf_man::skin-tone-1:'],
    '🧏🏼‍♂️': [':deaf_man_tone2:', ':deaf_man_medium_light_skin_tone:', ':deaf_man::skin-tone-2:'],
    '🧏🏽‍♂️': [':deaf_man_tone3:', ':deaf_man_medium_skin_tone:', ':deaf_man::skin-tone-3:'],
    '🧏🏾‍♂️': [':deaf_man_tone4:', ':deaf_man_medium_dark_skin_tone:', ':deaf_man::skin-tone-4:'],
    '🧏🏿‍♂️': [':deaf_man_tone5:', ':deaf_man_dark_skin_tone:', ':deaf_man::skin-tone-5:'],
    '🧏': [':deaf_person:'],
    '🧏🏻': [':deaf_person_tone1:', ':deaf_person_light_skin_tone:', ':deaf_person::skin-tone-1:'],
    '🧏🏼': [':deaf_person_tone2:', ':deaf_person_medium_light_skin_tone:', ':deaf_person::skin-tone-2:'],
    '🧏🏽': [':deaf_person_tone3:', ':deaf_person_medium_skin_tone:', ':deaf_person::skin-tone-3:'],
    '🧏🏾': [':deaf_person_tone4:', ':deaf_person_medium_dark_skin_tone:', ':deaf_person::skin-tone-4:'],
    '🧏🏿': [':deaf_person_tone5:', ':deaf_person_dark_skin_tone:', ':deaf_person::skin-tone-5:'],
    '🧏‍♀️': [':deaf_woman:'],
    '🧏🏻‍♀️': [':deaf_woman_tone1:', ':deaf_woman_light_skin_tone:', ':deaf_woman::skin-tone-1:'],
    '🧏🏼‍♀️': [':deaf_woman_tone2:', ':deaf_woman_medium_light_skin_tone:', ':deaf_woman::skin-tone-2:'],
    '🧏🏽‍♀️': [':deaf_woman_tone3:', ':deaf_woman_medium_skin_tone:', ':deaf_woman::skin-tone-3:'],
    '🧏🏾‍♀️': [':deaf_woman_tone4:', ':deaf_woman_medium_dark_skin_tone:', ':deaf_woman::skin-tone-4:'],
    '🧏🏿‍♀️': [':deaf_woman_tone5:', ':deaf_woman_dark_skin_tone:', ':deaf_woman::skin-tone-5:'],
    '🌳': [':deciduous_tree:'],
    '🦌': [':deer:'],
    '🏬': [':department_store:'],
    '🏜️': [':desert:'],
    '🖥️': [':desktop:', ':desktop_computer:'],
    '🕵️': [':detective:', ':spy:', ':sleuth_or_spy:'],
    '🕵🏻': [':detective_tone1:', ':spy_tone1:', ':sleuth_or_spy_tone1:', ':detective::skin-tone-1:', ':spy::skin-tone-1:', ':sleuth_or_spy::skin-tone-1:'],
    '🕵🏼': [':detective_tone2:', ':spy_tone2:', ':sleuth_or_spy_tone2:', ':detective::skin-tone-2:', ':spy::skin-tone-2:', ':sleuth_or_spy::skin-tone-2:'],
    '🕵🏽': [':detective_tone3:', ':spy_tone3:', ':sleuth_or_spy_tone3:', ':detective::skin-tone-3:', ':spy::skin-tone-3:', ':sleuth_or_spy::skin-tone-3:'],
    '🕵🏾': [':detective_tone4:', ':spy_tone4:', ':sleuth_or_spy_tone4:', ':detective::skin-tone-4:', ':spy::skin-tone-4:', ':sleuth_or_spy::skin-tone-4:'],
    '🕵🏿': [':detective_tone5:', ':spy_tone5:', ':sleuth_or_spy_tone5:', ':detective::skin-tone-5:', ':spy::skin-tone-5:', ':sleuth_or_spy::skin-tone-5:'],
    '💠': [':diamond_shape_with_a_dot_inside:'],
    '♦️': [':diamonds:'],
    '😞': [':disappointed:'],
    '😥': [':disappointed_relieved:'],
    '🥸': [':disguised_face:'],
    '🗂️': [':dividers:', ':card_index_dividers:'],
    '🤿': [':diving_mask:'],
    '🪔': [':diya_lamp:'],
    '💫': [':dizzy:'],
    '😵': [':dizzy_face:'],
    '🧬': [':dna:'],
    '🚯': [':do_not_litter:'],
    '🦤': [':dodo:'],
    '🐶': [':dog:'],
    '🐕': [':dog2:'],
    '💵': [':dollar:'],
    '🎎': [':dolls:'],
    '🐬': [':dolphin:'],
    '🚪': [':door:'],
    '🍩': [':doughnut:'],
    '🕊️': [':dove:', ':dove_of_peace:'],
    '🐉': [':dragon:'],
    '🐲': [':dragon_face:'],
    '👗': [':dress:'],
    '🐪': [':dromedary_camel:'],
    '🤤': [':drooling_face:', ':drool:'],
    '🩸': [':drop_of_blood:'],
    '💧': [':droplet:'],
    '🥁': [':drum:', ':drum_with_drumsticks:'],
    '🦆': [':duck:'],
    '🥟': [':dumpling:'],
    '📀': [':dvd:'],
    '📧': [':e_mail:', ':email:'],
    '🦅': [':eagle:'],
    '👂': [':ear:'],
    '🌾': [':ear_of_rice:'],
    '👂🏻': [':ear_tone1:', ':ear::skin-tone-1:'],
    '👂🏼': [':ear_tone2:', ':ear::skin-tone-2:'],
    '👂🏽': [':ear_tone3:', ':ear::skin-tone-3:'],
    '👂🏾': [':ear_tone4:', ':ear::skin-tone-4:'],
    '👂🏿': [':ear_tone5:', ':ear::skin-tone-5:'],
    '🦻': [':ear_with_hearing_aid:'],
    '🦻🏻': [':ear_with_hearing_aid_tone1:', ':ear_with_hearing_aid_light_skin_tone:', ':ear_with_hearing_aid::skin-tone-1:'],
    '🦻🏼': [':ear_with_hearing_aid_tone2:', ':ear_with_hearing_aid_medium_light_skin_tone:', ':ear_with_hearing_aid::skin-tone-2:'],
    '🦻🏽': [':ear_with_hearing_aid_tone3:', ':ear_with_hearing_aid_medium_skin_tone:', ':ear_with_hearing_aid::skin-tone-3:'],
    '🦻🏾': [':ear_with_hearing_aid_tone4:', ':ear_with_hearing_aid_medium_dark_skin_tone:', ':ear_with_hearing_aid::skin-tone-4:'],
    '🦻🏿': [':ear_with_hearing_aid_tone5:', ':ear_with_hearing_aid_dark_skin_tone:', ':ear_with_hearing_aid::skin-tone-5:'],
    '🌍': [':earth_africa:'],
    '🌎': [':earth_americas:'],
    '🌏': [':earth_asia:'],
    '🥚': [':egg:'],
    '🍆': [':eggplant:'],
    '8️⃣': [':eight:'],
    '✴️': [':eight_pointed_black_star:'],
    '✳️': [':eight_spoked_asterisk:'],
    '⏏️': [':eject:', ':eject_symbol:'],
    '🔌': [':electric_plug:'],
    '🐘': [':elephant:'],
    '🛗': [':elevator:'],
    '🧝': [':elf:'],
    '🧝🏻': [':elf_tone1:', ':elf_light_skin_tone:', ':elf::skin-tone-1:'],
    '🧝🏼': [':elf_tone2:', ':elf_medium_light_skin_tone:', ':elf::skin-tone-2:'],
    '🧝🏽': [':elf_tone3:', ':elf_medium_skin_tone:', ':elf::skin-tone-3:'],
    '🧝🏾': [':elf_tone4:', ':elf_medium_dark_skin_tone:', ':elf::skin-tone-4:'],
    '🧝🏿': [':elf_tone5:', ':elf_dark_skin_tone:', ':elf::skin-tone-5:'],
    '🔚': [':end:'],
    '🏴󠁧󠁢󠁥󠁮󠁧󠁿': [':england:'],
    '✉️': [':envelope:'],
    '📩': [':envelope_with_arrow:'],
    '💶': [':euro:'],
    '🏰': [':european_castle:'],
    '🏤': [':european_post_office:'],
    '🌲': [':evergreen_tree:'],
    '❗': [':exclamation:'],
    '🤯': [':exploding_head:'],
    '😑': [':expressionless:'],
    '👁️': [':eye:'],
    '👁‍🗨': [':eye_in_speech_bubble:'],
    '👓': [':eyeglasses:'],
    '👀': [':eyes:'],
    '🤮': [':face_vomiting:'],
    '🤭': [':face_with_hand_over_mouth:'],
    '🧐': [':face_with_monocle:'],
    '🤨': [':face_with_raised_eyebrow:'],
    '🤬': [':face_with_symbols_over_mouth:'],
    '🏭': [':factory:'],
    '🧑‍🏭': [':factory_worker:'],
    '🧑🏻‍🏭': [':factory_worker_tone1:', ':factory_worker_light_skin_tone:', ':factory_worker::skin-tone-1:'],
    '🧑🏼‍🏭': [':factory_worker_tone2:', ':factory_worker_medium_light_skin_tone:', ':factory_worker::skin-tone-2:'],
    '🧑🏽‍🏭': [':factory_worker_tone3:', ':factory_worker_medium_skin_tone:', ':factory_worker::skin-tone-3:'],
    '🧑🏾‍🏭': [':factory_worker_tone4:', ':factory_worker_medium_dark_skin_tone:', ':factory_worker::skin-tone-4:'],
    '🧑🏿‍🏭': [':factory_worker_tone5:', ':factory_worker_dark_skin_tone:', ':factory_worker::skin-tone-5:'],
    '🧚': [':fairy:'],
    '🧚🏻': [':fairy_tone1:', ':fairy_light_skin_tone:', ':fairy::skin-tone-1:'],
    '🧚🏼': [':fairy_tone2:', ':fairy_medium_light_skin_tone:', ':fairy::skin-tone-2:'],
    '🧚🏽': [':fairy_tone3:', ':fairy_medium_skin_tone:', ':fairy::skin-tone-3:'],
    '🧚🏾': [':fairy_tone4:', ':fairy_medium_dark_skin_tone:', ':fairy::skin-tone-4:'],
    '🧚🏿': [':fairy_tone5:', ':fairy_dark_skin_tone:', ':fairy::skin-tone-5:'],
    '🧆': [':falafel:'],
    '🍂': [':fallen_leaf:'],
    '👪': [':family:'],
    '👨‍👦': [':family_man_boy:'],
    '👨‍👦‍👦': [':family_man_boy_boy:'],
    '👨‍👧': [':family_man_girl:'],
    '👨‍👧‍👦': [':family_man_girl_boy:'],
    '👨‍👧‍👧': [':family_man_girl_girl:'],
    '👨‍👩‍👦': [':family_man_woman_boy:'],
    '👨‍👨‍👦': [':family_mmb:'],
    '👨‍👨‍👦‍👦': [':family_mmbb:'],
    '👨‍👨‍👧': [':family_mmg:'],
    '👨‍👨‍👧‍👦': [':family_mmgb:'],
    '👨‍👨‍👧‍👧': [':family_mmgg:'],
    '👨‍👩‍👦‍👦': [':family_mwbb:'],
    '👨‍👩‍👧': [':family_mwg:'],
    '👨‍👩‍👧‍👦': [':family_mwgb:'],
    '👨‍👩‍👧‍👧': [':family_mwgg:'],
    '👩‍👦': [':family_woman_boy:'],
    '👩‍👦‍👦': [':family_woman_boy_boy:'],
    '👩‍👧': [':family_woman_girl:'],
    '👩‍👧‍👦': [':family_woman_girl_boy:'],
    '👩‍👧‍👧': [':family_woman_girl_girl:'],
    '👩‍👩‍👦': [':family_wwb:'],
    '👩‍👩‍👦‍👦': [':family_wwbb:'],
    '👩‍👩‍👧': [':family_wwg:'],
    '👩‍👩‍👧‍👦': [':family_wwgb:'],
    '👩‍👩‍👧‍👧': [':family_wwgg:'],
    '🧑‍🌾': [':farmer:'],
    '🧑🏻‍🌾': [':farmer_tone1:', ':farmer_light_skin_tone:', ':farmer::skin-tone-1:'],
    '🧑🏼‍🌾': [':farmer_tone2:', ':farmer_medium_light_skin_tone:', ':farmer::skin-tone-2:'],
    '🧑🏽‍🌾': [':farmer_tone3:', ':farmer_medium_skin_tone:', ':farmer::skin-tone-3:'],
    '🧑🏾‍🌾': [':farmer_tone4:', ':farmer_medium_dark_skin_tone:', ':farmer::skin-tone-4:'],
    '🧑🏿‍🌾': [':farmer_tone5:', ':farmer_dark_skin_tone:', ':farmer::skin-tone-5:'],
    '⏩': [':fast_forward:'],
    '📠': [':fax:'],
    '😨': [':fearful:'],
    '🪶': [':feather:'],
    '🐾': [':feet:', ':paw_prints:'],
    '♀️': [':female_sign:'],
    '🎡': [':ferris_wheel:'],
    '⛴️': [':ferry:'],
    '🏑': [':field_hockey:'],
    '🗄️': [':file_cabinet:'],
    '📁': [':file_folder:'],
    '🎞️': [':film_frames:'],
    '🤞': [':fingers_crossed:', ':hand_with_index_and_middle_finger_crossed:'],
    '🤞🏻': [':fingers_crossed_tone1:', ':hand_with_index_and_middle_fingers_crossed_tone1:', ':fingers_crossed::skin-tone-1:', ':hand_with_index_and_middle_finger_crossed::skin-tone-1:'],
    '🤞🏼': [':fingers_crossed_tone2:', ':hand_with_index_and_middle_fingers_crossed_tone2:', ':fingers_crossed::skin-tone-2:', ':hand_with_index_and_middle_finger_crossed::skin-tone-2:'],
    '🤞🏽': [':fingers_crossed_tone3:', ':hand_with_index_and_middle_fingers_crossed_tone3:', ':fingers_crossed::skin-tone-3:', ':hand_with_index_and_middle_finger_crossed::skin-tone-3:'],
    '🤞🏾': [':fingers_crossed_tone4:', ':hand_with_index_and_middle_fingers_crossed_tone4:', ':fingers_crossed::skin-tone-4:', ':hand_with_index_and_middle_finger_crossed::skin-tone-4:'],
    '🤞🏿': [':fingers_crossed_tone5:', ':hand_with_index_and_middle_fingers_crossed_tone5:', ':fingers_crossed::skin-tone-5:', ':hand_with_index_and_middle_finger_crossed::skin-tone-5:'],
    '🔥': [':fire:', ':flame:'],
    '🚒': [':fire_engine:'],
    '🧯': [':fire_extinguisher:'],
    '🧨': [':firecracker:'],
    '🧑‍🚒': [':firefighter:'],
    '🧑🏻‍🚒': [':firefighter_tone1:', ':firefighter_light_skin_tone:', ':firefighter::skin-tone-1:'],
    '🧑🏼‍🚒': [':firefighter_tone2:', ':firefighter_medium_light_skin_tone:', ':firefighter::skin-tone-2:'],
    '🧑🏽‍🚒': [':firefighter_tone3:', ':firefighter_medium_skin_tone:', ':firefighter::skin-tone-3:'],
    '🧑🏾‍🚒': [':firefighter_tone4:', ':firefighter_medium_dark_skin_tone:', ':firefighter::skin-tone-4:'],
    '🧑🏿‍🚒': [':firefighter_tone5:', ':firefighter_dark_skin_tone:', ':firefighter::skin-tone-5:'],
    '🎆': [':fireworks:'],
    '🥇': [':first_place:', ':first_place_medal:'],
    '🌓': [':first_quarter_moon:'],
    '🌛': [':first_quarter_moon_with_face:'],
    '🐟': [':fish:'],
    '🍥': [':fish_cake:'],
    '🎣': [':fishing_pole_and_fish:'],
    '✊': [':fist:'],
    '✊🏻': [':fist_tone1:', ':fist::skin-tone-1:'],
    '✊🏼': [':fist_tone2:', ':fist::skin-tone-2:'],
    '✊🏽': [':fist_tone3:', ':fist::skin-tone-3:'],
    '✊🏾': [':fist_tone4:', ':fist::skin-tone-4:'],
    '✊🏿': [':fist_tone5:', ':fist::skin-tone-5:'],
    '5️⃣': [':five:'],
    '🇦🇨': [':flag_ac:'],
    '🇦🇩': [':flag_ad:'],
    '🇦🇪': [':flag_ae:'],
    '🇦🇫': [':flag_af:'],
    '🇦🇬': [':flag_ag:'],
    '🇦🇮': [':flag_ai:'],
    '🇦🇱': [':flag_al:'],
    '🇦🇲': [':flag_am:'],
    '🇦🇴': [':flag_ao:'],
    '🇦🇶': [':flag_aq:'],
    '🇦🇷': [':flag_ar:'],
    '🇦🇸': [':flag_as:'],
    '🇦🇹': [':flag_at:'],
    '🇦🇺': [':flag_au:'],
    '🇦🇼': [':flag_aw:'],
    '🇦🇽': [':flag_ax:'],
    '🇦🇿': [':flag_az:'],
    '🇧🇦': [':flag_ba:'],
    '🇧🇧': [':flag_bb:'],
    '🇧🇩': [':flag_bd:'],
    '🇧🇪': [':flag_be:'],
    '🇧🇫': [':flag_bf:'],
    '🇧🇬': [':flag_bg:'],
    '🇧🇭': [':flag_bh:'],
    '🇧🇮': [':flag_bi:'],
    '🇧🇯': [':flag_bj:'],
    '🇧🇱': [':flag_bl:'],
    '🏴': [':flag_black:'],
    '🇧🇲': [':flag_bm:'],
    '🇧🇳': [':flag_bn:'],
    '🇧🇴': [':flag_bo:'],
    '🇧🇶': [':flag_bq:'],
    '🇧🇷': [':flag_br:'],
    '🇧🇸': [':flag_bs:'],
    '🇧🇹': [':flag_bt:'],
    '🇧🇻': [':flag_bv:'],
    '🇧🇼': [':flag_bw:'],
    '🇧🇾': [':flag_by:'],
    '🇧🇿': [':flag_bz:'],
    '🇨🇦': [':flag_ca:'],
    '🇨🇨': [':flag_cc:'],
    '🇨🇩': [':flag_cd:'],
    '🇨🇫': [':flag_cf:'],
    '🇨🇬': [':flag_cg:'],
    '🇨🇭': [':flag_ch:'],
    '🇨🇮': [':flag_ci:'],
    '🇨🇰': [':flag_ck:'],
    '🇨🇱': [':flag_cl:'],
    '🇨🇲': [':flag_cm:'],
    '🇨🇳': [':flag_cn:'],
    '🇨🇴': [':flag_co:'],
    '🇨🇵': [':flag_cp:'],
    '🇨🇷': [':flag_cr:'],
    '🇨🇺': [':flag_cu:'],
    '🇨🇻': [':flag_cv:'],
    '🇨🇼': [':flag_cw:'],
    '🇨🇽': [':flag_cx:'],
    '🇨🇾': [':flag_cy:'],
    '🇨🇿': [':flag_cz:'],
    '🇩🇪': [':flag_de:'],
    '🇩🇬': [':flag_dg:'],
    '🇩🇯': [':flag_dj:'],
    '🇩🇰': [':flag_dk:'],
    '🇩🇲': [':flag_dm:'],
    '🇩🇴': [':flag_do:'],
    '🇩🇿': [':flag_dz:'],
    '🇪🇦': [':flag_ea:'],
    '🇪🇨': [':flag_ec:'],
    '🇪🇪': [':flag_ee:'],
    '🇪🇬': [':flag_eg:'],
    '🇪🇭': [':flag_eh:'],
    '🇪🇷': [':flag_er:'],
    '🇪🇸': [':flag_es:'],
    '🇪🇹': [':flag_et:'],
    '🇪🇺': [':flag_eu:'],
    '🇫🇮': [':flag_fi:'],
    '🇫🇯': [':flag_fj:'],
    '🇫🇰': [':flag_fk:'],
    '🇫🇲': [':flag_fm:'],
    '🇫🇴': [':flag_fo:'],
    '🇫🇷': [':flag_fr:'],
    '🇬🇦': [':flag_ga:'],
    '🇬🇧': [':flag_gb:'],
    '🇬🇩': [':flag_gd:'],
    '🇬🇪': [':flag_ge:'],
    '🇬🇫': [':flag_gf:'],
    '🇬🇬': [':flag_gg:'],
    '🇬🇭': [':flag_gh:'],
    '🇬🇮': [':flag_gi:'],
    '🇬🇱': [':flag_gl:'],
    '🇬🇲': [':flag_gm:'],
    '🇬🇳': [':flag_gn:'],
    '🇬🇵': [':flag_gp:'],
    '🇬🇶': [':flag_gq:'],
    '🇬🇷': [':flag_gr:'],
    '🇬🇸': [':flag_gs:'],
    '🇬🇹': [':flag_gt:'],
    '🇬🇺': [':flag_gu:'],
    '🇬🇼': [':flag_gw:'],
    '🇬🇾': [':flag_gy:'],
    '🇭🇰': [':flag_hk:'],
    '🇭🇲': [':flag_hm:'],
    '🇭🇳': [':flag_hn:'],
    '🇭🇷': [':flag_hr:'],
    '🇭🇹': [':flag_ht:'],
    '🇭🇺': [':flag_hu:'],
    '🇮🇨': [':flag_ic:'],
    '🇮🇩': [':flag_id:'],
    '🇮🇪': [':flag_ie:'],
    '🇮🇱': [':flag_il:'],
    '🇮🇲': [':flag_im:'],
    '🇮🇳': [':flag_in:'],
    '🇮🇴': [':flag_io:'],
    '🇮🇶': [':flag_iq:'],
    '🇮🇷': [':flag_ir:'],
    '🇮🇸': [':flag_is:'],
    '🇮🇹': [':flag_it:'],
    '🇯🇪': [':flag_je:'],
    '🇯🇲': [':flag_jm:'],
    '🇯🇴': [':flag_jo:'],
    '🇯🇵': [':flag_jp:'],
    '🇰🇪': [':flag_ke:'],
    '🇰🇬': [':flag_kg:'],
    '🇰🇭': [':flag_kh:'],
    '🇰🇮': [':flag_ki:'],
    '🇰🇲': [':flag_km:'],
    '🇰🇳': [':flag_kn:'],
    '🇰🇵': [':flag_kp:'],
    '🇰🇷': [':flag_kr:'],
    '🇰🇼': [':flag_kw:'],
    '🇰🇾': [':flag_ky:'],
    '🇰🇿': [':flag_kz:'],
    '🇱🇦': [':flag_la:'],
    '🇱🇧': [':flag_lb:'],
    '🇱🇨': [':flag_lc:'],
    '🇱🇮': [':flag_li:'],
    '🇱🇰': [':flag_lk:'],
    '🇱🇷': [':flag_lr:'],
    '🇱🇸': [':flag_ls:'],
    '🇱🇹': [':flag_lt:'],
    '🇱🇺': [':flag_lu:'],
    '🇱🇻': [':flag_lv:'],
    '🇱🇾': [':flag_ly:'],
    '🇲🇦': [':flag_ma:'],
    '🇲🇨': [':flag_mc:'],
    '🇲🇩': [':flag_md:'],
    '🇲🇪': [':flag_me:'],
    '🇲🇫': [':flag_mf:'],
    '🇲🇬': [':flag_mg:'],
    '🇲🇭': [':flag_mh:'],
    '🇲🇰': [':flag_mk:'],
    '🇲🇱': [':flag_ml:'],
    '🇲🇲': [':flag_mm:'],
    '🇲🇳': [':flag_mn:'],
    '🇲🇴': [':flag_mo:'],
    '🇲🇵': [':flag_mp:'],
    '🇲🇶': [':flag_mq:'],
    '🇲🇷': [':flag_mr:'],
    '🇲🇸': [':flag_ms:'],
    '🇲🇹': [':flag_mt:'],
    '🇲🇺': [':flag_mu:'],
    '🇲🇻': [':flag_mv:'],
    '🇲🇼': [':flag_mw:'],
    '🇲🇽': [':flag_mx:'],
    '🇲🇾': [':flag_my:'],
    '🇲🇿': [':flag_mz:'],
    '🇳🇦': [':flag_na:'],
    '🇳🇨': [':flag_nc:'],
    '🇳🇪': [':flag_ne:'],
    '🇳🇫': [':flag_nf:'],
    '🇳🇬': [':flag_ng:'],
    '🇳🇮': [':flag_ni:'],
    '🇳🇱': [':flag_nl:'],
    '🇳🇴': [':flag_no:'],
    '🇳🇵': [':flag_np:'],
    '🇳🇷': [':flag_nr:'],
    '🇳🇺': [':flag_nu:'],
    '🇳🇿': [':flag_nz:'],
    '🇴🇲': [':flag_om:'],
    '🇵🇦': [':flag_pa:'],
    '🇵🇪': [':flag_pe:'],
    '🇵🇫': [':flag_pf:'],
    '🇵🇬': [':flag_pg:'],
    '🇵🇭': [':flag_ph:'],
    '🇵🇰': [':flag_pk:'],
    '🇵🇱': [':flag_pl:'],
    '🇵🇲': [':flag_pm:'],
    '🇵🇳': [':flag_pn:'],
    '🇵🇷': [':flag_pr:'],
    '🇵🇸': [':flag_ps:'],
    '🇵🇹': [':flag_pt:'],
    '🇵🇼': [':flag_pw:'],
    '🇵🇾': [':flag_py:'],
    '🇶🇦': [':flag_qa:'],
    '🇷🇪': [':flag_re:'],
    '🇷🇴': [':flag_ro:'],
    '🇷🇸': [':flag_rs:'],
    '🇷🇺': [':flag_ru:'],
    '🇷🇼': [':flag_rw:'],
    '🇸🇦': [':flag_sa:'],
    '🇸🇧': [':flag_sb:'],
    '🇸🇨': [':flag_sc:'],
    '🇸🇩': [':flag_sd:'],
    '🇸🇪': [':flag_se:'],
    '🇸🇬': [':flag_sg:'],
    '🇸🇭': [':flag_sh:'],
    '🇸🇮': [':flag_si:'],
    '🇸🇯': [':flag_sj:'],
    '🇸🇰': [':flag_sk:'],
    '🇸🇱': [':flag_sl:'],
    '🇸🇲': [':flag_sm:'],
    '🇸🇳': [':flag_sn:'],
    '🇸🇴': [':flag_so:'],
    '🇸🇷': [':flag_sr:'],
    '🇸🇸': [':flag_ss:'],
    '🇸🇹': [':flag_st:'],
    '🇸🇻': [':flag_sv:'],
    '🇸🇽': [':flag_sx:'],
    '🇸🇾': [':flag_sy:'],
    '🇸🇿': [':flag_sz:'],
    '🇹🇦': [':flag_ta:'],
    '🇹🇨': [':flag_tc:'],
    '🇹🇩': [':flag_td:'],
    '🇹🇫': [':flag_tf:'],
    '🇹🇬': [':flag_tg:'],
    '🇹🇭': [':flag_th:'],
    '🇹🇯': [':flag_tj:'],
    '🇹🇰': [':flag_tk:'],
    '🇹🇱': [':flag_tl:'],
    '🇹🇲': [':flag_tm:'],
    '🇹🇳': [':flag_tn:'],
    '🇹🇴': [':flag_to:'],
    '🇹🇷': [':flag_tr:'],
    '🇹🇹': [':flag_tt:'],
    '🇹🇻': [':flag_tv:'],
    '🇹🇼': [':flag_tw:'],
    '🇹🇿': [':flag_tz:'],
    '🇺🇦': [':flag_ua:'],
    '🇺🇬': [':flag_ug:'],
    '🇺🇲': [':flag_um:'],
    '🇺🇸': [':flag_us:'],
    '🇺🇾': [':flag_uy:'],
    '🇺🇿': [':flag_uz:'],
    '🇻🇦': [':flag_va:'],
    '🇻🇨': [':flag_vc:'],
    '🇻🇪': [':flag_ve:'],
    '🇻🇬': [':flag_vg:'],
    '🇻🇮': [':flag_vi:'],
    '🇻🇳': [':flag_vn:'],
    '🇻🇺': [':flag_vu:'],
    '🇼🇫': [':flag_wf:'],
    '🏳️': [':flag_white:'],
    '🇼🇸': [':flag_ws:'],
    '🇽🇰': [':flag_xk:'],
    '🇾🇪': [':flag_ye:'],
    '🇾🇹': [':flag_yt:'],
    '🇿🇦': [':flag_za:'],
    '🇿🇲': [':flag_zm:'],
    '🇿🇼': [':flag_zw:'],
    '🎏': [':flags:'],
    '🦩': [':flamingo:'],
    '🔦': [':flashlight:'],
    '🫓': [':flatbread:'],
    '⚜️': [':fleur_de_lis:'],
    '💾': [':floppy_disk:'],
    '🎴': [':flower_playing_cards:'],
    '😳': [':flushed:'],
    '🪰': [':fly:'],
    '🥏': [':flying_disc:'],
    '🛸': [':flying_saucer:'],
    '🌫️': [':fog:'],
    '🌁': [':foggy:'],
    '🫕': [':fondue:'],
    '🦶': [':foot:'],
    '🦶🏻': [':foot_tone1:', ':foot_light_skin_tone:', ':foot::skin-tone-1:'],
    '🦶🏼': [':foot_tone2:', ':foot_medium_light_skin_tone:', ':foot::skin-tone-2:'],
    '🦶🏽': [':foot_tone3:', ':foot_medium_skin_tone:', ':foot::skin-tone-3:'],
    '🦶🏾': [':foot_tone4:', ':foot_medium_dark_skin_tone:', ':foot::skin-tone-4:'],
    '🦶🏿': [':foot_tone5:', ':foot_dark_skin_tone:', ':foot::skin-tone-5:'],
    '🏈': [':football:'],
    '👣': [':footprints:'],
    '🍴': [':fork_and_knife:'],
    '🍽️': [':fork_knife_plate:', ':fork_and_knife_with_plate:'],
    '🥠': [':fortune_cookie:'],
    '⛲': [':fountain:'],
    '4️⃣': [':four:'],
    '🍀': [':four_leaf_clover:'],
    '🦊': [':fox:', ':fox_face:'],
    '🖼️': [':frame_photo:', ':frame_with_picture:'],
    '🆓': [':free:'],
    '🥖': [':french_bread:', ':baguette_bread:'],
    '🍤': [':fried_shrimp:'],
    '🍟': [':fries:'],
    '🐸': [':frog:'],
    '😦': [':frowning:', ':(', ':-(', '(', '-('],
    '☹️': [':frowning2:', ':white_frowning_face:'],
    '⛽': [':fuelpump:'],
    '🌕': [':full_moon:'],
    '🌝': [':full_moon_with_face:'],
    '🎲': [':game_die:'],
    '🧄': [':garlic:'],
    '⚙️': [':gear:'],
    '💎': [':gem:'],
    '♊': [':gemini:'],
    '🧞': [':genie:'],
    '👻': [':ghost:'],
    '🎁': [':gift:'],
    '💝': [':gift_heart:'],
    '🦒': [':giraffe:'],
    '👧': [':girl:'],
    '👧🏻': [':girl_tone1:', ':girl::skin-tone-1:'],
    '👧🏼': [':girl_tone2:', ':girl::skin-tone-2:'],
    '👧🏽': [':girl_tone3:', ':girl::skin-tone-3:'],
    '👧🏾': [':girl_tone4:', ':girl::skin-tone-4:'],
    '👧🏿': [':girl_tone5:', ':girl::skin-tone-5:'],
    '🌐': [':globe_with_meridians:'],
    '🧤': [':gloves:'],
    '🥅': [':goal:', ':goal_net:'],
    '🐐': [':goat:'],
    '🥽': [':goggles:'],
    '⛳': [':golf:'],
    '🦍': [':gorilla:'],
    '🍇': [':grapes:'],
    '🍏': [':green_apple:'],
    '📗': [':green_book:'],
    '🟢': [':green_circle:'],
    '💚': [':green_heart:'],
    '🟩': [':green_square:'],
    '❕': [':grey_exclamation:'],
    '❔': [':grey_question:'],
    '😬': [':grimacing:'],
    '😁': [':grin:'],
    '😀': [':grinning:'],
    '💂': [':guard:', ':guardsman:'],
    '💂🏻': [':guard_tone1:', ':guardsman_tone1:', ':guard::skin-tone-1:', ':guardsman::skin-tone-1:'],
    '💂🏼': [':guard_tone2:', ':guardsman_tone2:', ':guard::skin-tone-2:', ':guardsman::skin-tone-2:'],
    '💂🏽': [':guard_tone3:', ':guardsman_tone3:', ':guard::skin-tone-3:', ':guardsman::skin-tone-3:'],
    '💂🏾': [':guard_tone4:', ':guardsman_tone4:', ':guard::skin-tone-4:', ':guardsman::skin-tone-4:'],
    '💂🏿': [':guard_tone5:', ':guardsman_tone5:', ':guard::skin-tone-5:', ':guardsman::skin-tone-5:'],
    '🦮': [':guide_dog:'],
    '🎸': [':guitar:'],
    '🔫': [':gun:'],
    '🍔': [':hamburger:'],
    '🔨': [':hammer:'],
    '⚒️': [':hammer_pick:', ':hammer_and_pick:'],
    '🐹': [':hamster:'],
    '🖐️': [':hand_splayed:', ':raised_hand_with_fingers_splayed:'],
    '🖐🏻': [':hand_splayed_tone1:', ':raised_hand_with_fingers_splayed_tone1:', ':hand_splayed::skin-tone-1:', ':raised_hand_with_fingers_splayed::skin-tone-1:'],
    '🖐🏼': [':hand_splayed_tone2:', ':raised_hand_with_fingers_splayed_tone2:', ':hand_splayed::skin-tone-2:', ':raised_hand_with_fingers_splayed::skin-tone-2:'],
    '🖐🏽': [':hand_splayed_tone3:', ':raised_hand_with_fingers_splayed_tone3:', ':hand_splayed::skin-tone-3:', ':raised_hand_with_fingers_splayed::skin-tone-3:'],
    '🖐🏾': [':hand_splayed_tone4:', ':raised_hand_with_fingers_splayed_tone4:', ':hand_splayed::skin-tone-4:', ':raised_hand_with_fingers_splayed::skin-tone-4:'],
    '🖐🏿': [':hand_splayed_tone5:', ':raised_hand_with_fingers_splayed_tone5:', ':hand_splayed::skin-tone-5:', ':raised_hand_with_fingers_splayed::skin-tone-5:'],
    '👜': [':handbag:'],
    '🤝': [':handshake:', ':shaking_hands:'],
    '#️⃣': [':hash:'],
    '🐥': [':hatched_chick:'],
    '🐣': [':hatching_chick:'],
    '🤕': [':head_bandage:', ':face_with_head_bandage:'],
    '🎧': [':headphones:'],
    '🪦': [':headstone:'],
    '🧑‍⚕️': [':health_worker:'],
    '🧑🏻‍⚕️': [':health_worker_tone1:', ':health_worker_light_skin_tone:', ':health_worker::skin-tone-1:'],
    '🧑🏼‍⚕️': [':health_worker_tone2:', ':health_worker_medium_light_skin_tone:', ':health_worker::skin-tone-2:'],
    '🧑🏽‍⚕️': [':health_worker_tone3:', ':health_worker_medium_skin_tone:', ':health_worker::skin-tone-3:'],
    '🧑🏾‍⚕️': [':health_worker_tone4:', ':health_worker_medium_dark_skin_tone:', ':health_worker::skin-tone-4:'],
    '🧑🏿‍⚕️': [':health_worker_tone5:', ':health_worker_dark_skin_tone:', ':health_worker::skin-tone-5:'],
    '🙉': [':hear_no_evil:'],
    '❤️': [':heart:', '<3', '♡'],
    '💟': [':heart_decoration:'],
    '❣️': [':heart_exclamation:', ':heavy_heart_exclamation_mark_ornament:'],
    '😍': [':heart_eyes:'],
    '😻': [':heart_eyes_cat:'],
    '💓': [':heartbeat:'],
    '💗': [':heartpulse:'],
    '♥️': [':hearts:'],
    '✔️': [':heavy_check_mark:'],
    '➗': [':heavy_division_sign:'],
    '💲': [':heavy_dollar_sign:'],
    '➖': [':heavy_minus_sign:'],
    '✖️': [':heavy_multiplication_x:'],
    '➕': [':heavy_plus_sign:'],
    '🦔': [':hedgehog:'],
    '🚁': [':helicopter:'],
    '⛑️': [':helmet_with_cross:', ':helmet_with_white_cross:'],
    '🌿': [':herb:'],
    '🌺': [':hibiscus:'],
    '🔆': [':high_brightness:'],
    '👠': [':high_heel:'],
    '🥾': [':hiking_boot:'],
    '🛕': [':hindu_temple:'],
    '🦛': [':hippopotamus:'],
    '🏒': [':hockey:'],
    '🕳️': [':hole:'],
    '🏘️': [':homes:', ':house_buildings:'],
    '🍯': [':honey_pot:'],
    '🪝': [':hook:'],
    '🐴': [':horse:'],
    '🏇': [':horse_racing:'],
    '🏇🏻': [':horse_racing_tone1:', ':horse_racing::skin-tone-1:'],
    '🏇🏼': [':horse_racing_tone2:', ':horse_racing::skin-tone-2:'],
    '🏇🏽': [':horse_racing_tone3:', ':horse_racing::skin-tone-3:'],
    '🏇🏾': [':horse_racing_tone4:', ':horse_racing::skin-tone-4:'],
    '🏇🏿': [':horse_racing_tone5:', ':horse_racing::skin-tone-5:'],
    '🏥': [':hospital:'],
    '🥵': [':hot_face:'],
    '🌶️': [':hot_pepper:'],
    '🌭': [':hotdog:', ':hot_dog:'],
    '🏨': [':hotel:'],
    '♨️': [':hotsprings:'],
    '⌛': [':hourglass:'],
    '⏳': [':hourglass_flowing_sand:'],
    '🏠': [':house:'],
    '🏚️': [':house_abandoned:', ':derelict_house_building:'],
    '🏡': [':house_with_garden:'],
    '🤗': [':hugging:', ':hugging_face:'],
    '😯': [':hushed:'],
    '🛖': [':hut:'],
    '🍨': [':ice_cream:'],
    '🧊': [':ice_cube:'],
    '⛸️': [':ice_skate:'],
    '🍦': [':icecream:'],
    '🆔': [':id:'],
    '🉐': [':ideograph_advantage:'],
    '👿': [':imp:', ':(', ':-(', '(', '-('],
    '📥': [':inbox_tray:'],
    '📨': [':incoming_envelope:'],
    '♾️': [':infinity:'],
    'ℹ️': [':information_source:'],
    '😇': [':innocent:', 'o:)', 'O:)', 'o:-)', 'O:-)', '0:)', '0:-)', 'o)', 'O)', 'o-)', 'O-)', '0)', '0-)'],
    '⁉️': [':interrobang:'],
    '🏝️': [':island:', ':desert_island:'],
    '🏮': [':izakaya_lantern:'],
    '🎃': [':jack_o_lantern:'],
    '🗾': [':japan:'],
    '🏯': [':japanese_castle:'],
    '👺': [':japanese_goblin:'],
    '👹': [':japanese_ogre:'],
    '👖': [':jeans:'],
    '🧩': [':jigsaw:'],
    '😂': [':joy:', ":')", ":'-)", ':)', ':-)', ":'D", ":'-D", ':D', ':-D', "')", "'-)", ')', '-)', "'D", "'-D", 'D', '-D'],
    '😹': [':joy_cat:'],
    '🕹️': [':joystick:'],
    '🧑‍⚖️': [':judge:'],
    '🧑🏻‍⚖️': [':judge_tone1:', ':judge_light_skin_tone:', ':judge::skin-tone-1:'],
    '🧑🏼‍⚖️': [':judge_tone2:', ':judge_medium_light_skin_tone:', ':judge::skin-tone-2:'],
    '🧑🏽‍⚖️': [':judge_tone3:', ':judge_medium_skin_tone:', ':judge::skin-tone-3:'],
    '🧑🏾‍⚖️': [':judge_tone4:', ':judge_medium_dark_skin_tone:', ':judge::skin-tone-4:'],
    '🧑🏿‍⚖️': [':judge_tone5:', ':judge_dark_skin_tone:', ':judge::skin-tone-5:'],
    '🕋': [':kaaba:'],
    '🦘': [':kangaroo:'],
    '🔑': [':key:'],
    '🗝️': [':key2:', ':old_key:'],
    '⌨️': [':keyboard:'],
    '🔟': [':keycap_ten:'],
    '👘': [':kimono:'],
    '💋': [':kiss:'],
    '👨‍❤️‍💋‍👨': [':kiss_mm:', ':couplekiss_mm:'],
    '👩‍❤️‍💋‍👨': [':kiss_woman_man:'],
    '👩‍❤️‍💋‍👩': [':kiss_ww:', ':couplekiss_ww:'],
    '😗': [':kissing:', ':*', ':-*', '*', '-*'],
    '😽': [':kissing_cat:'],
    '😚': [':kissing_closed_eyes:'],
    '😘': [':kissing_heart:'],
    '😙': [':kissing_smiling_eyes:'],
    '🪁': [':kite:'],
    '🥝': [':kiwi:', ':kiwifruit:'],
    '🔪': [':knife:'],
    '🪢': [':knot:'],
    '🐨': [':koala:'],
    '🈁': [':koko:'],
    '🥼': [':lab_coat:'],
    '🏷️': [':label:'],
    '🥍': [':lacrosse:'],
    '🪜': [':ladder:'],
    '🐞': [':lady_beetle:'],
    '🔷': [':large_blue_diamond:'],
    '🔶': [':large_orange_diamond:'],
    '🌗': [':last_quarter_moon:'],
    '🌜': [':last_quarter_moon_with_face:'],
    '😆': [':laughing:', ':satisfied:', 'x-)', 'X-)'],
    '🥬': [':leafy_green:'],
    '🍃': [':leaves:'],
    '📒': [':ledger:'],
    '🤛': [':left_facing_fist:', ':left_fist:'],
    '🤛🏻': [':left_facing_fist_tone1:', ':left_fist_tone1:', ':left_facing_fist::skin-tone-1:', ':left_fist::skin-tone-1:'],
    '🤛🏼': [':left_facing_fist_tone2:', ':left_fist_tone2:', ':left_facing_fist::skin-tone-2:', ':left_fist::skin-tone-2:'],
    '🤛🏽': [':left_facing_fist_tone3:', ':left_fist_tone3:', ':left_facing_fist::skin-tone-3:', ':left_fist::skin-tone-3:'],
    '🤛🏾': [':left_facing_fist_tone4:', ':left_fist_tone4:', ':left_facing_fist::skin-tone-4:', ':left_fist::skin-tone-4:'],
    '🤛🏿': [':left_facing_fist_tone5:', ':left_fist_tone5:', ':left_facing_fist::skin-tone-5:', ':left_fist::skin-tone-5:'],
    '🛅': [':left_luggage:'],
    '↔️': [':left_right_arrow:'],
    '↩️': [':leftwards_arrow_with_hook:'],
    '🦵': [':leg:'],
    '🦵🏻': [':leg_tone1:', ':leg_light_skin_tone:', ':leg::skin-tone-1:'],
    '🦵🏼': [':leg_tone2:', ':leg_medium_light_skin_tone:', ':leg::skin-tone-2:'],
    '🦵🏽': [':leg_tone3:', ':leg_medium_skin_tone:', ':leg::skin-tone-3:'],
    '🦵🏾': [':leg_tone4:', ':leg_medium_dark_skin_tone:', ':leg::skin-tone-4:'],
    '🦵🏿': [':leg_tone5:', ':leg_dark_skin_tone:', ':leg::skin-tone-5:'],
    '🍋': [':lemon:'],
    '♌': [':leo:'],
    '🐆': [':leopard:'],
    '🎚️': [':level_slider:'],
    '🕴️': [':levitate:', ':man_in_business_suit_levitating:'],
    '🕴🏻': [':levitate_tone1:', ':man_in_business_suit_levitating_tone1:', ':man_in_business_suit_levitating_light_skin_tone:', ':levitate::skin-tone-1:', ':man_in_business_suit_levitating::skin-tone-1:'],
    '🕴🏼': [':levitate_tone2:', ':man_in_business_suit_levitating_tone2:', ':man_in_business_suit_levitating_medium_light_skin_tone:', ':levitate::skin-tone-2:', ':man_in_business_suit_levitating::skin-tone-2:'],
    '🕴🏽': [':levitate_tone3:', ':man_in_business_suit_levitating_tone3:', ':man_in_business_suit_levitating_medium_skin_tone:', ':levitate::skin-tone-3:', ':man_in_business_suit_levitating::skin-tone-3:'],
    '🕴🏾': [':levitate_tone4:', ':man_in_business_suit_levitating_tone4:', ':man_in_business_suit_levitating_medium_dark_skin_tone:', ':levitate::skin-tone-4:', ':man_in_business_suit_levitating::skin-tone-4:'],
    '🕴🏿': [':levitate_tone5:', ':man_in_business_suit_levitating_tone5:', ':man_in_business_suit_levitating_dark_skin_tone:', ':levitate::skin-tone-5:', ':man_in_business_suit_levitating::skin-tone-5:'],
    '♎': [':libra:'],
    '🚈': [':light_rail:'],
    '🔗': [':link:'],
    '🦁': [':lion_face:', ':lion:'],
    '👄': [':lips:'],
    '💄': [':lipstick:'],
    '🦎': [':lizard:'],
    '🦙': [':llama:'],
    '🦞': [':lobster:'],
    '🔒': [':lock:'],
    '🔏': [':lock_with_ink_pen:'],
    '🍭': [':lollipop:'],
    '🪘': [':long_drum:'],
    '➿': [':loop:'],
    '🔊': [':loud_sound:'],
    '📢': [':loudspeaker:'],
    '🏩': [':love_hotel:'],
    '💌': [':love_letter:'],
    '🤟': [':love_you_gesture:'],
    '🤟🏻': [':love_you_gesture_tone1:', ':love_you_gesture_light_skin_tone:', ':love_you_gesture::skin-tone-1:'],
    '🤟🏼': [':love_you_gesture_tone2:', ':love_you_gesture_medium_light_skin_tone:', ':love_you_gesture::skin-tone-2:'],
    '🤟🏽': [':love_you_gesture_tone3:', ':love_you_gesture_medium_skin_tone:', ':love_you_gesture::skin-tone-3:'],
    '🤟🏾': [':love_you_gesture_tone4:', ':love_you_gesture_medium_dark_skin_tone:', ':love_you_gesture::skin-tone-4:'],
    '🤟🏿': [':love_you_gesture_tone5:', ':love_you_gesture_dark_skin_tone:', ':love_you_gesture::skin-tone-5:'],
    '🔅': [':low_brightness:'],
    '🧳': [':luggage:'],
    '🫁': [':lungs:'],
    '🤥': [':lying_face:', ':liar:'],
    'Ⓜ️': [':m:'],
    '🔍': [':mag:'],
    '🔎': [':mag_right:'],
    '🧙': [':mage:'],
    '🧙🏻': [':mage_tone1:', ':mage_light_skin_tone:', ':mage::skin-tone-1:'],
    '🧙🏼': [':mage_tone2:', ':mage_medium_light_skin_tone:', ':mage::skin-tone-2:'],
    '🧙🏽': [':mage_tone3:', ':mage_medium_skin_tone:', ':mage::skin-tone-3:'],
    '🧙🏾': [':mage_tone4:', ':mage_medium_dark_skin_tone:', ':mage::skin-tone-4:'],
    '🧙🏿': [':mage_tone5:', ':mage_dark_skin_tone:', ':mage::skin-tone-5:'],
    '🪄': [':magic_wand:'],
    '🧲': [':magnet:'],
    '🀄': [':mahjong:'],
    '📫': [':mailbox:'],
    '📪': [':mailbox_closed:'],
    '📬': [':mailbox_with_mail:'],
    '📭': [':mailbox_with_no_mail:'],
    '♂️': [':male_sign:'],
    '🦣': [':mammoth:'],
    '👨': [':man:'],
    '👨‍🎨': [':man_artist:'],
    '👨🏻‍🎨': [':man_artist_tone1:', ':man_artist_light_skin_tone:', ':man_artist::skin-tone-1:'],
    '👨🏼‍🎨': [':man_artist_tone2:', ':man_artist_medium_light_skin_tone:', ':man_artist::skin-tone-2:'],
    '👨🏽‍🎨': [':man_artist_tone3:', ':man_artist_medium_skin_tone:', ':man_artist::skin-tone-3:'],
    '👨🏾‍🎨': [':man_artist_tone4:', ':man_artist_medium_dark_skin_tone:', ':man_artist::skin-tone-4:'],
    '👨🏿‍🎨': [':man_artist_tone5:', ':man_artist_dark_skin_tone:', ':man_artist::skin-tone-5:'],
    '👨‍🚀': [':man_astronaut:'],
    '👨🏻‍🚀': [':man_astronaut_tone1:', ':man_astronaut_light_skin_tone:', ':man_astronaut::skin-tone-1:'],
    '👨🏼‍🚀': [':man_astronaut_tone2:', ':man_astronaut_medium_light_skin_tone:', ':man_astronaut::skin-tone-2:'],
    '👨🏽‍🚀': [':man_astronaut_tone3:', ':man_astronaut_medium_skin_tone:', ':man_astronaut::skin-tone-3:'],
    '👨🏾‍🚀': [':man_astronaut_tone4:', ':man_astronaut_medium_dark_skin_tone:', ':man_astronaut::skin-tone-4:'],
    '👨🏿‍🚀': [':man_astronaut_tone5:', ':man_astronaut_dark_skin_tone:', ':man_astronaut::skin-tone-5:'],
    '👨‍🦲': [':man_bald:'],
    '👨🏻‍🦲': [':man_bald_tone1:', ':man_bald_light_skin_tone:', ':man_bald::skin-tone-1:'],
    '👨🏼‍🦲': [':man_bald_tone2:', ':man_bald_medium_light_skin_tone:', ':man_bald::skin-tone-2:'],
    '👨🏽‍🦲': [':man_bald_tone3:', ':man_bald_medium_skin_tone:', ':man_bald::skin-tone-3:'],
    '👨🏾‍🦲': [':man_bald_tone4:', ':man_bald_medium_dark_skin_tone:', ':man_bald::skin-tone-4:'],
    '👨🏿‍🦲': [':man_bald_tone5:', ':man_bald_dark_skin_tone:', ':man_bald::skin-tone-5:'],
    '🚴‍♂️': [':man_biking:'],
    '🚴🏻‍♂️': [':man_biking_tone1:', ':man_biking_light_skin_tone:', ':man_biking::skin-tone-1:'],
    '🚴🏼‍♂️': [':man_biking_tone2:', ':man_biking_medium_light_skin_tone:', ':man_biking::skin-tone-2:'],
    '🚴🏽‍♂️': [':man_biking_tone3:', ':man_biking_medium_skin_tone:', ':man_biking::skin-tone-3:'],
    '🚴🏾‍♂️': [':man_biking_tone4:', ':man_biking_medium_dark_skin_tone:', ':man_biking::skin-tone-4:'],
    '🚴🏿‍♂️': [':man_biking_tone5:', ':man_biking_dark_skin_tone:', ':man_biking::skin-tone-5:'],
    '⛹️‍♂️': [':man_bouncing_ball:'],
    '⛹🏻‍♂️': [':man_bouncing_ball_tone1:', ':man_bouncing_ball_light_skin_tone:', ':man_bouncing_ball::skin-tone-1:'],
    '⛹🏼‍♂️': [':man_bouncing_ball_tone2:', ':man_bouncing_ball_medium_light_skin_tone:', ':man_bouncing_ball::skin-tone-2:'],
    '⛹🏽‍♂️': [':man_bouncing_ball_tone3:', ':man_bouncing_ball_medium_skin_tone:', ':man_bouncing_ball::skin-tone-3:'],
    '⛹🏾‍♂️': [':man_bouncing_ball_tone4:', ':man_bouncing_ball_medium_dark_skin_tone:', ':man_bouncing_ball::skin-tone-4:'],
    '⛹🏿‍♂️': [':man_bouncing_ball_tone5:', ':man_bouncing_ball_dark_skin_tone:', ':man_bouncing_ball::skin-tone-5:'],
    '🙇‍♂️': [':man_bowing:'],
    '🙇🏻‍♂️': [':man_bowing_tone1:', ':man_bowing_light_skin_tone:', ':man_bowing::skin-tone-1:'],
    '🙇🏼‍♂️': [':man_bowing_tone2:', ':man_bowing_medium_light_skin_tone:', ':man_bowing::skin-tone-2:'],
    '🙇🏽‍♂️': [':man_bowing_tone3:', ':man_bowing_medium_skin_tone:', ':man_bowing::skin-tone-3:'],
    '🙇🏾‍♂️': [':man_bowing_tone4:', ':man_bowing_medium_dark_skin_tone:', ':man_bowing::skin-tone-4:'],
    '🙇🏿‍♂️': [':man_bowing_tone5:', ':man_bowing_dark_skin_tone:', ':man_bowing::skin-tone-5:'],
    '🤸‍♂️': [':man_cartwheeling:'],
    '🤸🏻‍♂️': [':man_cartwheeling_tone1:', ':man_cartwheeling_light_skin_tone:', ':man_cartwheeling::skin-tone-1:'],
    '🤸🏼‍♂️': [':man_cartwheeling_tone2:', ':man_cartwheeling_medium_light_skin_tone:', ':man_cartwheeling::skin-tone-2:'],
    '🤸🏽‍♂️': [':man_cartwheeling_tone3:', ':man_cartwheeling_medium_skin_tone:', ':man_cartwheeling::skin-tone-3:'],
    '🤸🏾‍♂️': [':man_cartwheeling_tone4:', ':man_cartwheeling_medium_dark_skin_tone:', ':man_cartwheeling::skin-tone-4:'],
    '🤸🏿‍♂️': [':man_cartwheeling_tone5:', ':man_cartwheeling_dark_skin_tone:', ':man_cartwheeling::skin-tone-5:'],
    '🧗‍♂️': [':man_climbing:'],
    '🧗🏻‍♂️': [':man_climbing_tone1:', ':man_climbing_light_skin_tone:', ':man_climbing::skin-tone-1:'],
    '🧗🏼‍♂️': [':man_climbing_tone2:', ':man_climbing_medium_light_skin_tone:', ':man_climbing::skin-tone-2:'],
    '🧗🏽‍♂️': [':man_climbing_tone3:', ':man_climbing_medium_skin_tone:', ':man_climbing::skin-tone-3:'],
    '🧗🏾‍♂️': [':man_climbing_tone4:', ':man_climbing_medium_dark_skin_tone:', ':man_climbing::skin-tone-4:'],
    '🧗🏿‍♂️': [':man_climbing_tone5:', ':man_climbing_dark_skin_tone:', ':man_climbing::skin-tone-5:'],
    '👷‍♂️': [':man_construction_worker:'],
    '👷🏻‍♂️': [':man_construction_worker_tone1:', ':man_construction_worker_light_skin_tone:', ':man_construction_worker::skin-tone-1:'],
    '👷🏼‍♂️': [':man_construction_worker_tone2:', ':man_construction_worker_medium_light_skin_tone:', ':man_construction_worker::skin-tone-2:'],
    '👷🏽‍♂️': [':man_construction_worker_tone3:', ':man_construction_worker_medium_skin_tone:', ':man_construction_worker::skin-tone-3:'],
    '👷🏾‍♂️': [':man_construction_worker_tone4:', ':man_construction_worker_medium_dark_skin_tone:', ':man_construction_worker::skin-tone-4:'],
    '👷🏿‍♂️': [':man_construction_worker_tone5:', ':man_construction_worker_dark_skin_tone:', ':man_construction_worker::skin-tone-5:'],
    '👨‍🍳': [':man_cook:'],
    '👨🏻‍🍳': [':man_cook_tone1:', ':man_cook_light_skin_tone:', ':man_cook::skin-tone-1:'],
    '👨🏼‍🍳': [':man_cook_tone2:', ':man_cook_medium_light_skin_tone:', ':man_cook::skin-tone-2:'],
    '👨🏽‍🍳': [':man_cook_tone3:', ':man_cook_medium_skin_tone:', ':man_cook::skin-tone-3:'],
    '👨🏾‍🍳': [':man_cook_tone4:', ':man_cook_medium_dark_skin_tone:', ':man_cook::skin-tone-4:'],
    '👨🏿‍🍳': [':man_cook_tone5:', ':man_cook_dark_skin_tone:', ':man_cook::skin-tone-5:'],
    '👨‍🦱': [':man_curly_haired:'],
    '👨🏻‍🦱': [':man_curly_haired_tone1:', ':man_curly_haired_light_skin_tone:', ':man_curly_haired::skin-tone-1:'],
    '👨🏼‍🦱': [':man_curly_haired_tone2:', ':man_curly_haired_medium_light_skin_tone:', ':man_curly_haired::skin-tone-2:'],
    '👨🏽‍🦱': [':man_curly_haired_tone3:', ':man_curly_haired_medium_skin_tone:', ':man_curly_haired::skin-tone-3:'],
    '👨🏾‍🦱': [':man_curly_haired_tone4:', ':man_curly_haired_medium_dark_skin_tone:', ':man_curly_haired::skin-tone-4:'],
    '👨🏿‍🦱': [':man_curly_haired_tone5:', ':man_curly_haired_dark_skin_tone:', ':man_curly_haired::skin-tone-5:'],
    '🕺': [':man_dancing:', ':male_dancer:'],
    '🕺🏻': [':man_dancing_tone1:', ':male_dancer_tone1:', ':man_dancing::skin-tone-1:', ':male_dancer::skin-tone-1:'],
    '🕺🏼': [':man_dancing_tone2:', ':male_dancer_tone2:', ':man_dancing::skin-tone-2:', ':male_dancer::skin-tone-2:'],
    '🕺🏽': [':man_dancing_tone3:', ':male_dancer_tone3:', ':man_dancing::skin-tone-3:', ':male_dancer::skin-tone-3:'],
    '🕺🏾': [':man_dancing_tone4:', ':male_dancer_tone4:', ':man_dancing::skin-tone-4:', ':male_dancer::skin-tone-4:'],
    '🕺🏿': [':man_dancing_tone5:', ':male_dancer_tone5:', ':man_dancing::skin-tone-5:', ':male_dancer::skin-tone-5:'],
    '🕵️‍♂️': [':man_detective:'],
    '🕵🏻‍♂️': [':man_detective_tone1:', ':man_detective_light_skin_tone:', ':man_detective::skin-tone-1:'],
    '🕵🏼‍♂️': [':man_detective_tone2:', ':man_detective_medium_light_skin_tone:', ':man_detective::skin-tone-2:'],
    '🕵🏽‍♂️': [':man_detective_tone3:', ':man_detective_medium_skin_tone:', ':man_detective::skin-tone-3:'],
    '🕵🏾‍♂️': [':man_detective_tone4:', ':man_detective_medium_dark_skin_tone:', ':man_detective::skin-tone-4:'],
    '🕵🏿‍♂️': [':man_detective_tone5:', ':man_detective_dark_skin_tone:', ':man_detective::skin-tone-5:'],
    '🧝‍♂️': [':man_elf:'],
    '🧝🏻‍♂️': [':man_elf_tone1:', ':man_elf_light_skin_tone:', ':man_elf::skin-tone-1:'],
    '🧝🏼‍♂️': [':man_elf_tone2:', ':man_elf_medium_light_skin_tone:', ':man_elf::skin-tone-2:'],
    '🧝🏽‍♂️': [':man_elf_tone3:', ':man_elf_medium_skin_tone:', ':man_elf::skin-tone-3:'],
    '🧝🏾‍♂️': [':man_elf_tone4:', ':man_elf_medium_dark_skin_tone:', ':man_elf::skin-tone-4:'],
    '🧝🏿‍♂️': [':man_elf_tone5:', ':man_elf_dark_skin_tone:', ':man_elf::skin-tone-5:'],
    '🤦‍♂️': [':man_facepalming:'],
    '🤦🏻‍♂️': [':man_facepalming_tone1:', ':man_facepalming_light_skin_tone:', ':man_facepalming::skin-tone-1:'],
    '🤦🏼‍♂️': [':man_facepalming_tone2:', ':man_facepalming_medium_light_skin_tone:', ':man_facepalming::skin-tone-2:'],
    '🤦🏽‍♂️': [':man_facepalming_tone3:', ':man_facepalming_medium_skin_tone:', ':man_facepalming::skin-tone-3:'],
    '🤦🏾‍♂️': [':man_facepalming_tone4:', ':man_facepalming_medium_dark_skin_tone:', ':man_facepalming::skin-tone-4:'],
    '🤦🏿‍♂️': [':man_facepalming_tone5:', ':man_facepalming_dark_skin_tone:', ':man_facepalming::skin-tone-5:'],
    '👨‍🏭': [':man_factory_worker:'],
    '👨🏻‍🏭': [':man_factory_worker_tone1:', ':man_factory_worker_light_skin_tone:', ':man_factory_worker::skin-tone-1:'],
    '👨🏼‍🏭': [':man_factory_worker_tone2:', ':man_factory_worker_medium_light_skin_tone:', ':man_factory_worker::skin-tone-2:'],
    '👨🏽‍🏭': [':man_factory_worker_tone3:', ':man_factory_worker_medium_skin_tone:', ':man_factory_worker::skin-tone-3:'],
    '👨🏾‍🏭': [':man_factory_worker_tone4:', ':man_factory_worker_medium_dark_skin_tone:', ':man_factory_worker::skin-tone-4:'],
    '👨🏿‍🏭': [':man_factory_worker_tone5:', ':man_factory_worker_dark_skin_tone:', ':man_factory_worker::skin-tone-5:'],
    '🧚‍♂️': [':man_fairy:'],
    '🧚🏻‍♂️': [':man_fairy_tone1:', ':man_fairy_light_skin_tone:', ':man_fairy::skin-tone-1:'],
    '🧚🏼‍♂️': [':man_fairy_tone2:', ':man_fairy_medium_light_skin_tone:', ':man_fairy::skin-tone-2:'],
    '🧚🏽‍♂️': [':man_fairy_tone3:', ':man_fairy_medium_skin_tone:', ':man_fairy::skin-tone-3:'],
    '🧚🏾‍♂️': [':man_fairy_tone4:', ':man_fairy_medium_dark_skin_tone:', ':man_fairy::skin-tone-4:'],
    '🧚🏿‍♂️': [':man_fairy_tone5:', ':man_fairy_dark_skin_tone:', ':man_fairy::skin-tone-5:'],
    '👨‍🌾': [':man_farmer:'],
    '👨🏻‍🌾': [':man_farmer_tone1:', ':man_farmer_light_skin_tone:', ':man_farmer::skin-tone-1:'],
    '👨🏼‍🌾': [':man_farmer_tone2:', ':man_farmer_medium_light_skin_tone:', ':man_farmer::skin-tone-2:'],
    '👨🏽‍🌾': [':man_farmer_tone3:', ':man_farmer_medium_skin_tone:', ':man_farmer::skin-tone-3:'],
    '👨🏾‍🌾': [':man_farmer_tone4:', ':man_farmer_medium_dark_skin_tone:', ':man_farmer::skin-tone-4:'],
    '👨🏿‍🌾': [':man_farmer_tone5:', ':man_farmer_dark_skin_tone:', ':man_farmer::skin-tone-5:'],
    '👨‍🍼': [':man_feeding_baby:'],
    '👨🏻‍🍼': [':man_feeding_baby_tone1:', ':man_feeding_baby_light_skin_tone:', ':man_feeding_baby::skin-tone-1:'],
    '👨🏼‍🍼': [':man_feeding_baby_tone2:', ':man_feeding_baby_medium_light_skin_tone:', ':man_feeding_baby::skin-tone-2:'],
    '👨🏽‍🍼': [':man_feeding_baby_tone3:', ':man_feeding_baby_medium_skin_tone:', ':man_feeding_baby::skin-tone-3:'],
    '👨🏾‍🍼': [':man_feeding_baby_tone4:', ':man_feeding_baby_medium_dark_skin_tone:', ':man_feeding_baby::skin-tone-4:'],
    '👨🏿‍🍼': [':man_feeding_baby_tone5:', ':man_feeding_baby_dark_skin_tone:', ':man_feeding_baby::skin-tone-5:'],
    '👨‍🚒': [':man_firefighter:'],
    '👨🏻‍🚒': [':man_firefighter_tone1:', ':man_firefighter_light_skin_tone:', ':man_firefighter::skin-tone-1:'],
    '👨🏼‍🚒': [':man_firefighter_tone2:', ':man_firefighter_medium_light_skin_tone:', ':man_firefighter::skin-tone-2:'],
    '👨🏽‍🚒': [':man_firefighter_tone3:', ':man_firefighter_medium_skin_tone:', ':man_firefighter::skin-tone-3:'],
    '👨🏾‍🚒': [':man_firefighter_tone4:', ':man_firefighter_medium_dark_skin_tone:', ':man_firefighter::skin-tone-4:'],
    '👨🏿‍🚒': [':man_firefighter_tone5:', ':man_firefighter_dark_skin_tone:', ':man_firefighter::skin-tone-5:'],
    '🙍‍♂️': [':man_frowning:'],
    '🙍🏻‍♂️': [':man_frowning_tone1:', ':man_frowning_light_skin_tone:', ':man_frowning::skin-tone-1:'],
    '🙍🏼‍♂️': [':man_frowning_tone2:', ':man_frowning_medium_light_skin_tone:', ':man_frowning::skin-tone-2:'],
    '🙍🏽‍♂️': [':man_frowning_tone3:', ':man_frowning_medium_skin_tone:', ':man_frowning::skin-tone-3:'],
    '🙍🏾‍♂️': [':man_frowning_tone4:', ':man_frowning_medium_dark_skin_tone:', ':man_frowning::skin-tone-4:'],
    '🙍🏿‍♂️': [':man_frowning_tone5:', ':man_frowning_dark_skin_tone:', ':man_frowning::skin-tone-5:'],
    '🧞‍♂️': [':man_genie:'],
    '🙅‍♂️': [':man_gesturing_no:'],
    '🙅🏻‍♂️': [':man_gesturing_no_tone1:', ':man_gesturing_no_light_skin_tone:', ':man_gesturing_no::skin-tone-1:'],
    '🙅🏼‍♂️': [':man_gesturing_no_tone2:', ':man_gesturing_no_medium_light_skin_tone:', ':man_gesturing_no::skin-tone-2:'],
    '🙅🏽‍♂️': [':man_gesturing_no_tone3:', ':man_gesturing_no_medium_skin_tone:', ':man_gesturing_no::skin-tone-3:'],
    '🙅🏾‍♂️': [':man_gesturing_no_tone4:', ':man_gesturing_no_medium_dark_skin_tone:', ':man_gesturing_no::skin-tone-4:'],
    '🙅🏿‍♂️': [':man_gesturing_no_tone5:', ':man_gesturing_no_dark_skin_tone:', ':man_gesturing_no::skin-tone-5:'],
    '🙆‍♂️': [':man_gesturing_ok:'],
    '🙆🏻‍♂️': [':man_gesturing_ok_tone1:', ':man_gesturing_ok_light_skin_tone:', ':man_gesturing_ok::skin-tone-1:'],
    '🙆🏼‍♂️': [':man_gesturing_ok_tone2:', ':man_gesturing_ok_medium_light_skin_tone:', ':man_gesturing_ok::skin-tone-2:'],
    '🙆🏽‍♂️': [':man_gesturing_ok_tone3:', ':man_gesturing_ok_medium_skin_tone:', ':man_gesturing_ok::skin-tone-3:'],
    '🙆🏾‍♂️': [':man_gesturing_ok_tone4:', ':man_gesturing_ok_medium_dark_skin_tone:', ':man_gesturing_ok::skin-tone-4:'],
    '🙆🏿‍♂️': [':man_gesturing_ok_tone5:', ':man_gesturing_ok_dark_skin_tone:', ':man_gesturing_ok::skin-tone-5:'],
    '💆‍♂️': [':man_getting_face_massage:'],
    '💆🏻‍♂️': [':man_getting_face_massage_tone1:', ':man_getting_face_massage_light_skin_tone:', ':man_getting_face_massage::skin-tone-1:'],
    '💆🏼‍♂️': [':man_getting_face_massage_tone2:', ':man_getting_face_massage_medium_light_skin_tone:', ':man_getting_face_massage::skin-tone-2:'],
    '💆🏽‍♂️': [':man_getting_face_massage_tone3:', ':man_getting_face_massage_medium_skin_tone:', ':man_getting_face_massage::skin-tone-3:'],
    '💆🏾‍♂️': [':man_getting_face_massage_tone4:', ':man_getting_face_massage_medium_dark_skin_tone:', ':man_getting_face_massage::skin-tone-4:'],
    '💆🏿‍♂️': [':man_getting_face_massage_tone5:', ':man_getting_face_massage_dark_skin_tone:', ':man_getting_face_massage::skin-tone-5:'],
    '💇‍♂️': [':man_getting_haircut:'],
    '💇🏻‍♂️': [':man_getting_haircut_tone1:', ':man_getting_haircut_light_skin_tone:', ':man_getting_haircut::skin-tone-1:'],
    '💇🏼‍♂️': [':man_getting_haircut_tone2:', ':man_getting_haircut_medium_light_skin_tone:', ':man_getting_haircut::skin-tone-2:'],
    '💇🏽‍♂️': [':man_getting_haircut_tone3:', ':man_getting_haircut_medium_skin_tone:', ':man_getting_haircut::skin-tone-3:'],
    '💇🏾‍♂️': [':man_getting_haircut_tone4:', ':man_getting_haircut_medium_dark_skin_tone:', ':man_getting_haircut::skin-tone-4:'],
    '💇🏿‍♂️': [':man_getting_haircut_tone5:', ':man_getting_haircut_dark_skin_tone:', ':man_getting_haircut::skin-tone-5:'],
    '🏌️‍♂️': [':man_golfing:'],
    '🏌🏻‍♂️': [':man_golfing_tone1:', ':man_golfing_light_skin_tone:', ':man_golfing::skin-tone-1:'],
    '🏌🏼‍♂️': [':man_golfing_tone2:', ':man_golfing_medium_light_skin_tone:', ':man_golfing::skin-tone-2:'],
    '🏌🏽‍♂️': [':man_golfing_tone3:', ':man_golfing_medium_skin_tone:', ':man_golfing::skin-tone-3:'],
    '🏌🏾‍♂️': [':man_golfing_tone4:', ':man_golfing_medium_dark_skin_tone:', ':man_golfing::skin-tone-4:'],
    '🏌🏿‍♂️': [':man_golfing_tone5:', ':man_golfing_dark_skin_tone:', ':man_golfing::skin-tone-5:'],
    '💂‍♂️': [':man_guard:'],
    '💂🏻‍♂️': [':man_guard_tone1:', ':man_guard_light_skin_tone:', ':man_guard::skin-tone-1:'],
    '💂🏼‍♂️': [':man_guard_tone2:', ':man_guard_medium_light_skin_tone:', ':man_guard::skin-tone-2:'],
    '💂🏽‍♂️': [':man_guard_tone3:', ':man_guard_medium_skin_tone:', ':man_guard::skin-tone-3:'],
    '💂🏾‍♂️': [':man_guard_tone4:', ':man_guard_medium_dark_skin_tone:', ':man_guard::skin-tone-4:'],
    '💂🏿‍♂️': [':man_guard_tone5:', ':man_guard_dark_skin_tone:', ':man_guard::skin-tone-5:'],
    '👨‍⚕️': [':man_health_worker:'],
    '👨🏻‍⚕️': [':man_health_worker_tone1:', ':man_health_worker_light_skin_tone:', ':man_health_worker::skin-tone-1:'],
    '👨🏼‍⚕️': [':man_health_worker_tone2:', ':man_health_worker_medium_light_skin_tone:', ':man_health_worker::skin-tone-2:'],
    '👨🏽‍⚕️': [':man_health_worker_tone3:', ':man_health_worker_medium_skin_tone:', ':man_health_worker::skin-tone-3:'],
    '👨🏾‍⚕️': [':man_health_worker_tone4:', ':man_health_worker_medium_dark_skin_tone:', ':man_health_worker::skin-tone-4:'],
    '👨🏿‍⚕️': [':man_health_worker_tone5:', ':man_health_worker_dark_skin_tone:', ':man_health_worker::skin-tone-5:'],
    '🧘‍♂️': [':man_in_lotus_position:'],
    '🧘🏻‍♂️': [':man_in_lotus_position_tone1:', ':man_in_lotus_position_light_skin_tone:', ':man_in_lotus_position::skin-tone-1:'],
    '🧘🏼‍♂️': [':man_in_lotus_position_tone2:', ':man_in_lotus_position_medium_light_skin_tone:', ':man_in_lotus_position::skin-tone-2:'],
    '🧘🏽‍♂️': [':man_in_lotus_position_tone3:', ':man_in_lotus_position_medium_skin_tone:', ':man_in_lotus_position::skin-tone-3:'],
    '🧘🏾‍♂️': [':man_in_lotus_position_tone4:', ':man_in_lotus_position_medium_dark_skin_tone:', ':man_in_lotus_position::skin-tone-4:'],
    '🧘🏿‍♂️': [':man_in_lotus_position_tone5:', ':man_in_lotus_position_dark_skin_tone:', ':man_in_lotus_position::skin-tone-5:'],
    '👨‍🦽': [':man_in_manual_wheelchair:'],
    '👨🏻‍🦽': [':man_in_manual_wheelchair_tone1:', ':man_in_manual_wheelchair_light_skin_tone:', ':man_in_manual_wheelchair::skin-tone-1:'],
    '👨🏼‍🦽': [':man_in_manual_wheelchair_tone2:', ':man_in_manual_wheelchair_medium_light_skin_tone:', ':man_in_manual_wheelchair::skin-tone-2:'],
    '👨🏽‍🦽': [':man_in_manual_wheelchair_tone3:', ':man_in_manual_wheelchair_medium_skin_tone:', ':man_in_manual_wheelchair::skin-tone-3:'],
    '👨🏾‍🦽': [':man_in_manual_wheelchair_tone4:', ':man_in_manual_wheelchair_medium_dark_skin_tone:', ':man_in_manual_wheelchair::skin-tone-4:'],
    '👨🏿‍🦽': [':man_in_manual_wheelchair_tone5:', ':man_in_manual_wheelchair_dark_skin_tone:', ':man_in_manual_wheelchair::skin-tone-5:'],
    '👨‍🦼': [':man_in_motorized_wheelchair:'],
    '👨🏻‍🦼': [':man_in_motorized_wheelchair_tone1:', ':man_in_motorized_wheelchair_light_skin_tone:', ':man_in_motorized_wheelchair::skin-tone-1:'],
    '👨🏼‍🦼': [':man_in_motorized_wheelchair_tone2:', ':man_in_motorized_wheelchair_medium_light_skin_tone:', ':man_in_motorized_wheelchair::skin-tone-2:'],
    '👨🏽‍🦼': [':man_in_motorized_wheelchair_tone3:', ':man_in_motorized_wheelchair_medium_skin_tone:', ':man_in_motorized_wheelchair::skin-tone-3:'],
    '👨🏾‍🦼': [':man_in_motorized_wheelchair_tone4:', ':man_in_motorized_wheelchair_medium_dark_skin_tone:', ':man_in_motorized_wheelchair::skin-tone-4:'],
    '👨🏿‍🦼': [':man_in_motorized_wheelchair_tone5:', ':man_in_motorized_wheelchair_dark_skin_tone:', ':man_in_motorized_wheelchair::skin-tone-5:'],
    '🧖‍♂️': [':man_in_steamy_room:'],
    '🧖🏻‍♂️': [':man_in_steamy_room_tone1:', ':man_in_steamy_room_light_skin_tone:', ':man_in_steamy_room::skin-tone-1:'],
    '🧖🏼‍♂️': [':man_in_steamy_room_tone2:', ':man_in_steamy_room_medium_light_skin_tone:', ':man_in_steamy_room::skin-tone-2:'],
    '🧖🏽‍♂️': [':man_in_steamy_room_tone3:', ':man_in_steamy_room_medium_skin_tone:', ':man_in_steamy_room::skin-tone-3:'],
    '🧖🏾‍♂️': [':man_in_steamy_room_tone4:', ':man_in_steamy_room_medium_dark_skin_tone:', ':man_in_steamy_room::skin-tone-4:'],
    '🧖🏿‍♂️': [':man_in_steamy_room_tone5:', ':man_in_steamy_room_dark_skin_tone:', ':man_in_steamy_room::skin-tone-5:'],
    '🤵‍♂️': [':man_in_tuxedo:'],
    '🤵🏻‍♂️': [':man_in_tuxedo_tone1:', ':man_in_tuxedo_light_skin_tone:', ':man_in_tuxedo::skin-tone-1:'],
    '🤵🏼‍♂️': [':man_in_tuxedo_tone2:', ':man_in_tuxedo_medium_light_skin_tone:', ':man_in_tuxedo::skin-tone-2:'],
    '🤵🏽‍♂️': [':man_in_tuxedo_tone3:', ':man_in_tuxedo_medium_skin_tone:', ':man_in_tuxedo::skin-tone-3:'],
    '🤵🏾‍♂️': [':man_in_tuxedo_tone4:', ':man_in_tuxedo_medium_dark_skin_tone:', ':man_in_tuxedo::skin-tone-4:'],
    '🤵🏿‍♂️': [':man_in_tuxedo_tone5:', ':man_in_tuxedo_dark_skin_tone:', ':man_in_tuxedo::skin-tone-5:'],
    '👨‍⚖️': [':man_judge:'],
    '👨🏻‍⚖️': [':man_judge_tone1:', ':man_judge_light_skin_tone:', ':man_judge::skin-tone-1:'],
    '👨🏼‍⚖️': [':man_judge_tone2:', ':man_judge_medium_light_skin_tone:', ':man_judge::skin-tone-2:'],
    '👨🏽‍⚖️': [':man_judge_tone3:', ':man_judge_medium_skin_tone:', ':man_judge::skin-tone-3:'],
    '👨🏾‍⚖️': [':man_judge_tone4:', ':man_judge_medium_dark_skin_tone:', ':man_judge::skin-tone-4:'],
    '👨🏿‍⚖️': [':man_judge_tone5:', ':man_judge_dark_skin_tone:', ':man_judge::skin-tone-5:'],
    '🤹‍♂️': [':man_juggling:'],
    '🤹🏻‍♂️': [':man_juggling_tone1:', ':man_juggling_light_skin_tone:', ':man_juggling::skin-tone-1:'],
    '🤹🏼‍♂️': [':man_juggling_tone2:', ':man_juggling_medium_light_skin_tone:', ':man_juggling::skin-tone-2:'],
    '🤹🏽‍♂️': [':man_juggling_tone3:', ':man_juggling_medium_skin_tone:', ':man_juggling::skin-tone-3:'],
    '🤹🏾‍♂️': [':man_juggling_tone4:', ':man_juggling_medium_dark_skin_tone:', ':man_juggling::skin-tone-4:'],
    '🤹🏿‍♂️': [':man_juggling_tone5:', ':man_juggling_dark_skin_tone:', ':man_juggling::skin-tone-5:'],
    '🧎‍♂️': [':man_kneeling:'],
    '🧎🏻‍♂️': [':man_kneeling_tone1:', ':man_kneeling_light_skin_tone:', ':man_kneeling::skin-tone-1:'],
    '🧎🏼‍♂️': [':man_kneeling_tone2:', ':man_kneeling_medium_light_skin_tone:', ':man_kneeling::skin-tone-2:'],
    '🧎🏽‍♂️': [':man_kneeling_tone3:', ':man_kneeling_medium_skin_tone:', ':man_kneeling::skin-tone-3:'],
    '🧎🏾‍♂️': [':man_kneeling_tone4:', ':man_kneeling_medium_dark_skin_tone:', ':man_kneeling::skin-tone-4:'],
    '🧎🏿‍♂️': [':man_kneeling_tone5:', ':man_kneeling_dark_skin_tone:', ':man_kneeling::skin-tone-5:'],
    '🏋️‍♂️': [':man_lifting_weights:'],
    '🏋🏻‍♂️': [':man_lifting_weights_tone1:', ':man_lifting_weights_light_skin_tone:', ':man_lifting_weights::skin-tone-1:'],
    '🏋🏼‍♂️': [':man_lifting_weights_tone2:', ':man_lifting_weights_medium_light_skin_tone:', ':man_lifting_weights::skin-tone-2:'],
    '🏋🏽‍♂️': [':man_lifting_weights_tone3:', ':man_lifting_weights_medium_skin_tone:', ':man_lifting_weights::skin-tone-3:'],
    '🏋🏾‍♂️': [':man_lifting_weights_tone4:', ':man_lifting_weights_medium_dark_skin_tone:', ':man_lifting_weights::skin-tone-4:'],
    '🏋🏿‍♂️': [':man_lifting_weights_tone5:', ':man_lifting_weights_dark_skin_tone:', ':man_lifting_weights::skin-tone-5:'],
    '🧙‍♂️': [':man_mage:'],
    '🧙🏻‍♂️': [':man_mage_tone1:', ':man_mage_light_skin_tone:', ':man_mage::skin-tone-1:'],
    '🧙🏼‍♂️': [':man_mage_tone2:', ':man_mage_medium_light_skin_tone:', ':man_mage::skin-tone-2:'],
    '🧙🏽‍♂️': [':man_mage_tone3:', ':man_mage_medium_skin_tone:', ':man_mage::skin-tone-3:'],
    '🧙🏾‍♂️': [':man_mage_tone4:', ':man_mage_medium_dark_skin_tone:', ':man_mage::skin-tone-4:'],
    '🧙🏿‍♂️': [':man_mage_tone5:', ':man_mage_dark_skin_tone:', ':man_mage::skin-tone-5:'],
    '👨‍🔧': [':man_mechanic:'],
    '👨🏻‍🔧': [':man_mechanic_tone1:', ':man_mechanic_light_skin_tone:', ':man_mechanic::skin-tone-1:'],
    '👨🏼‍🔧': [':man_mechanic_tone2:', ':man_mechanic_medium_light_skin_tone:', ':man_mechanic::skin-tone-2:'],
    '👨🏽‍🔧': [':man_mechanic_tone3:', ':man_mechanic_medium_skin_tone:', ':man_mechanic::skin-tone-3:'],
    '👨🏾‍🔧': [':man_mechanic_tone4:', ':man_mechanic_medium_dark_skin_tone:', ':man_mechanic::skin-tone-4:'],
    '👨🏿‍🔧': [':man_mechanic_tone5:', ':man_mechanic_dark_skin_tone:', ':man_mechanic::skin-tone-5:'],
    '🚵‍♂️': [':man_mountain_biking:'],
    '🚵🏻‍♂️': [':man_mountain_biking_tone1:', ':man_mountain_biking_light_skin_tone:', ':man_mountain_biking::skin-tone-1:'],
    '🚵🏼‍♂️': [':man_mountain_biking_tone2:', ':man_mountain_biking_medium_light_skin_tone:', ':man_mountain_biking::skin-tone-2:'],
    '🚵🏽‍♂️': [':man_mountain_biking_tone3:', ':man_mountain_biking_medium_skin_tone:', ':man_mountain_biking::skin-tone-3:'],
    '🚵🏾‍♂️': [':man_mountain_biking_tone4:', ':man_mountain_biking_medium_dark_skin_tone:', ':man_mountain_biking::skin-tone-4:'],
    '🚵🏿‍♂️': [':man_mountain_biking_tone5:', ':man_mountain_biking_dark_skin_tone:', ':man_mountain_biking::skin-tone-5:'],
    '👨‍💼': [':man_office_worker:'],
    '👨🏻‍💼': [':man_office_worker_tone1:', ':man_office_worker_light_skin_tone:', ':man_office_worker::skin-tone-1:'],
    '👨🏼‍💼': [':man_office_worker_tone2:', ':man_office_worker_medium_light_skin_tone:', ':man_office_worker::skin-tone-2:'],
    '👨🏽‍💼': [':man_office_worker_tone3:', ':man_office_worker_medium_skin_tone:', ':man_office_worker::skin-tone-3:'],
    '👨🏾‍💼': [':man_office_worker_tone4:', ':man_office_worker_medium_dark_skin_tone:', ':man_office_worker::skin-tone-4:'],
    '👨🏿‍💼': [':man_office_worker_tone5:', ':man_office_worker_dark_skin_tone:', ':man_office_worker::skin-tone-5:'],
    '👨‍✈️': [':man_pilot:'],
    '👨🏻‍✈️': [':man_pilot_tone1:', ':man_pilot_light_skin_tone:', ':man_pilot::skin-tone-1:'],
    '👨🏼‍✈️': [':man_pilot_tone2:', ':man_pilot_medium_light_skin_tone:', ':man_pilot::skin-tone-2:'],
    '👨🏽‍✈️': [':man_pilot_tone3:', ':man_pilot_medium_skin_tone:', ':man_pilot::skin-tone-3:'],
    '👨🏾‍✈️': [':man_pilot_tone4:', ':man_pilot_medium_dark_skin_tone:', ':man_pilot::skin-tone-4:'],
    '👨🏿‍✈️': [':man_pilot_tone5:', ':man_pilot_dark_skin_tone:', ':man_pilot::skin-tone-5:'],
    '🤾‍♂️': [':man_playing_handball:'],
    '🤾🏻‍♂️': [':man_playing_handball_tone1:', ':man_playing_handball_light_skin_tone:', ':man_playing_handball::skin-tone-1:'],
    '🤾🏼‍♂️': [':man_playing_handball_tone2:', ':man_playing_handball_medium_light_skin_tone:', ':man_playing_handball::skin-tone-2:'],
    '🤾🏽‍♂️': [':man_playing_handball_tone3:', ':man_playing_handball_medium_skin_tone:', ':man_playing_handball::skin-tone-3:'],
    '🤾🏾‍♂️': [':man_playing_handball_tone4:', ':man_playing_handball_medium_dark_skin_tone:', ':man_playing_handball::skin-tone-4:'],
    '🤾🏿‍♂️': [':man_playing_handball_tone5:', ':man_playing_handball_dark_skin_tone:', ':man_playing_handball::skin-tone-5:'],
    '🤽‍♂️': [':man_playing_water_polo:'],
    '🤽🏻‍♂️': [':man_playing_water_polo_tone1:', ':man_playing_water_polo_light_skin_tone:', ':man_playing_water_polo::skin-tone-1:'],
    '🤽🏼‍♂️': [':man_playing_water_polo_tone2:', ':man_playing_water_polo_medium_light_skin_tone:', ':man_playing_water_polo::skin-tone-2:'],
    '🤽🏽‍♂️': [':man_playing_water_polo_tone3:', ':man_playing_water_polo_medium_skin_tone:', ':man_playing_water_polo::skin-tone-3:'],
    '🤽🏾‍♂️': [':man_playing_water_polo_tone4:', ':man_playing_water_polo_medium_dark_skin_tone:', ':man_playing_water_polo::skin-tone-4:'],
    '🤽🏿‍♂️': [':man_playing_water_polo_tone5:', ':man_playing_water_polo_dark_skin_tone:', ':man_playing_water_polo::skin-tone-5:'],
    '👮‍♂️': [':man_police_officer:'],
    '👮🏻‍♂️': [':man_police_officer_tone1:', ':man_police_officer_light_skin_tone:', ':man_police_officer::skin-tone-1:'],
    '👮🏼‍♂️': [':man_police_officer_tone2:', ':man_police_officer_medium_light_skin_tone:', ':man_police_officer::skin-tone-2:'],
    '👮🏽‍♂️': [':man_police_officer_tone3:', ':man_police_officer_medium_skin_tone:', ':man_police_officer::skin-tone-3:'],
    '👮🏾‍♂️': [':man_police_officer_tone4:', ':man_police_officer_medium_dark_skin_tone:', ':man_police_officer::skin-tone-4:'],
    '👮🏿‍♂️': [':man_police_officer_tone5:', ':man_police_officer_dark_skin_tone:', ':man_police_officer::skin-tone-5:'],
    '🙎‍♂️': [':man_pouting:'],
    '🙎🏻‍♂️': [':man_pouting_tone1:', ':man_pouting_light_skin_tone:', ':man_pouting::skin-tone-1:'],
    '🙎🏼‍♂️': [':man_pouting_tone2:', ':man_pouting_medium_light_skin_tone:', ':man_pouting::skin-tone-2:'],
    '🙎🏽‍♂️': [':man_pouting_tone3:', ':man_pouting_medium_skin_tone:', ':man_pouting::skin-tone-3:'],
    '🙎🏾‍♂️': [':man_pouting_tone4:', ':man_pouting_medium_dark_skin_tone:', ':man_pouting::skin-tone-4:'],
    '🙎🏿‍♂️': [':man_pouting_tone5:', ':man_pouting_dark_skin_tone:', ':man_pouting::skin-tone-5:'],
    '🙋‍♂️': [':man_raising_hand:'],
    '🙋🏻‍♂️': [':man_raising_hand_tone1:', ':man_raising_hand_light_skin_tone:', ':man_raising_hand::skin-tone-1:'],
    '🙋🏼‍♂️': [':man_raising_hand_tone2:', ':man_raising_hand_medium_light_skin_tone:', ':man_raising_hand::skin-tone-2:'],
    '🙋🏽‍♂️': [':man_raising_hand_tone3:', ':man_raising_hand_medium_skin_tone:', ':man_raising_hand::skin-tone-3:'],
    '🙋🏾‍♂️': [':man_raising_hand_tone4:', ':man_raising_hand_medium_dark_skin_tone:', ':man_raising_hand::skin-tone-4:'],
    '🙋🏿‍♂️': [':man_raising_hand_tone5:', ':man_raising_hand_dark_skin_tone:', ':man_raising_hand::skin-tone-5:'],
    '👨‍🦰': [':man_red_haired:'],
    '👨🏻‍🦰': [':man_red_haired_tone1:', ':man_red_haired_light_skin_tone:', ':man_red_haired::skin-tone-1:'],
    '👨🏼‍🦰': [':man_red_haired_tone2:', ':man_red_haired_medium_light_skin_tone:', ':man_red_haired::skin-tone-2:'],
    '👨🏽‍🦰': [':man_red_haired_tone3:', ':man_red_haired_medium_skin_tone:', ':man_red_haired::skin-tone-3:'],
    '👨🏾‍🦰': [':man_red_haired_tone4:', ':man_red_haired_medium_dark_skin_tone:', ':man_red_haired::skin-tone-4:'],
    '👨🏿‍🦰': [':man_red_haired_tone5:', ':man_red_haired_dark_skin_tone:', ':man_red_haired::skin-tone-5:'],
    '🚣‍♂️': [':man_rowing_boat:'],
    '🚣🏻‍♂️': [':man_rowing_boat_tone1:', ':man_rowing_boat_light_skin_tone:', ':man_rowing_boat::skin-tone-1:'],
    '🚣🏼‍♂️': [':man_rowing_boat_tone2:', ':man_rowing_boat_medium_light_skin_tone:', ':man_rowing_boat::skin-tone-2:'],
    '🚣🏽‍♂️': [':man_rowing_boat_tone3:', ':man_rowing_boat_medium_skin_tone:', ':man_rowing_boat::skin-tone-3:'],
    '🚣🏾‍♂️': [':man_rowing_boat_tone4:', ':man_rowing_boat_medium_dark_skin_tone:', ':man_rowing_boat::skin-tone-4:'],
    '🚣🏿‍♂️': [':man_rowing_boat_tone5:', ':man_rowing_boat_dark_skin_tone:', ':man_rowing_boat::skin-tone-5:'],
    '🏃‍♂️': [':man_running:'],
    '🏃🏻‍♂️': [':man_running_tone1:', ':man_running_light_skin_tone:', ':man_running::skin-tone-1:'],
    '🏃🏼‍♂️': [':man_running_tone2:', ':man_running_medium_light_skin_tone:', ':man_running::skin-tone-2:'],
    '🏃🏽‍♂️': [':man_running_tone3:', ':man_running_medium_skin_tone:', ':man_running::skin-tone-3:'],
    '🏃🏾‍♂️': [':man_running_tone4:', ':man_running_medium_dark_skin_tone:', ':man_running::skin-tone-4:'],
    '🏃🏿‍♂️': [':man_running_tone5:', ':man_running_dark_skin_tone:', ':man_running::skin-tone-5:'],
    '👨‍🔬': [':man_scientist:'],
    '👨🏻‍🔬': [':man_scientist_tone1:', ':man_scientist_light_skin_tone:', ':man_scientist::skin-tone-1:'],
    '👨🏼‍🔬': [':man_scientist_tone2:', ':man_scientist_medium_light_skin_tone:', ':man_scientist::skin-tone-2:'],
    '👨🏽‍🔬': [':man_scientist_tone3:', ':man_scientist_medium_skin_tone:', ':man_scientist::skin-tone-3:'],
    '👨🏾‍🔬': [':man_scientist_tone4:', ':man_scientist_medium_dark_skin_tone:', ':man_scientist::skin-tone-4:'],
    '👨🏿‍🔬': [':man_scientist_tone5:', ':man_scientist_dark_skin_tone:', ':man_scientist::skin-tone-5:'],
    '🤷‍♂️': [':man_shrugging:'],
    '🤷🏻‍♂️': [':man_shrugging_tone1:', ':man_shrugging_light_skin_tone:', ':man_shrugging::skin-tone-1:'],
    '🤷🏼‍♂️': [':man_shrugging_tone2:', ':man_shrugging_medium_light_skin_tone:', ':man_shrugging::skin-tone-2:'],
    '🤷🏽‍♂️': [':man_shrugging_tone3:', ':man_shrugging_medium_skin_tone:', ':man_shrugging::skin-tone-3:'],
    '🤷🏾‍♂️': [':man_shrugging_tone4:', ':man_shrugging_medium_dark_skin_tone:', ':man_shrugging::skin-tone-4:'],
    '🤷🏿‍♂️': [':man_shrugging_tone5:', ':man_shrugging_dark_skin_tone:', ':man_shrugging::skin-tone-5:'],
    '👨‍🎤': [':man_singer:'],
    '👨🏻‍🎤': [':man_singer_tone1:', ':man_singer_light_skin_tone:', ':man_singer::skin-tone-1:'],
    '👨🏼‍🎤': [':man_singer_tone2:', ':man_singer_medium_light_skin_tone:', ':man_singer::skin-tone-2:'],
    '👨🏽‍🎤': [':man_singer_tone3:', ':man_singer_medium_skin_tone:', ':man_singer::skin-tone-3:'],
    '👨🏾‍🎤': [':man_singer_tone4:', ':man_singer_medium_dark_skin_tone:', ':man_singer::skin-tone-4:'],
    '👨🏿‍🎤': [':man_singer_tone5:', ':man_singer_dark_skin_tone:', ':man_singer::skin-tone-5:'],
    '🧍‍♂️': [':man_standing:'],
    '🧍🏻‍♂️': [':man_standing_tone1:', ':man_standing_light_skin_tone:', ':man_standing::skin-tone-1:'],
    '🧍🏼‍♂️': [':man_standing_tone2:', ':man_standing_medium_light_skin_tone:', ':man_standing::skin-tone-2:'],
    '🧍🏽‍♂️': [':man_standing_tone3:', ':man_standing_medium_skin_tone:', ':man_standing::skin-tone-3:'],
    '🧍🏾‍♂️': [':man_standing_tone4:', ':man_standing_medium_dark_skin_tone:', ':man_standing::skin-tone-4:'],
    '🧍🏿‍♂️': [':man_standing_tone5:', ':man_standing_dark_skin_tone:', ':man_standing::skin-tone-5:'],
    '👨‍🎓': [':man_student:'],
    '👨🏻‍🎓': [':man_student_tone1:', ':man_student_light_skin_tone:', ':man_student::skin-tone-1:'],
    '👨🏼‍🎓': [':man_student_tone2:', ':man_student_medium_light_skin_tone:', ':man_student::skin-tone-2:'],
    '👨🏽‍🎓': [':man_student_tone3:', ':man_student_medium_skin_tone:', ':man_student::skin-tone-3:'],
    '👨🏾‍🎓': [':man_student_tone4:', ':man_student_medium_dark_skin_tone:', ':man_student::skin-tone-4:'],
    '👨🏿‍🎓': [':man_student_tone5:', ':man_student_dark_skin_tone:', ':man_student::skin-tone-5:'],
    '🦸‍♂️': [':man_superhero:'],
    '🦸🏻‍♂️': [':man_superhero_tone1:', ':man_superhero_light_skin_tone:', ':man_superhero::skin-tone-1:'],
    '🦸🏼‍♂️': [':man_superhero_tone2:', ':man_superhero_medium_light_skin_tone:', ':man_superhero::skin-tone-2:'],
    '🦸🏽‍♂️': [':man_superhero_tone3:', ':man_superhero_medium_skin_tone:', ':man_superhero::skin-tone-3:'],
    '🦸🏾‍♂️': [':man_superhero_tone4:', ':man_superhero_medium_dark_skin_tone:', ':man_superhero::skin-tone-4:'],
    '🦸🏿‍♂️': [':man_superhero_tone5:', ':man_superhero_dark_skin_tone:', ':man_superhero::skin-tone-5:'],
    '🦹‍♂️': [':man_supervillain:'],
    '🦹🏻‍♂️': [':man_supervillain_tone1:', ':man_supervillain_light_skin_tone:', ':man_supervillain::skin-tone-1:'],
    '🦹🏼‍♂️': [':man_supervillain_tone2:', ':man_supervillain_medium_light_skin_tone:', ':man_supervillain::skin-tone-2:'],
    '🦹🏽‍♂️': [':man_supervillain_tone3:', ':man_supervillain_medium_skin_tone:', ':man_supervillain::skin-tone-3:'],
    '🦹🏾‍♂️': [':man_supervillain_tone4:', ':man_supervillain_medium_dark_skin_tone:', ':man_supervillain::skin-tone-4:'],
    '🦹🏿‍♂️': [':man_supervillain_tone5:', ':man_supervillain_dark_skin_tone:', ':man_supervillain::skin-tone-5:'],
    '🏄‍♂️': [':man_surfing:'],
    '🏄🏻‍♂️': [':man_surfing_tone1:', ':man_surfing_light_skin_tone:', ':man_surfing::skin-tone-1:'],
    '🏄🏼‍♂️': [':man_surfing_tone2:', ':man_surfing_medium_light_skin_tone:', ':man_surfing::skin-tone-2:'],
    '🏄🏽‍♂️': [':man_surfing_tone3:', ':man_surfing_medium_skin_tone:', ':man_surfing::skin-tone-3:'],
    '🏄🏾‍♂️': [':man_surfing_tone4:', ':man_surfing_medium_dark_skin_tone:', ':man_surfing::skin-tone-4:'],
    '🏄🏿‍♂️': [':man_surfing_tone5:', ':man_surfing_dark_skin_tone:', ':man_surfing::skin-tone-5:'],
    '🏊‍♂️': [':man_swimming:'],
    '🏊🏻‍♂️': [':man_swimming_tone1:', ':man_swimming_light_skin_tone:', ':man_swimming::skin-tone-1:'],
    '🏊🏼‍♂️': [':man_swimming_tone2:', ':man_swimming_medium_light_skin_tone:', ':man_swimming::skin-tone-2:'],
    '🏊🏽‍♂️': [':man_swimming_tone3:', ':man_swimming_medium_skin_tone:', ':man_swimming::skin-tone-3:'],
    '🏊🏾‍♂️': [':man_swimming_tone4:', ':man_swimming_medium_dark_skin_tone:', ':man_swimming::skin-tone-4:'],
    '🏊🏿‍♂️': [':man_swimming_tone5:', ':man_swimming_dark_skin_tone:', ':man_swimming::skin-tone-5:'],
    '👨‍🏫': [':man_teacher:'],
    '👨🏻‍🏫': [':man_teacher_tone1:', ':man_teacher_light_skin_tone:', ':man_teacher::skin-tone-1:'],
    '👨🏼‍🏫': [':man_teacher_tone2:', ':man_teacher_medium_light_skin_tone:', ':man_teacher::skin-tone-2:'],
    '👨🏽‍🏫': [':man_teacher_tone3:', ':man_teacher_medium_skin_tone:', ':man_teacher::skin-tone-3:'],
    '👨🏾‍🏫': [':man_teacher_tone4:', ':man_teacher_medium_dark_skin_tone:', ':man_teacher::skin-tone-4:'],
    '👨🏿‍🏫': [':man_teacher_tone5:', ':man_teacher_dark_skin_tone:', ':man_teacher::skin-tone-5:'],
    '👨‍💻': [':man_technologist:'],
    '👨🏻‍💻': [':man_technologist_tone1:', ':man_technologist_light_skin_tone:', ':man_technologist::skin-tone-1:'],
    '👨🏼‍💻': [':man_technologist_tone2:', ':man_technologist_medium_light_skin_tone:', ':man_technologist::skin-tone-2:'],
    '👨🏽‍💻': [':man_technologist_tone3:', ':man_technologist_medium_skin_tone:', ':man_technologist::skin-tone-3:'],
    '👨🏾‍💻': [':man_technologist_tone4:', ':man_technologist_medium_dark_skin_tone:', ':man_technologist::skin-tone-4:'],
    '👨🏿‍💻': [':man_technologist_tone5:', ':man_technologist_dark_skin_tone:', ':man_technologist::skin-tone-5:'],
    '💁‍♂️': [':man_tipping_hand:'],
    '💁🏻‍♂️': [':man_tipping_hand_tone1:', ':man_tipping_hand_light_skin_tone:', ':man_tipping_hand::skin-tone-1:'],
    '💁🏼‍♂️': [':man_tipping_hand_tone2:', ':man_tipping_hand_medium_light_skin_tone:', ':man_tipping_hand::skin-tone-2:'],
    '💁🏽‍♂️': [':man_tipping_hand_tone3:', ':man_tipping_hand_medium_skin_tone:', ':man_tipping_hand::skin-tone-3:'],
    '💁🏾‍♂️': [':man_tipping_hand_tone4:', ':man_tipping_hand_medium_dark_skin_tone:', ':man_tipping_hand::skin-tone-4:'],
    '💁🏿‍♂️': [':man_tipping_hand_tone5:', ':man_tipping_hand_dark_skin_tone:', ':man_tipping_hand::skin-tone-5:'],
    '👨🏻': [':man_tone1:', ':man::skin-tone-1:'],
    '👨🏼': [':man_tone2:', ':man::skin-tone-2:'],
    '👨🏽': [':man_tone3:', ':man::skin-tone-3:'],
    '👨🏾': [':man_tone4:', ':man::skin-tone-4:'],
    '👨🏿': [':man_tone5:', ':man::skin-tone-5:'],
    '🧛‍♂️': [':man_vampire:'],
    '🧛🏻‍♂️': [':man_vampire_tone1:', ':man_vampire_light_skin_tone:', ':man_vampire::skin-tone-1:'],
    '🧛🏼‍♂️': [':man_vampire_tone2:', ':man_vampire_medium_light_skin_tone:', ':man_vampire::skin-tone-2:'],
    '🧛🏽‍♂️': [':man_vampire_tone3:', ':man_vampire_medium_skin_tone:', ':man_vampire::skin-tone-3:'],
    '🧛🏾‍♂️': [':man_vampire_tone4:', ':man_vampire_medium_dark_skin_tone:', ':man_vampire::skin-tone-4:'],
    '🧛🏿‍♂️': [':man_vampire_tone5:', ':man_vampire_dark_skin_tone:', ':man_vampire::skin-tone-5:'],
    '🚶‍♂️': [':man_walking:'],
    '🚶🏻‍♂️': [':man_walking_tone1:', ':man_walking_light_skin_tone:', ':man_walking::skin-tone-1:'],
    '🚶🏼‍♂️': [':man_walking_tone2:', ':man_walking_medium_light_skin_tone:', ':man_walking::skin-tone-2:'],
    '🚶🏽‍♂️': [':man_walking_tone3:', ':man_walking_medium_skin_tone:', ':man_walking::skin-tone-3:'],
    '🚶🏾‍♂️': [':man_walking_tone4:', ':man_walking_medium_dark_skin_tone:', ':man_walking::skin-tone-4:'],
    '🚶🏿‍♂️': [':man_walking_tone5:', ':man_walking_dark_skin_tone:', ':man_walking::skin-tone-5:'],
    '👳‍♂️': [':man_wearing_turban:'],
    '👳🏻‍♂️': [':man_wearing_turban_tone1:', ':man_wearing_turban_light_skin_tone:', ':man_wearing_turban::skin-tone-1:'],
    '👳🏼‍♂️': [':man_wearing_turban_tone2:', ':man_wearing_turban_medium_light_skin_tone:', ':man_wearing_turban::skin-tone-2:'],
    '👳🏽‍♂️': [':man_wearing_turban_tone3:', ':man_wearing_turban_medium_skin_tone:', ':man_wearing_turban::skin-tone-3:'],
    '👳🏾‍♂️': [':man_wearing_turban_tone4:', ':man_wearing_turban_medium_dark_skin_tone:', ':man_wearing_turban::skin-tone-4:'],
    '👳🏿‍♂️': [':man_wearing_turban_tone5:', ':man_wearing_turban_dark_skin_tone:', ':man_wearing_turban::skin-tone-5:'],
    '👨‍🦳': [':man_white_haired:'],
    '👨🏻‍🦳': [':man_white_haired_tone1:', ':man_white_haired_light_skin_tone:', ':man_white_haired::skin-tone-1:'],
    '👨🏼‍🦳': [':man_white_haired_tone2:', ':man_white_haired_medium_light_skin_tone:', ':man_white_haired::skin-tone-2:'],
    '👨🏽‍🦳': [':man_white_haired_tone3:', ':man_white_haired_medium_skin_tone:', ':man_white_haired::skin-tone-3:'],
    '👨🏾‍🦳': [':man_white_haired_tone4:', ':man_white_haired_medium_dark_skin_tone:', ':man_white_haired::skin-tone-4:'],
    '👨🏿‍🦳': [':man_white_haired_tone5:', ':man_white_haired_dark_skin_tone:', ':man_white_haired::skin-tone-5:'],
    '👲': [':man_with_chinese_cap:', ':man_with_gua_pi_mao:'],
    '👲🏻': [':man_with_chinese_cap_tone1:', ':man_with_gua_pi_mao_tone1:', ':man_with_chinese_cap::skin-tone-1:', ':man_with_gua_pi_mao::skin-tone-1:'],
    '👲🏼': [':man_with_chinese_cap_tone2:', ':man_with_gua_pi_mao_tone2:', ':man_with_chinese_cap::skin-tone-2:', ':man_with_gua_pi_mao::skin-tone-2:'],
    '👲🏽': [':man_with_chinese_cap_tone3:', ':man_with_gua_pi_mao_tone3:', ':man_with_chinese_cap::skin-tone-3:', ':man_with_gua_pi_mao::skin-tone-3:'],
    '👲🏾': [':man_with_chinese_cap_tone4:', ':man_with_gua_pi_mao_tone4:', ':man_with_chinese_cap::skin-tone-4:', ':man_with_gua_pi_mao::skin-tone-4:'],
    '👲🏿': [':man_with_chinese_cap_tone5:', ':man_with_gua_pi_mao_tone5:', ':man_with_chinese_cap::skin-tone-5:', ':man_with_gua_pi_mao::skin-tone-5:'],
    '👨‍🦯': [':man_with_probing_cane:'],
    '👨🏻‍🦯': [':man_with_probing_cane_tone1:', ':man_with_probing_cane_light_skin_tone:', ':man_with_probing_cane::skin-tone-1:'],
    '👨🏼‍🦯': [':man_with_probing_cane_tone2:', ':man_with_probing_cane_medium_light_skin_tone:', ':man_with_probing_cane::skin-tone-2:'],
    '👨🏽‍🦯': [':man_with_probing_cane_tone3:', ':man_with_probing_cane_medium_skin_tone:', ':man_with_probing_cane::skin-tone-3:'],
    '👨🏾‍🦯': [':man_with_probing_cane_tone4:', ':man_with_probing_cane_medium_dark_skin_tone:', ':man_with_probing_cane::skin-tone-4:'],
    '👨🏿‍🦯': [':man_with_probing_cane_tone5:', ':man_with_probing_cane_dark_skin_tone:', ':man_with_probing_cane::skin-tone-5:'],
    '👰‍♂️': [':man_with_veil:'],
    '👰🏻‍♂️': [':man_with_veil_tone1:', ':man_with_veil_light_skin_tone:', ':man_with_veil::skin-tone-1:'],
    '👰🏼‍♂️': [':man_with_veil_tone2:', ':man_with_veil_medium_light_skin_tone:', ':man_with_veil::skin-tone-2:'],
    '👰🏽‍♂️': [':man_with_veil_tone3:', ':man_with_veil_medium_skin_tone:', ':man_with_veil::skin-tone-3:'],
    '👰🏾‍♂️': [':man_with_veil_tone4:', ':man_with_veil_medium_dark_skin_tone:', ':man_with_veil::skin-tone-4:'],
    '👰🏿‍♂️': [':man_with_veil_tone5:', ':man_with_veil_dark_skin_tone:', ':man_with_veil::skin-tone-5:'],
    '🧟‍♂️': [':man_zombie:'],
    '🥭': [':mango:'],
    '👞': [':mans_shoe:'],
    '🦽': [':manual_wheelchair:'],
    '🗺️': [':map:', ':world_map:'],
    '🍁': [':maple_leaf:'],
    '🥋': [':martial_arts_uniform:', ':karate_uniform:'],
    '😷': [':mask:'],
    '🧉': [':mate:'],
    '🍖': [':meat_on_bone:'],
    '🧑‍🔧': [':mechanic:'],
    '🧑🏻‍🔧': [':mechanic_tone1:', ':mechanic_light_skin_tone:', ':mechanic::skin-tone-1:'],
    '🧑🏼‍🔧': [':mechanic_tone2:', ':mechanic_medium_light_skin_tone:', ':mechanic::skin-tone-2:'],
    '🧑🏽‍🔧': [':mechanic_tone3:', ':mechanic_medium_skin_tone:', ':mechanic::skin-tone-3:'],
    '🧑🏾‍🔧': [':mechanic_tone4:', ':mechanic_medium_dark_skin_tone:', ':mechanic::skin-tone-4:'],
    '🧑🏿‍🔧': [':mechanic_tone5:', ':mechanic_dark_skin_tone:', ':mechanic::skin-tone-5:'],
    '🦾': [':mechanical_arm:'],
    '🦿': [':mechanical_leg:'],
    '🏅': [':medal:', ':sports_medal:'],
    '⚕️': [':medical_symbol:'],
    '📣': [':mega:'],
    '🍈': [':melon:'],
    '👬': [':men_holding_hands_tone1:', ':men_holding_hands_light_skin_tone:', ':men_holding_hands_tone1_tone2:', ':men_holding_hands_light_skin_tone_medium_light_skin_tone:', ':men_holding_hands_tone1_tone3:', ':men_holding_hands_light_skin_tone_medium_skin_tone:', ':men_holding_hands_tone1_tone4:', ':men_holding_hands_light_skin_tone_medium_dark_skin_tone:', ':men_holding_hands_tone1_tone5:', ':men_holding_hands_light_skin_tone_dark_skin_tone:', ':men_holding_hands_tone2:', ':men_holding_hands_medium_light_skin_tone:', ':men_holding_hands_tone2_tone1:', ':men_holding_hands_medium_light_skin_tone_light_skin_tone:', ':men_holding_hands_tone2_tone3:', ':men_holding_hands_medium_light_skin_tone_medium_skin_tone:', ':men_holding_hands_tone2_tone4:', ':men_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:', ':men_holding_hands_tone2_tone5:', ':men_holding_hands_medium_light_skin_tone_dark_skin_tone:', ':men_holding_hands_tone3:', ':men_holding_hands_medium_skin_tone:', ':men_holding_hands_tone3_tone1:', ':men_holding_hands_medium_skin_tone_light_skin_tone:', ':men_holding_hands_tone3_tone2:', ':men_holding_hands_medium_skin_tone_medium_light_skin_tone:', ':men_holding_hands_tone3_tone4:', ':men_holding_hands_medium_skin_tone_medium_dark_skin_tone:', ':men_holding_hands_tone3_tone5:', ':men_holding_hands_medium_skin_tone_dark_skin_tone:', ':men_holding_hands_tone4:', ':men_holding_hands_medium_dark_skin_tone:', ':men_holding_hands_tone4_tone1:', ':men_holding_hands_medium_dark_skin_tone_light_skin_tone:', ':men_holding_hands_tone4_tone2:', ':men_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:', ':men_holding_hands_tone4_tone3:', ':men_holding_hands_medium_dark_skin_tone_medium_skin_tone:', ':men_holding_hands_tone4_tone5:', ':men_holding_hands_medium_dark_skin_tone_dark_skin_tone:', ':men_holding_hands_tone5:', ':men_holding_hands_dark_skin_tone:', ':men_holding_hands_tone5_tone1:', ':men_holding_hands_dark_skin_tone_light_skin_tone:', ':men_holding_hands_tone5_tone2:', ':men_holding_hands_dark_skin_tone_medium_light_skin_tone:', ':men_holding_hands_tone5_tone3:', ':men_holding_hands_dark_skin_tone_medium_skin_tone:', ':men_holding_hands_tone5_tone4:', ':men_holding_hands_dark_skin_tone_medium_dark_skin_tone:', ':two_men_holding_hands:'],
    '👯‍♂️': [':men_with_bunny_ears_partying:'],
    '🤼‍♂️': [':men_wrestling:'],
    '🕎': [':menorah:'],
    '🚹': [':mens:'],
    '🧜‍♀️': [':mermaid:'],
    '🧜🏻‍♀️': [':mermaid_tone1:', ':mermaid_light_skin_tone:', ':mermaid::skin-tone-1:'],
    '🧜🏼‍♀️': [':mermaid_tone2:', ':mermaid_medium_light_skin_tone:', ':mermaid::skin-tone-2:'],
    '🧜🏽‍♀️': [':mermaid_tone3:', ':mermaid_medium_skin_tone:', ':mermaid::skin-tone-3:'],
    '🧜🏾‍♀️': [':mermaid_tone4:', ':mermaid_medium_dark_skin_tone:', ':mermaid::skin-tone-4:'],
    '🧜🏿‍♀️': [':mermaid_tone5:', ':mermaid_dark_skin_tone:', ':mermaid::skin-tone-5:'],
    '🧜‍♂️': [':merman:'],
    '🧜🏻‍♂️': [':merman_tone1:', ':merman_light_skin_tone:', ':merman::skin-tone-1:'],
    '🧜🏼‍♂️': [':merman_tone2:', ':merman_medium_light_skin_tone:', ':merman::skin-tone-2:'],
    '🧜🏽‍♂️': [':merman_tone3:', ':merman_medium_skin_tone:', ':merman::skin-tone-3:'],
    '🧜🏾‍♂️': [':merman_tone4:', ':merman_medium_dark_skin_tone:', ':merman::skin-tone-4:'],
    '🧜🏿‍♂️': [':merman_tone5:', ':merman_dark_skin_tone:', ':merman::skin-tone-5:'],
    '🧜': [':merperson:'],
    '🧜🏻': [':merperson_tone1:', ':merperson_light_skin_tone:', ':merperson::skin-tone-1:'],
    '🧜🏼': [':merperson_tone2:', ':merperson_medium_light_skin_tone:', ':merperson::skin-tone-2:'],
    '🧜🏽': [':merperson_tone3:', ':merperson_medium_skin_tone:', ':merperson::skin-tone-3:'],
    '🧜🏾': [':merperson_tone4:', ':merperson_medium_dark_skin_tone:', ':merperson::skin-tone-4:'],
    '🧜🏿': [':merperson_tone5:', ':merperson_dark_skin_tone:', ':merperson::skin-tone-5:'],
    '🤘': [':metal:', ':sign_of_the_horns:'],
    '🤘🏻': [':metal_tone1:', ':sign_of_the_horns_tone1:', ':metal::skin-tone-1:', ':sign_of_the_horns::skin-tone-1:'],
    '🤘🏼': [':metal_tone2:', ':sign_of_the_horns_tone2:', ':metal::skin-tone-2:', ':sign_of_the_horns::skin-tone-2:'],
    '🤘🏽': [':metal_tone3:', ':sign_of_the_horns_tone3:', ':metal::skin-tone-3:', ':sign_of_the_horns::skin-tone-3:'],
    '🤘🏾': [':metal_tone4:', ':sign_of_the_horns_tone4:', ':metal::skin-tone-4:', ':sign_of_the_horns::skin-tone-4:'],
    '🤘🏿': [':metal_tone5:', ':sign_of_the_horns_tone5:', ':metal::skin-tone-5:', ':sign_of_the_horns::skin-tone-5:'],
    '🚇': [':metro:'],
    '🦠': [':microbe:'],
    '🎤': [':microphone:'],
    '🎙️': [':microphone2:', ':studio_microphone:'],
    '🔬': [':microscope:'],
    '🖕': [':middle_finger:', ':reversed_hand_with_middle_finger_extended:'],
    '🖕🏻': [':middle_finger_tone1:', ':reversed_hand_with_middle_finger_extended_tone1:', ':middle_finger::skin-tone-1:', ':reversed_hand_with_middle_finger_extended::skin-tone-1:'],
    '🖕🏼': [':middle_finger_tone2:', ':reversed_hand_with_middle_finger_extended_tone2:', ':middle_finger::skin-tone-2:', ':reversed_hand_with_middle_finger_extended::skin-tone-2:'],
    '🖕🏽': [':middle_finger_tone3:', ':reversed_hand_with_middle_finger_extended_tone3:', ':middle_finger::skin-tone-3:', ':reversed_hand_with_middle_finger_extended::skin-tone-3:'],
    '🖕🏾': [':middle_finger_tone4:', ':reversed_hand_with_middle_finger_extended_tone4:', ':middle_finger::skin-tone-4:', ':reversed_hand_with_middle_finger_extended::skin-tone-4:'],
    '🖕🏿': [':middle_finger_tone5:', ':reversed_hand_with_middle_finger_extended_tone5:', ':middle_finger::skin-tone-5:', ':reversed_hand_with_middle_finger_extended::skin-tone-5:'],
    '🪖': [':military_helmet:'],
    '🎖️': [':military_medal:'],
    '🥛': [':milk:', ':glass_of_milk:'],
    '🌌': [':milky_way:'],
    '🚐': [':minibus:'],
    '💽': [':minidisc:'],
    '🪞': [':mirror:'],
    '📱': [':mobile_phone:', ':iphone:'],
    '📴': [':mobile_phone_off:'],
    '🤑': [':money_mouth:', ':money_mouth_face:'],
    '💸': [':money_with_wings:'],
    '💰': [':moneybag:'],
    '🐒': [':monkey:'],
    '🐵': [':monkey_face:'],
    '🚝': [':monorail:'],
    '🥮': [':moon_cake:'],
    '🎓': [':mortar_board:'],
    '🕌': [':mosque:'],
    '🦟': [':mosquito:'],
    '🛵': [':motor_scooter:', ':motorbike:'],
    '🛥️': [':motorboat:'],
    '🏍️': [':motorcycle:', ':racing_motorcycle:'],
    '🦼': [':motorized_wheelchair:'],
    '🛣️': [':motorway:'],
    '🗻': [':mount_fuji:'],
    '⛰️': [':mountain:'],
    '🚠': [':mountain_cableway:'],
    '🚞': [':mountain_railway:'],
    '🏔️': [':mountain_snow:', ':snow_capped_mountain:'],
    '🐭': [':mouse:'],
    '🖱️': [':mouse_three_button:', ':three_button_mouse:'],
    '🪤': [':mouse_trap:'],
    '🐁': [':mouse2:'],
    '🎥': [':movie_camera:'],
    '🗿': [':moyai:'],
    '🤶': [':mrs_claus:', ':mother_christmas:'],
    '🤶🏻': [':mrs_claus_tone1:', ':mother_christmas_tone1:', ':mrs_claus::skin-tone-1:', ':mother_christmas::skin-tone-1:'],
    '🤶🏼': [':mrs_claus_tone2:', ':mother_christmas_tone2:', ':mrs_claus::skin-tone-2:', ':mother_christmas::skin-tone-2:'],
    '🤶🏽': [':mrs_claus_tone3:', ':mother_christmas_tone3:', ':mrs_claus::skin-tone-3:', ':mother_christmas::skin-tone-3:'],
    '🤶🏾': [':mrs_claus_tone4:', ':mother_christmas_tone4:', ':mrs_claus::skin-tone-4:', ':mother_christmas::skin-tone-4:'],
    '🤶🏿': [':mrs_claus_tone5:', ':mother_christmas_tone5:', ':mrs_claus::skin-tone-5:', ':mother_christmas::skin-tone-5:'],
    '💪': [':muscle:'],
    '💪🏻': [':muscle_tone1:', ':muscle::skin-tone-1:'],
    '💪🏼': [':muscle_tone2:', ':muscle::skin-tone-2:'],
    '💪🏽': [':muscle_tone3:', ':muscle::skin-tone-3:'],
    '💪🏾': [':muscle_tone4:', ':muscle::skin-tone-4:'],
    '💪🏿': [':muscle_tone5:', ':muscle::skin-tone-5:'],
    '🍄': [':mushroom:'],
    '🎹': [':musical_keyboard:'],
    '🎵': [':musical_note:'],
    '🎼': [':musical_score:'],
    '🔇': [':mute:'],
    '🧑‍🎄': [':mx_claus:'],
    '🧑🏻‍🎄': [':mx_claus_tone1:', ':mx_claus_light_skin_tone:', ':mx_claus::skin-tone-1:'],
    '🧑🏼‍🎄': [':mx_claus_tone2:', ':mx_claus_medium_light_skin_tone:', ':mx_claus::skin-tone-2:'],
    '🧑🏽‍🎄': [':mx_claus_tone3:', ':mx_claus_medium_skin_tone:', ':mx_claus::skin-tone-3:'],
    '🧑🏾‍🎄': [':mx_claus_tone4:', ':mx_claus_medium_dark_skin_tone:', ':mx_claus::skin-tone-4:'],
    '🧑🏿‍🎄': [':mx_claus_tone5:', ':mx_claus_dark_skin_tone:', ':mx_claus::skin-tone-5:'],
    '💅': [':nail_care:'],
    '💅🏻': [':nail_care_tone1:', ':nail_care::skin-tone-1:'],
    '💅🏼': [':nail_care_tone2:', ':nail_care::skin-tone-2:'],
    '💅🏽': [':nail_care_tone3:', ':nail_care::skin-tone-3:'],
    '💅🏾': [':nail_care_tone4:', ':nail_care::skin-tone-4:'],
    '💅🏿': [':nail_care_tone5:', ':nail_care::skin-tone-5:'],
    '📛': [':name_badge:'],
    '🤢': [':nauseated_face:', ':sick:'],
    '🧿': [':nazar_amulet:'],
    '👔': [':necktie:'],
    '❎': [':negative_squared_cross_mark:'],
    '🤓': [':nerd:', ':nerd_face:'],
    '🪆': [':nesting_dolls:'],
    '😐': [':neutral_face:', ':|', ':-|', '|', '-|'],
    '🆕': [':new:'],
    '🌑': [':new_moon:'],
    '🌚': [':new_moon_with_face:'],
    '📰': [':newspaper:'],
    '🗞️': [':newspaper2:', ':rolled_up_newspaper:'],
    '🆖': [':ng:'],
    '🌃': [':night_with_stars:'],
    '9️⃣': [':nine:'],
    '🥷': [':ninja:'],
    '🥷🏻': [':ninja_tone1:', ':ninja_light_skin_tone:', ':ninja::skin-tone-1:'],
    '🥷🏼': [':ninja_tone2:', ':ninja_medium_light_skin_tone:', ':ninja::skin-tone-2:'],
    '🥷🏽': [':ninja_tone3:', ':ninja_medium_skin_tone:', ':ninja::skin-tone-3:'],
    '🥷🏾': [':ninja_tone4:', ':ninja_medium_dark_skin_tone:', ':ninja::skin-tone-4:'],
    '🥷🏿': [':ninja_tone5:', ':ninja_dark_skin_tone:', ':ninja::skin-tone-5:'],
    '🔕': [':no_bell:'],
    '🚳': [':no_bicycles:'],
    '⛔': [':no_entry:'],
    '🚫': [':no_entry_sign:'],
    '📵': [':no_mobile_phones:'],
    '😶': [':no_mouth:'],
    '🚷': [':no_pedestrians:'],
    '🚭': [':no_smoking:'],
    '🚱': [':non_potable_water:'],
    '👃': [':nose:'],
    '👃🏻': [':nose_tone1:', ':nose::skin-tone-1:'],
    '👃🏼': [':nose_tone2:', ':nose::skin-tone-2:'],
    '👃🏽': [':nose_tone3:', ':nose::skin-tone-3:'],
    '👃🏾': [':nose_tone4:', ':nose::skin-tone-4:'],
    '👃🏿': [':nose_tone5:', ':nose::skin-tone-5:'],
    '📓': [':notebook:'],
    '📔': [':notebook_with_decorative_cover:'],
    '🗒️': [':notepad_spiral:', ':spiral_note_pad:'],
    '🎶': [':notes:'],
    '🔩': [':nut_and_bolt:'],
    '⭕': [':o:'],
    '🅾️': [':o2:'],
    '🌊': [':ocean:'],
    '🛑': [':octagonal_sign:', ':stop_sign:'],
    '🐙': [':octopus:'],
    '🍢': [':oden:'],
    '🏢': [':office:'],
    '🧑‍💼': [':office_worker:'],
    '🧑🏻‍💼': [':office_worker_tone1:', ':office_worker_light_skin_tone:', ':office_worker::skin-tone-1:'],
    '🧑🏼‍💼': [':office_worker_tone2:', ':office_worker_medium_light_skin_tone:', ':office_worker::skin-tone-2:'],
    '🧑🏽‍💼': [':office_worker_tone3:', ':office_worker_medium_skin_tone:', ':office_worker::skin-tone-3:'],
    '🧑🏾‍💼': [':office_worker_tone4:', ':office_worker_medium_dark_skin_tone:', ':office_worker::skin-tone-4:'],
    '🧑🏿‍💼': [':office_worker_tone5:', ':office_worker_dark_skin_tone:', ':office_worker::skin-tone-5:'],
    '🛢️': [':oil:', ':oil_drum:'],
    '🆗': [':ok:'],
    '👌': [':ok_hand:'],
    '👌🏻': [':ok_hand_tone1:', ':ok_hand::skin-tone-1:'],
    '👌🏼': [':ok_hand_tone2:', ':ok_hand::skin-tone-2:'],
    '👌🏽': [':ok_hand_tone3:', ':ok_hand::skin-tone-3:'],
    '👌🏾': [':ok_hand_tone4:', ':ok_hand::skin-tone-4:'],
    '👌🏿': [':ok_hand_tone5:', ':ok_hand::skin-tone-5:'],
    '🧓': [':older_adult:'],
    '🧓🏻': [':older_adult_tone1:', ':older_adult_light_skin_tone:', ':older_adult::skin-tone-1:'],
    '🧓🏼': [':older_adult_tone2:', ':older_adult_medium_light_skin_tone:', ':older_adult::skin-tone-2:'],
    '🧓🏽': [':older_adult_tone3:', ':older_adult_medium_skin_tone:', ':older_adult::skin-tone-3:'],
    '🧓🏾': [':older_adult_tone4:', ':older_adult_medium_dark_skin_tone:', ':older_adult::skin-tone-4:'],
    '🧓🏿': [':older_adult_tone5:', ':older_adult_dark_skin_tone:', ':older_adult::skin-tone-5:'],
    '👴': [':older_man:'],
    '👴🏻': [':older_man_tone1:', ':older_man::skin-tone-1:'],
    '👴🏼': [':older_man_tone2:', ':older_man::skin-tone-2:'],
    '👴🏽': [':older_man_tone3:', ':older_man::skin-tone-3:'],
    '👴🏾': [':older_man_tone4:', ':older_man::skin-tone-4:'],
    '👴🏿': [':older_man_tone5:', ':older_man::skin-tone-5:'],
    '👵': [':older_woman:', ':grandma:'],
    '👵🏻': [':older_woman_tone1:', ':grandma_tone1:', ':older_woman::skin-tone-1:', ':grandma::skin-tone-1:'],
    '👵🏼': [':older_woman_tone2:', ':grandma_tone2:', ':older_woman::skin-tone-2:', ':grandma::skin-tone-2:'],
    '👵🏽': [':older_woman_tone3:', ':grandma_tone3:', ':older_woman::skin-tone-3:', ':grandma::skin-tone-3:'],
    '👵🏾': [':older_woman_tone4:', ':grandma_tone4:', ':older_woman::skin-tone-4:', ':grandma::skin-tone-4:'],
    '👵🏿': [':older_woman_tone5:', ':grandma_tone5:', ':older_woman::skin-tone-5:', ':grandma::skin-tone-5:'],
    '🫒': [':olive:'],
    '🕉️': [':om_symbol:'],
    '🔛': [':on:'],
    '🚘': [':oncoming_automobile:'],
    '🚍': [':oncoming_bus:'],
    '🚔': [':oncoming_police_car:'],
    '🚖': [':oncoming_taxi:'],
    '1️⃣': [':one:'],
    '🩱': [':one_piece_swimsuit:'],
    '🧅': [':onion:'],
    '📂': [':open_file_folder:'],
    '👐': [':open_hands:'],
    '👐🏻': [':open_hands_tone1:', ':open_hands::skin-tone-1:'],
    '👐🏼': [':open_hands_tone2:', ':open_hands::skin-tone-2:'],
    '👐🏽': [':open_hands_tone3:', ':open_hands::skin-tone-3:'],
    '👐🏾': [':open_hands_tone4:', ':open_hands::skin-tone-4:'],
    '👐🏿': [':open_hands_tone5:', ':open_hands::skin-tone-5:'],
    '😮': [':open_mouth:', ':o', ':-o', ':O', ':-O', 'o', '-o', 'O', '-O'],
    '⛎': [':ophiuchus:'],
    '📙': [':orange_book:'],
    '🟠': [':orange_circle:'],
    '🧡': [':orange_heart:'],
    '🟧': [':orange_square:'],
    '🦧': [':orangutan:'],
    '☦️': [':orthodox_cross:'],
    '🦦': [':otter:'],
    '📤': [':outbox_tray:'],
    '🦉': [':owl:'],
    '🐂': [':ox:'],
    '🦪': [':oyster:'],
    '📦': [':package:'],
    '📄': [':page_facing_up:'],
    '📃': [':page_with_curl:'],
    '📟': [':pager:'],
    '🖌️': [':paintbrush:', ':lower_left_paintbrush:'],
    '🌴': [':palm_tree:'],
    '🤲': [':palms_up_together:'],
    '🤲🏻': [':palms_up_together_tone1:', ':palms_up_together_light_skin_tone:', ':palms_up_together::skin-tone-1:'],
    '🤲🏼': [':palms_up_together_tone2:', ':palms_up_together_medium_light_skin_tone:', ':palms_up_together::skin-tone-2:'],
    '🤲🏽': [':palms_up_together_tone3:', ':palms_up_together_medium_skin_tone:', ':palms_up_together::skin-tone-3:'],
    '🤲🏾': [':palms_up_together_tone4:', ':palms_up_together_medium_dark_skin_tone:', ':palms_up_together::skin-tone-4:'],
    '🤲🏿': [':palms_up_together_tone5:', ':palms_up_together_dark_skin_tone:', ':palms_up_together::skin-tone-5:'],
    '🥞': [':pancakes:'],
    '🐼': [':panda_face:'],
    '📎': [':paperclip:'],
    '🖇️': [':paperclips:', ':linked_paperclips:'],
    '🪂': [':parachute:'],
    '🏞️': [':park:', ':national_park:'],
    '🅿️': [':parking:'],
    '🦜': [':parrot:'],
    '〽️': [':part_alternation_mark:'],
    '⛅': [':partly_sunny:'],
    '🥳': [':partying_face:'],
    '🛂': [':passport_control:'],
    '⏸️': [':pause_button:', ':double_vertical_bar:'],
    '☮️': [':peace:', ':peace_symbol:'],
    '🍑': [':peach:'],
    '🦚': [':peacock:'],
    '🥜': [':peanuts:', ':shelled_peanut:'],
    '🍐': [':pear:'],
    '🖊️': [':pen_ballpoint:', ':lower_left_ballpoint_pen:'],
    '🖋️': [':pen_fountain:', ':lower_left_fountain_pen:'],
    '📝': [':pencil:', ':memo:'],
    '✏️': [':pencil2:'],
    '🐧': [':penguin:'],
    '😔': [':pensive:'],
    '🧑‍🤝‍🧑': [':people_holding_hands:', ':people_holding_hands_tone1:', ':people_holding_hands_light_skin_tone:', ':people_holding_hands_tone1_tone2:', ':people_holding_hands_light_skin_tone_medium_light_skin_tone:', ':people_holding_hands_tone1_tone3:', ':people_holding_hands_light_skin_tone_medium_skin_tone:', ':people_holding_hands_tone1_tone4:', ':people_holding_hands_light_skin_tone_medium_dark_skin_tone:', ':people_holding_hands_tone1_tone5:', ':people_holding_hands_light_skin_tone_dark_skin_tone:', ':people_holding_hands_tone2:', ':people_holding_hands_medium_light_skin_tone:', ':people_holding_hands_tone2_tone1:', ':people_holding_hands_medium_light_skin_tone_light_skin_tone:', ':people_holding_hands_tone2_tone3:', ':people_holding_hands_medium_light_skin_tone_medium_skin_tone:', ':people_holding_hands_tone2_tone4:', ':people_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:', ':people_holding_hands_tone2_tone5:', ':people_holding_hands_medium_light_skin_tone_dark_skin_tone:', ':people_holding_hands_tone3:', ':people_holding_hands_medium_skin_tone:', ':people_holding_hands_tone3_tone1:', ':people_holding_hands_medium_skin_tone_light_skin_tone:', ':people_holding_hands_tone3_tone2:', ':people_holding_hands_medium_skin_tone_medium_light_skin_tone:', ':people_holding_hands_tone3_tone4:', ':people_holding_hands_medium_skin_tone_medium_dark_skin_tone:', ':people_holding_hands_tone3_tone5:', ':people_holding_hands_medium_skin_tone_dark_skin_tone:', ':people_holding_hands_tone4:', ':people_holding_hands_medium_dark_skin_tone:', ':people_holding_hands_tone4_tone1:', ':people_holding_hands_medium_dark_skin_tone_light_skin_tone:', ':people_holding_hands_tone4_tone2:', ':people_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:', ':people_holding_hands_tone4_tone3:', ':people_holding_hands_medium_dark_skin_tone_medium_skin_tone:', ':people_holding_hands_tone4_tone5:', ':people_holding_hands_medium_dark_skin_tone_dark_skin_tone:', ':people_holding_hands_tone5:', ':people_holding_hands_dark_skin_tone:', ':people_holding_hands_tone5_tone1:', ':people_holding_hands_dark_skin_tone_light_skin_tone:', ':people_holding_hands_tone5_tone2:', ':people_holding_hands_dark_skin_tone_medium_light_skin_tone:', ':people_holding_hands_tone5_tone3:', ':people_holding_hands_dark_skin_tone_medium_skin_tone:', ':people_holding_hands_tone5_tone4:', ':people_holding_hands_dark_skin_tone_medium_dark_skin_tone:'],
    '🫂': [':people_hugging:'],
    '👯': [':people_with_bunny_ears_partying:', ':dancers:'],
    '🤼': [':people_wrestling:', ':wrestlers:', ':wrestling:'],
    '🎭': [':performing_arts:'],
    '😣': [':persevere:'],
    '🧑‍🦲': [':person_bald:'],
    '🚴': [':person_biking:', ':bicyclist:'],
    '🚴🏻': [':person_biking_tone1:', ':bicyclist_tone1:', ':person_biking::skin-tone-1:', ':bicyclist::skin-tone-1:'],
    '🚴🏼': [':person_biking_tone2:', ':bicyclist_tone2:', ':person_biking::skin-tone-2:', ':bicyclist::skin-tone-2:'],
    '🚴🏽': [':person_biking_tone3:', ':bicyclist_tone3:', ':person_biking::skin-tone-3:', ':bicyclist::skin-tone-3:'],
    '🚴🏾': [':person_biking_tone4:', ':bicyclist_tone4:', ':person_biking::skin-tone-4:', ':bicyclist::skin-tone-4:'],
    '🚴🏿': [':person_biking_tone5:', ':bicyclist_tone5:', ':person_biking::skin-tone-5:', ':bicyclist::skin-tone-5:'],
    '⛹️': [':person_bouncing_ball:', ':basketball_player:', ':person_with_ball:'],
    '⛹🏻': [':person_bouncing_ball_tone1:', ':basketball_player_tone1:', ':person_with_ball_tone1:', ':person_bouncing_ball::skin-tone-1:', ':basketball_player::skin-tone-1:', ':person_with_ball::skin-tone-1:'],
    '⛹🏼': [':person_bouncing_ball_tone2:', ':basketball_player_tone2:', ':person_with_ball_tone2:', ':person_bouncing_ball::skin-tone-2:', ':basketball_player::skin-tone-2:', ':person_with_ball::skin-tone-2:'],
    '⛹🏽': [':person_bouncing_ball_tone3:', ':basketball_player_tone3:', ':person_with_ball_tone3:', ':person_bouncing_ball::skin-tone-3:', ':basketball_player::skin-tone-3:', ':person_with_ball::skin-tone-3:'],
    '⛹🏾': [':person_bouncing_ball_tone4:', ':basketball_player_tone4:', ':person_with_ball_tone4:', ':person_bouncing_ball::skin-tone-4:', ':basketball_player::skin-tone-4:', ':person_with_ball::skin-tone-4:'],
    '⛹🏿': [':person_bouncing_ball_tone5:', ':basketball_player_tone5:', ':person_with_ball_tone5:', ':person_bouncing_ball::skin-tone-5:', ':basketball_player::skin-tone-5:', ':person_with_ball::skin-tone-5:'],
    '🙇': [':person_bowing:', ':bow:'],
    '🙇🏻': [':person_bowing_tone1:', ':bow_tone1:', ':person_bowing::skin-tone-1:', ':bow::skin-tone-1:'],
    '🙇🏼': [':person_bowing_tone2:', ':bow_tone2:', ':person_bowing::skin-tone-2:', ':bow::skin-tone-2:'],
    '🙇🏽': [':person_bowing_tone3:', ':bow_tone3:', ':person_bowing::skin-tone-3:', ':bow::skin-tone-3:'],
    '🙇🏾': [':person_bowing_tone4:', ':bow_tone4:', ':person_bowing::skin-tone-4:', ':bow::skin-tone-4:'],
    '🙇🏿': [':person_bowing_tone5:', ':bow_tone5:', ':person_bowing::skin-tone-5:', ':bow::skin-tone-5:'],
    '🧗': [':person_climbing:'],
    '🧗🏻': [':person_climbing_tone1:', ':person_climbing_light_skin_tone:', ':person_climbing::skin-tone-1:'],
    '🧗🏼': [':person_climbing_tone2:', ':person_climbing_medium_light_skin_tone:', ':person_climbing::skin-tone-2:'],
    '🧗🏽': [':person_climbing_tone3:', ':person_climbing_medium_skin_tone:', ':person_climbing::skin-tone-3:'],
    '🧗🏾': [':person_climbing_tone4:', ':person_climbing_medium_dark_skin_tone:', ':person_climbing::skin-tone-4:'],
    '🧗🏿': [':person_climbing_tone5:', ':person_climbing_dark_skin_tone:', ':person_climbing::skin-tone-5:'],
    '🧑‍🦱': [':person_curly_hair:'],
    '🤸': [':person_doing_cartwheel:', ':cartwheel:'],
    '🤸🏻': [':person_doing_cartwheel_tone1:', ':cartwheel_tone1:', ':person_doing_cartwheel::skin-tone-1:', ':cartwheel::skin-tone-1:'],
    '🤸🏼': [':person_doing_cartwheel_tone2:', ':cartwheel_tone2:', ':person_doing_cartwheel::skin-tone-2:', ':cartwheel::skin-tone-2:'],
    '🤸🏽': [':person_doing_cartwheel_tone3:', ':cartwheel_tone3:', ':person_doing_cartwheel::skin-tone-3:', ':cartwheel::skin-tone-3:'],
    '🤸🏾': [':person_doing_cartwheel_tone4:', ':cartwheel_tone4:', ':person_doing_cartwheel::skin-tone-4:', ':cartwheel::skin-tone-4:'],
    '🤸🏿': [':person_doing_cartwheel_tone5:', ':cartwheel_tone5:', ':person_doing_cartwheel::skin-tone-5:', ':cartwheel::skin-tone-5:'],
    '🤦': [':person_facepalming:', ':face_palm:', ':facepalm:'],
    '🤦🏻': [':person_facepalming_tone1:', ':face_palm_tone1:', ':facepalm_tone1:', ':person_facepalming::skin-tone-1:', ':face_palm::skin-tone-1:', ':facepalm::skin-tone-1:'],
    '🤦🏼': [':person_facepalming_tone2:', ':face_palm_tone2:', ':facepalm_tone2:', ':person_facepalming::skin-tone-2:', ':face_palm::skin-tone-2:', ':facepalm::skin-tone-2:'],
    '🤦🏽': [':person_facepalming_tone3:', ':face_palm_tone3:', ':facepalm_tone3:', ':person_facepalming::skin-tone-3:', ':face_palm::skin-tone-3:', ':facepalm::skin-tone-3:'],
    '🤦🏾': [':person_facepalming_tone4:', ':face_palm_tone4:', ':facepalm_tone4:', ':person_facepalming::skin-tone-4:', ':face_palm::skin-tone-4:', ':facepalm::skin-tone-4:'],
    '🤦🏿': [':person_facepalming_tone5:', ':face_palm_tone5:', ':facepalm_tone5:', ':person_facepalming::skin-tone-5:', ':face_palm::skin-tone-5:', ':facepalm::skin-tone-5:'],
    '🧑‍🍼': [':person_feeding_baby:'],
    '🧑🏻‍🍼': [':person_feeding_baby_tone1:', ':person_feeding_baby_light_skin_tone:', ':person_feeding_baby::skin-tone-1:'],
    '🧑🏼‍🍼': [':person_feeding_baby_tone2:', ':person_feeding_baby_medium_light_skin_tone:', ':person_feeding_baby::skin-tone-2:'],
    '🧑🏽‍🍼': [':person_feeding_baby_tone3:', ':person_feeding_baby_medium_skin_tone:', ':person_feeding_baby::skin-tone-3:'],
    '🧑🏾‍🍼': [':person_feeding_baby_tone4:', ':person_feeding_baby_medium_dark_skin_tone:', ':person_feeding_baby::skin-tone-4:'],
    '🧑🏿‍🍼': [':person_feeding_baby_tone5:', ':person_feeding_baby_dark_skin_tone:', ':person_feeding_baby::skin-tone-5:'],
    '🤺': [':person_fencing:', ':fencer:', ':fencing:'],
    '🙍': [':person_frowning:'],
    '🙍🏻': [':person_frowning_tone1:', ':person_frowning::skin-tone-1:'],
    '🙍🏼': [':person_frowning_tone2:', ':person_frowning::skin-tone-2:'],
    '🙍🏽': [':person_frowning_tone3:', ':person_frowning::skin-tone-3:'],
    '🙍🏾': [':person_frowning_tone4:', ':person_frowning::skin-tone-4:'],
    '🙍🏿': [':person_frowning_tone5:', ':person_frowning::skin-tone-5:'],
    '🙅': [':person_gesturing_no:', ':no_good:'],
    '🙅🏻': [':person_gesturing_no_tone1:', ':no_good_tone1:', ':person_gesturing_no::skin-tone-1:', ':no_good::skin-tone-1:'],
    '🙅🏼': [':person_gesturing_no_tone2:', ':no_good_tone2:', ':person_gesturing_no::skin-tone-2:', ':no_good::skin-tone-2:'],
    '🙅🏽': [':person_gesturing_no_tone3:', ':no_good_tone3:', ':person_gesturing_no::skin-tone-3:', ':no_good::skin-tone-3:'],
    '🙅🏾': [':person_gesturing_no_tone4:', ':no_good_tone4:', ':person_gesturing_no::skin-tone-4:', ':no_good::skin-tone-4:'],
    '🙅🏿': [':person_gesturing_no_tone5:', ':no_good_tone5:', ':person_gesturing_no::skin-tone-5:', ':no_good::skin-tone-5:'],
    '🙆': [':person_gesturing_ok:', ':ok_woman:'],
    '🙆🏻': [':person_gesturing_ok_tone1:', ':ok_woman_tone1:', ':person_gesturing_ok::skin-tone-1:', ':ok_woman::skin-tone-1:'],
    '🙆🏼': [':person_gesturing_ok_tone2:', ':ok_woman_tone2:', ':person_gesturing_ok::skin-tone-2:', ':ok_woman::skin-tone-2:'],
    '🙆🏽': [':person_gesturing_ok_tone3:', ':ok_woman_tone3:', ':person_gesturing_ok::skin-tone-3:', ':ok_woman::skin-tone-3:'],
    '🙆🏾': [':person_gesturing_ok_tone4:', ':ok_woman_tone4:', ':person_gesturing_ok::skin-tone-4:', ':ok_woman::skin-tone-4:'],
    '🙆🏿': [':person_gesturing_ok_tone5:', ':ok_woman_tone5:', ':person_gesturing_ok::skin-tone-5:', ':ok_woman::skin-tone-5:'],
    '💇': [':person_getting_haircut:', ':haircut:'],
    '💇🏻': [':person_getting_haircut_tone1:', ':haircut_tone1:', ':person_getting_haircut::skin-tone-1:', ':haircut::skin-tone-1:'],
    '💇🏼': [':person_getting_haircut_tone2:', ':haircut_tone2:', ':person_getting_haircut::skin-tone-2:', ':haircut::skin-tone-2:'],
    '💇🏽': [':person_getting_haircut_tone3:', ':haircut_tone3:', ':person_getting_haircut::skin-tone-3:', ':haircut::skin-tone-3:'],
    '💇🏾': [':person_getting_haircut_tone4:', ':haircut_tone4:', ':person_getting_haircut::skin-tone-4:', ':haircut::skin-tone-4:'],
    '💇🏿': [':person_getting_haircut_tone5:', ':haircut_tone5:', ':person_getting_haircut::skin-tone-5:', ':haircut::skin-tone-5:'],
    '💆': [':person_getting_massage:', ':massage:'],
    '💆🏻': [':person_getting_massage_tone1:', ':massage_tone1:', ':person_getting_massage::skin-tone-1:', ':massage::skin-tone-1:'],
    '💆🏼': [':person_getting_massage_tone2:', ':massage_tone2:', ':person_getting_massage::skin-tone-2:', ':massage::skin-tone-2:'],
    '💆🏽': [':person_getting_massage_tone3:', ':massage_tone3:', ':person_getting_massage::skin-tone-3:', ':massage::skin-tone-3:'],
    '💆🏾': [':person_getting_massage_tone4:', ':massage_tone4:', ':person_getting_massage::skin-tone-4:', ':massage::skin-tone-4:'],
    '💆🏿': [':person_getting_massage_tone5:', ':massage_tone5:', ':person_getting_massage::skin-tone-5:', ':massage::skin-tone-5:'],
    '🏌️': [':person_golfing:', ':golfer:'],
    '🏌🏻': [':person_golfing_tone1:', ':person_golfing_light_skin_tone:', ':person_golfing::skin-tone-1:', ':golfer::skin-tone-1:'],
    '🏌🏼': [':person_golfing_tone2:', ':person_golfing_medium_light_skin_tone:', ':person_golfing::skin-tone-2:', ':golfer::skin-tone-2:'],
    '🏌🏽': [':person_golfing_tone3:', ':person_golfing_medium_skin_tone:', ':person_golfing::skin-tone-3:', ':golfer::skin-tone-3:'],
    '🏌🏾': [':person_golfing_tone4:', ':person_golfing_medium_dark_skin_tone:', ':person_golfing::skin-tone-4:', ':golfer::skin-tone-4:'],
    '🏌🏿': [':person_golfing_tone5:', ':person_golfing_dark_skin_tone:', ':person_golfing::skin-tone-5:', ':golfer::skin-tone-5:'],
    '🛌🏻': [':person_in_bed_tone1:', ':person_in_bed_light_skin_tone:', ':sleeping_accommodation::skin-tone-1:'],
    '🛌🏼': [':person_in_bed_tone2:', ':person_in_bed_medium_light_skin_tone:', ':sleeping_accommodation::skin-tone-2:'],
    '🛌🏽': [':person_in_bed_tone3:', ':person_in_bed_medium_skin_tone:', ':sleeping_accommodation::skin-tone-3:'],
    '🛌🏾': [':person_in_bed_tone4:', ':person_in_bed_medium_dark_skin_tone:', ':sleeping_accommodation::skin-tone-4:'],
    '🛌🏿': [':person_in_bed_tone5:', ':person_in_bed_dark_skin_tone:', ':sleeping_accommodation::skin-tone-5:'],
    '🧘': [':person_in_lotus_position:'],
    '🧘🏻': [':person_in_lotus_position_tone1:', ':person_in_lotus_position_light_skin_tone:', ':person_in_lotus_position::skin-tone-1:'],
    '🧘🏼': [':person_in_lotus_position_tone2:', ':person_in_lotus_position_medium_light_skin_tone:', ':person_in_lotus_position::skin-tone-2:'],
    '🧘🏽': [':person_in_lotus_position_tone3:', ':person_in_lotus_position_medium_skin_tone:', ':person_in_lotus_position::skin-tone-3:'],
    '🧘🏾': [':person_in_lotus_position_tone4:', ':person_in_lotus_position_medium_dark_skin_tone:', ':person_in_lotus_position::skin-tone-4:'],
    '🧘🏿': [':person_in_lotus_position_tone5:', ':person_in_lotus_position_dark_skin_tone:', ':person_in_lotus_position::skin-tone-5:'],
    '🧑‍🦽': [':person_in_manual_wheelchair:'],
    '🧑🏻‍🦽': [':person_in_manual_wheelchair_tone1:', ':person_in_manual_wheelchair_light_skin_tone:', ':person_in_manual_wheelchair::skin-tone-1:'],
    '🧑🏼‍🦽': [':person_in_manual_wheelchair_tone2:', ':person_in_manual_wheelchair_medium_light_skin_tone:', ':person_in_manual_wheelchair::skin-tone-2:'],
    '🧑🏽‍🦽': [':person_in_manual_wheelchair_tone3:', ':person_in_manual_wheelchair_medium_skin_tone:', ':person_in_manual_wheelchair::skin-tone-3:'],
    '🧑🏾‍🦽': [':person_in_manual_wheelchair_tone4:', ':person_in_manual_wheelchair_medium_dark_skin_tone:', ':person_in_manual_wheelchair::skin-tone-4:'],
    '🧑🏿‍🦽': [':person_in_manual_wheelchair_tone5:', ':person_in_manual_wheelchair_dark_skin_tone:', ':person_in_manual_wheelchair::skin-tone-5:'],
    '🧑‍🦼': [':person_in_motorized_wheelchair:'],
    '🧑🏻‍🦼': [':person_in_motorized_wheelchair_tone1:', ':person_in_motorized_wheelchair_light_skin_tone:', ':person_in_motorized_wheelchair::skin-tone-1:'],
    '🧑🏼‍🦼': [':person_in_motorized_wheelchair_tone2:', ':person_in_motorized_wheelchair_medium_light_skin_tone:', ':person_in_motorized_wheelchair::skin-tone-2:'],
    '🧑🏽‍🦼': [':person_in_motorized_wheelchair_tone3:', ':person_in_motorized_wheelchair_medium_skin_tone:', ':person_in_motorized_wheelchair::skin-tone-3:'],
    '🧑🏾‍🦼': [':person_in_motorized_wheelchair_tone4:', ':person_in_motorized_wheelchair_medium_dark_skin_tone:', ':person_in_motorized_wheelchair::skin-tone-4:'],
    '🧑🏿‍🦼': [':person_in_motorized_wheelchair_tone5:', ':person_in_motorized_wheelchair_dark_skin_tone:', ':person_in_motorized_wheelchair::skin-tone-5:'],
    '🧖': [':person_in_steamy_room:'],
    '🧖🏻': [':person_in_steamy_room_tone1:', ':person_in_steamy_room_light_skin_tone:', ':person_in_steamy_room::skin-tone-1:'],
    '🧖🏼': [':person_in_steamy_room_tone2:', ':person_in_steamy_room_medium_light_skin_tone:', ':person_in_steamy_room::skin-tone-2:'],
    '🧖🏽': [':person_in_steamy_room_tone3:', ':person_in_steamy_room_medium_skin_tone:', ':person_in_steamy_room::skin-tone-3:'],
    '🧖🏾': [':person_in_steamy_room_tone4:', ':person_in_steamy_room_medium_dark_skin_tone:', ':person_in_steamy_room::skin-tone-4:'],
    '🧖🏿': [':person_in_steamy_room_tone5:', ':person_in_steamy_room_dark_skin_tone:', ':person_in_steamy_room::skin-tone-5:'],
    '🤵': [':person_in_tuxedo:'],
    '🤵🏻': [':person_in_tuxedo_tone1:', ':tuxedo_tone1:', ':person_in_tuxedo::skin-tone-1:'],
    '🤵🏼': [':person_in_tuxedo_tone2:', ':tuxedo_tone2:', ':person_in_tuxedo::skin-tone-2:'],
    '🤵🏽': [':person_in_tuxedo_tone3:', ':tuxedo_tone3:', ':person_in_tuxedo::skin-tone-3:'],
    '🤵🏾': [':person_in_tuxedo_tone4:', ':tuxedo_tone4:', ':person_in_tuxedo::skin-tone-4:'],
    '🤵🏿': [':person_in_tuxedo_tone5:', ':tuxedo_tone5:', ':person_in_tuxedo::skin-tone-5:'],
    '🤹': [':person_juggling:', ':juggling:', ':juggler:'],
    '🤹🏻': [':person_juggling_tone1:', ':juggling_tone1:', ':juggler_tone1:', ':person_juggling::skin-tone-1:', ':juggling::skin-tone-1:', ':juggler::skin-tone-1:'],
    '🤹🏼': [':person_juggling_tone2:', ':juggling_tone2:', ':juggler_tone2:', ':person_juggling::skin-tone-2:', ':juggling::skin-tone-2:', ':juggler::skin-tone-2:'],
    '🤹🏽': [':person_juggling_tone3:', ':juggling_tone3:', ':juggler_tone3:', ':person_juggling::skin-tone-3:', ':juggling::skin-tone-3:', ':juggler::skin-tone-3:'],
    '🤹🏾': [':person_juggling_tone4:', ':juggling_tone4:', ':juggler_tone4:', ':person_juggling::skin-tone-4:', ':juggling::skin-tone-4:', ':juggler::skin-tone-4:'],
    '🤹🏿': [':person_juggling_tone5:', ':juggling_tone5:', ':juggler_tone5:', ':person_juggling::skin-tone-5:', ':juggling::skin-tone-5:', ':juggler::skin-tone-5:'],
    '🧎': [':person_kneeling:'],
    '🧎🏻': [':person_kneeling_tone1:', ':person_kneeling_light_skin_tone:', ':person_kneeling::skin-tone-1:'],
    '🧎🏼': [':person_kneeling_tone2:', ':person_kneeling_medium_light_skin_tone:', ':person_kneeling::skin-tone-2:'],
    '🧎🏽': [':person_kneeling_tone3:', ':person_kneeling_medium_skin_tone:', ':person_kneeling::skin-tone-3:'],
    '🧎🏾': [':person_kneeling_tone4:', ':person_kneeling_medium_dark_skin_tone:', ':person_kneeling::skin-tone-4:'],
    '🧎🏿': [':person_kneeling_tone5:', ':person_kneeling_dark_skin_tone:', ':person_kneeling::skin-tone-5:'],
    '🏋️': [':person_lifting_weights:', ':lifter:', ':weight_lifter:'],
    '🏋🏻': [':person_lifting_weights_tone1:', ':lifter_tone1:', ':weight_lifter_tone1:', ':person_lifting_weights::skin-tone-1:', ':lifter::skin-tone-1:', ':weight_lifter::skin-tone-1:'],
    '🏋🏼': [':person_lifting_weights_tone2:', ':lifter_tone2:', ':weight_lifter_tone2:', ':person_lifting_weights::skin-tone-2:', ':lifter::skin-tone-2:', ':weight_lifter::skin-tone-2:'],
    '🏋🏽': [':person_lifting_weights_tone3:', ':lifter_tone3:', ':weight_lifter_tone3:', ':person_lifting_weights::skin-tone-3:', ':lifter::skin-tone-3:', ':weight_lifter::skin-tone-3:'],
    '🏋🏾': [':person_lifting_weights_tone4:', ':lifter_tone4:', ':weight_lifter_tone4:', ':person_lifting_weights::skin-tone-4:', ':lifter::skin-tone-4:', ':weight_lifter::skin-tone-4:'],
    '🏋🏿': [':person_lifting_weights_tone5:', ':lifter_tone5:', ':weight_lifter_tone5:', ':person_lifting_weights::skin-tone-5:', ':lifter::skin-tone-5:', ':weight_lifter::skin-tone-5:'],
    '🚵': [':person_mountain_biking:', ':mountain_bicyclist:'],
    '🚵🏻': [':person_mountain_biking_tone1:', ':mountain_bicyclist_tone1:', ':person_mountain_biking::skin-tone-1:', ':mountain_bicyclist::skin-tone-1:'],
    '🚵🏼': [':person_mountain_biking_tone2:', ':mountain_bicyclist_tone2:', ':person_mountain_biking::skin-tone-2:', ':mountain_bicyclist::skin-tone-2:'],
    '🚵🏽': [':person_mountain_biking_tone3:', ':mountain_bicyclist_tone3:', ':person_mountain_biking::skin-tone-3:', ':mountain_bicyclist::skin-tone-3:'],
    '🚵🏾': [':person_mountain_biking_tone4:', ':mountain_bicyclist_tone4:', ':person_mountain_biking::skin-tone-4:', ':mountain_bicyclist::skin-tone-4:'],
    '🚵🏿': [':person_mountain_biking_tone5:', ':mountain_bicyclist_tone5:', ':person_mountain_biking::skin-tone-5:', ':mountain_bicyclist::skin-tone-5:'],
    '🤾': [':person_playing_handball:', ':handball:'],
    '🤾🏻': [':person_playing_handball_tone1:', ':handball_tone1:', ':person_playing_handball::skin-tone-1:', ':handball::skin-tone-1:'],
    '🤾🏼': [':person_playing_handball_tone2:', ':handball_tone2:', ':person_playing_handball::skin-tone-2:', ':handball::skin-tone-2:'],
    '🤾🏽': [':person_playing_handball_tone3:', ':handball_tone3:', ':person_playing_handball::skin-tone-3:', ':handball::skin-tone-3:'],
    '🤾🏾': [':person_playing_handball_tone4:', ':handball_tone4:', ':person_playing_handball::skin-tone-4:', ':handball::skin-tone-4:'],
    '🤾🏿': [':person_playing_handball_tone5:', ':handball_tone5:', ':person_playing_handball::skin-tone-5:', ':handball::skin-tone-5:'],
    '🤽': [':person_playing_water_polo:', ':water_polo:'],
    '🤽🏻': [':person_playing_water_polo_tone1:', ':water_polo_tone1:', ':person_playing_water_polo::skin-tone-1:', ':water_polo::skin-tone-1:'],
    '🤽🏼': [':person_playing_water_polo_tone2:', ':water_polo_tone2:', ':person_playing_water_polo::skin-tone-2:', ':water_polo::skin-tone-2:'],
    '🤽🏽': [':person_playing_water_polo_tone3:', ':water_polo_tone3:', ':person_playing_water_polo::skin-tone-3:', ':water_polo::skin-tone-3:'],
    '🤽🏾': [':person_playing_water_polo_tone4:', ':water_polo_tone4:', ':person_playing_water_polo::skin-tone-4:', ':water_polo::skin-tone-4:'],
    '🤽🏿': [':person_playing_water_polo_tone5:', ':water_polo_tone5:', ':person_playing_water_polo::skin-tone-5:', ':water_polo::skin-tone-5:'],
    '🙎': [':person_pouting:', ':person_with_pouting_face:'],
    '🙎🏻': [':person_pouting_tone1:', ':person_with_pouting_face_tone1:', ':person_pouting::skin-tone-1:', ':person_with_pouting_face::skin-tone-1:'],
    '🙎🏼': [':person_pouting_tone2:', ':person_with_pouting_face_tone2:', ':person_pouting::skin-tone-2:', ':person_with_pouting_face::skin-tone-2:'],
    '🙎🏽': [':person_pouting_tone3:', ':person_with_pouting_face_tone3:', ':person_pouting::skin-tone-3:', ':person_with_pouting_face::skin-tone-3:'],
    '🙎🏾': [':person_pouting_tone4:', ':person_with_pouting_face_tone4:', ':person_pouting::skin-tone-4:', ':person_with_pouting_face::skin-tone-4:'],
    '🙎🏿': [':person_pouting_tone5:', ':person_with_pouting_face_tone5:', ':person_pouting::skin-tone-5:', ':person_with_pouting_face::skin-tone-5:'],
    '🙋': [':person_raising_hand:', ':raising_hand:'],
    '🙋🏻': [':person_raising_hand_tone1:', ':raising_hand_tone1:', ':person_raising_hand::skin-tone-1:', ':raising_hand::skin-tone-1:'],
    '🙋🏼': [':person_raising_hand_tone2:', ':raising_hand_tone2:', ':person_raising_hand::skin-tone-2:', ':raising_hand::skin-tone-2:'],
    '🙋🏽': [':person_raising_hand_tone3:', ':raising_hand_tone3:', ':person_raising_hand::skin-tone-3:', ':raising_hand::skin-tone-3:'],
    '🙋🏾': [':person_raising_hand_tone4:', ':raising_hand_tone4:', ':person_raising_hand::skin-tone-4:', ':raising_hand::skin-tone-4:'],
    '🙋🏿': [':person_raising_hand_tone5:', ':raising_hand_tone5:', ':person_raising_hand::skin-tone-5:', ':raising_hand::skin-tone-5:'],
    '🧑‍🦰': [':person_red_hair:'],
    '🚣': [':person_rowing_boat:', ':rowboat:'],
    '🚣🏻': [':person_rowing_boat_tone1:', ':rowboat_tone1:', ':person_rowing_boat::skin-tone-1:', ':rowboat::skin-tone-1:'],
    '🚣🏼': [':person_rowing_boat_tone2:', ':rowboat_tone2:', ':person_rowing_boat::skin-tone-2:', ':rowboat::skin-tone-2:'],
    '🚣🏽': [':person_rowing_boat_tone3:', ':rowboat_tone3:', ':person_rowing_boat::skin-tone-3:', ':rowboat::skin-tone-3:'],
    '🚣🏾': [':person_rowing_boat_tone4:', ':rowboat_tone4:', ':person_rowing_boat::skin-tone-4:', ':rowboat::skin-tone-4:'],
    '🚣🏿': [':person_rowing_boat_tone5:', ':rowboat_tone5:', ':person_rowing_boat::skin-tone-5:', ':rowboat::skin-tone-5:'],
    '🏃': [':person_running:', ':runner:'],
    '🏃🏻': [':person_running_tone1:', ':runner_tone1:', ':person_running::skin-tone-1:', ':runner::skin-tone-1:'],
    '🏃🏼': [':person_running_tone2:', ':runner_tone2:', ':person_running::skin-tone-2:', ':runner::skin-tone-2:'],
    '🏃🏽': [':person_running_tone3:', ':runner_tone3:', ':person_running::skin-tone-3:', ':runner::skin-tone-3:'],
    '🏃🏾': [':person_running_tone4:', ':runner_tone4:', ':person_running::skin-tone-4:', ':runner::skin-tone-4:'],
    '🏃🏿': [':person_running_tone5:', ':runner_tone5:', ':person_running::skin-tone-5:', ':runner::skin-tone-5:'],
    '🤷': [':person_shrugging:', ':shrug:'],
    '🤷🏻': [':person_shrugging_tone1:', ':shrug_tone1:', ':person_shrugging::skin-tone-1:', ':shrug::skin-tone-1:'],
    '🤷🏼': [':person_shrugging_tone2:', ':shrug_tone2:', ':person_shrugging::skin-tone-2:', ':shrug::skin-tone-2:'],
    '🤷🏽': [':person_shrugging_tone3:', ':shrug_tone3:', ':person_shrugging::skin-tone-3:', ':shrug::skin-tone-3:'],
    '🤷🏾': [':person_shrugging_tone4:', ':shrug_tone4:', ':person_shrugging::skin-tone-4:', ':shrug::skin-tone-4:'],
    '🤷🏿': [':person_shrugging_tone5:', ':shrug_tone5:', ':person_shrugging::skin-tone-5:', ':shrug::skin-tone-5:'],
    '🧍': [':person_standing:'],
    '🧍🏻': [':person_standing_tone1:', ':person_standing_light_skin_tone:', ':person_standing::skin-tone-1:'],
    '🧍🏼': [':person_standing_tone2:', ':person_standing_medium_light_skin_tone:', ':person_standing::skin-tone-2:'],
    '🧍🏽': [':person_standing_tone3:', ':person_standing_medium_skin_tone:', ':person_standing::skin-tone-3:'],
    '🧍🏾': [':person_standing_tone4:', ':person_standing_medium_dark_skin_tone:', ':person_standing::skin-tone-4:'],
    '🧍🏿': [':person_standing_tone5:', ':person_standing_dark_skin_tone:', ':person_standing::skin-tone-5:'],
    '🏄': [':person_surfing:', ':surfer:'],
    '🏄🏻': [':person_surfing_tone1:', ':surfer_tone1:', ':person_surfing::skin-tone-1:', ':surfer::skin-tone-1:'],
    '🏄🏼': [':person_surfing_tone2:', ':surfer_tone2:', ':person_surfing::skin-tone-2:', ':surfer::skin-tone-2:'],
    '🏄🏽': [':person_surfing_tone3:', ':surfer_tone3:', ':person_surfing::skin-tone-3:', ':surfer::skin-tone-3:'],
    '🏄🏾': [':person_surfing_tone4:', ':surfer_tone4:', ':person_surfing::skin-tone-4:', ':surfer::skin-tone-4:'],
    '🏄🏿': [':person_surfing_tone5:', ':surfer_tone5:', ':person_surfing::skin-tone-5:', ':surfer::skin-tone-5:'],
    '🏊': [':person_swimming:', ':swimmer:'],
    '🏊🏻': [':person_swimming_tone1:', ':swimmer_tone1:', ':person_swimming::skin-tone-1:', ':swimmer::skin-tone-1:'],
    '🏊🏼': [':person_swimming_tone2:', ':swimmer_tone2:', ':person_swimming::skin-tone-2:', ':swimmer::skin-tone-2:'],
    '🏊🏽': [':person_swimming_tone3:', ':swimmer_tone3:', ':person_swimming::skin-tone-3:', ':swimmer::skin-tone-3:'],
    '🏊🏾': [':person_swimming_tone4:', ':swimmer_tone4:', ':person_swimming::skin-tone-4:', ':swimmer::skin-tone-4:'],
    '🏊🏿': [':person_swimming_tone5:', ':swimmer_tone5:', ':person_swimming::skin-tone-5:', ':swimmer::skin-tone-5:'],
    '💁': [':person_tipping_hand:', ':information_desk_person:'],
    '💁🏻': [':person_tipping_hand_tone1:', ':information_desk_person_tone1:', ':person_tipping_hand::skin-tone-1:', ':information_desk_person::skin-tone-1:'],
    '💁🏼': [':person_tipping_hand_tone2:', ':information_desk_person_tone2:', ':person_tipping_hand::skin-tone-2:', ':information_desk_person::skin-tone-2:'],
    '💁🏽': [':person_tipping_hand_tone3:', ':information_desk_person_tone3:', ':person_tipping_hand::skin-tone-3:', ':information_desk_person::skin-tone-3:'],
    '💁🏾': [':person_tipping_hand_tone4:', ':information_desk_person_tone4:', ':person_tipping_hand::skin-tone-4:', ':information_desk_person::skin-tone-4:'],
    '💁🏿': [':person_tipping_hand_tone5:', ':information_desk_person_tone5:', ':person_tipping_hand::skin-tone-5:', ':information_desk_person::skin-tone-5:'],
    '🧑🏻‍🦲': [':person_tone1_bald:', ':person_light_skin_tone_bald:', ':person_bald::skin-tone-1:'],
    '🧑🏻‍🦱': [':person_tone1_curly_hair:', ':person_light_skin_tone_curly_hair:', ':person_curly_hair::skin-tone-1:'],
    '🧑🏻‍🦰': [':person_tone1_red_hair:', ':person_light_skin_tone_red_hair:', ':person_red_hair::skin-tone-1:'],
    '🧑🏻‍🦳': [':person_tone1_white_hair:', ':person_light_skin_tone_white_hair:', ':person_white_hair::skin-tone-1:'],
    '🧑🏼‍🦲': [':person_tone2_bald:', ':person_medium_light_skin_tone_bald:', ':person_bald::skin-tone-2:'],
    '🧑🏼‍🦱': [':person_tone2_curly_hair:', ':person_medium_light_skin_tone_curly_hair:', ':person_curly_hair::skin-tone-2:'],
    '🧑🏼‍🦰': [':person_tone2_red_hair:', ':person_medium_light_skin_tone_red_hair:', ':person_red_hair::skin-tone-2:'],
    '🧑🏼‍🦳': [':person_tone2_white_hair:', ':person_medium_light_skin_tone_white_hair:', ':person_white_hair::skin-tone-2:'],
    '🧑🏽‍🦲': [':person_tone3_bald:', ':person_medium_skin_tone_bald:', ':person_bald::skin-tone-3:'],
    '🧑🏽‍🦱': [':person_tone3_curly_hair:', ':person_medium_skin_tone_curly_hair:', ':person_curly_hair::skin-tone-3:'],
    '🧑🏽‍🦰': [':person_tone3_red_hair:', ':person_medium_skin_tone_red_hair:', ':person_red_hair::skin-tone-3:'],
    '🧑🏽‍🦳': [':person_tone3_white_hair:', ':person_medium_skin_tone_white_hair:', ':person_white_hair::skin-tone-3:'],
    '🧑🏾‍🦲': [':person_tone4_bald:', ':person_medium_dark_skin_tone_bald:', ':person_bald::skin-tone-4:'],
    '🧑🏾‍🦱': [':person_tone4_curly_hair:', ':person_medium_dark_skin_tone_curly_hair:', ':person_curly_hair::skin-tone-4:'],
    '🧑🏾‍🦰': [':person_tone4_red_hair:', ':person_medium_dark_skin_tone_red_hair:', ':person_red_hair::skin-tone-4:'],
    '🧑🏾‍🦳': [':person_tone4_white_hair:', ':person_medium_dark_skin_tone_white_hair:', ':person_white_hair::skin-tone-4:'],
    '🧑🏿‍🦲': [':person_tone5_bald:', ':person_dark_skin_tone_bald:', ':person_bald::skin-tone-5:'],
    '🧑🏿‍🦱': [':person_tone5_curly_hair:', ':person_dark_skin_tone_curly_hair:', ':person_curly_hair::skin-tone-5:'],
    '🧑🏿‍🦰': [':person_tone5_red_hair:', ':person_dark_skin_tone_red_hair:', ':person_red_hair::skin-tone-5:'],
    '🧑🏿‍🦳': [':person_tone5_white_hair:', ':person_dark_skin_tone_white_hair:', ':person_white_hair::skin-tone-5:'],
    '🚶': [':person_walking:', ':walking:'],
    '🚶🏻': [':person_walking_tone1:', ':walking_tone1:', ':person_walking::skin-tone-1:', ':walking::skin-tone-1:'],
    '🚶🏼': [':person_walking_tone2:', ':walking_tone2:', ':person_walking::skin-tone-2:', ':walking::skin-tone-2:'],
    '🚶🏽': [':person_walking_tone3:', ':walking_tone3:', ':person_walking::skin-tone-3:', ':walking::skin-tone-3:'],
    '🚶🏾': [':person_walking_tone4:', ':walking_tone4:', ':person_walking::skin-tone-4:', ':walking::skin-tone-4:'],
    '🚶🏿': [':person_walking_tone5:', ':walking_tone5:', ':person_walking::skin-tone-5:', ':walking::skin-tone-5:'],
    '👳': [':person_wearing_turban:', ':man_with_turban:'],
    '👳🏻': [':person_wearing_turban_tone1:', ':man_with_turban_tone1:', ':person_wearing_turban::skin-tone-1:', ':man_with_turban::skin-tone-1:'],
    '👳🏼': [':person_wearing_turban_tone2:', ':man_with_turban_tone2:', ':person_wearing_turban::skin-tone-2:', ':man_with_turban::skin-tone-2:'],
    '👳🏽': [':person_wearing_turban_tone3:', ':man_with_turban_tone3:', ':person_wearing_turban::skin-tone-3:', ':man_with_turban::skin-tone-3:'],
    '👳🏾': [':person_wearing_turban_tone4:', ':man_with_turban_tone4:', ':person_wearing_turban::skin-tone-4:', ':man_with_turban::skin-tone-4:'],
    '👳🏿': [':person_wearing_turban_tone5:', ':man_with_turban_tone5:', ':person_wearing_turban::skin-tone-5:', ':man_with_turban::skin-tone-5:'],
    '🧑‍🦳': [':person_white_hair:'],
    '🧑‍🦯': [':person_with_probing_cane:'],
    '🧑🏻‍🦯': [':person_with_probing_cane_tone1:', ':person_with_probing_cane_light_skin_tone:', ':person_with_probing_cane::skin-tone-1:'],
    '🧑🏼‍🦯': [':person_with_probing_cane_tone2:', ':person_with_probing_cane_medium_light_skin_tone:', ':person_with_probing_cane::skin-tone-2:'],
    '🧑🏽‍🦯': [':person_with_probing_cane_tone3:', ':person_with_probing_cane_medium_skin_tone:', ':person_with_probing_cane::skin-tone-3:'],
    '🧑🏾‍🦯': [':person_with_probing_cane_tone4:', ':person_with_probing_cane_medium_dark_skin_tone:', ':person_with_probing_cane::skin-tone-4:'],
    '🧑🏿‍🦯': [':person_with_probing_cane_tone5:', ':person_with_probing_cane_dark_skin_tone:', ':person_with_probing_cane::skin-tone-5:'],
    '👰': [':person_with_veil:'],
    '👰🏻': [':person_with_veil_tone1:', ':person_with_veil::skin-tone-1:'],
    '👰🏼': [':person_with_veil_tone2:', ':person_with_veil::skin-tone-2:'],
    '👰🏽': [':person_with_veil_tone3:', ':person_with_veil::skin-tone-3:'],
    '👰🏾': [':person_with_veil_tone4:', ':person_with_veil::skin-tone-4:'],
    '👰🏿': [':person_with_veil_tone5:', ':person_with_veil::skin-tone-5:'],
    '🧫': [':petri_dish:'],
    '⛏️': [':pick:'],
    '🛻': [':pickup_truck:'],
    '🥧': [':pie:'],
    '🐷': [':pig:'],
    '🐽': [':pig_nose:'],
    '🐖': [':pig2:'],
    '💊': [':pill:'],
    '🧑‍✈️': [':pilot:'],
    '🧑🏻‍✈️': [':pilot_tone1:', ':pilot_light_skin_tone:', ':pilot::skin-tone-1:'],
    '🧑🏼‍✈️': [':pilot_tone2:', ':pilot_medium_light_skin_tone:', ':pilot::skin-tone-2:'],
    '🧑🏽‍✈️': [':pilot_tone3:', ':pilot_medium_skin_tone:', ':pilot::skin-tone-3:'],
    '🧑🏾‍✈️': [':pilot_tone4:', ':pilot_medium_dark_skin_tone:', ':pilot::skin-tone-4:'],
    '🧑🏿‍✈️': [':pilot_tone5:', ':pilot_dark_skin_tone:', ':pilot::skin-tone-5:'],
    '🪅': [':piñata:'],
    '🤌': [':pinched_fingers:'],
    '🤌🏻': [':pinched_fingers_tone1:', ':pinched_fingers_light_skin_tone:', ':pinched_fingers::skin-tone-1:'],
    '🤌🏼': [':pinched_fingers_tone2:', ':pinched_fingers_medium_light_skin_tone:', ':pinched_fingers::skin-tone-2:'],
    '🤌🏽': [':pinched_fingers_tone3:', ':pinched_fingers_medium_skin_tone:', ':pinched_fingers::skin-tone-3:'],
    '🤌🏾': [':pinched_fingers_tone4:', ':pinched_fingers_medium_dark_skin_tone:', ':pinched_fingers::skin-tone-4:'],
    '🤌🏿': [':pinched_fingers_tone5:', ':pinched_fingers_dark_skin_tone:', ':pinched_fingers::skin-tone-5:'],
    '🤏': [':pinching_hand:'],
    '🤏🏻': [':pinching_hand_tone1:', ':pinching_hand_light_skin_tone:', ':pinching_hand::skin-tone-1:'],
    '🤏🏼': [':pinching_hand_tone2:', ':pinching_hand_medium_light_skin_tone:', ':pinching_hand::skin-tone-2:'],
    '🤏🏽': [':pinching_hand_tone3:', ':pinching_hand_medium_skin_tone:', ':pinching_hand::skin-tone-3:'],
    '🤏🏾': [':pinching_hand_tone4:', ':pinching_hand_medium_dark_skin_tone:', ':pinching_hand::skin-tone-4:'],
    '🤏🏿': [':pinching_hand_tone5:', ':pinching_hand_dark_skin_tone:', ':pinching_hand::skin-tone-5:'],
    '🍍': [':pineapple:'],
    '🏓': [':ping_pong:', ':table_tennis:'],
    '🏴‍☠️': [':pirate_flag:'],
    '♓': [':pisces:'],
    '🍕': [':pizza:'],
    '🪧': [':placard:'],
    '🛐': [':place_of_worship:', ':worship_symbol:'],
    '⏯️': [':play_pause:'],
    '🥺': [':pleading_face:'],
    '🪠': [':plunger:'],
    '👇': [':point_down:'],
    '👇🏻': [':point_down_tone1:', ':point_down::skin-tone-1:'],
    '👇🏼': [':point_down_tone2:', ':point_down::skin-tone-2:'],
    '👇🏽': [':point_down_tone3:', ':point_down::skin-tone-3:'],
    '👇🏾': [':point_down_tone4:', ':point_down::skin-tone-4:'],
    '👇🏿': [':point_down_tone5:', ':point_down::skin-tone-5:'],
    '👈': [':point_left:'],
    '👈🏻': [':point_left_tone1:', ':point_left::skin-tone-1:'],
    '👈🏼': [':point_left_tone2:', ':point_left::skin-tone-2:'],
    '👈🏽': [':point_left_tone3:', ':point_left::skin-tone-3:'],
    '👈🏾': [':point_left_tone4:', ':point_left::skin-tone-4:'],
    '👈🏿': [':point_left_tone5:', ':point_left::skin-tone-5:'],
    '👉': [':point_right:'],
    '👉🏻': [':point_right_tone1:', ':point_right::skin-tone-1:'],
    '👉🏼': [':point_right_tone2:', ':point_right::skin-tone-2:'],
    '👉🏽': [':point_right_tone3:', ':point_right::skin-tone-3:'],
    '👉🏾': [':point_right_tone4:', ':point_right::skin-tone-4:'],
    '👉🏿': [':point_right_tone5:', ':point_right::skin-tone-5:'],
    '☝️': [':point_up:'],
    '👆': [':point_up_2:'],
    '👆🏻': [':point_up_2_tone1:', ':point_up_2::skin-tone-1:'],
    '👆🏼': [':point_up_2_tone2:', ':point_up_2::skin-tone-2:'],
    '👆🏽': [':point_up_2_tone3:', ':point_up_2::skin-tone-3:'],
    '👆🏾': [':point_up_2_tone4:', ':point_up_2::skin-tone-4:'],
    '👆🏿': [':point_up_2_tone5:', ':point_up_2::skin-tone-5:'],
    '☝🏻': [':point_up_tone1:', ':point_up::skin-tone-1:'],
    '☝🏼': [':point_up_tone2:', ':point_up::skin-tone-2:'],
    '☝🏽': [':point_up_tone3:', ':point_up::skin-tone-3:'],
    '☝🏾': [':point_up_tone4:', ':point_up::skin-tone-4:'],
    '☝🏿': [':point_up_tone5:', ':point_up::skin-tone-5:'],
    '🐻‍❄️': [':polar_bear:'],
    '🚓': [':police_car:'],
    '👮': [':police_officer:', ':cop:'],
    '👮🏻': [':police_officer_tone1:', ':cop_tone1:', ':police_officer::skin-tone-1:', ':cop::skin-tone-1:'],
    '👮🏼': [':police_officer_tone2:', ':cop_tone2:', ':police_officer::skin-tone-2:', ':cop::skin-tone-2:'],
    '👮🏽': [':police_officer_tone3:', ':cop_tone3:', ':police_officer::skin-tone-3:', ':cop::skin-tone-3:'],
    '👮🏾': [':police_officer_tone4:', ':cop_tone4:', ':police_officer::skin-tone-4:', ':cop::skin-tone-4:'],
    '👮🏿': [':police_officer_tone5:', ':cop_tone5:', ':police_officer::skin-tone-5:', ':cop::skin-tone-5:'],
    '🐩': [':poodle:'],
    '💩': [':poop:', ':shit:', ':hankey:', ':poo:'],
    '🍿': [':popcorn:'],
    '🏣': [':post_office:'],
    '📯': [':postal_horn:'],
    '📮': [':postbox:'],
    '🚰': [':potable_water:'],
    '🥔': [':potato:'],
    '🪴': [':potted_plant:'],
    '👝': [':pouch:'],
    '🍗': [':poultry_leg:'],
    '💷': [':pound:'],
    '😾': [':pouting_cat:'],
    '🙏': [':pray:'],
    '🙏🏻': [':pray_tone1:', ':pray::skin-tone-1:'],
    '🙏🏼': [':pray_tone2:', ':pray::skin-tone-2:'],
    '🙏🏽': [':pray_tone3:', ':pray::skin-tone-3:'],
    '🙏🏾': [':pray_tone4:', ':pray::skin-tone-4:'],
    '🙏🏿': [':pray_tone5:', ':pray::skin-tone-5:'],
    '📿': [':prayer_beads:'],
    '🤰': [':pregnant_woman:', ':expecting_woman:'],
    '🤰🏻': [':pregnant_woman_tone1:', ':expecting_woman_tone1:', ':pregnant_woman::skin-tone-1:', ':expecting_woman::skin-tone-1:'],
    '🤰🏼': [':pregnant_woman_tone2:', ':expecting_woman_tone2:', ':pregnant_woman::skin-tone-2:', ':expecting_woman::skin-tone-2:'],
    '🤰🏽': [':pregnant_woman_tone3:', ':expecting_woman_tone3:', ':pregnant_woman::skin-tone-3:', ':expecting_woman::skin-tone-3:'],
    '🤰🏾': [':pregnant_woman_tone4:', ':expecting_woman_tone4:', ':pregnant_woman::skin-tone-4:', ':expecting_woman::skin-tone-4:'],
    '🤰🏿': [':pregnant_woman_tone5:', ':expecting_woman_tone5:', ':pregnant_woman::skin-tone-5:', ':expecting_woman::skin-tone-5:'],
    '🥨': [':pretzel:'],
    '🤴': [':prince:'],
    '🤴🏻': [':prince_tone1:', ':prince::skin-tone-1:'],
    '🤴🏼': [':prince_tone2:', ':prince::skin-tone-2:'],
    '🤴🏽': [':prince_tone3:', ':prince::skin-tone-3:'],
    '🤴🏾': [':prince_tone4:', ':prince::skin-tone-4:'],
    '🤴🏿': [':prince_tone5:', ':prince::skin-tone-5:'],
    '👸': [':princess:'],
    '👸🏻': [':princess_tone1:', ':princess::skin-tone-1:'],
    '👸🏼': [':princess_tone2:', ':princess::skin-tone-2:'],
    '👸🏽': [':princess_tone3:', ':princess::skin-tone-3:'],
    '👸🏾': [':princess_tone4:', ':princess::skin-tone-4:'],
    '👸🏿': [':princess_tone5:', ':princess::skin-tone-5:'],
    '🖨️': [':printer:'],
    '🦯': [':probing_cane:'],
    '📽️': [':projector:', ':film_projector:'],
    '👊': [':punch:'],
    '👊🏻': [':punch_tone1:', ':punch::skin-tone-1:'],
    '👊🏼': [':punch_tone2:', ':punch::skin-tone-2:'],
    '👊🏽': [':punch_tone3:', ':punch::skin-tone-3:'],
    '👊🏾': [':punch_tone4:', ':punch::skin-tone-4:'],
    '👊🏿': [':punch_tone5:', ':punch::skin-tone-5:'],
    '🟣': [':purple_circle:'],
    '💜': [':purple_heart:'],
    '🟪': [':purple_square:'],
    '👛': [':purse:'],
    '📌': [':pushpin:'],
    '🚮': [':put_litter_in_its_place:'],
    '❓': [':question:'],
    '🐰': [':rabbit:'],
    '🐇': [':rabbit2:'],
    '🦝': [':raccoon:'],
    '🏎️': [':race_car:', ':racing_car:'],
    '🐎': [':racehorse:'],
    '📻': [':radio:'],
    '🔘': [':radio_button:'],
    '☢️': [':radioactive:', ':radioactive_sign:'],
    '😡': [':rage:', ':@', ':-@', '@', '-@'],
    '🚃': [':railway_car:'],
    '🛤️': [':railway_track:', ':railroad_track:'],
    '🌈': [':rainbow:'],
    '🏳️‍🌈': [':rainbow_flag:', ':gay_pride_flag:'],
    '🤚': [':raised_back_of_hand:', ':back_of_hand:'],
    '🤚🏻': [':raised_back_of_hand_tone1:', ':back_of_hand_tone1:', ':raised_back_of_hand::skin-tone-1:', ':back_of_hand::skin-tone-1:'],
    '🤚🏼': [':raised_back_of_hand_tone2:', ':back_of_hand_tone2:', ':raised_back_of_hand::skin-tone-2:', ':back_of_hand::skin-tone-2:'],
    '🤚🏽': [':raised_back_of_hand_tone3:', ':back_of_hand_tone3:', ':raised_back_of_hand::skin-tone-3:', ':back_of_hand::skin-tone-3:'],
    '🤚🏾': [':raised_back_of_hand_tone4:', ':back_of_hand_tone4:', ':raised_back_of_hand::skin-tone-4:', ':back_of_hand::skin-tone-4:'],
    '🤚🏿': [':raised_back_of_hand_tone5:', ':back_of_hand_tone5:', ':raised_back_of_hand::skin-tone-5:', ':back_of_hand::skin-tone-5:'],
    '✋': [':raised_hand:'],
    '✋🏻': [':raised_hand_tone1:', ':raised_hand::skin-tone-1:'],
    '✋🏼': [':raised_hand_tone2:', ':raised_hand::skin-tone-2:'],
    '✋🏽': [':raised_hand_tone3:', ':raised_hand::skin-tone-3:'],
    '✋🏾': [':raised_hand_tone4:', ':raised_hand::skin-tone-4:'],
    '✋🏿': [':raised_hand_tone5:', ':raised_hand::skin-tone-5:'],
    '🙌': [':raised_hands:'],
    '🙌🏻': [':raised_hands_tone1:', ':raised_hands::skin-tone-1:'],
    '🙌🏼': [':raised_hands_tone2:', ':raised_hands::skin-tone-2:'],
    '🙌🏽': [':raised_hands_tone3:', ':raised_hands::skin-tone-3:'],
    '🙌🏾': [':raised_hands_tone4:', ':raised_hands::skin-tone-4:'],
    '🙌🏿': [':raised_hands_tone5:', ':raised_hands::skin-tone-5:'],
    '🐏': [':ram:'],
    '🍜': [':ramen:'],
    '🐀': [':rat:'],
    '🪒': [':razor:'],
    '🧾': [':receipt:'],
    '⏺️': [':record_button:'],
    '♻️': [':recycle:'],
    '🚗': [':red_car:'],
    '🔴': [':red_circle:'],
    '🧧': [':red_envelope:'],
    '🟥': [':red_square:'],
    '🇦': [':regional_indicator_a:'],
    '🇧': [':regional_indicator_b:'],
    '🇨': [':regional_indicator_c:'],
    '🇩': [':regional_indicator_d:'],
    '🇪': [':regional_indicator_e:'],
    '🇫': [':regional_indicator_f:'],
    '🇬': [':regional_indicator_g:'],
    '🇭': [':regional_indicator_h:'],
    '🇮': [':regional_indicator_i:'],
    '🇯': [':regional_indicator_j:'],
    '🇰': [':regional_indicator_k:'],
    '🇱': [':regional_indicator_l:'],
    '🇲': [':regional_indicator_m:'],
    '🇳': [':regional_indicator_n:'],
    '🇴': [':regional_indicator_o:'],
    '🇵': [':regional_indicator_p:'],
    '🇶': [':regional_indicator_q:'],
    '🇷': [':regional_indicator_r:'],
    '🇸': [':regional_indicator_s:'],
    '🇹': [':regional_indicator_t:'],
    '🇺': [':regional_indicator_u:'],
    '🇻': [':regional_indicator_v:'],
    '🇼': [':regional_indicator_w:'],
    '🇽': [':regional_indicator_x:'],
    '🇾': [':regional_indicator_y:'],
    '🇿': [':regional_indicator_z:'],
    '®️': [':registered:'],
    '☺️': [':relaxed:'],
    '😌': [':relieved:'],
    '🎗️': [':reminder_ribbon:'],
    '🔁': [':repeat:'],
    '🔂': [':repeat_one:'],
    '🚻': [':restroom:'],
    '💞': [':revolving_hearts:'],
    '⏪': [':rewind:'],
    '🦏': [':rhino:', ':rhinoceros:'],
    '🎀': [':ribbon:'],
    '🍚': [':rice:'],
    '🍙': [':rice_ball:'],
    '🍘': [':rice_cracker:'],
    '🎑': [':rice_scene:'],
    '🤜': [':right_facing_fist:', ':right_fist:'],
    '🤜🏻': [':right_facing_fist_tone1:', ':right_fist_tone1:', ':right_facing_fist::skin-tone-1:', ':right_fist::skin-tone-1:'],
    '🤜🏼': [':right_facing_fist_tone2:', ':right_fist_tone2:', ':right_facing_fist::skin-tone-2:', ':right_fist::skin-tone-2:'],
    '🤜🏽': [':right_facing_fist_tone3:', ':right_fist_tone3:', ':right_facing_fist::skin-tone-3:', ':right_fist::skin-tone-3:'],
    '🤜🏾': [':right_facing_fist_tone4:', ':right_fist_tone4:', ':right_facing_fist::skin-tone-4:', ':right_fist::skin-tone-4:'],
    '🤜🏿': [':right_facing_fist_tone5:', ':right_fist_tone5:', ':right_facing_fist::skin-tone-5:', ':right_fist::skin-tone-5:'],
    '💍': [':ring:'],
    '🪐': [':ringed_planet:'],
    '🤖': [':robot:', ':robot_face:'],
    '🪨': [':rock:'],
    '🚀': [':rocket:'],
    '🤣': [':rofl:', ':rolling_on_the_floor_laughing:'],
    '🧻': [':roll_of_paper:'],
    '🎢': [':roller_coaster:'],
    '🛼': [':roller_skate:'],
    '🙄': [':rolling_eyes:', ':face_with_rolling_eyes:'],
    '🐓': [':rooster:'],
    '🌹': [':rose:'],
    '🏵️': [':rosette:'],
    '🚨': [':rotating_light:'],
    '📍': [':round_pushpin:'],
    '🏉': [':rugby_football:'],
    '🎽': [':running_shirt_with_sash:'],
    '🈂️': [':sa:'],
    '🧷': [':safety_pin:'],
    '🦺': [':safety_vest:'],
    '♐': [':sagittarius:'],
    '⛵': [':sailboat:'],
    '🍶': [':sake:'],
    '🥗': [':salad:', ':green_salad:'],
    '🧂': [':salt:'],
    '👡': [':sandal:'],
    '🥪': [':sandwich:'],
    '🎅': [':santa:'],
    '🎅🏻': [':santa_tone1:', ':santa::skin-tone-1:'],
    '🎅🏼': [':santa_tone2:', ':santa::skin-tone-2:'],
    '🎅🏽': [':santa_tone3:', ':santa::skin-tone-3:'],
    '🎅🏾': [':santa_tone4:', ':santa::skin-tone-4:'],
    '🎅🏿': [':santa_tone5:', ':santa::skin-tone-5:'],
    '🥻': [':sari:'],
    '📡': [':satellite:'],
    '🛰️': [':satellite_orbital:'],
    '🦕': [':sauropod:'],
    '🎷': [':saxophone:'],
    '⚖️': [':scales:'],
    '🧣': [':scarf:'],
    '🏫': [':school:'],
    '🎒': [':school_satchel:'],
    '🧑‍🔬': [':scientist:'],
    '🧑🏻‍🔬': [':scientist_tone1:', ':scientist_light_skin_tone:', ':scientist::skin-tone-1:'],
    '🧑🏼‍🔬': [':scientist_tone2:', ':scientist_medium_light_skin_tone:', ':scientist::skin-tone-2:'],
    '🧑🏽‍🔬': [':scientist_tone3:', ':scientist_medium_skin_tone:', ':scientist::skin-tone-3:'],
    '🧑🏾‍🔬': [':scientist_tone4:', ':scientist_medium_dark_skin_tone:', ':scientist::skin-tone-4:'],
    '🧑🏿‍🔬': [':scientist_tone5:', ':scientist_dark_skin_tone:', ':scientist::skin-tone-5:'],
    '✂️': [':scissors:'],
    '🛴': [':scooter:'],
    '🦂': [':scorpion:'],
    '♏': [':scorpius:'],
    '🏴󠁧󠁢󠁳󠁣󠁴󠁿': [':scotland:'],
    '😱': [':scream:'],
    '🙀': [':scream_cat:'],
    '🪛': [':screwdriver:'],
    '📜': [':scroll:'],
    '🦭': [':seal:'],
    '💺': [':seat:'],
    '🥈': [':second_place:', ':second_place_medal:'],
    '㊙️': [':secret:'],
    '🙈': [':see_no_evil:'],
    '🌱': [':seedling:'],
    '🤳': [':selfie:'],
    '🤳🏻': [':selfie_tone1:', ':selfie::skin-tone-1:'],
    '🤳🏼': [':selfie_tone2:', ':selfie::skin-tone-2:'],
    '🤳🏽': [':selfie_tone3:', ':selfie::skin-tone-3:'],
    '🤳🏾': [':selfie_tone4:', ':selfie::skin-tone-4:'],
    '🤳🏿': [':selfie_tone5:', ':selfie::skin-tone-5:'],
    '🐕‍🦺': [':service_dog:'],
    '7️⃣': [':seven:'],
    '🪡': [':sewing_needle:'],
    '🥘': [':shallow_pan_of_food:', ':paella:'],
    '☘️': [':shamrock:'],
    '🦈': [':shark:'],
    '🍧': [':shaved_ice:'],
    '🐑': [':sheep:'],
    '🐚': [':shell:'],
    '🛡️': [':shield:'],
    '⛩️': [':shinto_shrine:'],
    '🚢': [':ship:'],
    '👕': [':shirt:'],
    '🛍️': [':shopping_bags:'],
    '🛒': [':shopping_cart:', ':shopping_trolley:'],
    '🩳': [':shorts:'],
    '🚿': [':shower:'],
    '🦐': [':shrimp:'],
    '🤫': [':shushing_face:'],
    '📶': [':signal_strength:'],
    '🧑‍🎤': [':singer:'],
    '🧑🏻‍🎤': [':singer_tone1:', ':singer_light_skin_tone:', ':singer::skin-tone-1:'],
    '🧑🏼‍🎤': [':singer_tone2:', ':singer_medium_light_skin_tone:', ':singer::skin-tone-2:'],
    '🧑🏽‍🎤': [':singer_tone3:', ':singer_medium_skin_tone:', ':singer::skin-tone-3:'],
    '🧑🏾‍🎤': [':singer_tone4:', ':singer_medium_dark_skin_tone:', ':singer::skin-tone-4:'],
    '🧑🏿‍🎤': [':singer_tone5:', ':singer_dark_skin_tone:', ':singer::skin-tone-5:'],
    '6️⃣': [':six:'],
    '🔯': [':six_pointed_star:'],
    '🛹': [':skateboard:'],
    '🎿': [':ski:'],
    '⛷️': [':skier:'],
    '💀': [':skull:', ':skeleton:'],
    '☠️': [':skull_crossbones:', ':skull_and_crossbones:'],
    '🦨': [':skunk:'],
    '🛷': [':sled:'],
    '😴': [':sleeping:'],
    '🛌': [':sleeping_accommodation:'],
    '😪': [':sleepy:'],
    '🙁': [':slight_frown:', ':slightly_frowning_face:'],
    '🙂': [':slight_smile:', ':slightly_smiling_face:', ':)', ':-)', ')', '-)'],
    '🎰': [':slot_machine:'],
    '🦥': [':sloth:'],
    '🔹': [':small_blue_diamond:'],
    '🔸': [':small_orange_diamond:'],
    '🔺': [':small_red_triangle:'],
    '🔻': [':small_red_triangle_down:'],
    '😄': [':smile:', ':D', ':-D', 'D', '-D'],
    '😸': [':smile_cat:'],
    '😃': [':smiley:'],
    '😺': [':smiley_cat:'],
    '🥰': [':smiling_face_with_3_hearts:'],
    '🥲': [':smiling_face_with_tear:'],
    '😈': [':smiling_imp:', ':)', ':-)', ')', '-)'],
    '😏': [':smirk:'],
    '😼': [':smirk_cat:'],
    '🚬': [':smoking:'],
    '🐌': [':snail:'],
    '🐍': [':snake:'],
    '🤧': [':sneezing_face:', ':sneeze:'],
    '🏂': [':snowboarder:'],
    '🏂🏻': [':snowboarder_tone1:', ':snowboarder_light_skin_tone:', ':snowboarder::skin-tone-1:'],
    '🏂🏼': [':snowboarder_tone2:', ':snowboarder_medium_light_skin_tone:', ':snowboarder::skin-tone-2:'],
    '🏂🏽': [':snowboarder_tone3:', ':snowboarder_medium_skin_tone:', ':snowboarder::skin-tone-3:'],
    '🏂🏾': [':snowboarder_tone4:', ':snowboarder_medium_dark_skin_tone:', ':snowboarder::skin-tone-4:'],
    '🏂🏿': [':snowboarder_tone5:', ':snowboarder_dark_skin_tone:', ':snowboarder::skin-tone-5:'],
    '❄️': [':snowflake:'],
    '⛄': [':snowman:'],
    '☃️': [':snowman2:'],
    '🧼': [':soap:'],
    '😭': [':sob:', ":'(", ":'-(", ';(', ';-(', "'(", "'-("],
    '⚽': [':soccer:'],
    '🧦': [':socks:'],
    '🥎': [':softball:'],
    '🔜': [':soon:'],
    '🆘': [':sos:'],
    '🔉': [':sound:'],
    '👾': [':space_invader:'],
    '♠️': [':spades:'],
    '🍝': [':spaghetti:'],
    '❇️': [':sparkle:'],
    '🎇': [':sparkler:'],
    '✨': [':sparkles:'],
    '💖': [':sparkling_heart:'],
    '🙊': [':speak_no_evil:'],
    '🔈': [':speaker:'],
    '🗣️': [':speaking_head:', ':speaking_head_in_silhouette:'],
    '💬': [':speech_balloon:'],
    '🗨️': [':speech_left:', ':left_speech_bubble:'],
    '🚤': [':speedboat:'],
    '🕷️': [':spider:'],
    '🕸️': [':spider_web:'],
    '🧽': [':sponge:'],
    '🥄': [':spoon:'],
    '🧴': [':squeeze_bottle:'],
    '🦑': [':squid:'],
    '🏟️': [':stadium:'],
    '⭐': [':star:'],
    '☪️': [':star_and_crescent:'],
    '✡️': [':star_of_david:'],
    '🤩': [':star_struck:'],
    '🌟': [':star2:'],
    '🌠': [':stars:'],
    '🚉': [':station:'],
    '🗽': [':statue_of_liberty:'],
    '🚂': [':steam_locomotive:'],
    '🩺': [':stethoscope:'],
    '🍲': [':stew:'],
    '⏹️': [':stop_button:'],
    '⏱️': [':stopwatch:'],
    '📏': [':straight_ruler:'],
    '🍓': [':strawberry:'],
    '😛': [':stuck_out_tongue:', ':P', ':-P', 'P', '-P'],
    '😝': [':stuck_out_tongue_closed_eyes:'],
    '😜': [':stuck_out_tongue_winking_eye:'],
    '🧑‍🎓': [':student:'],
    '🧑🏻‍🎓': [':student_tone1:', ':student_light_skin_tone:', ':student::skin-tone-1:'],
    '🧑🏼‍🎓': [':student_tone2:', ':student_medium_light_skin_tone:', ':student::skin-tone-2:'],
    '🧑🏽‍🎓': [':student_tone3:', ':student_medium_skin_tone:', ':student::skin-tone-3:'],
    '🧑🏾‍🎓': [':student_tone4:', ':student_medium_dark_skin_tone:', ':student::skin-tone-4:'],
    '🧑🏿‍🎓': [':student_tone5:', ':student_dark_skin_tone:', ':student::skin-tone-5:'],
    '🥙': [':stuffed_flatbread:', ':stuffed_pita:'],
    '🌞': [':sun_with_face:'],
    '🌻': [':sunflower:'],
    '😎': [':sunglasses:', '8-)', 'B-)'],
    '☀️': [':sunny:'],
    '🌅': [':sunrise:'],
    '🌄': [':sunrise_over_mountains:'],
    '🦸': [':superhero:'],
    '🦸🏻': [':superhero_tone1:', ':superhero_light_skin_tone:', ':superhero::skin-tone-1:'],
    '🦸🏼': [':superhero_tone2:', ':superhero_medium_light_skin_tone:', ':superhero::skin-tone-2:'],
    '🦸🏽': [':superhero_tone3:', ':superhero_medium_skin_tone:', ':superhero::skin-tone-3:'],
    '🦸🏾': [':superhero_tone4:', ':superhero_medium_dark_skin_tone:', ':superhero::skin-tone-4:'],
    '🦸🏿': [':superhero_tone5:', ':superhero_dark_skin_tone:', ':superhero::skin-tone-5:'],
    '🦹': [':supervillain:'],
    '🦹🏻': [':supervillain_tone1:', ':supervillain_light_skin_tone:', ':supervillain::skin-tone-1:'],
    '🦹🏼': [':supervillain_tone2:', ':supervillain_medium_light_skin_tone:', ':supervillain::skin-tone-2:'],
    '🦹🏽': [':supervillain_tone3:', ':supervillain_medium_skin_tone:', ':supervillain::skin-tone-3:'],
    '🦹🏾': [':supervillain_tone4:', ':supervillain_medium_dark_skin_tone:', ':supervillain::skin-tone-4:'],
    '🦹🏿': [':supervillain_tone5:', ':supervillain_dark_skin_tone:', ':supervillain::skin-tone-5:'],
    '🍣': [':sushi:'],
    '🚟': [':suspension_railway:'],
    '🦢': [':swan:'],
    '😓': [':sweat:', ':(', ':-(', '(', '-('],
    '💦': [':sweat_drops:'],
    '😅': [':sweat_smile:', ':)', ':-)', ')', '-)'],
    '🍠': [':sweet_potato:'],
    '🔣': [':symbols:'],
    '🕍': [':synagogue:'],
    '💉': [':syringe:'],
    '🦖': [':t_rex:'],
    '🌮': [':taco:'],
    '🎉': [':tada:'],
    '🥡': [':takeout_box:'],
    '🫔': [':tamale:'],
    '🎋': [':tanabata_tree:'],
    '🍊': [':tangerine:'],
    '♉': [':taurus:'],
    '🚕': [':taxi:'],
    '🍵': [':tea:'],
    '🧑‍🏫': [':teacher:'],
    '🧑🏻‍🏫': [':teacher_tone1:', ':teacher_light_skin_tone:', ':teacher::skin-tone-1:'],
    '🧑🏼‍🏫': [':teacher_tone2:', ':teacher_medium_light_skin_tone:', ':teacher::skin-tone-2:'],
    '🧑🏽‍🏫': [':teacher_tone3:', ':teacher_medium_skin_tone:', ':teacher::skin-tone-3:'],
    '🧑🏾‍🏫': [':teacher_tone4:', ':teacher_medium_dark_skin_tone:', ':teacher::skin-tone-4:'],
    '🧑🏿‍🏫': [':teacher_tone5:', ':teacher_dark_skin_tone:', ':teacher::skin-tone-5:'],
    '🫖': [':teapot:'],
    '🧑‍💻': [':technologist:'],
    '🧑🏻‍💻': [':technologist_tone1:', ':technologist_light_skin_tone:', ':technologist::skin-tone-1:'],
    '🧑🏼‍💻': [':technologist_tone2:', ':technologist_medium_light_skin_tone:', ':technologist::skin-tone-2:'],
    '🧑🏽‍💻': [':technologist_tone3:', ':technologist_medium_skin_tone:', ':technologist::skin-tone-3:'],
    '🧑🏾‍💻': [':technologist_tone4:', ':technologist_medium_dark_skin_tone:', ':technologist::skin-tone-4:'],
    '🧑🏿‍💻': [':technologist_tone5:', ':technologist_dark_skin_tone:', ':technologist::skin-tone-5:'],
    '🧸': [':teddy_bear:'],
    '☎️': [':telephone:'],
    '📞': [':telephone_receiver:'],
    '🔭': [':telescope:'],
    '🎾': [':tennis:'],
    '⛺': [':tent:'],
    '🧪': [':test_tube:'],
    '🌡️': [':thermometer:'],
    '🤒': [':thermometer_face:', ':face_with_thermometer:'],
    '🤔': [':thinking:', ':thinking_face:'],
    '🥉': [':third_place:', ':third_place_medal:'],
    '🩴': [':thong_sandal:'],
    '💭': [':thought_balloon:'],
    '🧵': [':thread:'],
    '3️⃣': [':three:'],
    '👎': [':thumbsdown:', ':-1:', ':thumbdown:'],
    '👎🏻': [':thumbsdown_tone1:', ':_1_tone1:', ':thumbdown_tone1:', ':thumbsdown::skin-tone-1:', ':-1::skin-tone-1:', ':thumbdown::skin-tone-1:'],
    '👎🏼': [':thumbsdown_tone2:', ':_1_tone2:', ':thumbdown_tone2:', ':thumbsdown::skin-tone-2:', ':-1::skin-tone-2:', ':thumbdown::skin-tone-2:'],
    '👎🏽': [':thumbsdown_tone3:', ':_1_tone3:', ':thumbdown_tone3:', ':thumbsdown::skin-tone-3:', ':-1::skin-tone-3:', ':thumbdown::skin-tone-3:'],
    '👎🏾': [':thumbsdown_tone4:', ':_1_tone4:', ':thumbdown_tone4:', ':thumbsdown::skin-tone-4:', ':-1::skin-tone-4:', ':thumbdown::skin-tone-4:'],
    '👎🏿': [':thumbsdown_tone5:', ':_1_tone5:', ':thumbdown_tone5:', ':thumbsdown::skin-tone-5:', ':-1::skin-tone-5:', ':thumbdown::skin-tone-5:'],
    '👍': [':thumbsup:', ':+1:', ':thumbup:'],
    '👍🏻': [':thumbsup_tone1:', ':+1_tone1:', ':thumbup_tone1:', ':thumbsup::skin-tone-1:', ':+1::skin-tone-1:', ':thumbup::skin-tone-1:'],
    '👍🏼': [':thumbsup_tone2:', ':+1_tone2:', ':thumbup_tone2:', ':thumbsup::skin-tone-2:', ':+1::skin-tone-2:', ':thumbup::skin-tone-2:'],
    '👍🏽': [':thumbsup_tone3:', ':+1_tone3:', ':thumbup_tone3:', ':thumbsup::skin-tone-3:', ':+1::skin-tone-3:', ':thumbup::skin-tone-3:'],
    '👍🏾': [':thumbsup_tone4:', ':+1_tone4:', ':thumbup_tone4:', ':thumbsup::skin-tone-4:', ':+1::skin-tone-4:', ':thumbup::skin-tone-4:'],
    '👍🏿': [':thumbsup_tone5:', ':+1_tone5:', ':thumbup_tone5:', ':thumbsup::skin-tone-5:', ':+1::skin-tone-5:', ':thumbup::skin-tone-5:'],
    '⛈️': [':thunder_cloud_rain:', ':thunder_cloud_and_rain:'],
    '🎫': [':ticket:'],
    '🎟️': [':tickets:', ':admission_tickets:'],
    '🐯': [':tiger:'],
    '🐅': [':tiger2:'],
    '⏲️': [':timer:', ':timer_clock:'],
    '😫': [':tired_face:'],
    '™️': [':tm:'],
    '🚽': [':toilet:'],
    '🗼': [':tokyo_tower:'],
    '🍅': [':tomato:'],
    '👅': [':tongue:'],
    '🧰': [':toolbox:'],
    '🛠️': [':tools:', ':hammer_and_wrench:'],
    '🦷': [':tooth:'],
    '🪥': [':toothbrush:'],
    '🔝': [':top:'],
    '🎩': [':tophat:'],
    '⏭️': [':track_next:', ':next_track:'],
    '⏮️': [':track_previous:', ':previous_track:'],
    '🖲️': [':trackball:'],
    '🚜': [':tractor:'],
    '🚥': [':traffic_light:'],
    '🚋': [':train:'],
    '🚆': [':train2:'],
    '🚊': [':tram:'],
    '🏳️‍⚧️': [':transgender_flag:'],
    '⚧': [':transgender_symbol:'],
    '🚩': [':triangular_flag_on_post:'],
    '📐': [':triangular_ruler:'],
    '🔱': [':trident:'],
    '😤': [':triumph:'],
    '🚎': [':trolleybus:'],
    '🏆': [':trophy:'],
    '🍹': [':tropical_drink:'],
    '🐠': [':tropical_fish:'],
    '🚚': [':truck:'],
    '🎺': [':trumpet:'],
    '🌷': [':tulip:'],
    '🥃': [':tumbler_glass:', ':whisky:'],
    '🦃': [':turkey:'],
    '🐢': [':turtle:'],
    '📺': [':tv:'],
    '🔀': [':twisted_rightwards_arrows:'],
    '2️⃣': [':two:'],
    '💕': [':two_hearts:'],
    '👭': [':two_women_holding_hands:', ':women_holding_hands_tone1:', ':women_holding_hands_light_skin_tone:', ':women_holding_hands_tone1_tone2:', ':women_holding_hands_light_skin_tone_medium_light_skin_tone:', ':women_holding_hands_tone1_tone3:', ':women_holding_hands_light_skin_tone_medium_skin_tone:', ':women_holding_hands_tone1_tone4:', ':women_holding_hands_light_skin_tone_medium_dark_skin_tone:', ':women_holding_hands_tone1_tone5:', ':women_holding_hands_light_skin_tone_dark_skin_tone:', ':women_holding_hands_tone2:', ':women_holding_hands_medium_light_skin_tone:', ':women_holding_hands_tone2_tone1:', ':women_holding_hands_medium_light_skin_tone_light_skin_tone:', ':women_holding_hands_tone2_tone3:', ':women_holding_hands_medium_light_skin_tone_medium_skin_tone:', ':women_holding_hands_tone2_tone4:', ':women_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:', ':women_holding_hands_tone2_tone5:', ':women_holding_hands_medium_light_skin_tone_dark_skin_tone:', ':women_holding_hands_tone3:', ':women_holding_hands_medium_skin_tone:', ':women_holding_hands_tone3_tone1:', ':women_holding_hands_medium_skin_tone_light_skin_tone:', ':women_holding_hands_tone3_tone2:', ':women_holding_hands_medium_skin_tone_medium_light_skin_tone:', ':women_holding_hands_tone3_tone4:', ':women_holding_hands_medium_skin_tone_medium_dark_skin_tone:', ':women_holding_hands_tone3_tone5:', ':women_holding_hands_medium_skin_tone_dark_skin_tone:', ':women_holding_hands_tone4:', ':women_holding_hands_medium_dark_skin_tone:', ':women_holding_hands_tone4_tone1:', ':women_holding_hands_medium_dark_skin_tone_light_skin_tone:', ':women_holding_hands_tone4_tone2:', ':women_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:', ':women_holding_hands_tone4_tone3:', ':women_holding_hands_medium_dark_skin_tone_medium_skin_tone:', ':women_holding_hands_tone4_tone5:', ':women_holding_hands_medium_dark_skin_tone_dark_skin_tone:', ':women_holding_hands_tone5:', ':women_holding_hands_dark_skin_tone:', ':women_holding_hands_tone5_tone1:', ':women_holding_hands_dark_skin_tone_light_skin_tone:', ':women_holding_hands_tone5_tone2:', ':women_holding_hands_dark_skin_tone_medium_light_skin_tone:', ':women_holding_hands_tone5_tone3:', ':women_holding_hands_dark_skin_tone_medium_skin_tone:', ':women_holding_hands_tone5_tone4:', ':women_holding_hands_dark_skin_tone_medium_dark_skin_tone:'],
    '🈹': [':u5272:'],
    '🈴': [':u5408:'],
    '🈺': [':u55b6:'],
    '🈯': [':u6307:'],
    '🈷️': [':u6708:'],
    '🈶': [':u6709:'],
    '🈵': [':u6e80:'],
    '🈚': [':u7121:'],
    '🈸': [':u7533:'],
    '🈲': [':u7981:'],
    '🈳': [':u7a7a:'],
    '☔': [':umbrella:'],
    '☂️': [':umbrella2:'],
    '😒': [':unamused:', ':s', ':-S', ':z', ':-Z', ':$', ':-$', 's', '-S', 'z', '-Z', '$', '-$'],
    '🔞': [':underage:'],
    '🦄': [':unicorn:', ':unicorn_face:'],
    '🇺🇳': [':united_nations:'],
    '🔓': [':unlock:'],
    '🆙': [':up:'],
    '🙃': [':upside_down:', ':upside_down_face:'],
    '⚱️': [':urn:', ':funeral_urn:'],
    '✌️': [':v:'],
    '✌🏻': [':v_tone1:', ':v::skin-tone-1:'],
    '✌🏼': [':v_tone2:', ':v::skin-tone-2:'],
    '✌🏽': [':v_tone3:', ':v::skin-tone-3:'],
    '✌🏾': [':v_tone4:', ':v::skin-tone-4:'],
    '✌🏿': [':v_tone5:', ':v::skin-tone-5:'],
    '🧛': [':vampire:'],
    '🧛🏻': [':vampire_tone1:', ':vampire_light_skin_tone:', ':vampire::skin-tone-1:'],
    '🧛🏼': [':vampire_tone2:', ':vampire_medium_light_skin_tone:', ':vampire::skin-tone-2:'],
    '🧛🏽': [':vampire_tone3:', ':vampire_medium_skin_tone:', ':vampire::skin-tone-3:'],
    '🧛🏾': [':vampire_tone4:', ':vampire_medium_dark_skin_tone:', ':vampire::skin-tone-4:'],
    '🧛🏿': [':vampire_tone5:', ':vampire_dark_skin_tone:', ':vampire::skin-tone-5:'],
    '🚦': [':vertical_traffic_light:'],
    '📼': [':vhs:'],
    '📳': [':vibration_mode:'],
    '📹': [':video_camera:'],
    '🎮': [':video_game:'],
    '🎻': [':violin:'],
    '♍': [':virgo:'],
    '🌋': [':volcano:'],
    '🏐': [':volleyball:'],
    '🆚': [':vs:'],
    '🖖': [':vulcan:', ':raised_hand_with_part_between_middle_and_ring_fingers:'],
    '🖖🏻': [':vulcan_tone1:', ':raised_hand_with_part_between_middle_and_ring_fingers_tone1:', ':vulcan::skin-tone-1:', ':raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-1:'],
    '🖖🏼': [':vulcan_tone2:', ':raised_hand_with_part_between_middle_and_ring_fingers_tone2:', ':vulcan::skin-tone-2:', ':raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-2:'],
    '🖖🏽': [':vulcan_tone3:', ':raised_hand_with_part_between_middle_and_ring_fingers_tone3:', ':vulcan::skin-tone-3:', ':raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-3:'],
    '🖖🏾': [':vulcan_tone4:', ':raised_hand_with_part_between_middle_and_ring_fingers_tone4:', ':vulcan::skin-tone-4:', ':raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-4:'],
    '🖖🏿': [':vulcan_tone5:', ':raised_hand_with_part_between_middle_and_ring_fingers_tone5:', ':vulcan::skin-tone-5:', ':raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-5:'],
    '🧇': [':waffle:'],
    '🏴󠁧󠁢󠁷󠁬󠁳󠁿': [':wales:'],
    '🌘': [':waning_crescent_moon:'],
    '🌖': [':waning_gibbous_moon:'],
    '⚠️': [':warning:'],
    '🗑️': [':wastebasket:'],
    '⌚': [':watch:'],
    '🐃': [':water_buffalo:'],
    '🍉': [':watermelon:'],
    '👋': [':wave:'],
    '👋🏻': [':wave_tone1:', ':wave::skin-tone-1:'],
    '👋🏼': [':wave_tone2:', ':wave::skin-tone-2:'],
    '👋🏽': [':wave_tone3:', ':wave::skin-tone-3:'],
    '👋🏾': [':wave_tone4:', ':wave::skin-tone-4:'],
    '👋🏿': [':wave_tone5:', ':wave::skin-tone-5:'],
    '〰️': [':wavy_dash:'],
    '🌒': [':waxing_crescent_moon:'],
    '🌔': [':waxing_gibbous_moon:'],
    '🚾': [':wc:'],
    '😩': [':weary:'],
    '💒': [':wedding:'],
    '🐳': [':whale:'],
    '🐋': [':whale2:'],
    '☸️': [':wheel_of_dharma:'],
    '♿': [':wheelchair:'],
    '✅': [':white_check_mark:'],
    '⚪': [':white_circle:'],
    '💮': [':white_flower:'],
    '🤍': [':white_heart:'],
    '⬜': [':white_large_square:'],
    '◽': [':white_medium_small_square:'],
    '◻️': [':white_medium_square:'],
    '▫️': [':white_small_square:'],
    '🔳': [':white_square_button:'],
    '🌥️': [':white_sun_cloud:', ':white_sun_behind_cloud:'],
    '🌦️': [':white_sun_rain_cloud:', ':white_sun_behind_cloud_with_rain:'],
    '🌤️': [':white_sun_small_cloud:', ':white_sun_with_small_cloud:'],
    '🥀': [':wilted_rose:', ':wilted_flower:'],
    '🌬️': [':wind_blowing_face:'],
    '🎐': [':wind_chime:'],
    '🪟': [':window:'],
    '🍷': [':wine_glass:'],
    '😉': [':wink:', ';)', ';-)'],
    '🐺': [':wolf:'],
    '👩': [':woman:'],
    '👩‍🎨': [':woman_artist:'],
    '👩🏻‍🎨': [':woman_artist_tone1:', ':woman_artist_light_skin_tone:', ':woman_artist::skin-tone-1:'],
    '👩🏼‍🎨': [':woman_artist_tone2:', ':woman_artist_medium_light_skin_tone:', ':woman_artist::skin-tone-2:'],
    '👩🏽‍🎨': [':woman_artist_tone3:', ':woman_artist_medium_skin_tone:', ':woman_artist::skin-tone-3:'],
    '👩🏾‍🎨': [':woman_artist_tone4:', ':woman_artist_medium_dark_skin_tone:', ':woman_artist::skin-tone-4:'],
    '👩🏿‍🎨': [':woman_artist_tone5:', ':woman_artist_dark_skin_tone:', ':woman_artist::skin-tone-5:'],
    '👩‍🚀': [':woman_astronaut:'],
    '👩🏻‍🚀': [':woman_astronaut_tone1:', ':woman_astronaut_light_skin_tone:', ':woman_astronaut::skin-tone-1:'],
    '👩🏼‍🚀': [':woman_astronaut_tone2:', ':woman_astronaut_medium_light_skin_tone:', ':woman_astronaut::skin-tone-2:'],
    '👩🏽‍🚀': [':woman_astronaut_tone3:', ':woman_astronaut_medium_skin_tone:', ':woman_astronaut::skin-tone-3:'],
    '👩🏾‍🚀': [':woman_astronaut_tone4:', ':woman_astronaut_medium_dark_skin_tone:', ':woman_astronaut::skin-tone-4:'],
    '👩🏿‍🚀': [':woman_astronaut_tone5:', ':woman_astronaut_dark_skin_tone:', ':woman_astronaut::skin-tone-5:'],
    '👩‍🦲': [':woman_bald:'],
    '👩🏻‍🦲': [':woman_bald_tone1:', ':woman_bald_light_skin_tone:', ':woman_bald::skin-tone-1:'],
    '👩🏼‍🦲': [':woman_bald_tone2:', ':woman_bald_medium_light_skin_tone:', ':woman_bald::skin-tone-2:'],
    '👩🏽‍🦲': [':woman_bald_tone3:', ':woman_bald_medium_skin_tone:', ':woman_bald::skin-tone-3:'],
    '👩🏾‍🦲': [':woman_bald_tone4:', ':woman_bald_medium_dark_skin_tone:', ':woman_bald::skin-tone-4:'],
    '👩🏿‍🦲': [':woman_bald_tone5:', ':woman_bald_dark_skin_tone:', ':woman_bald::skin-tone-5:'],
    '🚴‍♀️': [':woman_biking:'],
    '🚴🏻‍♀️': [':woman_biking_tone1:', ':woman_biking_light_skin_tone:', ':woman_biking::skin-tone-1:'],
    '🚴🏼‍♀️': [':woman_biking_tone2:', ':woman_biking_medium_light_skin_tone:', ':woman_biking::skin-tone-2:'],
    '🚴🏽‍♀️': [':woman_biking_tone3:', ':woman_biking_medium_skin_tone:', ':woman_biking::skin-tone-3:'],
    '🚴🏾‍♀️': [':woman_biking_tone4:', ':woman_biking_medium_dark_skin_tone:', ':woman_biking::skin-tone-4:'],
    '🚴🏿‍♀️': [':woman_biking_tone5:', ':woman_biking_dark_skin_tone:', ':woman_biking::skin-tone-5:'],
    '⛹️‍♀️': [':woman_bouncing_ball:'],
    '⛹🏻‍♀️': [':woman_bouncing_ball_tone1:', ':woman_bouncing_ball_light_skin_tone:', ':woman_bouncing_ball::skin-tone-1:'],
    '⛹🏼‍♀️': [':woman_bouncing_ball_tone2:', ':woman_bouncing_ball_medium_light_skin_tone:', ':woman_bouncing_ball::skin-tone-2:'],
    '⛹🏽‍♀️': [':woman_bouncing_ball_tone3:', ':woman_bouncing_ball_medium_skin_tone:', ':woman_bouncing_ball::skin-tone-3:'],
    '⛹🏾‍♀️': [':woman_bouncing_ball_tone4:', ':woman_bouncing_ball_medium_dark_skin_tone:', ':woman_bouncing_ball::skin-tone-4:'],
    '⛹🏿‍♀️': [':woman_bouncing_ball_tone5:', ':woman_bouncing_ball_dark_skin_tone:', ':woman_bouncing_ball::skin-tone-5:'],
    '🙇‍♀️': [':woman_bowing:'],
    '🙇🏻‍♀️': [':woman_bowing_tone1:', ':woman_bowing_light_skin_tone:', ':woman_bowing::skin-tone-1:'],
    '🙇🏼‍♀️': [':woman_bowing_tone2:', ':woman_bowing_medium_light_skin_tone:', ':woman_bowing::skin-tone-2:'],
    '🙇🏽‍♀️': [':woman_bowing_tone3:', ':woman_bowing_medium_skin_tone:', ':woman_bowing::skin-tone-3:'],
    '🙇🏾‍♀️': [':woman_bowing_tone4:', ':woman_bowing_medium_dark_skin_tone:', ':woman_bowing::skin-tone-4:'],
    '🙇🏿‍♀️': [':woman_bowing_tone5:', ':woman_bowing_dark_skin_tone:', ':woman_bowing::skin-tone-5:'],
    '🤸‍♀️': [':woman_cartwheeling:'],
    '🤸🏻‍♀️': [':woman_cartwheeling_tone1:', ':woman_cartwheeling_light_skin_tone:', ':woman_cartwheeling::skin-tone-1:'],
    '🤸🏼‍♀️': [':woman_cartwheeling_tone2:', ':woman_cartwheeling_medium_light_skin_tone:', ':woman_cartwheeling::skin-tone-2:'],
    '🤸🏽‍♀️': [':woman_cartwheeling_tone3:', ':woman_cartwheeling_medium_skin_tone:', ':woman_cartwheeling::skin-tone-3:'],
    '🤸🏾‍♀️': [':woman_cartwheeling_tone4:', ':woman_cartwheeling_medium_dark_skin_tone:', ':woman_cartwheeling::skin-tone-4:'],
    '🤸🏿‍♀️': [':woman_cartwheeling_tone5:', ':woman_cartwheeling_dark_skin_tone:', ':woman_cartwheeling::skin-tone-5:'],
    '🧗‍♀️': [':woman_climbing:'],
    '🧗🏻‍♀️': [':woman_climbing_tone1:', ':woman_climbing_light_skin_tone:', ':woman_climbing::skin-tone-1:'],
    '🧗🏼‍♀️': [':woman_climbing_tone2:', ':woman_climbing_medium_light_skin_tone:', ':woman_climbing::skin-tone-2:'],
    '🧗🏽‍♀️': [':woman_climbing_tone3:', ':woman_climbing_medium_skin_tone:', ':woman_climbing::skin-tone-3:'],
    '🧗🏾‍♀️': [':woman_climbing_tone4:', ':woman_climbing_medium_dark_skin_tone:', ':woman_climbing::skin-tone-4:'],
    '🧗🏿‍♀️': [':woman_climbing_tone5:', ':woman_climbing_dark_skin_tone:', ':woman_climbing::skin-tone-5:'],
    '👷‍♀️': [':woman_construction_worker:'],
    '👷🏻‍♀️': [':woman_construction_worker_tone1:', ':woman_construction_worker_light_skin_tone:', ':woman_construction_worker::skin-tone-1:'],
    '👷🏼‍♀️': [':woman_construction_worker_tone2:', ':woman_construction_worker_medium_light_skin_tone:', ':woman_construction_worker::skin-tone-2:'],
    '👷🏽‍♀️': [':woman_construction_worker_tone3:', ':woman_construction_worker_medium_skin_tone:', ':woman_construction_worker::skin-tone-3:'],
    '👷🏾‍♀️': [':woman_construction_worker_tone4:', ':woman_construction_worker_medium_dark_skin_tone:', ':woman_construction_worker::skin-tone-4:'],
    '👷🏿‍♀️': [':woman_construction_worker_tone5:', ':woman_construction_worker_dark_skin_tone:', ':woman_construction_worker::skin-tone-5:'],
    '👩‍🍳': [':woman_cook:'],
    '👩🏻‍🍳': [':woman_cook_tone1:', ':woman_cook_light_skin_tone:', ':woman_cook::skin-tone-1:'],
    '👩🏼‍🍳': [':woman_cook_tone2:', ':woman_cook_medium_light_skin_tone:', ':woman_cook::skin-tone-2:'],
    '👩🏽‍🍳': [':woman_cook_tone3:', ':woman_cook_medium_skin_tone:', ':woman_cook::skin-tone-3:'],
    '👩🏾‍🍳': [':woman_cook_tone4:', ':woman_cook_medium_dark_skin_tone:', ':woman_cook::skin-tone-4:'],
    '👩🏿‍🍳': [':woman_cook_tone5:', ':woman_cook_dark_skin_tone:', ':woman_cook::skin-tone-5:'],
    '👩‍🦱': [':woman_curly_haired:'],
    '👩🏻‍🦱': [':woman_curly_haired_tone1:', ':woman_curly_haired_light_skin_tone:', ':woman_curly_haired::skin-tone-1:'],
    '👩🏼‍🦱': [':woman_curly_haired_tone2:', ':woman_curly_haired_medium_light_skin_tone:', ':woman_curly_haired::skin-tone-2:'],
    '👩🏽‍🦱': [':woman_curly_haired_tone3:', ':woman_curly_haired_medium_skin_tone:', ':woman_curly_haired::skin-tone-3:'],
    '👩🏾‍🦱': [':woman_curly_haired_tone4:', ':woman_curly_haired_medium_dark_skin_tone:', ':woman_curly_haired::skin-tone-4:'],
    '👩🏿‍🦱': [':woman_curly_haired_tone5:', ':woman_curly_haired_dark_skin_tone:', ':woman_curly_haired::skin-tone-5:'],
    '🕵️‍♀️': [':woman_detective:'],
    '🕵🏻‍♀️': [':woman_detective_tone1:', ':woman_detective_light_skin_tone:', ':woman_detective::skin-tone-1:'],
    '🕵🏼‍♀️': [':woman_detective_tone2:', ':woman_detective_medium_light_skin_tone:', ':woman_detective::skin-tone-2:'],
    '🕵🏽‍♀️': [':woman_detective_tone3:', ':woman_detective_medium_skin_tone:', ':woman_detective::skin-tone-3:'],
    '🕵🏾‍♀️': [':woman_detective_tone4:', ':woman_detective_medium_dark_skin_tone:', ':woman_detective::skin-tone-4:'],
    '🕵🏿‍♀️': [':woman_detective_tone5:', ':woman_detective_dark_skin_tone:', ':woman_detective::skin-tone-5:'],
    '🧝‍♀️': [':woman_elf:'],
    '🧝🏻‍♀️': [':woman_elf_tone1:', ':woman_elf_light_skin_tone:', ':woman_elf::skin-tone-1:'],
    '🧝🏼‍♀️': [':woman_elf_tone2:', ':woman_elf_medium_light_skin_tone:', ':woman_elf::skin-tone-2:'],
    '🧝🏽‍♀️': [':woman_elf_tone3:', ':woman_elf_medium_skin_tone:', ':woman_elf::skin-tone-3:'],
    '🧝🏾‍♀️': [':woman_elf_tone4:', ':woman_elf_medium_dark_skin_tone:', ':woman_elf::skin-tone-4:'],
    '🧝🏿‍♀️': [':woman_elf_tone5:', ':woman_elf_dark_skin_tone:', ':woman_elf::skin-tone-5:'],
    '🤦‍♀️': [':woman_facepalming:'],
    '🤦🏻‍♀️': [':woman_facepalming_tone1:', ':woman_facepalming_light_skin_tone:', ':woman_facepalming::skin-tone-1:'],
    '🤦🏼‍♀️': [':woman_facepalming_tone2:', ':woman_facepalming_medium_light_skin_tone:', ':woman_facepalming::skin-tone-2:'],
    '🤦🏽‍♀️': [':woman_facepalming_tone3:', ':woman_facepalming_medium_skin_tone:', ':woman_facepalming::skin-tone-3:'],
    '🤦🏾‍♀️': [':woman_facepalming_tone4:', ':woman_facepalming_medium_dark_skin_tone:', ':woman_facepalming::skin-tone-4:'],
    '🤦🏿‍♀️': [':woman_facepalming_tone5:', ':woman_facepalming_dark_skin_tone:', ':woman_facepalming::skin-tone-5:'],
    '👩‍🏭': [':woman_factory_worker:'],
    '👩🏻‍🏭': [':woman_factory_worker_tone1:', ':woman_factory_worker_light_skin_tone:', ':woman_factory_worker::skin-tone-1:'],
    '👩🏼‍🏭': [':woman_factory_worker_tone2:', ':woman_factory_worker_medium_light_skin_tone:', ':woman_factory_worker::skin-tone-2:'],
    '👩🏽‍🏭': [':woman_factory_worker_tone3:', ':woman_factory_worker_medium_skin_tone:', ':woman_factory_worker::skin-tone-3:'],
    '👩🏾‍🏭': [':woman_factory_worker_tone4:', ':woman_factory_worker_medium_dark_skin_tone:', ':woman_factory_worker::skin-tone-4:'],
    '👩🏿‍🏭': [':woman_factory_worker_tone5:', ':woman_factory_worker_dark_skin_tone:', ':woman_factory_worker::skin-tone-5:'],
    '🧚‍♀️': [':woman_fairy:'],
    '🧚🏻‍♀️': [':woman_fairy_tone1:', ':woman_fairy_light_skin_tone:', ':woman_fairy::skin-tone-1:'],
    '🧚🏼‍♀️': [':woman_fairy_tone2:', ':woman_fairy_medium_light_skin_tone:', ':woman_fairy::skin-tone-2:'],
    '🧚🏽‍♀️': [':woman_fairy_tone3:', ':woman_fairy_medium_skin_tone:', ':woman_fairy::skin-tone-3:'],
    '🧚🏾‍♀️': [':woman_fairy_tone4:', ':woman_fairy_medium_dark_skin_tone:', ':woman_fairy::skin-tone-4:'],
    '🧚🏿‍♀️': [':woman_fairy_tone5:', ':woman_fairy_dark_skin_tone:', ':woman_fairy::skin-tone-5:'],
    '👩‍🌾': [':woman_farmer:'],
    '👩🏻‍🌾': [':woman_farmer_tone1:', ':woman_farmer_light_skin_tone:', ':woman_farmer::skin-tone-1:'],
    '👩🏼‍🌾': [':woman_farmer_tone2:', ':woman_farmer_medium_light_skin_tone:', ':woman_farmer::skin-tone-2:'],
    '👩🏽‍🌾': [':woman_farmer_tone3:', ':woman_farmer_medium_skin_tone:', ':woman_farmer::skin-tone-3:'],
    '👩🏾‍🌾': [':woman_farmer_tone4:', ':woman_farmer_medium_dark_skin_tone:', ':woman_farmer::skin-tone-4:'],
    '👩🏿‍🌾': [':woman_farmer_tone5:', ':woman_farmer_dark_skin_tone:', ':woman_farmer::skin-tone-5:'],
    '👩‍🍼': [':woman_feeding_baby:'],
    '👩🏻‍🍼': [':woman_feeding_baby_tone1:', ':woman_feeding_baby_light_skin_tone:', ':woman_feeding_baby::skin-tone-1:'],
    '👩🏼‍🍼': [':woman_feeding_baby_tone2:', ':woman_feeding_baby_medium_light_skin_tone:', ':woman_feeding_baby::skin-tone-2:'],
    '👩🏽‍🍼': [':woman_feeding_baby_tone3:', ':woman_feeding_baby_medium_skin_tone:', ':woman_feeding_baby::skin-tone-3:'],
    '👩🏾‍🍼': [':woman_feeding_baby_tone4:', ':woman_feeding_baby_medium_dark_skin_tone:', ':woman_feeding_baby::skin-tone-4:'],
    '👩🏿‍🍼': [':woman_feeding_baby_tone5:', ':woman_feeding_baby_dark_skin_tone:', ':woman_feeding_baby::skin-tone-5:'],
    '👩‍🚒': [':woman_firefighter:'],
    '👩🏻‍🚒': [':woman_firefighter_tone1:', ':woman_firefighter_light_skin_tone:', ':woman_firefighter::skin-tone-1:'],
    '👩🏼‍🚒': [':woman_firefighter_tone2:', ':woman_firefighter_medium_light_skin_tone:', ':woman_firefighter::skin-tone-2:'],
    '👩🏽‍🚒': [':woman_firefighter_tone3:', ':woman_firefighter_medium_skin_tone:', ':woman_firefighter::skin-tone-3:'],
    '👩🏾‍🚒': [':woman_firefighter_tone4:', ':woman_firefighter_medium_dark_skin_tone:', ':woman_firefighter::skin-tone-4:'],
    '👩🏿‍🚒': [':woman_firefighter_tone5:', ':woman_firefighter_dark_skin_tone:', ':woman_firefighter::skin-tone-5:'],
    '🙍‍♀️': [':woman_frowning:'],
    '🙍🏻‍♀️': [':woman_frowning_tone1:', ':woman_frowning_light_skin_tone:', ':woman_frowning::skin-tone-1:'],
    '🙍🏼‍♀️': [':woman_frowning_tone2:', ':woman_frowning_medium_light_skin_tone:', ':woman_frowning::skin-tone-2:'],
    '🙍🏽‍♀️': [':woman_frowning_tone3:', ':woman_frowning_medium_skin_tone:', ':woman_frowning::skin-tone-3:'],
    '🙍🏾‍♀️': [':woman_frowning_tone4:', ':woman_frowning_medium_dark_skin_tone:', ':woman_frowning::skin-tone-4:'],
    '🙍🏿‍♀️': [':woman_frowning_tone5:', ':woman_frowning_dark_skin_tone:', ':woman_frowning::skin-tone-5:'],
    '🧞‍♀️': [':woman_genie:'],
    '🙅‍♀️': [':woman_gesturing_no:'],
    '🙅🏻‍♀️': [':woman_gesturing_no_tone1:', ':woman_gesturing_no_light_skin_tone:', ':woman_gesturing_no::skin-tone-1:'],
    '🙅🏼‍♀️': [':woman_gesturing_no_tone2:', ':woman_gesturing_no_medium_light_skin_tone:', ':woman_gesturing_no::skin-tone-2:'],
    '🙅🏽‍♀️': [':woman_gesturing_no_tone3:', ':woman_gesturing_no_medium_skin_tone:', ':woman_gesturing_no::skin-tone-3:'],
    '🙅🏾‍♀️': [':woman_gesturing_no_tone4:', ':woman_gesturing_no_medium_dark_skin_tone:', ':woman_gesturing_no::skin-tone-4:'],
    '🙅🏿‍♀️': [':woman_gesturing_no_tone5:', ':woman_gesturing_no_dark_skin_tone:', ':woman_gesturing_no::skin-tone-5:'],
    '🙆‍♀️': [':woman_gesturing_ok:'],
    '🙆🏻‍♀️': [':woman_gesturing_ok_tone1:', ':woman_gesturing_ok_light_skin_tone:', ':woman_gesturing_ok::skin-tone-1:'],
    '🙆🏼‍♀️': [':woman_gesturing_ok_tone2:', ':woman_gesturing_ok_medium_light_skin_tone:', ':woman_gesturing_ok::skin-tone-2:'],
    '🙆🏽‍♀️': [':woman_gesturing_ok_tone3:', ':woman_gesturing_ok_medium_skin_tone:', ':woman_gesturing_ok::skin-tone-3:'],
    '🙆🏾‍♀️': [':woman_gesturing_ok_tone4:', ':woman_gesturing_ok_medium_dark_skin_tone:', ':woman_gesturing_ok::skin-tone-4:'],
    '🙆🏿‍♀️': [':woman_gesturing_ok_tone5:', ':woman_gesturing_ok_dark_skin_tone:', ':woman_gesturing_ok::skin-tone-5:'],
    '💆‍♀️': [':woman_getting_face_massage:'],
    '💆🏻‍♀️': [':woman_getting_face_massage_tone1:', ':woman_getting_face_massage_light_skin_tone:', ':woman_getting_face_massage::skin-tone-1:'],
    '💆🏼‍♀️': [':woman_getting_face_massage_tone2:', ':woman_getting_face_massage_medium_light_skin_tone:', ':woman_getting_face_massage::skin-tone-2:'],
    '💆🏽‍♀️': [':woman_getting_face_massage_tone3:', ':woman_getting_face_massage_medium_skin_tone:', ':woman_getting_face_massage::skin-tone-3:'],
    '💆🏾‍♀️': [':woman_getting_face_massage_tone4:', ':woman_getting_face_massage_medium_dark_skin_tone:', ':woman_getting_face_massage::skin-tone-4:'],
    '💆🏿‍♀️': [':woman_getting_face_massage_tone5:', ':woman_getting_face_massage_dark_skin_tone:', ':woman_getting_face_massage::skin-tone-5:'],
    '💇‍♀️': [':woman_getting_haircut:'],
    '💇🏻‍♀️': [':woman_getting_haircut_tone1:', ':woman_getting_haircut_light_skin_tone:', ':woman_getting_haircut::skin-tone-1:'],
    '💇🏼‍♀️': [':woman_getting_haircut_tone2:', ':woman_getting_haircut_medium_light_skin_tone:', ':woman_getting_haircut::skin-tone-2:'],
    '💇🏽‍♀️': [':woman_getting_haircut_tone3:', ':woman_getting_haircut_medium_skin_tone:', ':woman_getting_haircut::skin-tone-3:'],
    '💇🏾‍♀️': [':woman_getting_haircut_tone4:', ':woman_getting_haircut_medium_dark_skin_tone:', ':woman_getting_haircut::skin-tone-4:'],
    '💇🏿‍♀️': [':woman_getting_haircut_tone5:', ':woman_getting_haircut_dark_skin_tone:', ':woman_getting_haircut::skin-tone-5:'],
    '🏌️‍♀️': [':woman_golfing:'],
    '🏌🏻‍♀️': [':woman_golfing_tone1:', ':woman_golfing_light_skin_tone:', ':woman_golfing::skin-tone-1:'],
    '🏌🏼‍♀️': [':woman_golfing_tone2:', ':woman_golfing_medium_light_skin_tone:', ':woman_golfing::skin-tone-2:'],
    '🏌🏽‍♀️': [':woman_golfing_tone3:', ':woman_golfing_medium_skin_tone:', ':woman_golfing::skin-tone-3:'],
    '🏌🏾‍♀️': [':woman_golfing_tone4:', ':woman_golfing_medium_dark_skin_tone:', ':woman_golfing::skin-tone-4:'],
    '🏌🏿‍♀️': [':woman_golfing_tone5:', ':woman_golfing_dark_skin_tone:', ':woman_golfing::skin-tone-5:'],
    '💂‍♀️': [':woman_guard:'],
    '💂🏻‍♀️': [':woman_guard_tone1:', ':woman_guard_light_skin_tone:', ':woman_guard::skin-tone-1:'],
    '💂🏼‍♀️': [':woman_guard_tone2:', ':woman_guard_medium_light_skin_tone:', ':woman_guard::skin-tone-2:'],
    '💂🏽‍♀️': [':woman_guard_tone3:', ':woman_guard_medium_skin_tone:', ':woman_guard::skin-tone-3:'],
    '💂🏾‍♀️': [':woman_guard_tone4:', ':woman_guard_medium_dark_skin_tone:', ':woman_guard::skin-tone-4:'],
    '💂🏿‍♀️': [':woman_guard_tone5:', ':woman_guard_dark_skin_tone:', ':woman_guard::skin-tone-5:'],
    '👩‍⚕️': [':woman_health_worker:'],
    '👩🏻‍⚕️': [':woman_health_worker_tone1:', ':woman_health_worker_light_skin_tone:', ':woman_health_worker::skin-tone-1:'],
    '👩🏼‍⚕️': [':woman_health_worker_tone2:', ':woman_health_worker_medium_light_skin_tone:', ':woman_health_worker::skin-tone-2:'],
    '👩🏽‍⚕️': [':woman_health_worker_tone3:', ':woman_health_worker_medium_skin_tone:', ':woman_health_worker::skin-tone-3:'],
    '👩🏾‍⚕️': [':woman_health_worker_tone4:', ':woman_health_worker_medium_dark_skin_tone:', ':woman_health_worker::skin-tone-4:'],
    '👩🏿‍⚕️': [':woman_health_worker_tone5:', ':woman_health_worker_dark_skin_tone:', ':woman_health_worker::skin-tone-5:'],
    '🧘‍♀️': [':woman_in_lotus_position:'],
    '🧘🏻‍♀️': [':woman_in_lotus_position_tone1:', ':woman_in_lotus_position_light_skin_tone:', ':woman_in_lotus_position::skin-tone-1:'],
    '🧘🏼‍♀️': [':woman_in_lotus_position_tone2:', ':woman_in_lotus_position_medium_light_skin_tone:', ':woman_in_lotus_position::skin-tone-2:'],
    '🧘🏽‍♀️': [':woman_in_lotus_position_tone3:', ':woman_in_lotus_position_medium_skin_tone:', ':woman_in_lotus_position::skin-tone-3:'],
    '🧘🏾‍♀️': [':woman_in_lotus_position_tone4:', ':woman_in_lotus_position_medium_dark_skin_tone:', ':woman_in_lotus_position::skin-tone-4:'],
    '🧘🏿‍♀️': [':woman_in_lotus_position_tone5:', ':woman_in_lotus_position_dark_skin_tone:', ':woman_in_lotus_position::skin-tone-5:'],
    '👩‍🦽': [':woman_in_manual_wheelchair:'],
    '👩🏻‍🦽': [':woman_in_manual_wheelchair_tone1:', ':woman_in_manual_wheelchair_light_skin_tone:', ':woman_in_manual_wheelchair::skin-tone-1:'],
    '👩🏼‍🦽': [':woman_in_manual_wheelchair_tone2:', ':woman_in_manual_wheelchair_medium_light_skin_tone:', ':woman_in_manual_wheelchair::skin-tone-2:'],
    '👩🏽‍🦽': [':woman_in_manual_wheelchair_tone3:', ':woman_in_manual_wheelchair_medium_skin_tone:', ':woman_in_manual_wheelchair::skin-tone-3:'],
    '👩🏾‍🦽': [':woman_in_manual_wheelchair_tone4:', ':woman_in_manual_wheelchair_medium_dark_skin_tone:', ':woman_in_manual_wheelchair::skin-tone-4:'],
    '👩🏿‍🦽': [':woman_in_manual_wheelchair_tone5:', ':woman_in_manual_wheelchair_dark_skin_tone:', ':woman_in_manual_wheelchair::skin-tone-5:'],
    '👩‍🦼': [':woman_in_motorized_wheelchair:'],
    '👩🏻‍🦼': [':woman_in_motorized_wheelchair_tone1:', ':woman_in_motorized_wheelchair_light_skin_tone:', ':woman_in_motorized_wheelchair::skin-tone-1:'],
    '👩🏼‍🦼': [':woman_in_motorized_wheelchair_tone2:', ':woman_in_motorized_wheelchair_medium_light_skin_tone:', ':woman_in_motorized_wheelchair::skin-tone-2:'],
    '👩🏽‍🦼': [':woman_in_motorized_wheelchair_tone3:', ':woman_in_motorized_wheelchair_medium_skin_tone:', ':woman_in_motorized_wheelchair::skin-tone-3:'],
    '👩🏾‍🦼': [':woman_in_motorized_wheelchair_tone4:', ':woman_in_motorized_wheelchair_medium_dark_skin_tone:', ':woman_in_motorized_wheelchair::skin-tone-4:'],
    '👩🏿‍🦼': [':woman_in_motorized_wheelchair_tone5:', ':woman_in_motorized_wheelchair_dark_skin_tone:', ':woman_in_motorized_wheelchair::skin-tone-5:'],
    '🧖‍♀️': [':woman_in_steamy_room:'],
    '🧖🏻‍♀️': [':woman_in_steamy_room_tone1:', ':woman_in_steamy_room_light_skin_tone:', ':woman_in_steamy_room::skin-tone-1:'],
    '🧖🏼‍♀️': [':woman_in_steamy_room_tone2:', ':woman_in_steamy_room_medium_light_skin_tone:', ':woman_in_steamy_room::skin-tone-2:'],
    '🧖🏽‍♀️': [':woman_in_steamy_room_tone3:', ':woman_in_steamy_room_medium_skin_tone:', ':woman_in_steamy_room::skin-tone-3:'],
    '🧖🏾‍♀️': [':woman_in_steamy_room_tone4:', ':woman_in_steamy_room_medium_dark_skin_tone:', ':woman_in_steamy_room::skin-tone-4:'],
    '🧖🏿‍♀️': [':woman_in_steamy_room_tone5:', ':woman_in_steamy_room_dark_skin_tone:', ':woman_in_steamy_room::skin-tone-5:'],
    '🤵‍♀️': [':woman_in_tuxedo:'],
    '🤵🏻‍♀️': [':woman_in_tuxedo_tone1:', ':woman_in_tuxedo_light_skin_tone:', ':woman_in_tuxedo::skin-tone-1:'],
    '🤵🏼‍♀️': [':woman_in_tuxedo_tone2:', ':woman_in_tuxedo_medium_light_skin_tone:', ':woman_in_tuxedo::skin-tone-2:'],
    '🤵🏽‍♀️': [':woman_in_tuxedo_tone3:', ':woman_in_tuxedo_medium_skin_tone:', ':woman_in_tuxedo::skin-tone-3:'],
    '🤵🏾‍♀️': [':woman_in_tuxedo_tone4:', ':woman_in_tuxedo_medium_dark_skin_tone:', ':woman_in_tuxedo::skin-tone-4:'],
    '🤵🏿‍♀️': [':woman_in_tuxedo_tone5:', ':woman_in_tuxedo_dark_skin_tone:', ':woman_in_tuxedo::skin-tone-5:'],
    '👩‍⚖️': [':woman_judge:'],
    '👩🏻‍⚖️': [':woman_judge_tone1:', ':woman_judge_light_skin_tone:', ':woman_judge::skin-tone-1:'],
    '👩🏼‍⚖️': [':woman_judge_tone2:', ':woman_judge_medium_light_skin_tone:', ':woman_judge::skin-tone-2:'],
    '👩🏽‍⚖️': [':woman_judge_tone3:', ':woman_judge_medium_skin_tone:', ':woman_judge::skin-tone-3:'],
    '👩🏾‍⚖️': [':woman_judge_tone4:', ':woman_judge_medium_dark_skin_tone:', ':woman_judge::skin-tone-4:'],
    '👩🏿‍⚖️': [':woman_judge_tone5:', ':woman_judge_dark_skin_tone:', ':woman_judge::skin-tone-5:'],
    '🤹‍♀️': [':woman_juggling:'],
    '🤹🏻‍♀️': [':woman_juggling_tone1:', ':woman_juggling_light_skin_tone:', ':woman_juggling::skin-tone-1:'],
    '🤹🏼‍♀️': [':woman_juggling_tone2:', ':woman_juggling_medium_light_skin_tone:', ':woman_juggling::skin-tone-2:'],
    '🤹🏽‍♀️': [':woman_juggling_tone3:', ':woman_juggling_medium_skin_tone:', ':woman_juggling::skin-tone-3:'],
    '🤹🏾‍♀️': [':woman_juggling_tone4:', ':woman_juggling_medium_dark_skin_tone:', ':woman_juggling::skin-tone-4:'],
    '🤹🏿‍♀️': [':woman_juggling_tone5:', ':woman_juggling_dark_skin_tone:', ':woman_juggling::skin-tone-5:'],
    '🧎‍♀️': [':woman_kneeling:'],
    '🧎🏻‍♀️': [':woman_kneeling_tone1:', ':woman_kneeling_light_skin_tone:', ':woman_kneeling::skin-tone-1:'],
    '🧎🏼‍♀️': [':woman_kneeling_tone2:', ':woman_kneeling_medium_light_skin_tone:', ':woman_kneeling::skin-tone-2:'],
    '🧎🏽‍♀️': [':woman_kneeling_tone3:', ':woman_kneeling_medium_skin_tone:', ':woman_kneeling::skin-tone-3:'],
    '🧎🏾‍♀️': [':woman_kneeling_tone4:', ':woman_kneeling_medium_dark_skin_tone:', ':woman_kneeling::skin-tone-4:'],
    '🧎🏿‍♀️': [':woman_kneeling_tone5:', ':woman_kneeling_dark_skin_tone:', ':woman_kneeling::skin-tone-5:'],
    '🏋️‍♀️': [':woman_lifting_weights:'],
    '🏋🏻‍♀️': [':woman_lifting_weights_tone1:', ':woman_lifting_weights_light_skin_tone:', ':woman_lifting_weights::skin-tone-1:'],
    '🏋🏼‍♀️': [':woman_lifting_weights_tone2:', ':woman_lifting_weights_medium_light_skin_tone:', ':woman_lifting_weights::skin-tone-2:'],
    '🏋🏽‍♀️': [':woman_lifting_weights_tone3:', ':woman_lifting_weights_medium_skin_tone:', ':woman_lifting_weights::skin-tone-3:'],
    '🏋🏾‍♀️': [':woman_lifting_weights_tone4:', ':woman_lifting_weights_medium_dark_skin_tone:', ':woman_lifting_weights::skin-tone-4:'],
    '🏋🏿‍♀️': [':woman_lifting_weights_tone5:', ':woman_lifting_weights_dark_skin_tone:', ':woman_lifting_weights::skin-tone-5:'],
    '🧙‍♀️': [':woman_mage:'],
    '🧙🏻‍♀️': [':woman_mage_tone1:', ':woman_mage_light_skin_tone:', ':woman_mage::skin-tone-1:'],
    '🧙🏼‍♀️': [':woman_mage_tone2:', ':woman_mage_medium_light_skin_tone:', ':woman_mage::skin-tone-2:'],
    '🧙🏽‍♀️': [':woman_mage_tone3:', ':woman_mage_medium_skin_tone:', ':woman_mage::skin-tone-3:'],
    '🧙🏾‍♀️': [':woman_mage_tone4:', ':woman_mage_medium_dark_skin_tone:', ':woman_mage::skin-tone-4:'],
    '🧙🏿‍♀️': [':woman_mage_tone5:', ':woman_mage_dark_skin_tone:', ':woman_mage::skin-tone-5:'],
    '👩‍🔧': [':woman_mechanic:'],
    '👩🏻‍🔧': [':woman_mechanic_tone1:', ':woman_mechanic_light_skin_tone:', ':woman_mechanic::skin-tone-1:'],
    '👩🏼‍🔧': [':woman_mechanic_tone2:', ':woman_mechanic_medium_light_skin_tone:', ':woman_mechanic::skin-tone-2:'],
    '👩🏽‍🔧': [':woman_mechanic_tone3:', ':woman_mechanic_medium_skin_tone:', ':woman_mechanic::skin-tone-3:'],
    '👩🏾‍🔧': [':woman_mechanic_tone4:', ':woman_mechanic_medium_dark_skin_tone:', ':woman_mechanic::skin-tone-4:'],
    '👩🏿‍🔧': [':woman_mechanic_tone5:', ':woman_mechanic_dark_skin_tone:', ':woman_mechanic::skin-tone-5:'],
    '🚵‍♀️': [':woman_mountain_biking:'],
    '🚵🏻‍♀️': [':woman_mountain_biking_tone1:', ':woman_mountain_biking_light_skin_tone:', ':woman_mountain_biking::skin-tone-1:'],
    '🚵🏼‍♀️': [':woman_mountain_biking_tone2:', ':woman_mountain_biking_medium_light_skin_tone:', ':woman_mountain_biking::skin-tone-2:'],
    '🚵🏽‍♀️': [':woman_mountain_biking_tone3:', ':woman_mountain_biking_medium_skin_tone:', ':woman_mountain_biking::skin-tone-3:'],
    '🚵🏾‍♀️': [':woman_mountain_biking_tone4:', ':woman_mountain_biking_medium_dark_skin_tone:', ':woman_mountain_biking::skin-tone-4:'],
    '🚵🏿‍♀️': [':woman_mountain_biking_tone5:', ':woman_mountain_biking_dark_skin_tone:', ':woman_mountain_biking::skin-tone-5:'],
    '👩‍💼': [':woman_office_worker:'],
    '👩🏻‍💼': [':woman_office_worker_tone1:', ':woman_office_worker_light_skin_tone:', ':woman_office_worker::skin-tone-1:'],
    '👩🏼‍💼': [':woman_office_worker_tone2:', ':woman_office_worker_medium_light_skin_tone:', ':woman_office_worker::skin-tone-2:'],
    '👩🏽‍💼': [':woman_office_worker_tone3:', ':woman_office_worker_medium_skin_tone:', ':woman_office_worker::skin-tone-3:'],
    '👩🏾‍💼': [':woman_office_worker_tone4:', ':woman_office_worker_medium_dark_skin_tone:', ':woman_office_worker::skin-tone-4:'],
    '👩🏿‍💼': [':woman_office_worker_tone5:', ':woman_office_worker_dark_skin_tone:', ':woman_office_worker::skin-tone-5:'],
    '👩‍✈️': [':woman_pilot:'],
    '👩🏻‍✈️': [':woman_pilot_tone1:', ':woman_pilot_light_skin_tone:', ':woman_pilot::skin-tone-1:'],
    '👩🏼‍✈️': [':woman_pilot_tone2:', ':woman_pilot_medium_light_skin_tone:', ':woman_pilot::skin-tone-2:'],
    '👩🏽‍✈️': [':woman_pilot_tone3:', ':woman_pilot_medium_skin_tone:', ':woman_pilot::skin-tone-3:'],
    '👩🏾‍✈️': [':woman_pilot_tone4:', ':woman_pilot_medium_dark_skin_tone:', ':woman_pilot::skin-tone-4:'],
    '👩🏿‍✈️': [':woman_pilot_tone5:', ':woman_pilot_dark_skin_tone:', ':woman_pilot::skin-tone-5:'],
    '🤾‍♀️': [':woman_playing_handball:'],
    '🤾🏻‍♀️': [':woman_playing_handball_tone1:', ':woman_playing_handball_light_skin_tone:', ':woman_playing_handball::skin-tone-1:'],
    '🤾🏼‍♀️': [':woman_playing_handball_tone2:', ':woman_playing_handball_medium_light_skin_tone:', ':woman_playing_handball::skin-tone-2:'],
    '🤾🏽‍♀️': [':woman_playing_handball_tone3:', ':woman_playing_handball_medium_skin_tone:', ':woman_playing_handball::skin-tone-3:'],
    '🤾🏾‍♀️': [':woman_playing_handball_tone4:', ':woman_playing_handball_medium_dark_skin_tone:', ':woman_playing_handball::skin-tone-4:'],
    '🤾🏿‍♀️': [':woman_playing_handball_tone5:', ':woman_playing_handball_dark_skin_tone:', ':woman_playing_handball::skin-tone-5:'],
    '🤽‍♀️': [':woman_playing_water_polo:'],
    '🤽🏻‍♀️': [':woman_playing_water_polo_tone1:', ':woman_playing_water_polo_light_skin_tone:', ':woman_playing_water_polo::skin-tone-1:'],
    '🤽🏼‍♀️': [':woman_playing_water_polo_tone2:', ':woman_playing_water_polo_medium_light_skin_tone:', ':woman_playing_water_polo::skin-tone-2:'],
    '🤽🏽‍♀️': [':woman_playing_water_polo_tone3:', ':woman_playing_water_polo_medium_skin_tone:', ':woman_playing_water_polo::skin-tone-3:'],
    '🤽🏾‍♀️': [':woman_playing_water_polo_tone4:', ':woman_playing_water_polo_medium_dark_skin_tone:', ':woman_playing_water_polo::skin-tone-4:'],
    '🤽🏿‍♀️': [':woman_playing_water_polo_tone5:', ':woman_playing_water_polo_dark_skin_tone:', ':woman_playing_water_polo::skin-tone-5:'],
    '👮‍♀️': [':woman_police_officer:'],
    '👮🏻‍♀️': [':woman_police_officer_tone1:', ':woman_police_officer_light_skin_tone:', ':woman_police_officer::skin-tone-1:'],
    '👮🏼‍♀️': [':woman_police_officer_tone2:', ':woman_police_officer_medium_light_skin_tone:', ':woman_police_officer::skin-tone-2:'],
    '👮🏽‍♀️': [':woman_police_officer_tone3:', ':woman_police_officer_medium_skin_tone:', ':woman_police_officer::skin-tone-3:'],
    '👮🏾‍♀️': [':woman_police_officer_tone4:', ':woman_police_officer_medium_dark_skin_tone:', ':woman_police_officer::skin-tone-4:'],
    '👮🏿‍♀️': [':woman_police_officer_tone5:', ':woman_police_officer_dark_skin_tone:', ':woman_police_officer::skin-tone-5:'],
    '🙎‍♀️': [':woman_pouting:'],
    '🙎🏻‍♀️': [':woman_pouting_tone1:', ':woman_pouting_light_skin_tone:', ':woman_pouting::skin-tone-1:'],
    '🙎🏼‍♀️': [':woman_pouting_tone2:', ':woman_pouting_medium_light_skin_tone:', ':woman_pouting::skin-tone-2:'],
    '🙎🏽‍♀️': [':woman_pouting_tone3:', ':woman_pouting_medium_skin_tone:', ':woman_pouting::skin-tone-3:'],
    '🙎🏾‍♀️': [':woman_pouting_tone4:', ':woman_pouting_medium_dark_skin_tone:', ':woman_pouting::skin-tone-4:'],
    '🙎🏿‍♀️': [':woman_pouting_tone5:', ':woman_pouting_dark_skin_tone:', ':woman_pouting::skin-tone-5:'],
    '🙋‍♀️': [':woman_raising_hand:'],
    '🙋🏻‍♀️': [':woman_raising_hand_tone1:', ':woman_raising_hand_light_skin_tone:', ':woman_raising_hand::skin-tone-1:'],
    '🙋🏼‍♀️': [':woman_raising_hand_tone2:', ':woman_raising_hand_medium_light_skin_tone:', ':woman_raising_hand::skin-tone-2:'],
    '🙋🏽‍♀️': [':woman_raising_hand_tone3:', ':woman_raising_hand_medium_skin_tone:', ':woman_raising_hand::skin-tone-3:'],
    '🙋🏾‍♀️': [':woman_raising_hand_tone4:', ':woman_raising_hand_medium_dark_skin_tone:', ':woman_raising_hand::skin-tone-4:'],
    '🙋🏿‍♀️': [':woman_raising_hand_tone5:', ':woman_raising_hand_dark_skin_tone:', ':woman_raising_hand::skin-tone-5:'],
    '👩‍🦰': [':woman_red_haired:'],
    '👩🏻‍🦰': [':woman_red_haired_tone1:', ':woman_red_haired_light_skin_tone:', ':woman_red_haired::skin-tone-1:'],
    '👩🏼‍🦰': [':woman_red_haired_tone2:', ':woman_red_haired_medium_light_skin_tone:', ':woman_red_haired::skin-tone-2:'],
    '👩🏽‍🦰': [':woman_red_haired_tone3:', ':woman_red_haired_medium_skin_tone:', ':woman_red_haired::skin-tone-3:'],
    '👩🏾‍🦰': [':woman_red_haired_tone4:', ':woman_red_haired_medium_dark_skin_tone:', ':woman_red_haired::skin-tone-4:'],
    '👩🏿‍🦰': [':woman_red_haired_tone5:', ':woman_red_haired_dark_skin_tone:', ':woman_red_haired::skin-tone-5:'],
    '🚣‍♀️': [':woman_rowing_boat:'],
    '🚣🏻‍♀️': [':woman_rowing_boat_tone1:', ':woman_rowing_boat_light_skin_tone:', ':woman_rowing_boat::skin-tone-1:'],
    '🚣🏼‍♀️': [':woman_rowing_boat_tone2:', ':woman_rowing_boat_medium_light_skin_tone:', ':woman_rowing_boat::skin-tone-2:'],
    '🚣🏽‍♀️': [':woman_rowing_boat_tone3:', ':woman_rowing_boat_medium_skin_tone:', ':woman_rowing_boat::skin-tone-3:'],
    '🚣🏾‍♀️': [':woman_rowing_boat_tone4:', ':woman_rowing_boat_medium_dark_skin_tone:', ':woman_rowing_boat::skin-tone-4:'],
    '🚣🏿‍♀️': [':woman_rowing_boat_tone5:', ':woman_rowing_boat_dark_skin_tone:', ':woman_rowing_boat::skin-tone-5:'],
    '🏃‍♀️': [':woman_running:'],
    '🏃🏻‍♀️': [':woman_running_tone1:', ':woman_running_light_skin_tone:', ':woman_running::skin-tone-1:'],
    '🏃🏼‍♀️': [':woman_running_tone2:', ':woman_running_medium_light_skin_tone:', ':woman_running::skin-tone-2:'],
    '🏃🏽‍♀️': [':woman_running_tone3:', ':woman_running_medium_skin_tone:', ':woman_running::skin-tone-3:'],
    '🏃🏾‍♀️': [':woman_running_tone4:', ':woman_running_medium_dark_skin_tone:', ':woman_running::skin-tone-4:'],
    '🏃🏿‍♀️': [':woman_running_tone5:', ':woman_running_dark_skin_tone:', ':woman_running::skin-tone-5:'],
    '👩‍🔬': [':woman_scientist:'],
    '👩🏻‍🔬': [':woman_scientist_tone1:', ':woman_scientist_light_skin_tone:', ':woman_scientist::skin-tone-1:'],
    '👩🏼‍🔬': [':woman_scientist_tone2:', ':woman_scientist_medium_light_skin_tone:', ':woman_scientist::skin-tone-2:'],
    '👩🏽‍🔬': [':woman_scientist_tone3:', ':woman_scientist_medium_skin_tone:', ':woman_scientist::skin-tone-3:'],
    '👩🏾‍🔬': [':woman_scientist_tone4:', ':woman_scientist_medium_dark_skin_tone:', ':woman_scientist::skin-tone-4:'],
    '👩🏿‍🔬': [':woman_scientist_tone5:', ':woman_scientist_dark_skin_tone:', ':woman_scientist::skin-tone-5:'],
    '🤷‍♀️': [':woman_shrugging:'],
    '🤷🏻‍♀️': [':woman_shrugging_tone1:', ':woman_shrugging_light_skin_tone:', ':woman_shrugging::skin-tone-1:'],
    '🤷🏼‍♀️': [':woman_shrugging_tone2:', ':woman_shrugging_medium_light_skin_tone:', ':woman_shrugging::skin-tone-2:'],
    '🤷🏽‍♀️': [':woman_shrugging_tone3:', ':woman_shrugging_medium_skin_tone:', ':woman_shrugging::skin-tone-3:'],
    '🤷🏾‍♀️': [':woman_shrugging_tone4:', ':woman_shrugging_medium_dark_skin_tone:', ':woman_shrugging::skin-tone-4:'],
    '🤷🏿‍♀️': [':woman_shrugging_tone5:', ':woman_shrugging_dark_skin_tone:', ':woman_shrugging::skin-tone-5:'],
    '👩‍🎤': [':woman_singer:'],
    '👩🏻‍🎤': [':woman_singer_tone1:', ':woman_singer_light_skin_tone:', ':woman_singer::skin-tone-1:'],
    '👩🏼‍🎤': [':woman_singer_tone2:', ':woman_singer_medium_light_skin_tone:', ':woman_singer::skin-tone-2:'],
    '👩🏽‍🎤': [':woman_singer_tone3:', ':woman_singer_medium_skin_tone:', ':woman_singer::skin-tone-3:'],
    '👩🏾‍🎤': [':woman_singer_tone4:', ':woman_singer_medium_dark_skin_tone:', ':woman_singer::skin-tone-4:'],
    '👩🏿‍🎤': [':woman_singer_tone5:', ':woman_singer_dark_skin_tone:', ':woman_singer::skin-tone-5:'],
    '🧍‍♀️': [':woman_standing:'],
    '🧍🏻‍♀️': [':woman_standing_tone1:', ':woman_standing_light_skin_tone:', ':woman_standing::skin-tone-1:'],
    '🧍🏼‍♀️': [':woman_standing_tone2:', ':woman_standing_medium_light_skin_tone:', ':woman_standing::skin-tone-2:'],
    '🧍🏽‍♀️': [':woman_standing_tone3:', ':woman_standing_medium_skin_tone:', ':woman_standing::skin-tone-3:'],
    '🧍🏾‍♀️': [':woman_standing_tone4:', ':woman_standing_medium_dark_skin_tone:', ':woman_standing::skin-tone-4:'],
    '🧍🏿‍♀️': [':woman_standing_tone5:', ':woman_standing_dark_skin_tone:', ':woman_standing::skin-tone-5:'],
    '👩‍🎓': [':woman_student:'],
    '👩🏻‍🎓': [':woman_student_tone1:', ':woman_student_light_skin_tone:', ':woman_student::skin-tone-1:'],
    '👩🏼‍🎓': [':woman_student_tone2:', ':woman_student_medium_light_skin_tone:', ':woman_student::skin-tone-2:'],
    '👩🏽‍🎓': [':woman_student_tone3:', ':woman_student_medium_skin_tone:', ':woman_student::skin-tone-3:'],
    '👩🏾‍🎓': [':woman_student_tone4:', ':woman_student_medium_dark_skin_tone:', ':woman_student::skin-tone-4:'],
    '👩🏿‍🎓': [':woman_student_tone5:', ':woman_student_dark_skin_tone:', ':woman_student::skin-tone-5:'],
    '🦸‍♀️': [':woman_superhero:'],
    '🦸🏻‍♀️': [':woman_superhero_tone1:', ':woman_superhero_light_skin_tone:', ':woman_superhero::skin-tone-1:'],
    '🦸🏼‍♀️': [':woman_superhero_tone2:', ':woman_superhero_medium_light_skin_tone:', ':woman_superhero::skin-tone-2:'],
    '🦸🏽‍♀️': [':woman_superhero_tone3:', ':woman_superhero_medium_skin_tone:', ':woman_superhero::skin-tone-3:'],
    '🦸🏾‍♀️': [':woman_superhero_tone4:', ':woman_superhero_medium_dark_skin_tone:', ':woman_superhero::skin-tone-4:'],
    '🦸🏿‍♀️': [':woman_superhero_tone5:', ':woman_superhero_dark_skin_tone:', ':woman_superhero::skin-tone-5:'],
    '🦹‍♀️': [':woman_supervillain:'],
    '🦹🏻‍♀️': [':woman_supervillain_tone1:', ':woman_supervillain_light_skin_tone:', ':woman_supervillain::skin-tone-1:'],
    '🦹🏼‍♀️': [':woman_supervillain_tone2:', ':woman_supervillain_medium_light_skin_tone:', ':woman_supervillain::skin-tone-2:'],
    '🦹🏽‍♀️': [':woman_supervillain_tone3:', ':woman_supervillain_medium_skin_tone:', ':woman_supervillain::skin-tone-3:'],
    '🦹🏾‍♀️': [':woman_supervillain_tone4:', ':woman_supervillain_medium_dark_skin_tone:', ':woman_supervillain::skin-tone-4:'],
    '🦹🏿‍♀️': [':woman_supervillain_tone5:', ':woman_supervillain_dark_skin_tone:', ':woman_supervillain::skin-tone-5:'],
    '🏄‍♀️': [':woman_surfing:'],
    '🏄🏻‍♀️': [':woman_surfing_tone1:', ':woman_surfing_light_skin_tone:', ':woman_surfing::skin-tone-1:'],
    '🏄🏼‍♀️': [':woman_surfing_tone2:', ':woman_surfing_medium_light_skin_tone:', ':woman_surfing::skin-tone-2:'],
    '🏄🏽‍♀️': [':woman_surfing_tone3:', ':woman_surfing_medium_skin_tone:', ':woman_surfing::skin-tone-3:'],
    '🏄🏾‍♀️': [':woman_surfing_tone4:', ':woman_surfing_medium_dark_skin_tone:', ':woman_surfing::skin-tone-4:'],
    '🏄🏿‍♀️': [':woman_surfing_tone5:', ':woman_surfing_dark_skin_tone:', ':woman_surfing::skin-tone-5:'],
    '🏊‍♀️': [':woman_swimming:'],
    '🏊🏻‍♀️': [':woman_swimming_tone1:', ':woman_swimming_light_skin_tone:', ':woman_swimming::skin-tone-1:'],
    '🏊🏼‍♀️': [':woman_swimming_tone2:', ':woman_swimming_medium_light_skin_tone:', ':woman_swimming::skin-tone-2:'],
    '🏊🏽‍♀️': [':woman_swimming_tone3:', ':woman_swimming_medium_skin_tone:', ':woman_swimming::skin-tone-3:'],
    '🏊🏾‍♀️': [':woman_swimming_tone4:', ':woman_swimming_medium_dark_skin_tone:', ':woman_swimming::skin-tone-4:'],
    '🏊🏿‍♀️': [':woman_swimming_tone5:', ':woman_swimming_dark_skin_tone:', ':woman_swimming::skin-tone-5:'],
    '👩‍🏫': [':woman_teacher:'],
    '👩🏻‍🏫': [':woman_teacher_tone1:', ':woman_teacher_light_skin_tone:', ':woman_teacher::skin-tone-1:'],
    '👩🏼‍🏫': [':woman_teacher_tone2:', ':woman_teacher_medium_light_skin_tone:', ':woman_teacher::skin-tone-2:'],
    '👩🏽‍🏫': [':woman_teacher_tone3:', ':woman_teacher_medium_skin_tone:', ':woman_teacher::skin-tone-3:'],
    '👩🏾‍🏫': [':woman_teacher_tone4:', ':woman_teacher_medium_dark_skin_tone:', ':woman_teacher::skin-tone-4:'],
    '👩🏿‍🏫': [':woman_teacher_tone5:', ':woman_teacher_dark_skin_tone:', ':woman_teacher::skin-tone-5:'],
    '👩‍💻': [':woman_technologist:'],
    '👩🏻‍💻': [':woman_technologist_tone1:', ':woman_technologist_light_skin_tone:', ':woman_technologist::skin-tone-1:'],
    '👩🏼‍💻': [':woman_technologist_tone2:', ':woman_technologist_medium_light_skin_tone:', ':woman_technologist::skin-tone-2:'],
    '👩🏽‍💻': [':woman_technologist_tone3:', ':woman_technologist_medium_skin_tone:', ':woman_technologist::skin-tone-3:'],
    '👩🏾‍💻': [':woman_technologist_tone4:', ':woman_technologist_medium_dark_skin_tone:', ':woman_technologist::skin-tone-4:'],
    '👩🏿‍💻': [':woman_technologist_tone5:', ':woman_technologist_dark_skin_tone:', ':woman_technologist::skin-tone-5:'],
    '💁‍♀️': [':woman_tipping_hand:'],
    '💁🏻‍♀️': [':woman_tipping_hand_tone1:', ':woman_tipping_hand_light_skin_tone:', ':woman_tipping_hand::skin-tone-1:'],
    '💁🏼‍♀️': [':woman_tipping_hand_tone2:', ':woman_tipping_hand_medium_light_skin_tone:', ':woman_tipping_hand::skin-tone-2:'],
    '💁🏽‍♀️': [':woman_tipping_hand_tone3:', ':woman_tipping_hand_medium_skin_tone:', ':woman_tipping_hand::skin-tone-3:'],
    '💁🏾‍♀️': [':woman_tipping_hand_tone4:', ':woman_tipping_hand_medium_dark_skin_tone:', ':woman_tipping_hand::skin-tone-4:'],
    '💁🏿‍♀️': [':woman_tipping_hand_tone5:', ':woman_tipping_hand_dark_skin_tone:', ':woman_tipping_hand::skin-tone-5:'],
    '👩🏻': [':woman_tone1:', ':woman::skin-tone-1:'],
    '👩🏼': [':woman_tone2:', ':woman::skin-tone-2:'],
    '👩🏽': [':woman_tone3:', ':woman::skin-tone-3:'],
    '👩🏾': [':woman_tone4:', ':woman::skin-tone-4:'],
    '👩🏿': [':woman_tone5:', ':woman::skin-tone-5:'],
    '🧛‍♀️': [':woman_vampire:'],
    '🧛🏻‍♀️': [':woman_vampire_tone1:', ':woman_vampire_light_skin_tone:', ':woman_vampire::skin-tone-1:'],
    '🧛🏼‍♀️': [':woman_vampire_tone2:', ':woman_vampire_medium_light_skin_tone:', ':woman_vampire::skin-tone-2:'],
    '🧛🏽‍♀️': [':woman_vampire_tone3:', ':woman_vampire_medium_skin_tone:', ':woman_vampire::skin-tone-3:'],
    '🧛🏾‍♀️': [':woman_vampire_tone4:', ':woman_vampire_medium_dark_skin_tone:', ':woman_vampire::skin-tone-4:'],
    '🧛🏿‍♀️': [':woman_vampire_tone5:', ':woman_vampire_dark_skin_tone:', ':woman_vampire::skin-tone-5:'],
    '🚶‍♀️': [':woman_walking:'],
    '🚶🏻‍♀️': [':woman_walking_tone1:', ':woman_walking_light_skin_tone:', ':woman_walking::skin-tone-1:'],
    '🚶🏼‍♀️': [':woman_walking_tone2:', ':woman_walking_medium_light_skin_tone:', ':woman_walking::skin-tone-2:'],
    '🚶🏽‍♀️': [':woman_walking_tone3:', ':woman_walking_medium_skin_tone:', ':woman_walking::skin-tone-3:'],
    '🚶🏾‍♀️': [':woman_walking_tone4:', ':woman_walking_medium_dark_skin_tone:', ':woman_walking::skin-tone-4:'],
    '🚶🏿‍♀️': [':woman_walking_tone5:', ':woman_walking_dark_skin_tone:', ':woman_walking::skin-tone-5:'],
    '👳‍♀️': [':woman_wearing_turban:'],
    '👳🏻‍♀️': [':woman_wearing_turban_tone1:', ':woman_wearing_turban_light_skin_tone:', ':woman_wearing_turban::skin-tone-1:'],
    '👳🏼‍♀️': [':woman_wearing_turban_tone2:', ':woman_wearing_turban_medium_light_skin_tone:', ':woman_wearing_turban::skin-tone-2:'],
    '👳🏽‍♀️': [':woman_wearing_turban_tone3:', ':woman_wearing_turban_medium_skin_tone:', ':woman_wearing_turban::skin-tone-3:'],
    '👳🏾‍♀️': [':woman_wearing_turban_tone4:', ':woman_wearing_turban_medium_dark_skin_tone:', ':woman_wearing_turban::skin-tone-4:'],
    '👳🏿‍♀️': [':woman_wearing_turban_tone5:', ':woman_wearing_turban_dark_skin_tone:', ':woman_wearing_turban::skin-tone-5:'],
    '👩‍🦳': [':woman_white_haired:'],
    '👩🏻‍🦳': [':woman_white_haired_tone1:', ':woman_white_haired_light_skin_tone:', ':woman_white_haired::skin-tone-1:'],
    '👩🏼‍🦳': [':woman_white_haired_tone2:', ':woman_white_haired_medium_light_skin_tone:', ':woman_white_haired::skin-tone-2:'],
    '👩🏽‍🦳': [':woman_white_haired_tone3:', ':woman_white_haired_medium_skin_tone:', ':woman_white_haired::skin-tone-3:'],
    '👩🏾‍🦳': [':woman_white_haired_tone4:', ':woman_white_haired_medium_dark_skin_tone:', ':woman_white_haired::skin-tone-4:'],
    '👩🏿‍🦳': [':woman_white_haired_tone5:', ':woman_white_haired_dark_skin_tone:', ':woman_white_haired::skin-tone-5:'],
    '🧕': [':woman_with_headscarf:'],
    '🧕🏻': [':woman_with_headscarf_tone1:', ':woman_with_headscarf_light_skin_tone:', ':woman_with_headscarf::skin-tone-1:'],
    '🧕🏼': [':woman_with_headscarf_tone2:', ':woman_with_headscarf_medium_light_skin_tone:', ':woman_with_headscarf::skin-tone-2:'],
    '🧕🏽': [':woman_with_headscarf_tone3:', ':woman_with_headscarf_medium_skin_tone:', ':woman_with_headscarf::skin-tone-3:'],
    '🧕🏾': [':woman_with_headscarf_tone4:', ':woman_with_headscarf_medium_dark_skin_tone:', ':woman_with_headscarf::skin-tone-4:'],
    '🧕🏿': [':woman_with_headscarf_tone5:', ':woman_with_headscarf_dark_skin_tone:', ':woman_with_headscarf::skin-tone-5:'],
    '👩‍🦯': [':woman_with_probing_cane:'],
    '👩🏻‍🦯': [':woman_with_probing_cane_tone1:', ':woman_with_probing_cane_light_skin_tone:', ':woman_with_probing_cane::skin-tone-1:'],
    '👩🏼‍🦯': [':woman_with_probing_cane_tone2:', ':woman_with_probing_cane_medium_light_skin_tone:', ':woman_with_probing_cane::skin-tone-2:'],
    '👩🏽‍🦯': [':woman_with_probing_cane_tone3:', ':woman_with_probing_cane_medium_skin_tone:', ':woman_with_probing_cane::skin-tone-3:'],
    '👩🏾‍🦯': [':woman_with_probing_cane_tone4:', ':woman_with_probing_cane_medium_dark_skin_tone:', ':woman_with_probing_cane::skin-tone-4:'],
    '👩🏿‍🦯': [':woman_with_probing_cane_tone5:', ':woman_with_probing_cane_dark_skin_tone:', ':woman_with_probing_cane::skin-tone-5:'],
    '👰‍♀️': [':woman_with_veil:', ':bride_with_veil:'],
    '👰🏻‍♀️': [':woman_with_veil_tone1:', ':woman_with_veil_light_skin_tone:', ':woman_with_veil::skin-tone-1:', ':bride_with_veil::skin-tone-1:'],
    '👰🏼‍♀️': [':woman_with_veil_tone2:', ':woman_with_veil_medium_light_skin_tone:', ':woman_with_veil::skin-tone-2:', ':bride_with_veil::skin-tone-2:'],
    '👰🏽‍♀️': [':woman_with_veil_tone3:', ':woman_with_veil_medium_skin_tone:', ':woman_with_veil::skin-tone-3:', ':bride_with_veil::skin-tone-3:'],
    '👰🏾‍♀️': [':woman_with_veil_tone4:', ':woman_with_veil_medium_dark_skin_tone:', ':woman_with_veil::skin-tone-4:', ':bride_with_veil::skin-tone-4:'],
    '👰🏿‍♀️': [':woman_with_veil_tone5:', ':woman_with_veil_dark_skin_tone:', ':woman_with_veil::skin-tone-5:', ':bride_with_veil::skin-tone-5:'],
    '🧟‍♀️': [':woman_zombie:'],
    '👚': [':womans_clothes:'],
    '🥿': [':womans_flat_shoe:'],
    '👒': [':womans_hat:'],
    '👯‍♀️': [':women_with_bunny_ears_partying:'],
    '🤼‍♀️': [':women_wrestling:'],
    '🚺': [':womens:'],
    '🪵': [':wood:'],
    '🥴': [':woozy_face:'],
    '🪱': [':worm:'],
    '😟': [':worried:'],
    '🔧': [':wrench:'],
    '✍️': [':writing_hand:'],
    '✍🏻': [':writing_hand_tone1:', ':writing_hand::skin-tone-1:'],
    '✍🏼': [':writing_hand_tone2:', ':writing_hand::skin-tone-2:'],
    '✍🏽': [':writing_hand_tone3:', ':writing_hand::skin-tone-3:'],
    '✍🏾': [':writing_hand_tone4:', ':writing_hand::skin-tone-4:'],
    '✍🏿': [':writing_hand_tone5:', ':writing_hand::skin-tone-5:'],
    '❌': [':x:'],
    '🧶': [':yarn:'],
    '🥱': [':yawning_face:'],
    '🟡': [':yellow_circle:'],
    '💛': [':yellow_heart:'],
    '🟨': [':yellow_square:'],
    '💴': [':yen:'],
    '☯️': [':yin_yang:'],
    '🪀': [':yo_yo:'],
    '😋': [':yum:'],
    '🤪': [':zany_face:'],
    '⚡': [':zap:'],
    '🦓': [':zebra:'],
    '0️⃣': [':zero:'],
    '🤐': [':zipper_mouth:', ':zipper_mouth_face:'],
    '🧟': [':zombie:'],
    '💤': [':zzz:']
}

sijome = {
    ":100:": "💯",
    ":1234:": "🔢",
    ":8ball:": "🎱",
    ":a:": "🅰️",
    ":ab:": "🆎",
    ":abacus:": "🧮",
    ":abc:": "🔤",
    ":abcd:": "🔡",
    ":accept:": "🉑",
    ":accordion:": "🪗",
    ":adhesive_bandage:": "🩹",
    ":adult:": "🧑",
    ":adult_tone1:": "🧑🏻",
    ":adult_light_skin_tone:": "🧑🏻",
    ":adult::skin-tone-1:": "🧑🏻",
    ":adult_tone2:": "🧑🏼",
    ":adult_medium_light_skin_tone:": "🧑🏼",
    ":adult::skin-tone-2:": "🧑🏼",
    ":adult_tone3:": "🧑🏽",
    ":adult_medium_skin_tone:": "🧑🏽",
    ":adult::skin-tone-3:": "🧑🏽",
    ":adult_tone4:": "🧑🏾",
    ":adult_medium_dark_skin_tone:": "🧑🏾",
    ":adult::skin-tone-4:": "🧑🏾",
    ":adult_tone5:": "🧑🏿",
    ":adult_dark_skin_tone:": "🧑🏿",
    ":adult::skin-tone-5:": "🧑🏿",
    ":aerial_tramway:": "🚡",
    ":airplane:": "✈️",
    ":airplane_arriving:": "🛬",
    ":airplane_departure:": "🛫",
    ":airplane_small:": "🛩️",
    ":small_airplane:": "🛩️",
    ":alarm_clock:": "⏰",
    ":alembic:": "⚗️",
    ":alien:": "👽",
    ":ambulance:": "🚑",
    ":amphora:": "🏺",
    ":anatomical_heart:": "🫀",
    ":anchor:": "⚓",
    ":angel:": "👼",
    ":angel_tone1:": "👼🏻",
    ":angel::skin-tone-1:": "👼🏻",
    ":angel_tone2:": "👼🏼",
    ":angel::skin-tone-2:": "👼🏼",
    ":angel_tone3:": "👼🏽",
    ":angel::skin-tone-3:": "👼🏽",
    ":angel_tone4:": "👼🏾",
    ":angel::skin-tone-4:": "👼🏾",
    ":angel_tone5:": "👼🏿",
    ":angel::skin-tone-5:": "👼🏿",
    ":anger:": "💢",
    ":anger_right:": "🗯️",
    ":right_anger_bubble:": "🗯️",
    ":angry:": "😠",
    ">:(": "😠",
    ">:-(": "😠",
    ">(": "😠",
    ">-(": "😠",
    ":anguished:": "😧",
    ":ant:": "🐜",
    ":apple:": "🍎",
    ":aquarius:": "♒",
    ":aries:": "♈",
    ":arrow_backward:": "◀️",
    ":arrow_double_down:": "⏬",
    ":arrow_double_up:": "⏫",
    ":arrow_down:": "⬇️",
    ":arrow_down_small:": "🔽",
    ":arrow_forward:": "▶️",
    ":arrow_heading_down:": "⤵️",
    ":arrow_heading_up:": "⤴️",
    ":arrow_left:": "⬅️",
    ":arrow_lower_left:": "↙️",
    ":arrow_lower_right:": "↘️",
    ":arrow_right:": "➡️",
    ":arrow_right_hook:": "↪️",
    ":arrow_up:": "⬆️",
    ":arrow_up_down:": "↕️",
    ":arrow_up_small:": "🔼",
    ":arrow_upper_left:": "↖️",
    ":arrow_upper_right:": "↗️",
    ":arrows_clockwise:": "🔃",
    ":arrows_counterclockwise:": "🔄",
    ":art:": "🎨",
    ":articulated_lorry:": "🚛",
    ":artist:": "🧑‍🎨",
    ":artist_tone1:": "🧑🏻‍🎨",
    ":artist_light_skin_tone:": "🧑🏻‍🎨",
    ":artist::skin-tone-1:": "🧑🏻‍🎨",
    ":artist_tone2:": "🧑🏼‍🎨",
    ":artist_medium_light_skin_tone:": "🧑🏼‍🎨",
    ":artist::skin-tone-2:": "🧑🏼‍🎨",
    ":artist_tone3:": "🧑🏽‍🎨",
    ":artist_medium_skin_tone:": "🧑🏽‍🎨",
    ":artist::skin-tone-3:": "🧑🏽‍🎨",
    ":artist_tone4:": "🧑🏾‍🎨",
    ":artist_medium_dark_skin_tone:": "🧑🏾‍🎨",
    ":artist::skin-tone-4:": "🧑🏾‍🎨",
    ":artist_tone5:": "🧑🏿‍🎨",
    ":artist_dark_skin_tone:": "🧑🏿‍🎨",
    ":artist::skin-tone-5:": "🧑🏿‍🎨",
    ":asterisk:": "*️⃣",
    ":keycap_asterisk:": "*️⃣",
    ":astonished:": "😲",
    ":astronaut:": "🧑‍🚀",
    ":astronaut_tone1:": "🧑🏻‍🚀",
    ":astronaut_light_skin_tone:": "🧑🏻‍🚀",
    ":astronaut::skin-tone-1:": "🧑🏻‍🚀",
    ":astronaut_tone2:": "🧑🏼‍🚀",
    ":astronaut_medium_light_skin_tone:": "🧑🏼‍🚀",
    ":astronaut::skin-tone-2:": "🧑🏼‍🚀",
    ":astronaut_tone3:": "🧑🏽‍🚀",
    ":astronaut_medium_skin_tone:": "🧑🏽‍🚀",
    ":astronaut::skin-tone-3:": "🧑🏽‍🚀",
    ":astronaut_tone4:": "🧑🏾‍🚀",
    ":astronaut_medium_dark_skin_tone:": "🧑🏾‍🚀",
    ":astronaut::skin-tone-4:": "🧑🏾‍🚀",
    ":astronaut_tone5:": "🧑🏿‍🚀",
    ":astronaut_dark_skin_tone:": "🧑🏿‍🚀",
    ":astronaut::skin-tone-5:": "🧑🏿‍🚀",
    ":athletic_shoe:": "👟",
    ":atm:": "🏧",
    ":atom:": "⚛️",
    ":atom_symbol:": "⚛️",
    ":auto_rickshaw:": "🛺",
    ":avocado:": "🥑",
    ":axe:": "🪓",
    ":b:": "🅱️",
    ":baby:": "👶",
    ":baby_bottle:": "🍼",
    ":baby_chick:": "🐤",
    ":baby_symbol:": "🚼",
    ":baby_tone1:": "👶🏻",
    ":baby::skin-tone-1:": "👶🏻",
    ":baby_tone2:": "👶🏼",
    ":baby::skin-tone-2:": "👶🏼",
    ":baby_tone3:": "👶🏽",
    ":baby::skin-tone-3:": "👶🏽",
    ":baby_tone4:": "👶🏾",
    ":baby::skin-tone-4:": "👶🏾",
    ":baby_tone5:": "👶🏿",
    ":baby::skin-tone-5:": "👶🏿",
    ":back:": "🔙",
    ":bacon:": "🥓",
    ":badger:": "🦡",
    ":badminton:": "🏸",
    ":bagel:": "🥯",
    ":baggage_claim:": "🛄",
    ":ballet_shoes:": "🩰",
    ":balloon:": "🎈",
    ":ballot_box:": "🗳️",
    ":ballot_box_with_ballot:": "🗳️",
    ":ballot_box_with_check:": "☑️",
    ":bamboo:": "🎍",
    ":banana:": "🍌",
    ":bangbang:": "‼️",
    ":banjo:": "🪕",
    ":bank:": "🏦",
    ":bar_chart:": "📊",
    ":barber:": "💈",
    ":baseball:": "⚾",
    ":basket:": "🧺",
    ":basketball:": "🏀",
    ":bat:": "🦇",
    ":bath:": "🛀",
    ":bath_tone1:": "🛀🏻",
    ":bath::skin-tone-1:": "🛀🏻",
    ":bath_tone2:": "🛀🏼",
    ":bath::skin-tone-2:": "🛀🏼",
    ":bath_tone3:": "🛀🏽",
    ":bath::skin-tone-3:": "🛀🏽",
    ":bath_tone4:": "🛀🏾",
    ":bath::skin-tone-4:": "🛀🏾",
    ":bath_tone5:": "🛀🏿",
    ":bath::skin-tone-5:": "🛀🏿",
    ":bathtub:": "🛁",
    ":battery:": "🔋",
    ":beach:": "🏖️",
    ":beach_with_umbrella:": "🏖️",
    ":beach_umbrella:": "⛱️",
    ":umbrella_on_ground:": "⛱️",
    ":bear:": "🐻",
    ":bearded_person:": "🧔",
    ":bearded_person_tone1:": "🧔🏻",
    ":bearded_person_light_skin_tone:": "🧔🏻",
    ":bearded_person::skin-tone-1:": "🧔🏻",
    ":bearded_person_tone2:": "🧔🏼",
    ":bearded_person_medium_light_skin_tone:": "🧔🏼",
    ":bearded_person::skin-tone-2:": "🧔🏼",
    ":bearded_person_tone3:": "🧔🏽",
    ":bearded_person_medium_skin_tone:": "🧔🏽",
    ":bearded_person::skin-tone-3:": "🧔🏽",
    ":bearded_person_tone4:": "🧔🏾",
    ":bearded_person_medium_dark_skin_tone:": "🧔🏾",
    ":bearded_person::skin-tone-4:": "🧔🏾",
    ":bearded_person_tone5:": "🧔🏿",
    ":bearded_person_dark_skin_tone:": "🧔🏿",
    ":bearded_person::skin-tone-5:": "🧔🏿",
    ":beaver:": "🦫",
    ":bed:": "🛏️",
    ":bee:": "🐝",
    ":beer:": "🍺",
    ":beers:": "🍻",
    ":beetle:": "🪲",
    ":beginner:": "🔰",
    ":bell:": "🔔",
    ":bell_pepper:": "🫑",
    ":bellhop:": "🛎️",
    ":bellhop_bell:": "🛎️",
    ":bento:": "🍱",
    ":beverage_box:": "🧃",
    ":bike:": "🚲",
    ":bikini:": "👙",
    ":billed_cap:": "🧢",
    ":biohazard:": "☣️",
    ":biohazard_sign:": "☣️",
    ":bird:": "🐦",
    ":birthday:": "🎂",
    ":bison:": "🦬",
    ":black_cat:": "🐈‍⬛",
    ":black_circle:": "⚫",
    ":black_heart:": "🖤",
    ":black_joker:": "🃏",
    ":black_large_square:": "⬛",
    ":black_medium_small_square:": "◾",
    ":black_medium_square:": "◼️",
    ":black_nib:": "✒️",
    ":black_small_square:": "▪️",
    ":black_square_button:": "🔲",
    ":blond_haired_man:": "👱‍♂️",
    ":blond_haired_man_tone1:": "👱🏻‍♂️",
    ":blond_haired_man_light_skin_tone:": "👱🏻‍♂️",
    ":blond_haired_man::skin-tone-1:": "👱🏻‍♂️",
    ":blond_haired_man_tone2:": "👱🏼‍♂️",
    ":blond_haired_man_medium_light_skin_tone:": "👱🏼‍♂️",
    ":blond_haired_man::skin-tone-2:": "👱🏼‍♂️",
    ":blond_haired_man_tone3:": "👱🏽‍♂️",
    ":blond_haired_man_medium_skin_tone:": "👱🏽‍♂️",
    ":blond_haired_man::skin-tone-3:": "👱🏽‍♂️",
    ":blond_haired_man_tone4:": "👱🏾‍♂️",
    ":blond_haired_man_medium_dark_skin_tone:": "👱🏾‍♂️",
    ":blond_haired_man::skin-tone-4:": "👱🏾‍♂️",
    ":blond_haired_man_tone5:": "👱🏿‍♂️",
    ":blond_haired_man_dark_skin_tone:": "👱🏿‍♂️",
    ":blond_haired_man::skin-tone-5:": "👱🏿‍♂️",
    ":blond_haired_person:": "👱",
    ":person_with_blond_hair:": "👱",
    ":blond_haired_person_tone1:": "👱🏻",
    ":person_with_blond_hair_tone1:": "👱🏻",
    ":blond_haired_person::skin-tone-1:": "👱🏻",
    ":person_with_blond_hair::skin-tone-1:": "👱🏻",
    ":blond_haired_person_tone2:": "👱🏼",
    ":person_with_blond_hair_tone2:": "👱🏼",
    ":blond_haired_person::skin-tone-2:": "👱🏼",
    ":person_with_blond_hair::skin-tone-2:": "👱🏼",
    ":blond_haired_person_tone3:": "👱🏽",
    ":person_with_blond_hair_tone3:": "👱🏽",
    ":blond_haired_person::skin-tone-3:": "👱🏽",
    ":person_with_blond_hair::skin-tone-3:": "👱🏽",
    ":blond_haired_person_tone4:": "👱🏾",
    ":person_with_blond_hair_tone4:": "👱🏾",
    ":blond_haired_person::skin-tone-4:": "👱🏾",
    ":person_with_blond_hair::skin-tone-4:": "👱🏾",
    ":blond_haired_person_tone5:": "👱🏿",
    ":person_with_blond_hair_tone5:": "👱🏿",
    ":blond_haired_person::skin-tone-5:": "👱🏿",
    ":person_with_blond_hair::skin-tone-5:": "👱🏿",
    ":blond_haired_woman:": "👱‍♀️",
    ":blond_haired_woman_tone1:": "👱🏻‍♀️",
    ":blond_haired_woman_light_skin_tone:": "👱🏻‍♀️",
    ":blond_haired_woman::skin-tone-1:": "👱🏻‍♀️",
    ":blond_haired_woman_tone2:": "👱🏼‍♀️",
    ":blond_haired_woman_medium_light_skin_tone:": "👱🏼‍♀️",
    ":blond_haired_woman::skin-tone-2:": "👱🏼‍♀️",
    ":blond_haired_woman_tone3:": "👱🏽‍♀️",
    ":blond_haired_woman_medium_skin_tone:": "👱🏽‍♀️",
    ":blond_haired_woman::skin-tone-3:": "👱🏽‍♀️",
    ":blond_haired_woman_tone4:": "👱🏾‍♀️",
    ":blond_haired_woman_medium_dark_skin_tone:": "👱🏾‍♀️",
    ":blond_haired_woman::skin-tone-4:": "👱🏾‍♀️",
    ":blond_haired_woman_tone5:": "👱🏿‍♀️",
    ":blond_haired_woman_dark_skin_tone:": "👱🏿‍♀️",
    ":blond_haired_woman::skin-tone-5:": "👱🏿‍♀️",
    ":blossom:": "🌼",
    ":blowfish:": "🐡",
    ":blue_book:": "📘",
    ":blue_car:": "🚙",
    ":blue_circle:": "🔵",
    ":blue_heart:": "💙",
    ":blue_square:": "🟦",
    ":blueberries:": "🫐",
    ":blush:": "😊",
    ":\)": "😊",
    ":-\)": "😊",
    "\)": "😊",
    "-\)": "😊",
    ":boar:": "🐗",
    ":bomb:": "💣",
    ":bone:": "🦴",
    ":book:": "📖",
    ":bookmark:": "🔖",
    ":bookmark_tabs:": "📑",
    ":books:": "📚",
    ":boom:": "💥",
    ":boomerang:": "🪃",
    ":boot:": "👢",
    ":bouquet:": "💐",
    ":bow_and_arrow:": "🏹",
    ":archery:": "🏹",
    ":bowl_with_spoon:": "🥣",
    ":bowling:": "🎳",
    ":boxing_glove:": "🥊",
    ":boxing_gloves:": "🥊",
    ":boy:": "👦",
    ":boy_tone1:": "👦🏻",
    ":boy::skin-tone-1:": "👦🏻",
    ":boy_tone2:": "👦🏼",
    ":boy::skin-tone-2:": "👦🏼",
    ":boy_tone3:": "👦🏽",
    ":boy::skin-tone-3:": "👦🏽",
    ":boy_tone4:": "👦🏾",
    ":boy::skin-tone-4:": "👦🏾",
    ":boy_tone5:": "👦🏿",
    ":boy::skin-tone-5:": "👦🏿",
    ":brain:": "🧠",
    ":bread:": "🍞",
    ":breast_feeding:": "🤱",
    ":breast_feeding_tone1:": "🤱🏻",
    ":breast_feeding_light_skin_tone:": "🤱🏻",
    ":breast_feeding::skin-tone-1:": "🤱🏻",
    ":breast_feeding_tone2:": "🤱🏼",
    ":breast_feeding_medium_light_skin_tone:": "🤱🏼",
    ":breast_feeding::skin-tone-2:": "🤱🏼",
    ":breast_feeding_tone3:": "🤱🏽",
    ":breast_feeding_medium_skin_tone:": "🤱🏽",
    ":breast_feeding::skin-tone-3:": "🤱🏽",
    ":breast_feeding_tone4:": "🤱🏾",
    ":breast_feeding_medium_dark_skin_tone:": "🤱🏾",
    ":breast_feeding::skin-tone-4:": "🤱🏾",
    ":breast_feeding_tone5:": "🤱🏿",
    ":breast_feeding_dark_skin_tone:": "🤱🏿",
    ":breast_feeding::skin-tone-5:": "🤱🏿",
    ":bricks:": "🧱",
    ":bridge_at_night:": "🌉",
    ":briefcase:": "💼",
    ":briefs:": "🩲",
    ":broccoli:": "🥦",
    ":broken_heart:": "💔",
    "</3": "💔",
    "<\\3": "💔",
    ":broom:": "🧹",
    ":brown_circle:": "🟤",
    ":brown_heart:": "🤎",
    ":brown_square:": "🟫",
    ":bubble_tea:": "🧋",
    ":bucket:": "🪣",
    ":bug:": "🐛",
    ":bulb:": "💡",
    ":bullettrain_front:": "🚅",
    ":bullettrain_side:": "🚄",
    ":burrito:": "🌯",
    ":bus:": "🚌",
    ":busstop:": "🚏",
    ":bust_in_silhouette:": "👤",
    ":busts_in_silhouette:": "👥",
    ":butter:": "🧈",
    ":butterfly:": "🦋",
    ":cactus:": "🌵",
    ":cake:": "🍰",
    ":calendar:": "📆",
    ":calendar_spiral:": "🗓️",
    ":spiral_calendar_pad:": "🗓️",
    ":call_me:": "🤙",
    ":call_me_hand:": "🤙",
    ":call_me_tone1:": "🤙🏻",
    ":call_me_hand_tone1:": "🤙🏻",
    ":call_me::skin-tone-1:": "🤙🏻",
    ":call_me_hand::skin-tone-1:": "🤙🏻",
    ":call_me_tone2:": "🤙🏼",
    ":call_me_hand_tone2:": "🤙🏼",
    ":call_me::skin-tone-2:": "🤙🏼",
    ":call_me_hand::skin-tone-2:": "🤙🏼",
    ":call_me_tone3:": "🤙🏽",
    ":call_me_hand_tone3:": "🤙🏽",
    ":call_me::skin-tone-3:": "🤙🏽",
    ":call_me_hand::skin-tone-3:": "🤙🏽",
    ":call_me_tone4:": "🤙🏾",
    ":call_me_hand_tone4:": "🤙🏾",
    ":call_me::skin-tone-4:": "🤙🏾",
    ":call_me_hand::skin-tone-4:": "🤙🏾",
    ":call_me_tone5:": "🤙🏿",
    ":call_me_hand_tone5:": "🤙🏿",
    ":call_me::skin-tone-5:": "🤙🏿",
    ":call_me_hand::skin-tone-5:": "🤙🏿",
    ":calling:": "📲",
    ":camel:": "🐫",
    ":camera:": "📷",
    ":camera_with_flash:": "📸",
    ":camping:": "🏕️",
    ":cancer:": "♋",
    ":candle:": "🕯️",
    ":candy:": "🍬",
    ":canned_food:": "🥫",
    ":canoe:": "🛶",
    ":kayak:": "🛶",
    ":capital_abcd:": "🔠",
    ":capricorn:": "♑",
    ":card_box:": "🗃️",
    ":card_file_box:": "🗃️",
    ":card_index:": "📇",
    ":carousel_horse:": "🎠",
    ":carpentry_saw:": "🪚",
    ":carrot:": "🥕",
    ":cat:": "🐱",
    ":cat2:": "🐈",
    ":cd:": "💿",
    ":chains:": "⛓️",
    ":chair:": "🪑",
    ":champagne:": "🍾",
    ":bottle_with_popping_cork:": "🍾",
    ":champagne_glass:": "🥂",
    ":clinking_glass:": "🥂",
    ":chart:": "💹",
    ":chart_with_downwards_trend:": "📉",
    ":chart_with_upwards_trend:": "📈",
    ":checkered_flag:": "🏁",
    ":cheese:": "🧀",
    ":cheese_wedge:": "🧀",
    ":cherries:": "🍒",
    ":cherry_blossom:": "🌸",
    ":chess_pawn:": "♟️",
    ":chestnut:": "🌰",
    ":chicken:": "🐔",
    ":child:": "🧒",
    ":child_tone1:": "🧒🏻",
    ":child_light_skin_tone:": "🧒🏻",
    ":child::skin-tone-1:": "🧒🏻",
    ":child_tone2:": "🧒🏼",
    ":child_medium_light_skin_tone:": "🧒🏼",
    ":child::skin-tone-2:": "🧒🏼",
    ":child_tone3:": "🧒🏽",
    ":child_medium_skin_tone:": "🧒🏽",
    ":child::skin-tone-3:": "🧒🏽",
    ":child_tone4:": "🧒🏾",
    ":child_medium_dark_skin_tone:": "🧒🏾",
    ":child::skin-tone-4:": "🧒🏾",
    ":child_tone5:": "🧒🏿",
    ":child_dark_skin_tone:": "🧒🏿",
    ":child::skin-tone-5:": "🧒🏿",
    ":children_crossing:": "🚸",
    ":chipmunk:": "🐿️",
    ":chocolate_bar:": "🍫",
    ":chopsticks:": "🥢",
    ":christmas_tree:": "🎄",
    ":church:": "⛪",
    ":cinema:": "🎦",
    ":circus_tent:": "🎪",
    ":city_dusk:": "🌆",
    ":city_sunset:": "🌇",
    ":city_sunrise:": "🌇",
    ":cityscape:": "🏙️",
    ":cl:": "🆑",
    ":clap:": "👏",
    ":clap_tone1:": "👏🏻",
    ":clap::skin-tone-1:": "👏🏻",
    ":clap_tone2:": "👏🏼",
    ":clap::skin-tone-2:": "👏🏼",
    ":clap_tone3:": "👏🏽",
    ":clap::skin-tone-3:": "👏🏽",
    ":clap_tone4:": "👏🏾",
    ":clap::skin-tone-4:": "👏🏾",
    ":clap_tone5:": "👏🏿",
    ":clap::skin-tone-5:": "👏🏿",
    ":clapper:": "🎬",
    ":classical_building:": "🏛️",
    ":clipboard:": "📋",
    ":clock:": "🕰️",
    ":mantlepiece_clock:": "🕰️",
    ":clock1:": "🕐",
    ":clock10:": "🕙",
    ":clock1030:": "🕥",
    ":clock11:": "🕚",
    ":clock1130:": "🕦",
    ":clock12:": "🕛",
    ":clock1230:": "🕧",
    ":clock130:": "🕜",
    ":clock2:": "🕑",
    ":clock230:": "🕝",
    ":clock3:": "🕒",
    ":clock330:": "🕞",
    ":clock4:": "🕓",
    ":clock430:": "🕟",
    ":clock5:": "🕔",
    ":clock530:": "🕠",
    ":clock6:": "🕕",
    ":clock630:": "🕡",
    ":clock7:": "🕖",
    ":clock730:": "🕢",
    ":clock8:": "🕗",
    ":clock830:": "🕣",
    ":clock9:": "🕘",
    ":clock930:": "🕤",
    ":closed_book:": "📕",
    ":closed_lock_with_key:": "🔐",
    ":closed_umbrella:": "🌂",
    ":cloud:": "☁️",
    ":cloud_lightning:": "🌩️",
    ":cloud_with_lightning:": "🌩️",
    ":cloud_rain:": "🌧️",
    ":cloud_with_rain:": "🌧️",
    ":cloud_snow:": "🌨️",
    ":cloud_with_snow:": "🌨️",
    ":cloud_tornado:": "🌪️",
    ":cloud_with_tornado:": "🌪️",
    ":clown:": "🤡",
    ":clown_face:": "🤡",
    ":clubs:": "♣️",
    ":coat:": "🧥",
    ":cockroach:": "🪳",
    ":cocktail:": "🍸",
    ":coconut:": "🥥",
    ":coffee:": "☕",
    ":coffin:": "⚰️",
    ":coin:": "🪙",
    ":cold_face:": "🥶",
    ":cold_sweat:": "😰",
    ":comet:": "☄️",
    ":compass:": "🧭",
    ":compression:": "🗜️",
    ":computer:": "💻",
    ":confetti_ball:": "🎊",
    ":confounded:": "😖",
    ":confused:": "😕",
    ":-\\": "😕",
    ":-/": "😕",
    "-\\": "😕",
    "-/": "😕",
    ":congratulations:": "㊗️",
    ":construction:": "🚧",
    ":construction_site:": "🏗️",
    ":building_construction:": "🏗️",
    ":construction_worker:": "👷",
    ":construction_worker_tone1:": "👷🏻",
    ":construction_worker::skin-tone-1:": "👷🏻",
    ":construction_worker_tone2:": "👷🏼",
    ":construction_worker::skin-tone-2:": "👷🏼",
    ":construction_worker_tone3:": "👷🏽",
    ":construction_worker::skin-tone-3:": "👷🏽",
    ":construction_worker_tone4:": "👷🏾",
    ":construction_worker::skin-tone-4:": "👷🏾",
    ":construction_worker_tone5:": "👷🏿",
    ":construction_worker::skin-tone-5:": "👷🏿",
    ":control_knobs:": "🎛️",
    ":convenience_store:": "🏪",
    ":cook:": "🧑‍🍳",
    ":cook_tone1:": "🧑🏻‍🍳",
    ":cook_light_skin_tone:": "🧑🏻‍🍳",
    ":cook::skin-tone-1:": "🧑🏻‍🍳",
    ":cook_tone2:": "🧑🏼‍🍳",
    ":cook_medium_light_skin_tone:": "🧑🏼‍🍳",
    ":cook::skin-tone-2:": "🧑🏼‍🍳",
    ":cook_tone3:": "🧑🏽‍🍳",
    ":cook_medium_skin_tone:": "🧑🏽‍🍳",
    ":cook::skin-tone-3:": "🧑🏽‍🍳",
    ":cook_tone4:": "🧑🏾‍🍳",
    ":cook_medium_dark_skin_tone:": "🧑🏾‍🍳",
    ":cook::skin-tone-4:": "🧑🏾‍🍳",
    ":cook_tone5:": "🧑🏿‍🍳",
    ":cook_dark_skin_tone:": "🧑🏿‍🍳",
    ":cook::skin-tone-5:": "🧑🏿‍🍳",
    ":cookie:": "🍪",
    ":cooking:": "🍳",
    ":cool:": "🆒",
    ":copyright:": "©️",
    ":corn:": "🌽",
    ":couch:": "🛋️",
    ":couch_and_lamp:": "🛋️",
    ":couple:": "👫",
    ":woman_and_man_holding_hands_tone1:": "👫",
    ":woman_and_man_holding_hands_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone1_tone2:": "👫",
    ":woman_and_man_holding_hands_light_skin_tone_medium_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone1_tone3:": "👫",
    ":woman_and_man_holding_hands_light_skin_tone_medium_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone1_tone4:": "👫",
    ":woman_and_man_holding_hands_light_skin_tone_medium_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone1_tone5:": "👫",
    ":woman_and_man_holding_hands_light_skin_tone_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone2:": "👫",
    ":woman_and_man_holding_hands_medium_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone2_tone1:": "👫",
    ":woman_and_man_holding_hands_medium_light_skin_tone_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone2_tone3:": "👫",
    ":woman_and_man_holding_hands_medium_light_skin_tone_medium_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone2_tone4:": "👫",
    ":woman_and_man_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone2_tone5:": "👫",
    ":woman_and_man_holding_hands_medium_light_skin_tone_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone3:": "👫",
    ":woman_and_man_holding_hands_medium_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone3_tone1:": "👫",
    ":woman_and_man_holding_hands_medium_skin_tone_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone3_tone2:": "👫",
    ":woman_and_man_holding_hands_medium_skin_tone_medium_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone3_tone4:": "👫",
    ":woman_and_man_holding_hands_medium_skin_tone_medium_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone3_tone5:": "👫",
    ":woman_and_man_holding_hands_medium_skin_tone_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone4:": "👫",
    ":woman_and_man_holding_hands_medium_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone4_tone1:": "👫",
    ":woman_and_man_holding_hands_medium_dark_skin_tone_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone4_tone2:": "👫",
    ":woman_and_man_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone4_tone3:": "👫",
    ":woman_and_man_holding_hands_medium_dark_skin_tone_medium_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone4_tone5:": "👫",
    ":woman_and_man_holding_hands_medium_dark_skin_tone_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone5:": "👫",
    ":woman_and_man_holding_hands_dark_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone5_tone1:": "👫",
    ":woman_and_man_holding_hands_dark_skin_tone_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone5_tone2:": "👫",
    ":woman_and_man_holding_hands_dark_skin_tone_medium_light_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone5_tone3:": "👫",
    ":woman_and_man_holding_hands_dark_skin_tone_medium_skin_tone:": "👫",
    ":woman_and_man_holding_hands_tone5_tone4:": "👫",
    ":woman_and_man_holding_hands_dark_skin_tone_medium_dark_skin_tone:": "👫",
    ":couple_mm:": "👨‍❤️‍👨",
    ":couple_with_heart_mm:": "👨‍❤️‍👨",
    ":couple_with_heart:": "💑",
    ":couple_with_heart_woman_man:": "👩‍❤️‍👨",
    ":couple_ww:": "👩‍❤️‍👩",
    ":couple_with_heart_ww:": "👩‍❤️‍👩",
    ":couplekiss:": "💏",
    ":cow:": "🐮",
    ":cow2:": "🐄",
    ":cowboy:": "🤠",
    ":face_with_cowboy_hat:": "🤠",
    ":crab:": "🦀",
    ":crayon:": "🖍️",
    ":lower_left_crayon:": "🖍️",
    ":credit_card:": "💳",
    ":crescent_moon:": "🌙",
    ":cricket:": "🦗",
    ":cricket_game:": "🏏",
    ":cricket_bat_ball:": "🏏",
    ":crocodile:": "🐊",
    ":croissant:": "🥐",
    ":cross:": "✝️",
    ":latin_cross:": "✝️",
    ":crossed_flags:": "🎌",
    ":crossed_swords:": "⚔️",
    ":crown:": "👑",
    ":cruise_ship:": "🛳️",
    ":passenger_ship:": "🛳️",
    ":cry:": "😢",
    ":'(": "😭",
    ":'-(": "😭",
    ":(": "😓",
    ":-(": "😓",
    "'(": "😭",
    "'-(": "😭",
    "(": "😓",
    "-(": "😓",
    ":crying_cat_face:": "😿",
    ":crystal_ball:": "🔮",
    ":cucumber:": "🥒",
    ":cup_with_straw:": "🥤",
    ":cupcake:": "🧁",
    ":cupid:": "💘",
    ":curling_stone:": "🥌",
    ":curly_loop:": "➰",
    ":currency_exchange:": "💱",
    ":curry:": "🍛",
    ":custard:": "🍮",
    ":pudding:": "🍮",
    ":flan:": "🍮",
    ":customs:": "🛃",
    ":cut_of_meat:": "🥩",
    ":cyclone:": "🌀",
    ":dagger:": "🗡️",
    ":dagger_knife:": "🗡️",
    ":dancer:": "💃",
    ":dancer_tone1:": "💃🏻",
    ":dancer::skin-tone-1:": "💃🏻",
    ":dancer_tone2:": "💃🏼",
    ":dancer::skin-tone-2:": "💃🏼",
    ":dancer_tone3:": "💃🏽",
    ":dancer::skin-tone-3:": "💃🏽",
    ":dancer_tone4:": "💃🏾",
    ":dancer::skin-tone-4:": "💃🏾",
    ":dancer_tone5:": "💃🏿",
    ":dancer::skin-tone-5:": "💃🏿",
    ":dango:": "🍡",
    ":dark_sunglasses:": "🕶️",
    ":dart:": "🎯",
    ":dash:": "💨",
    ":date:": "📅",
    ":deaf_man:": "🧏‍♂️",
    ":deaf_man_tone1:": "🧏🏻‍♂️",
    ":deaf_man_light_skin_tone:": "🧏🏻‍♂️",
    ":deaf_man::skin-tone-1:": "🧏🏻‍♂️",
    ":deaf_man_tone2:": "🧏🏼‍♂️",
    ":deaf_man_medium_light_skin_tone:": "🧏🏼‍♂️",
    ":deaf_man::skin-tone-2:": "🧏🏼‍♂️",
    ":deaf_man_tone3:": "🧏🏽‍♂️",
    ":deaf_man_medium_skin_tone:": "🧏🏽‍♂️",
    ":deaf_man::skin-tone-3:": "🧏🏽‍♂️",
    ":deaf_man_tone4:": "🧏🏾‍♂️",
    ":deaf_man_medium_dark_skin_tone:": "🧏🏾‍♂️",
    ":deaf_man::skin-tone-4:": "🧏🏾‍♂️",
    ":deaf_man_tone5:": "🧏🏿‍♂️",
    ":deaf_man_dark_skin_tone:": "🧏🏿‍♂️",
    ":deaf_man::skin-tone-5:": "🧏🏿‍♂️",
    ":deaf_person:": "🧏",
    ":deaf_person_tone1:": "🧏🏻",
    ":deaf_person_light_skin_tone:": "🧏🏻",
    ":deaf_person::skin-tone-1:": "🧏🏻",
    ":deaf_person_tone2:": "🧏🏼",
    ":deaf_person_medium_light_skin_tone:": "🧏🏼",
    ":deaf_person::skin-tone-2:": "🧏🏼",
    ":deaf_person_tone3:": "🧏🏽",
    ":deaf_person_medium_skin_tone:": "🧏🏽",
    ":deaf_person::skin-tone-3:": "🧏🏽",
    ":deaf_person_tone4:": "🧏🏾",
    ":deaf_person_medium_dark_skin_tone:": "🧏🏾",
    ":deaf_person::skin-tone-4:": "🧏🏾",
    ":deaf_person_tone5:": "🧏🏿",
    ":deaf_person_dark_skin_tone:": "🧏🏿",
    ":deaf_person::skin-tone-5:": "🧏🏿",
    ":deaf_woman:": "🧏‍♀️",
    ":deaf_woman_tone1:": "🧏🏻‍♀️",
    ":deaf_woman_light_skin_tone:": "🧏🏻‍♀️",
    ":deaf_woman::skin-tone-1:": "🧏🏻‍♀️",
    ":deaf_woman_tone2:": "🧏🏼‍♀️",
    ":deaf_woman_medium_light_skin_tone:": "🧏🏼‍♀️",
    ":deaf_woman::skin-tone-2:": "🧏🏼‍♀️",
    ":deaf_woman_tone3:": "🧏🏽‍♀️",
    ":deaf_woman_medium_skin_tone:": "🧏🏽‍♀️",
    ":deaf_woman::skin-tone-3:": "🧏🏽‍♀️",
    ":deaf_woman_tone4:": "🧏🏾‍♀️",
    ":deaf_woman_medium_dark_skin_tone:": "🧏🏾‍♀️",
    ":deaf_woman::skin-tone-4:": "🧏🏾‍♀️",
    ":deaf_woman_tone5:": "🧏🏿‍♀️",
    ":deaf_woman_dark_skin_tone:": "🧏🏿‍♀️",
    ":deaf_woman::skin-tone-5:": "🧏🏿‍♀️",
    ":deciduous_tree:": "🌳",
    ":deer:": "🦌",
    ":department_store:": "🏬",
    ":desert:": "🏜️",
    ":desktop:": "🖥️",
    ":desktop_computer:": "🖥️",
    ":detective:": "🕵️",
    ":spy:": "🕵️",
    ":sleuth_or_spy:": "🕵️",
    ":detective_tone1:": "🕵🏻",
    ":spy_tone1:": "🕵🏻",
    ":sleuth_or_spy_tone1:": "🕵🏻",
    ":detective::skin-tone-1:": "🕵🏻",
    ":spy::skin-tone-1:": "🕵🏻",
    ":sleuth_or_spy::skin-tone-1:": "🕵🏻",
    ":detective_tone2:": "🕵🏼",
    ":spy_tone2:": "🕵🏼",
    ":sleuth_or_spy_tone2:": "🕵🏼",
    ":detective::skin-tone-2:": "🕵🏼",
    ":spy::skin-tone-2:": "🕵🏼",
    ":sleuth_or_spy::skin-tone-2:": "🕵🏼",
    ":detective_tone3:": "🕵🏽",
    ":spy_tone3:": "🕵🏽",
    ":sleuth_or_spy_tone3:": "🕵🏽",
    ":detective::skin-tone-3:": "🕵🏽",
    ":spy::skin-tone-3:": "🕵🏽",
    ":sleuth_or_spy::skin-tone-3:": "🕵🏽",
    ":detective_tone4:": "🕵🏾",
    ":spy_tone4:": "🕵🏾",
    ":sleuth_or_spy_tone4:": "🕵🏾",
    ":detective::skin-tone-4:": "🕵🏾",
    ":spy::skin-tone-4:": "🕵🏾",
    ":sleuth_or_spy::skin-tone-4:": "🕵🏾",
    ":detective_tone5:": "🕵🏿",
    ":spy_tone5:": "🕵🏿",
    ":sleuth_or_spy_tone5:": "🕵🏿",
    ":detective::skin-tone-5:": "🕵🏿",
    ":spy::skin-tone-5:": "🕵🏿",
    ":sleuth_or_spy::skin-tone-5:": "🕵🏿",
    ":diamond_shape_with_a_dot_inside:": "💠",
    ":diamonds:": "♦️",
    ":disappointed:": "😞",
    ":disappointed_relieved:": "😥",
    ":disguised_face:": "🥸",
    ":dividers:": "🗂️",
    ":card_index_dividers:": "🗂️",
    ":diving_mask:": "🤿",
    ":diya_lamp:": "🪔",
    ":dizzy:": "💫",
    ":dizzy_face:": "😵",
    ":dna:": "🧬",
    ":do_not_litter:": "🚯",
    ":dodo:": "🦤",
    ":dog:": "🐶",
    ":dog2:": "🐕",
    ":dollar:": "💵",
    ":dolls:": "🎎",
    ":dolphin:": "🐬",
    ":door:": "🚪",
    ":doughnut:": "🍩",
    ":dove:": "🕊️",
    ":dove_of_peace:": "🕊️",
    ":dragon:": "🐉",
    ":dragon_face:": "🐲",
    ":dress:": "👗",
    ":dromedary_camel:": "🐪",
    ":drooling_face:": "🤤",
    ":drool:": "🤤",
    ":drop_of_blood:": "🩸",
    ":droplet:": "💧",
    ":drum:": "🥁",
    ":drum_with_drumsticks:": "🥁",
    ":duck:": "🦆",
    ":dumpling:": "🥟",
    ":dvd:": "📀",
    ":e_mail:": "📧",
    ":email:": "📧",
    ":eagle:": "🦅",
    ":ear:": "👂",
    ":ear_of_rice:": "🌾",
    ":ear_tone1:": "👂🏻",
    ":ear::skin-tone-1:": "👂🏻",
    ":ear_tone2:": "👂🏼",
    ":ear::skin-tone-2:": "👂🏼",
    ":ear_tone3:": "👂🏽",
    ":ear::skin-tone-3:": "👂🏽",
    ":ear_tone4:": "👂🏾",
    ":ear::skin-tone-4:": "👂🏾",
    ":ear_tone5:": "👂🏿",
    ":ear::skin-tone-5:": "👂🏿",
    ":ear_with_hearing_aid:": "🦻",
    ":ear_with_hearing_aid_tone1:": "🦻🏻",
    ":ear_with_hearing_aid_light_skin_tone:": "🦻🏻",
    ":ear_with_hearing_aid::skin-tone-1:": "🦻🏻",
    ":ear_with_hearing_aid_tone2:": "🦻🏼",
    ":ear_with_hearing_aid_medium_light_skin_tone:": "🦻🏼",
    ":ear_with_hearing_aid::skin-tone-2:": "🦻🏼",
    ":ear_with_hearing_aid_tone3:": "🦻🏽",
    ":ear_with_hearing_aid_medium_skin_tone:": "🦻🏽",
    ":ear_with_hearing_aid::skin-tone-3:": "🦻🏽",
    ":ear_with_hearing_aid_tone4:": "🦻🏾",
    ":ear_with_hearing_aid_medium_dark_skin_tone:": "🦻🏾",
    ":ear_with_hearing_aid::skin-tone-4:": "🦻🏾",
    ":ear_with_hearing_aid_tone5:": "🦻🏿",
    ":ear_with_hearing_aid_dark_skin_tone:": "🦻🏿",
    ":ear_with_hearing_aid::skin-tone-5:": "🦻🏿",
    ":earth_africa:": "🌍",
    ":earth_americas:": "🌎",
    ":earth_asia:": "🌏",
    ":egg:": "🥚",
    ":eggplant:": "🍆",
    ":eight:": "8️⃣",
    ":eight_pointed_black_star:": "✴️",
    ":eight_spoked_asterisk:": "✳️",
    ":eject:": "⏏️",
    ":eject_symbol:": "⏏️",
    ":electric_plug:": "🔌",
    ":elephant:": "🐘",
    ":elevator:": "🛗",
    ":elf:": "🧝",
    ":elf_tone1:": "🧝🏻",
    ":elf_light_skin_tone:": "🧝🏻",
    ":elf::skin-tone-1:": "🧝🏻",
    ":elf_tone2:": "🧝🏼",
    ":elf_medium_light_skin_tone:": "🧝🏼",
    ":elf::skin-tone-2:": "🧝🏼",
    ":elf_tone3:": "🧝🏽",
    ":elf_medium_skin_tone:": "🧝🏽",
    ":elf::skin-tone-3:": "🧝🏽",
    ":elf_tone4:": "🧝🏾",
    ":elf_medium_dark_skin_tone:": "🧝🏾",
    ":elf::skin-tone-4:": "🧝🏾",
    ":elf_tone5:": "🧝🏿",
    ":elf_dark_skin_tone:": "🧝🏿",
    ":elf::skin-tone-5:": "🧝🏿",
    ":end:": "🔚",
    ":england:": "🏴󠁧󠁢󠁥󠁮󠁧󠁿",
    ":envelope:": "✉️",
    ":envelope_with_arrow:": "📩",
    ":euro:": "💶",
    ":european_castle:": "🏰",
    ":european_post_office:": "🏤",
    ":evergreen_tree:": "🌲",
    ":exclamation:": "❗",
    ":exploding_head:": "🤯",
    ":expressionless:": "😑",
    ":eye:": "👁️",
    ":eye_in_speech_bubble:": "👁‍🗨",
    ":eyeglasses:": "👓",
    ":eyes:": "👀",
    ":face_vomiting:": "🤮",
    ":face_with_hand_over_mouth:": "🤭",
    ":face_with_monocle:": "🧐",
    ":face_with_raised_eyebrow:": "🤨",
    ":face_with_symbols_over_mouth:": "🤬",
    ":factory:": "🏭",
    ":factory_worker:": "🧑‍🏭",
    ":factory_worker_tone1:": "🧑🏻‍🏭",
    ":factory_worker_light_skin_tone:": "🧑🏻‍🏭",
    ":factory_worker::skin-tone-1:": "🧑🏻‍🏭",
    ":factory_worker_tone2:": "🧑🏼‍🏭",
    ":factory_worker_medium_light_skin_tone:": "🧑🏼‍🏭",
    ":factory_worker::skin-tone-2:": "🧑🏼‍🏭",
    ":factory_worker_tone3:": "🧑🏽‍🏭",
    ":factory_worker_medium_skin_tone:": "🧑🏽‍🏭",
    ":factory_worker::skin-tone-3:": "🧑🏽‍🏭",
    ":factory_worker_tone4:": "🧑🏾‍🏭",
    ":factory_worker_medium_dark_skin_tone:": "🧑🏾‍🏭",
    ":factory_worker::skin-tone-4:": "🧑🏾‍🏭",
    ":factory_worker_tone5:": "🧑🏿‍🏭",
    ":factory_worker_dark_skin_tone:": "🧑🏿‍🏭",
    ":factory_worker::skin-tone-5:": "🧑🏿‍🏭",
    ":fairy:": "🧚",
    ":fairy_tone1:": "🧚🏻",
    ":fairy_light_skin_tone:": "🧚🏻",
    ":fairy::skin-tone-1:": "🧚🏻",
    ":fairy_tone2:": "🧚🏼",
    ":fairy_medium_light_skin_tone:": "🧚🏼",
    ":fairy::skin-tone-2:": "🧚🏼",
    ":fairy_tone3:": "🧚🏽",
    ":fairy_medium_skin_tone:": "🧚🏽",
    ":fairy::skin-tone-3:": "🧚🏽",
    ":fairy_tone4:": "🧚🏾",
    ":fairy_medium_dark_skin_tone:": "🧚🏾",
    ":fairy::skin-tone-4:": "🧚🏾",
    ":fairy_tone5:": "🧚🏿",
    ":fairy_dark_skin_tone:": "🧚🏿",
    ":fairy::skin-tone-5:": "🧚🏿",
    ":falafel:": "🧆",
    ":fallen_leaf:": "🍂",
    ":family:": "👪",
    ":family_man_boy:": "👨‍👦",
    ":family_man_boy_boy:": "👨‍👦‍👦",
    ":family_man_girl:": "👨‍👧",
    ":family_man_girl_boy:": "👨‍👧‍👦",
    ":family_man_girl_girl:": "👨‍👧‍👧",
    ":family_man_woman_boy:": "👨‍👩‍👦",
    ":family_mmb:": "👨‍👨‍👦",
    ":family_mmbb:": "👨‍👨‍👦‍👦",
    ":family_mmg:": "👨‍👨‍👧",
    ":family_mmgb:": "👨‍👨‍👧‍👦",
    ":family_mmgg:": "👨‍👨‍👧‍👧",
    ":family_mwbb:": "👨‍👩‍👦‍👦",
    ":family_mwg:": "👨‍👩‍👧",
    ":family_mwgb:": "👨‍👩‍👧‍👦",
    ":family_mwgg:": "👨‍👩‍👧‍👧",
    ":family_woman_boy:": "👩‍👦",
    ":family_woman_boy_boy:": "👩‍👦‍👦",
    ":family_woman_girl:": "👩‍👧",
    ":family_woman_girl_boy:": "👩‍👧‍👦",
    ":family_woman_girl_girl:": "👩‍👧‍👧",
    ":family_wwb:": "👩‍👩‍👦",
    ":family_wwbb:": "👩‍👩‍👦‍👦",
    ":family_wwg:": "👩‍👩‍👧",
    ":family_wwgb:": "👩‍👩‍👧‍👦",
    ":family_wwgg:": "👩‍👩‍👧‍👧",
    ":farmer:": "🧑‍🌾",
    ":farmer_tone1:": "🧑🏻‍🌾",
    ":farmer_light_skin_tone:": "🧑🏻‍🌾",
    ":farmer::skin-tone-1:": "🧑🏻‍🌾",
    ":farmer_tone2:": "🧑🏼‍🌾",
    ":farmer_medium_light_skin_tone:": "🧑🏼‍🌾",
    ":farmer::skin-tone-2:": "🧑🏼‍🌾",
    ":farmer_tone3:": "🧑🏽‍🌾",
    ":farmer_medium_skin_tone:": "🧑🏽‍🌾",
    ":farmer::skin-tone-3:": "🧑🏽‍🌾",
    ":farmer_tone4:": "🧑🏾‍🌾",
    ":farmer_medium_dark_skin_tone:": "🧑🏾‍🌾",
    ":farmer::skin-tone-4:": "🧑🏾‍🌾",
    ":farmer_tone5:": "🧑🏿‍🌾",
    ":farmer_dark_skin_tone:": "🧑🏿‍🌾",
    ":farmer::skin-tone-5:": "🧑🏿‍🌾",
    ":fast_forward:": "⏩",
    ":fax:": "📠",
    ":fearful:": "😨",
    ":feather:": "🪶",
    ":feet:": "🐾",
    ":paw_prints:": "🐾",
    ":female_sign:": "♀️",
    ":ferris_wheel:": "🎡",
    ":ferry:": "⛴️",
    ":field_hockey:": "🏑",
    ":file_cabinet:": "🗄️",
    ":file_folder:": "📁",
    ":film_frames:": "🎞️",
    ":fingers_crossed:": "🤞",
    ":hand_with_index_and_middle_finger_crossed:": "🤞",
    ":fingers_crossed_tone1:": "🤞🏻",
    ":hand_with_index_and_middle_fingers_crossed_tone1:": "🤞🏻",
    ":fingers_crossed::skin-tone-1:": "🤞🏻",
    ":hand_with_index_and_middle_finger_crossed::skin-tone-1:": "🤞🏻",
    ":fingers_crossed_tone2:": "🤞🏼",
    ":hand_with_index_and_middle_fingers_crossed_tone2:": "🤞🏼",
    ":fingers_crossed::skin-tone-2:": "🤞🏼",
    ":hand_with_index_and_middle_finger_crossed::skin-tone-2:": "🤞🏼",
    ":fingers_crossed_tone3:": "🤞🏽",
    ":hand_with_index_and_middle_fingers_crossed_tone3:": "🤞🏽",
    ":fingers_crossed::skin-tone-3:": "🤞🏽",
    ":hand_with_index_and_middle_finger_crossed::skin-tone-3:": "🤞🏽",
    ":fingers_crossed_tone4:": "🤞🏾",
    ":hand_with_index_and_middle_fingers_crossed_tone4:": "🤞🏾",
    ":fingers_crossed::skin-tone-4:": "🤞🏾",
    ":hand_with_index_and_middle_finger_crossed::skin-tone-4:": "🤞🏾",
    ":fingers_crossed_tone5:": "🤞🏿",
    ":hand_with_index_and_middle_fingers_crossed_tone5:": "🤞🏿",
    ":fingers_crossed::skin-tone-5:": "🤞🏿",
    ":hand_with_index_and_middle_finger_crossed::skin-tone-5:": "🤞🏿",
    ":fire:": "🔥",
    ":flame:": "🔥",
    ":fire_engine:": "🚒",
    ":fire_extinguisher:": "🧯",
    ":firecracker:": "🧨",
    ":firefighter:": "🧑‍🚒",
    ":firefighter_tone1:": "🧑🏻‍🚒",
    ":firefighter_light_skin_tone:": "🧑🏻‍🚒",
    ":firefighter::skin-tone-1:": "🧑🏻‍🚒",
    ":firefighter_tone2:": "🧑🏼‍🚒",
    ":firefighter_medium_light_skin_tone:": "🧑🏼‍🚒",
    ":firefighter::skin-tone-2:": "🧑🏼‍🚒",
    ":firefighter_tone3:": "🧑🏽‍🚒",
    ":firefighter_medium_skin_tone:": "🧑🏽‍🚒",
    ":firefighter::skin-tone-3:": "🧑🏽‍🚒",
    ":firefighter_tone4:": "🧑🏾‍🚒",
    ":firefighter_medium_dark_skin_tone:": "🧑🏾‍🚒",
    ":firefighter::skin-tone-4:": "🧑🏾‍🚒",
    ":firefighter_tone5:": "🧑🏿‍🚒",
    ":firefighter_dark_skin_tone:": "🧑🏿‍🚒",
    ":firefighter::skin-tone-5:": "🧑🏿‍🚒",
    ":fireworks:": "🎆",
    ":first_place:": "🥇",
    ":first_place_medal:": "🥇",
    ":first_quarter_moon:": "🌓",
    ":first_quarter_moon_with_face:": "🌛",
    ":fish:": "🐟",
    ":fish_cake:": "🍥",
    ":fishing_pole_and_fish:": "🎣",
    ":fist:": "✊",
    ":fist_tone1:": "✊🏻",
    ":fist::skin-tone-1:": "✊🏻",
    ":fist_tone2:": "✊🏼",
    ":fist::skin-tone-2:": "✊🏼",
    ":fist_tone3:": "✊🏽",
    ":fist::skin-tone-3:": "✊🏽",
    ":fist_tone4:": "✊🏾",
    ":fist::skin-tone-4:": "✊🏾",
    ":fist_tone5:": "✊🏿",
    ":fist::skin-tone-5:": "✊🏿",
    ":five:": "5️⃣",
    ":flag_ac:": "🇦🇨",
    ":flag_ad:": "🇦🇩",
    ":flag_ae:": "🇦🇪",
    ":flag_af:": "🇦🇫",
    ":flag_ag:": "🇦🇬",
    ":flag_ai:": "🇦🇮",
    ":flag_al:": "🇦🇱",
    ":flag_am:": "🇦🇲",
    ":flag_ao:": "🇦🇴",
    ":flag_aq:": "🇦🇶",
    ":flag_ar:": "🇦🇷",
    ":flag_as:": "🇦🇸",
    ":flag_at:": "🇦🇹",
    ":flag_au:": "🇦🇺",
    ":flag_aw:": "🇦🇼",
    ":flag_ax:": "🇦🇽",
    ":flag_az:": "🇦🇿",
    ":flag_ba:": "🇧🇦",
    ":flag_bb:": "🇧🇧",
    ":flag_bd:": "🇧🇩",
    ":flag_be:": "🇧🇪",
    ":flag_bf:": "🇧🇫",
    ":flag_bg:": "🇧🇬",
    ":flag_bh:": "🇧🇭",
    ":flag_bi:": "🇧🇮",
    ":flag_bj:": "🇧🇯",
    ":flag_bl:": "🇧🇱",
    ":flag_black:": "🏴",
    ":flag_bm:": "🇧🇲",
    ":flag_bn:": "🇧🇳",
    ":flag_bo:": "🇧🇴",
    ":flag_bq:": "🇧🇶",
    ":flag_br:": "🇧🇷",
    ":flag_bs:": "🇧🇸",
    ":flag_bt:": "🇧🇹",
    ":flag_bv:": "🇧🇻",
    ":flag_bw:": "🇧🇼",
    ":flag_by:": "🇧🇾",
    ":flag_bz:": "🇧🇿",
    ":flag_ca:": "🇨🇦",
    ":flag_cc:": "🇨🇨",
    ":flag_cd:": "🇨🇩",
    ":flag_cf:": "🇨🇫",
    ":flag_cg:": "🇨🇬",
    ":flag_ch:": "🇨🇭",
    ":flag_ci:": "🇨🇮",
    ":flag_ck:": "🇨🇰",
    ":flag_cl:": "🇨🇱",
    ":flag_cm:": "🇨🇲",
    ":flag_cn:": "🇨🇳",
    ":flag_co:": "🇨🇴",
    ":flag_cp:": "🇨🇵",
    ":flag_cr:": "🇨🇷",
    ":flag_cu:": "🇨🇺",
    ":flag_cv:": "🇨🇻",
    ":flag_cw:": "🇨🇼",
    ":flag_cx:": "🇨🇽",
    ":flag_cy:": "🇨🇾",
    ":flag_cz:": "🇨🇿",
    ":flag_de:": "🇩🇪",
    ":flag_dg:": "🇩🇬",
    ":flag_dj:": "🇩🇯",
    ":flag_dk:": "🇩🇰",
    ":flag_dm:": "🇩🇲",
    ":flag_do:": "🇩🇴",
    ":flag_dz:": "🇩🇿",
    ":flag_ea:": "🇪🇦",
    ":flag_ec:": "🇪🇨",
    ":flag_ee:": "🇪🇪",
    ":flag_eg:": "🇪🇬",
    ":flag_eh:": "🇪🇭",
    ":flag_er:": "🇪🇷",
    ":flag_es:": "🇪🇸",
    ":flag_et:": "🇪🇹",
    ":flag_eu:": "🇪🇺",
    ":flag_fi:": "🇫🇮",
    ":flag_fj:": "🇫🇯",
    ":flag_fk:": "🇫🇰",
    ":flag_fm:": "🇫🇲",
    ":flag_fo:": "🇫🇴",
    ":flag_fr:": "🇫🇷",
    ":flag_ga:": "🇬🇦",
    ":flag_gb:": "🇬🇧",
    ":flag_gd:": "🇬🇩",
    ":flag_ge:": "🇬🇪",
    ":flag_gf:": "🇬🇫",
    ":flag_gg:": "🇬🇬",
    ":flag_gh:": "🇬🇭",
    ":flag_gi:": "🇬🇮",
    ":flag_gl:": "🇬🇱",
    ":flag_gm:": "🇬🇲",
    ":flag_gn:": "🇬🇳",
    ":flag_gp:": "🇬🇵",
    ":flag_gq:": "🇬🇶",
    ":flag_gr:": "🇬🇷",
    ":flag_gs:": "🇬🇸",
    ":flag_gt:": "🇬🇹",
    ":flag_gu:": "🇬🇺",
    ":flag_gw:": "🇬🇼",
    ":flag_gy:": "🇬🇾",
    ":flag_hk:": "🇭🇰",
    ":flag_hm:": "🇭🇲",
    ":flag_hn:": "🇭🇳",
    ":flag_hr:": "🇭🇷",
    ":flag_ht:": "🇭🇹",
    ":flag_hu:": "🇭🇺",
    ":flag_ic:": "🇮🇨",
    ":flag_id:": "🇮🇩",
    ":flag_ie:": "🇮🇪",
    ":flag_il:": "🇮🇱",
    ":flag_im:": "🇮🇲",
    ":flag_in:": "🇮🇳",
    ":flag_io:": "🇮🇴",
    ":flag_iq:": "🇮🇶",
    ":flag_ir:": "🇮🇷",
    ":flag_is:": "🇮🇸",
    ":flag_it:": "🇮🇹",
    ":flag_je:": "🇯🇪",
    ":flag_jm:": "🇯🇲",
    ":flag_jo:": "🇯🇴",
    ":flag_jp:": "🇯🇵",
    ":flag_ke:": "🇰🇪",
    ":flag_kg:": "🇰🇬",
    ":flag_kh:": "🇰🇭",
    ":flag_ki:": "🇰🇮",
    ":flag_km:": "🇰🇲",
    ":flag_kn:": "🇰🇳",
    ":flag_kp:": "🇰🇵",
    ":flag_kr:": "🇰🇷",
    ":flag_kw:": "🇰🇼",
    ":flag_ky:": "🇰🇾",
    ":flag_kz:": "🇰🇿",
    ":flag_la:": "🇱🇦",
    ":flag_lb:": "🇱🇧",
    ":flag_lc:": "🇱🇨",
    ":flag_li:": "🇱🇮",
    ":flag_lk:": "🇱🇰",
    ":flag_lr:": "🇱🇷",
    ":flag_ls:": "🇱🇸",
    ":flag_lt:": "🇱🇹",
    ":flag_lu:": "🇱🇺",
    ":flag_lv:": "🇱🇻",
    ":flag_ly:": "🇱🇾",
    ":flag_ma:": "🇲🇦",
    ":flag_mc:": "🇲🇨",
    ":flag_md:": "🇲🇩",
    ":flag_me:": "🇲🇪",
    ":flag_mf:": "🇲🇫",
    ":flag_mg:": "🇲🇬",
    ":flag_mh:": "🇲🇭",
    ":flag_mk:": "🇲🇰",
    ":flag_ml:": "🇲🇱",
    ":flag_mm:": "🇲🇲",
    ":flag_mn:": "🇲🇳",
    ":flag_mo:": "🇲🇴",
    ":flag_mp:": "🇲🇵",
    ":flag_mq:": "🇲🇶",
    ":flag_mr:": "🇲🇷",
    ":flag_ms:": "🇲🇸",
    ":flag_mt:": "🇲🇹",
    ":flag_mu:": "🇲🇺",
    ":flag_mv:": "🇲🇻",
    ":flag_mw:": "🇲🇼",
    ":flag_mx:": "🇲🇽",
    ":flag_my:": "🇲🇾",
    ":flag_mz:": "🇲🇿",
    ":flag_na:": "🇳🇦",
    ":flag_nc:": "🇳🇨",
    ":flag_ne:": "🇳🇪",
    ":flag_nf:": "🇳🇫",
    ":flag_ng:": "🇳🇬",
    ":flag_ni:": "🇳🇮",
    ":flag_nl:": "🇳🇱",
    ":flag_no:": "🇳🇴",
    ":flag_np:": "🇳🇵",
    ":flag_nr:": "🇳🇷",
    ":flag_nu:": "🇳🇺",
    ":flag_nz:": "🇳🇿",
    ":flag_om:": "🇴🇲",
    ":flag_pa:": "🇵🇦",
    ":flag_pe:": "🇵🇪",
    ":flag_pf:": "🇵🇫",
    ":flag_pg:": "🇵🇬",
    ":flag_ph:": "🇵🇭",
    ":flag_pk:": "🇵🇰",
    ":flag_pl:": "🇵🇱",
    ":flag_pm:": "🇵🇲",
    ":flag_pn:": "🇵🇳",
    ":flag_pr:": "🇵🇷",
    ":flag_ps:": "🇵🇸",
    ":flag_pt:": "🇵🇹",
    ":flag_pw:": "🇵🇼",
    ":flag_py:": "🇵🇾",
    ":flag_qa:": "🇶🇦",
    ":flag_re:": "🇷🇪",
    ":flag_ro:": "🇷🇴",
    ":flag_rs:": "🇷🇸",
    ":flag_ru:": "🇷🇺",
    ":flag_rw:": "🇷🇼",
    ":flag_sa:": "🇸🇦",
    ":flag_sb:": "🇸🇧",
    ":flag_sc:": "🇸🇨",
    ":flag_sd:": "🇸🇩",
    ":flag_se:": "🇸🇪",
    ":flag_sg:": "🇸🇬",
    ":flag_sh:": "🇸🇭",
    ":flag_si:": "🇸🇮",
    ":flag_sj:": "🇸🇯",
    ":flag_sk:": "🇸🇰",
    ":flag_sl:": "🇸🇱",
    ":flag_sm:": "🇸🇲",
    ":flag_sn:": "🇸🇳",
    ":flag_so:": "🇸🇴",
    ":flag_sr:": "🇸🇷",
    ":flag_ss:": "🇸🇸",
    ":flag_st:": "🇸🇹",
    ":flag_sv:": "🇸🇻",
    ":flag_sx:": "🇸🇽",
    ":flag_sy:": "🇸🇾",
    ":flag_sz:": "🇸🇿",
    ":flag_ta:": "🇹🇦",
    ":flag_tc:": "🇹🇨",
    ":flag_td:": "🇹🇩",
    ":flag_tf:": "🇹🇫",
    ":flag_tg:": "🇹🇬",
    ":flag_th:": "🇹🇭",
    ":flag_tj:": "🇹🇯",
    ":flag_tk:": "🇹🇰",
    ":flag_tl:": "🇹🇱",
    ":flag_tm:": "🇹🇲",
    ":flag_tn:": "🇹🇳",
    ":flag_to:": "🇹🇴",
    ":flag_tr:": "🇹🇷",
    ":flag_tt:": "🇹🇹",
    ":flag_tv:": "🇹🇻",
    ":flag_tw:": "🇹🇼",
    ":flag_tz:": "🇹🇿",
    ":flag_ua:": "🇺🇦",
    ":flag_ug:": "🇺🇬",
    ":flag_um:": "🇺🇲",
    ":flag_us:": "🇺🇸",
    ":flag_uy:": "🇺🇾",
    ":flag_uz:": "🇺🇿",
    ":flag_va:": "🇻🇦",
    ":flag_vc:": "🇻🇨",
    ":flag_ve:": "🇻🇪",
    ":flag_vg:": "🇻🇬",
    ":flag_vi:": "🇻🇮",
    ":flag_vn:": "🇻🇳",
    ":flag_vu:": "🇻🇺",
    ":flag_wf:": "🇼🇫",
    ":flag_white:": "🏳️",
    ":flag_ws:": "🇼🇸",
    ":flag_xk:": "🇽🇰",
    ":flag_ye:": "🇾🇪",
    ":flag_yt:": "🇾🇹",
    ":flag_za:": "🇿🇦",
    ":flag_zm:": "🇿🇲",
    ":flag_zw:": "🇿🇼",
    ":flags:": "🎏",
    ":flamingo:": "🦩",
    ":flashlight:": "🔦",
    ":flatbread:": "🫓",
    ":fleur_de_lis:": "⚜️",
    ":floppy_disk:": "💾",
    ":flower_playing_cards:": "🎴",
    ":flushed:": "😳",
    ":fly:": "🪰",
    ":flying_disc:": "🥏",
    ":flying_saucer:": "🛸",
    ":fog:": "🌫️",
    ":foggy:": "🌁",
    ":fondue:": "🫕",
    ":foot:": "🦶",
    ":foot_tone1:": "🦶🏻",
    ":foot_light_skin_tone:": "🦶🏻",
    ":foot::skin-tone-1:": "🦶🏻",
    ":foot_tone2:": "🦶🏼",
    ":foot_medium_light_skin_tone:": "🦶🏼",
    ":foot::skin-tone-2:": "🦶🏼",
    ":foot_tone3:": "🦶🏽",
    ":foot_medium_skin_tone:": "🦶🏽",
    ":foot::skin-tone-3:": "🦶🏽",
    ":foot_tone4:": "🦶🏾",
    ":foot_medium_dark_skin_tone:": "🦶🏾",
    ":foot::skin-tone-4:": "🦶🏾",
    ":foot_tone5:": "🦶🏿",
    ":foot_dark_skin_tone:": "🦶🏿",
    ":foot::skin-tone-5:": "🦶🏿",
    ":football:": "🏈",
    ":footprints:": "👣",
    ":fork_and_knife:": "🍴",
    ":fork_knife_plate:": "🍽️",
    ":fork_and_knife_with_plate:": "🍽️",
    ":fortune_cookie:": "🥠",
    ":fountain:": "⛲",
    ":four:": "4️⃣",
    ":four_leaf_clover:": "🍀",
    ":fox:": "🦊",
    ":fox_face:": "🦊",
    ":frame_photo:": "🖼️",
    ":frame_with_picture:": "🖼️",
    ":free:": "🆓",
    ":french_bread:": "🥖",
    ":baguette_bread:": "🥖",
    ":fried_shrimp:": "🍤",
    ":fries:": "🍟",
    ":frog:": "🐸",
    ":frowning:": "😦",
    ":frowning2:": "☹️",
    ":white_frowning_face:": "☹️",
    ":fuelpump:": "⛽",
    ":full_moon:": "🌕",
    ":full_moon_with_face:": "🌝",
    ":game_die:": "🎲",
    ":garlic:": "🧄",
    ":gear:": "⚙️",
    ":gem:": "💎",
    ":gemini:": "♊",
    ":genie:": "🧞",
    ":ghost:": "👻",
    ":gift:": "🎁",
    ":gift_heart:": "💝",
    ":giraffe:": "🦒",
    ":girl:": "👧",
    ":girl_tone1:": "👧🏻",
    ":girl::skin-tone-1:": "👧🏻",
    ":girl_tone2:": "👧🏼",
    ":girl::skin-tone-2:": "👧🏼",
    ":girl_tone3:": "👧🏽",
    ":girl::skin-tone-3:": "👧🏽",
    ":girl_tone4:": "👧🏾",
    ":girl::skin-tone-4:": "👧🏾",
    ":girl_tone5:": "👧🏿",
    ":girl::skin-tone-5:": "👧🏿",
    ":globe_with_meridians:": "🌐",
    ":gloves:": "🧤",
    ":goal:": "🥅",
    ":goal_net:": "🥅",
    ":goat:": "🐐",
    ":goggles:": "🥽",
    ":golf:": "⛳",
    ":gorilla:": "🦍",
    ":grapes:": "🍇",
    ":green_apple:": "🍏",
    ":green_book:": "📗",
    ":green_circle:": "🟢",
    ":green_heart:": "💚",
    ":green_square:": "🟩",
    ":grey_exclamation:": "❕",
    ":grey_question:": "❔",
    ":grimacing:": "😬",
    ":grin:": "😁",
    ":grinning:": "😀",
    ":guard:": "💂",
    ":guardsman:": "💂",
    ":guard_tone1:": "💂🏻",
    ":guardsman_tone1:": "💂🏻",
    ":guard::skin-tone-1:": "💂🏻",
    ":guardsman::skin-tone-1:": "💂🏻",
    ":guard_tone2:": "💂🏼",
    ":guardsman_tone2:": "💂🏼",
    ":guard::skin-tone-2:": "💂🏼",
    ":guardsman::skin-tone-2:": "💂🏼",
    ":guard_tone3:": "💂🏽",
    ":guardsman_tone3:": "💂🏽",
    ":guard::skin-tone-3:": "💂🏽",
    ":guardsman::skin-tone-3:": "💂🏽",
    ":guard_tone4:": "💂🏾",
    ":guardsman_tone4:": "💂🏾",
    ":guard::skin-tone-4:": "💂🏾",
    ":guardsman::skin-tone-4:": "💂🏾",
    ":guard_tone5:": "💂🏿",
    ":guardsman_tone5:": "💂🏿",
    ":guard::skin-tone-5:": "💂🏿",
    ":guardsman::skin-tone-5:": "💂🏿",
    ":guide_dog:": "🦮",
    ":guitar:": "🎸",
    ":gun:": "🔫",
    ":hamburger:": "🍔",
    ":hammer:": "🔨",
    ":hammer_pick:": "⚒️",
    ":hammer_and_pick:": "⚒️",
    ":hamster:": "🐹",
    ":hand_splayed:": "🖐️",
    ":raised_hand_with_fingers_splayed:": "🖐️",
    ":hand_splayed_tone1:": "🖐🏻",
    ":raised_hand_with_fingers_splayed_tone1:": "🖐🏻",
    ":hand_splayed::skin-tone-1:": "🖐🏻",
    ":raised_hand_with_fingers_splayed::skin-tone-1:": "🖐🏻",
    ":hand_splayed_tone2:": "🖐🏼",
    ":raised_hand_with_fingers_splayed_tone2:": "🖐🏼",
    ":hand_splayed::skin-tone-2:": "🖐🏼",
    ":raised_hand_with_fingers_splayed::skin-tone-2:": "🖐🏼",
    ":hand_splayed_tone3:": "🖐🏽",
    ":raised_hand_with_fingers_splayed_tone3:": "🖐🏽",
    ":hand_splayed::skin-tone-3:": "🖐🏽",
    ":raised_hand_with_fingers_splayed::skin-tone-3:": "🖐🏽",
    ":hand_splayed_tone4:": "🖐🏾",
    ":raised_hand_with_fingers_splayed_tone4:": "🖐🏾",
    ":hand_splayed::skin-tone-4:": "🖐🏾",
    ":raised_hand_with_fingers_splayed::skin-tone-4:": "🖐🏾",
    ":hand_splayed_tone5:": "🖐🏿",
    ":raised_hand_with_fingers_splayed_tone5:": "🖐🏿",
    ":hand_splayed::skin-tone-5:": "🖐🏿",
    ":raised_hand_with_fingers_splayed::skin-tone-5:": "🖐🏿",
    ":handbag:": "👜",
    ":handshake:": "🤝",
    ":shaking_hands:": "🤝",
    ":hash:": "#️⃣",
    ":hatched_chick:": "🐥",
    ":hatching_chick:": "🐣",
    ":head_bandage:": "🤕",
    ":face_with_head_bandage:": "🤕",
    ":headphones:": "🎧",
    ":headstone:": "🪦",
    ":health_worker:": "🧑‍⚕️",
    ":health_worker_tone1:": "🧑🏻‍⚕️",
    ":health_worker_light_skin_tone:": "🧑🏻‍⚕️",
    ":health_worker::skin-tone-1:": "🧑🏻‍⚕️",
    ":health_worker_tone2:": "🧑🏼‍⚕️",
    ":health_worker_medium_light_skin_tone:": "🧑🏼‍⚕️",
    ":health_worker::skin-tone-2:": "🧑🏼‍⚕️",
    ":health_worker_tone3:": "🧑🏽‍⚕️",
    ":health_worker_medium_skin_tone:": "🧑🏽‍⚕️",
    ":health_worker::skin-tone-3:": "🧑🏽‍⚕️",
    ":health_worker_tone4:": "🧑🏾‍⚕️",
    ":health_worker_medium_dark_skin_tone:": "🧑🏾‍⚕️",
    ":health_worker::skin-tone-4:": "🧑🏾‍⚕️",
    ":health_worker_tone5:": "🧑🏿‍⚕️",
    ":health_worker_dark_skin_tone:": "🧑🏿‍⚕️",
    ":health_worker::skin-tone-5:": "🧑🏿‍⚕️",
    ":hear_no_evil:": "🙉",
    ":heart:": "❤️",
    "<3": "❤️",
    "♡": "❤️",
    ":heart_decoration:": "💟",
    ":heart_exclamation:": "❣️",
    ":heavy_heart_exclamation_mark_ornament:": "❣️",
    ":heart_eyes:": "😍",
    ":heart_eyes_cat:": "😻",
    ":heartbeat:": "💓",
    ":heartpulse:": "💗",
    ":hearts:": "♥️",
    ":heavy_check_mark:": "✔️",
    ":heavy_division_sign:": "➗",
    ":heavy_dollar_sign:": "💲",
    ":heavy_minus_sign:": "➖",
    ":heavy_multiplication_x:": "✖️",
    ":heavy_plus_sign:": "➕",
    ":hedgehog:": "🦔",
    ":helicopter:": "🚁",
    ":helmet_with_cross:": "⛑️",
    ":helmet_with_white_cross:": "⛑️",
    ":herb:": "🌿",
    ":hibiscus:": "🌺",
    ":high_brightness:": "🔆",
    ":high_heel:": "👠",
    ":hiking_boot:": "🥾",
    ":hindu_temple:": "🛕",
    ":hippopotamus:": "🦛",
    ":hockey:": "🏒",
    ":hole:": "🕳️",
    ":homes:": "🏘️",
    ":house_buildings:": "🏘️",
    ":honey_pot:": "🍯",
    ":hook:": "🪝",
    ":horse:": "🐴",
    ":horse_racing:": "🏇",
    ":horse_racing_tone1:": "🏇🏻",
    ":horse_racing::skin-tone-1:": "🏇🏻",
    ":horse_racing_tone2:": "🏇🏼",
    ":horse_racing::skin-tone-2:": "🏇🏼",
    ":horse_racing_tone3:": "🏇🏽",
    ":horse_racing::skin-tone-3:": "🏇🏽",
    ":horse_racing_tone4:": "🏇🏾",
    ":horse_racing::skin-tone-4:": "🏇🏾",
    ":horse_racing_tone5:": "🏇🏿",
    ":horse_racing::skin-tone-5:": "🏇🏿",
    ":hospital:": "🏥",
    ":hot_face:": "🥵",
    ":hot_pepper:": "🌶️",
    ":hotdog:": "🌭",
    ":hot_dog:": "🌭",
    ":hotel:": "🏨",
    ":hotsprings:": "♨️",
    ":hourglass:": "⌛",
    ":hourglass_flowing_sand:": "⏳",
    ":house:": "🏠",
    ":house_abandoned:": "🏚️",
    ":derelict_house_building:": "🏚️",
    ":house_with_garden:": "🏡",
    ":hugging:": "🤗",
    ":hugging_face:": "🤗",
    ":hushed:": "😯",
    ":hut:": "🛖",
    ":ice_cream:": "🍨",
    ":ice_cube:": "🧊",
    ":ice_skate:": "⛸️",
    ":icecream:": "🍦",
    ":id:": "🆔",
    ":ideograph_advantage:": "🉐",
    ":imp:": "👿",
    ":inbox_tray:": "📥",
    ":incoming_envelope:": "📨",
    ":infinity:": "♾️",
    ":information_source:": "ℹ️",
    ":innocent:": "😇",
    "o:)": "😇",
    "O:)": "😇",
    "o:-)": "😇",
    "O:-)": "😇",
    "0:)": "😇",
    "0:-)": "😇",
    "o)": "😇",
    "O)": "😇",
    "o-)": "😇",
    "O-)": "😇",
    "0)": "😇",
    "0-)": "😇",
    ":interrobang:": "⁉️",
    ":island:": "🏝️",
    ":desert_island:": "🏝️",
    ":izakaya_lantern:": "🏮",
    ":jack_o_lantern:": "🎃",
    ":japan:": "🗾",
    ":japanese_castle:": "🏯",
    ":japanese_goblin:": "👺",
    ":japanese_ogre:": "👹",
    ":jeans:": "👖",
    ":jigsaw:": "🧩",
    ":joy:": "😂",
    ":')": "😂",
    ":'-)": "😂",
    ":)": "😅",
    ":-)": "😅",
    ":'D": "😂",
    ":'-D": "😂",
    ":D": "😄",
    ":-D": "😄",
    "')": "😂",
    "'-)": "😂",
    ")": "😅",
    "-)": "😅",
    "'D": "😂",
    "'-D": "😂",
    "D": "😄",
    "-D": "😄",
    ":joy_cat:": "😹",
    ":joystick:": "🕹️",
    ":judge:": "🧑‍⚖️",
    ":judge_tone1:": "🧑🏻‍⚖️",
    ":judge_light_skin_tone:": "🧑🏻‍⚖️",
    ":judge::skin-tone-1:": "🧑🏻‍⚖️",
    ":judge_tone2:": "🧑🏼‍⚖️",
    ":judge_medium_light_skin_tone:": "🧑🏼‍⚖️",
    ":judge::skin-tone-2:": "🧑🏼‍⚖️",
    ":judge_tone3:": "🧑🏽‍⚖️",
    ":judge_medium_skin_tone:": "🧑🏽‍⚖️",
    ":judge::skin-tone-3:": "🧑🏽‍⚖️",
    ":judge_tone4:": "🧑🏾‍⚖️",
    ":judge_medium_dark_skin_tone:": "🧑🏾‍⚖️",
    ":judge::skin-tone-4:": "🧑🏾‍⚖️",
    ":judge_tone5:": "🧑🏿‍⚖️",
    ":judge_dark_skin_tone:": "🧑🏿‍⚖️",
    ":judge::skin-tone-5:": "🧑🏿‍⚖️",
    ":kaaba:": "🕋",
    ":kangaroo:": "🦘",
    ":key:": "🔑",
    ":key2:": "🗝️",
    ":old_key:": "🗝️",
    ":keyboard:": "⌨️",
    ":keycap_ten:": "🔟",
    ":kimono:": "👘",
    ":kiss:": "💋",
    ":kiss_mm:": "👨‍❤️‍💋‍👨",
    ":couplekiss_mm:": "👨‍❤️‍💋‍👨",
    ":kiss_woman_man:": "👩‍❤️‍💋‍👨",
    ":kiss_ww:": "👩‍❤️‍💋‍👩",
    ":couplekiss_ww:": "👩‍❤️‍💋‍👩",
    ":kissing:": "😗",
    ":*": "😗",
    ":-*": "😗",
    "*": "😗",
    "-*": "😗",
    ":kissing_cat:": "😽",
    ":kissing_closed_eyes:": "😚",
    ":kissing_heart:": "😘",
    ":kissing_smiling_eyes:": "😙",
    ":kite:": "🪁",
    ":kiwi:": "🥝",
    ":kiwifruit:": "🥝",
    ":knife:": "🔪",
    ":knot:": "🪢",
    ":koala:": "🐨",
    ":koko:": "🈁",
    ":lab_coat:": "🥼",
    ":label:": "🏷️",
    ":lacrosse:": "🥍",
    ":ladder:": "🪜",
    ":lady_beetle:": "🐞",
    ":large_blue_diamond:": "🔷",
    ":large_orange_diamond:": "🔶",
    ":last_quarter_moon:": "🌗",
    ":last_quarter_moon_with_face:": "🌜",
    ":laughing:": "😆",
    ":satisfied:": "😆",
    "x-)": "😆",
    "X-)": "😆",
    ":leafy_green:": "🥬",
    ":leaves:": "🍃",
    ":ledger:": "📒",
    ":left_facing_fist:": "🤛",
    ":left_fist:": "🤛",
    ":left_facing_fist_tone1:": "🤛🏻",
    ":left_fist_tone1:": "🤛🏻",
    ":left_facing_fist::skin-tone-1:": "🤛🏻",
    ":left_fist::skin-tone-1:": "🤛🏻",
    ":left_facing_fist_tone2:": "🤛🏼",
    ":left_fist_tone2:": "🤛🏼",
    ":left_facing_fist::skin-tone-2:": "🤛🏼",
    ":left_fist::skin-tone-2:": "🤛🏼",
    ":left_facing_fist_tone3:": "🤛🏽",
    ":left_fist_tone3:": "🤛🏽",
    ":left_facing_fist::skin-tone-3:": "🤛🏽",
    ":left_fist::skin-tone-3:": "🤛🏽",
    ":left_facing_fist_tone4:": "🤛🏾",
    ":left_fist_tone4:": "🤛🏾",
    ":left_facing_fist::skin-tone-4:": "🤛🏾",
    ":left_fist::skin-tone-4:": "🤛🏾",
    ":left_facing_fist_tone5:": "🤛🏿",
    ":left_fist_tone5:": "🤛🏿",
    ":left_facing_fist::skin-tone-5:": "🤛🏿",
    ":left_fist::skin-tone-5:": "🤛🏿",
    ":left_luggage:": "🛅",
    ":left_right_arrow:": "↔️",
    ":leftwards_arrow_with_hook:": "↩️",
    ":leg:": "🦵",
    ":leg_tone1:": "🦵🏻",
    ":leg_light_skin_tone:": "🦵🏻",
    ":leg::skin-tone-1:": "🦵🏻",
    ":leg_tone2:": "🦵🏼",
    ":leg_medium_light_skin_tone:": "🦵🏼",
    ":leg::skin-tone-2:": "🦵🏼",
    ":leg_tone3:": "🦵🏽",
    ":leg_medium_skin_tone:": "🦵🏽",
    ":leg::skin-tone-3:": "🦵🏽",
    ":leg_tone4:": "🦵🏾",
    ":leg_medium_dark_skin_tone:": "🦵🏾",
    ":leg::skin-tone-4:": "🦵🏾",
    ":leg_tone5:": "🦵🏿",
    ":leg_dark_skin_tone:": "🦵🏿",
    ":leg::skin-tone-5:": "🦵🏿",
    ":lemon:": "🍋",
    ":leo:": "♌",
    ":leopard:": "🐆",
    ":level_slider:": "🎚️",
    ":levitate:": "🕴️",
    ":man_in_business_suit_levitating:": "🕴️",
    ":levitate_tone1:": "🕴🏻",
    ":man_in_business_suit_levitating_tone1:": "🕴🏻",
    ":man_in_business_suit_levitating_light_skin_tone:": "🕴🏻",
    ":levitate::skin-tone-1:": "🕴🏻",
    ":man_in_business_suit_levitating::skin-tone-1:": "🕴🏻",
    ":levitate_tone2:": "🕴🏼",
    ":man_in_business_suit_levitating_tone2:": "🕴🏼",
    ":man_in_business_suit_levitating_medium_light_skin_tone:": "🕴🏼",
    ":levitate::skin-tone-2:": "🕴🏼",
    ":man_in_business_suit_levitating::skin-tone-2:": "🕴🏼",
    ":levitate_tone3:": "🕴🏽",
    ":man_in_business_suit_levitating_tone3:": "🕴🏽",
    ":man_in_business_suit_levitating_medium_skin_tone:": "🕴🏽",
    ":levitate::skin-tone-3:": "🕴🏽",
    ":man_in_business_suit_levitating::skin-tone-3:": "🕴🏽",
    ":levitate_tone4:": "🕴🏾",
    ":man_in_business_suit_levitating_tone4:": "🕴🏾",
    ":man_in_business_suit_levitating_medium_dark_skin_tone:": "🕴🏾",
    ":levitate::skin-tone-4:": "🕴🏾",
    ":man_in_business_suit_levitating::skin-tone-4:": "🕴🏾",
    ":levitate_tone5:": "🕴🏿",
    ":man_in_business_suit_levitating_tone5:": "🕴🏿",
    ":man_in_business_suit_levitating_dark_skin_tone:": "🕴🏿",
    ":levitate::skin-tone-5:": "🕴🏿",
    ":man_in_business_suit_levitating::skin-tone-5:": "🕴🏿",
    ":libra:": "♎",
    ":light_rail:": "🚈",
    ":link:": "🔗",
    ":lion_face:": "🦁",
    ":lion:": "🦁",
    ":lips:": "👄",
    ":lipstick:": "💄",
    ":lizard:": "🦎",
    ":llama:": "🦙",
    ":lobster:": "🦞",
    ":lock:": "🔒",
    ":lock_with_ink_pen:": "🔏",
    ":lollipop:": "🍭",
    ":long_drum:": "🪘",
    ":loop:": "➿",
    ":loud_sound:": "🔊",
    ":loudspeaker:": "📢",
    ":love_hotel:": "🏩",
    ":love_letter:": "💌",
    ":love_you_gesture:": "🤟",
    ":love_you_gesture_tone1:": "🤟🏻",
    ":love_you_gesture_light_skin_tone:": "🤟🏻",
    ":love_you_gesture::skin-tone-1:": "🤟🏻",
    ":love_you_gesture_tone2:": "🤟🏼",
    ":love_you_gesture_medium_light_skin_tone:": "🤟🏼",
    ":love_you_gesture::skin-tone-2:": "🤟🏼",
    ":love_you_gesture_tone3:": "🤟🏽",
    ":love_you_gesture_medium_skin_tone:": "🤟🏽",
    ":love_you_gesture::skin-tone-3:": "🤟🏽",
    ":love_you_gesture_tone4:": "🤟🏾",
    ":love_you_gesture_medium_dark_skin_tone:": "🤟🏾",
    ":love_you_gesture::skin-tone-4:": "🤟🏾",
    ":love_you_gesture_tone5:": "🤟🏿",
    ":love_you_gesture_dark_skin_tone:": "🤟🏿",
    ":love_you_gesture::skin-tone-5:": "🤟🏿",
    ":low_brightness:": "🔅",
    ":luggage:": "🧳",
    ":lungs:": "🫁",
    ":lying_face:": "🤥",
    ":liar:": "🤥",
    ":m:": "Ⓜ️",
    ":mag:": "🔍",
    ":mag_right:": "🔎",
    ":mage:": "🧙",
    ":mage_tone1:": "🧙🏻",
    ":mage_light_skin_tone:": "🧙🏻",
    ":mage::skin-tone-1:": "🧙🏻",
    ":mage_tone2:": "🧙🏼",
    ":mage_medium_light_skin_tone:": "🧙🏼",
    ":mage::skin-tone-2:": "🧙🏼",
    ":mage_tone3:": "🧙🏽",
    ":mage_medium_skin_tone:": "🧙🏽",
    ":mage::skin-tone-3:": "🧙🏽",
    ":mage_tone4:": "🧙🏾",
    ":mage_medium_dark_skin_tone:": "🧙🏾",
    ":mage::skin-tone-4:": "🧙🏾",
    ":mage_tone5:": "🧙🏿",
    ":mage_dark_skin_tone:": "🧙🏿",
    ":mage::skin-tone-5:": "🧙🏿",
    ":magic_wand:": "🪄",
    ":magnet:": "🧲",
    ":mahjong:": "🀄",
    ":mailbox:": "📫",
    ":mailbox_closed:": "📪",
    ":mailbox_with_mail:": "📬",
    ":mailbox_with_no_mail:": "📭",
    ":male_sign:": "♂️",
    ":mammoth:": "🦣",
    ":man:": "👨",
    ":man_artist:": "👨‍🎨",
    ":man_artist_tone1:": "👨🏻‍🎨",
    ":man_artist_light_skin_tone:": "👨🏻‍🎨",
    ":man_artist::skin-tone-1:": "👨🏻‍🎨",
    ":man_artist_tone2:": "👨🏼‍🎨",
    ":man_artist_medium_light_skin_tone:": "👨🏼‍🎨",
    ":man_artist::skin-tone-2:": "👨🏼‍🎨",
    ":man_artist_tone3:": "👨🏽‍🎨",
    ":man_artist_medium_skin_tone:": "👨🏽‍🎨",
    ":man_artist::skin-tone-3:": "👨🏽‍🎨",
    ":man_artist_tone4:": "👨🏾‍🎨",
    ":man_artist_medium_dark_skin_tone:": "👨🏾‍🎨",
    ":man_artist::skin-tone-4:": "👨🏾‍🎨",
    ":man_artist_tone5:": "👨🏿‍🎨",
    ":man_artist_dark_skin_tone:": "👨🏿‍🎨",
    ":man_artist::skin-tone-5:": "👨🏿‍🎨",
    ":man_astronaut:": "👨‍🚀",
    ":man_astronaut_tone1:": "👨🏻‍🚀",
    ":man_astronaut_light_skin_tone:": "👨🏻‍🚀",
    ":man_astronaut::skin-tone-1:": "👨🏻‍🚀",
    ":man_astronaut_tone2:": "👨🏼‍🚀",
    ":man_astronaut_medium_light_skin_tone:": "👨🏼‍🚀",
    ":man_astronaut::skin-tone-2:": "👨🏼‍🚀",
    ":man_astronaut_tone3:": "👨🏽‍🚀",
    ":man_astronaut_medium_skin_tone:": "👨🏽‍🚀",
    ":man_astronaut::skin-tone-3:": "👨🏽‍🚀",
    ":man_astronaut_tone4:": "👨🏾‍🚀",
    ":man_astronaut_medium_dark_skin_tone:": "👨🏾‍🚀",
    ":man_astronaut::skin-tone-4:": "👨🏾‍🚀",
    ":man_astronaut_tone5:": "👨🏿‍🚀",
    ":man_astronaut_dark_skin_tone:": "👨🏿‍🚀",
    ":man_astronaut::skin-tone-5:": "👨🏿‍🚀",
    ":man_bald:": "👨‍🦲",
    ":man_bald_tone1:": "👨🏻‍🦲",
    ":man_bald_light_skin_tone:": "👨🏻‍🦲",
    ":man_bald::skin-tone-1:": "👨🏻‍🦲",
    ":man_bald_tone2:": "👨🏼‍🦲",
    ":man_bald_medium_light_skin_tone:": "👨🏼‍🦲",
    ":man_bald::skin-tone-2:": "👨🏼‍🦲",
    ":man_bald_tone3:": "👨🏽‍🦲",
    ":man_bald_medium_skin_tone:": "👨🏽‍🦲",
    ":man_bald::skin-tone-3:": "👨🏽‍🦲",
    ":man_bald_tone4:": "👨🏾‍🦲",
    ":man_bald_medium_dark_skin_tone:": "👨🏾‍🦲",
    ":man_bald::skin-tone-4:": "👨🏾‍🦲",
    ":man_bald_tone5:": "👨🏿‍🦲",
    ":man_bald_dark_skin_tone:": "👨🏿‍🦲",
    ":man_bald::skin-tone-5:": "👨🏿‍🦲",
    ":man_biking:": "🚴‍♂️",
    ":man_biking_tone1:": "🚴🏻‍♂️",
    ":man_biking_light_skin_tone:": "🚴🏻‍♂️",
    ":man_biking::skin-tone-1:": "🚴🏻‍♂️",
    ":man_biking_tone2:": "🚴🏼‍♂️",
    ":man_biking_medium_light_skin_tone:": "🚴🏼‍♂️",
    ":man_biking::skin-tone-2:": "🚴🏼‍♂️",
    ":man_biking_tone3:": "🚴🏽‍♂️",
    ":man_biking_medium_skin_tone:": "🚴🏽‍♂️",
    ":man_biking::skin-tone-3:": "🚴🏽‍♂️",
    ":man_biking_tone4:": "🚴🏾‍♂️",
    ":man_biking_medium_dark_skin_tone:": "🚴🏾‍♂️",
    ":man_biking::skin-tone-4:": "🚴🏾‍♂️",
    ":man_biking_tone5:": "🚴🏿‍♂️",
    ":man_biking_dark_skin_tone:": "🚴🏿‍♂️",
    ":man_biking::skin-tone-5:": "🚴🏿‍♂️",
    ":man_bouncing_ball:": "⛹️‍♂️",
    ":man_bouncing_ball_tone1:": "⛹🏻‍♂️",
    ":man_bouncing_ball_light_skin_tone:": "⛹🏻‍♂️",
    ":man_bouncing_ball::skin-tone-1:": "⛹🏻‍♂️",
    ":man_bouncing_ball_tone2:": "⛹🏼‍♂️",
    ":man_bouncing_ball_medium_light_skin_tone:": "⛹🏼‍♂️",
    ":man_bouncing_ball::skin-tone-2:": "⛹🏼‍♂️",
    ":man_bouncing_ball_tone3:": "⛹🏽‍♂️",
    ":man_bouncing_ball_medium_skin_tone:": "⛹🏽‍♂️",
    ":man_bouncing_ball::skin-tone-3:": "⛹🏽‍♂️",
    ":man_bouncing_ball_tone4:": "⛹🏾‍♂️",
    ":man_bouncing_ball_medium_dark_skin_tone:": "⛹🏾‍♂️",
    ":man_bouncing_ball::skin-tone-4:": "⛹🏾‍♂️",
    ":man_bouncing_ball_tone5:": "⛹🏿‍♂️",
    ":man_bouncing_ball_dark_skin_tone:": "⛹🏿‍♂️",
    ":man_bouncing_ball::skin-tone-5:": "⛹🏿‍♂️",
    ":man_bowing:": "🙇‍♂️",
    ":man_bowing_tone1:": "🙇🏻‍♂️",
    ":man_bowing_light_skin_tone:": "🙇🏻‍♂️",
    ":man_bowing::skin-tone-1:": "🙇🏻‍♂️",
    ":man_bowing_tone2:": "🙇🏼‍♂️",
    ":man_bowing_medium_light_skin_tone:": "🙇🏼‍♂️",
    ":man_bowing::skin-tone-2:": "🙇🏼‍♂️",
    ":man_bowing_tone3:": "🙇🏽‍♂️",
    ":man_bowing_medium_skin_tone:": "🙇🏽‍♂️",
    ":man_bowing::skin-tone-3:": "🙇🏽‍♂️",
    ":man_bowing_tone4:": "🙇🏾‍♂️",
    ":man_bowing_medium_dark_skin_tone:": "🙇🏾‍♂️",
    ":man_bowing::skin-tone-4:": "🙇🏾‍♂️",
    ":man_bowing_tone5:": "🙇🏿‍♂️",
    ":man_bowing_dark_skin_tone:": "🙇🏿‍♂️",
    ":man_bowing::skin-tone-5:": "🙇🏿‍♂️",
    ":man_cartwheeling:": "🤸‍♂️",
    ":man_cartwheeling_tone1:": "🤸🏻‍♂️",
    ":man_cartwheeling_light_skin_tone:": "🤸🏻‍♂️",
    ":man_cartwheeling::skin-tone-1:": "🤸🏻‍♂️",
    ":man_cartwheeling_tone2:": "🤸🏼‍♂️",
    ":man_cartwheeling_medium_light_skin_tone:": "🤸🏼‍♂️",
    ":man_cartwheeling::skin-tone-2:": "🤸🏼‍♂️",
    ":man_cartwheeling_tone3:": "🤸🏽‍♂️",
    ":man_cartwheeling_medium_skin_tone:": "🤸🏽‍♂️",
    ":man_cartwheeling::skin-tone-3:": "🤸🏽‍♂️",
    ":man_cartwheeling_tone4:": "🤸🏾‍♂️",
    ":man_cartwheeling_medium_dark_skin_tone:": "🤸🏾‍♂️",
    ":man_cartwheeling::skin-tone-4:": "🤸🏾‍♂️",
    ":man_cartwheeling_tone5:": "🤸🏿‍♂️",
    ":man_cartwheeling_dark_skin_tone:": "🤸🏿‍♂️",
    ":man_cartwheeling::skin-tone-5:": "🤸🏿‍♂️",
    ":man_climbing:": "🧗‍♂️",
    ":man_climbing_tone1:": "🧗🏻‍♂️",
    ":man_climbing_light_skin_tone:": "🧗🏻‍♂️",
    ":man_climbing::skin-tone-1:": "🧗🏻‍♂️",
    ":man_climbing_tone2:": "🧗🏼‍♂️",
    ":man_climbing_medium_light_skin_tone:": "🧗🏼‍♂️",
    ":man_climbing::skin-tone-2:": "🧗🏼‍♂️",
    ":man_climbing_tone3:": "🧗🏽‍♂️",
    ":man_climbing_medium_skin_tone:": "🧗🏽‍♂️",
    ":man_climbing::skin-tone-3:": "🧗🏽‍♂️",
    ":man_climbing_tone4:": "🧗🏾‍♂️",
    ":man_climbing_medium_dark_skin_tone:": "🧗🏾‍♂️",
    ":man_climbing::skin-tone-4:": "🧗🏾‍♂️",
    ":man_climbing_tone5:": "🧗🏿‍♂️",
    ":man_climbing_dark_skin_tone:": "🧗🏿‍♂️",
    ":man_climbing::skin-tone-5:": "🧗🏿‍♂️",
    ":man_construction_worker:": "👷‍♂️",
    ":man_construction_worker_tone1:": "👷🏻‍♂️",
    ":man_construction_worker_light_skin_tone:": "👷🏻‍♂️",
    ":man_construction_worker::skin-tone-1:": "👷🏻‍♂️",
    ":man_construction_worker_tone2:": "👷🏼‍♂️",
    ":man_construction_worker_medium_light_skin_tone:": "👷🏼‍♂️",
    ":man_construction_worker::skin-tone-2:": "👷🏼‍♂️",
    ":man_construction_worker_tone3:": "👷🏽‍♂️",
    ":man_construction_worker_medium_skin_tone:": "👷🏽‍♂️",
    ":man_construction_worker::skin-tone-3:": "👷🏽‍♂️",
    ":man_construction_worker_tone4:": "👷🏾‍♂️",
    ":man_construction_worker_medium_dark_skin_tone:": "👷🏾‍♂️",
    ":man_construction_worker::skin-tone-4:": "👷🏾‍♂️",
    ":man_construction_worker_tone5:": "👷🏿‍♂️",
    ":man_construction_worker_dark_skin_tone:": "👷🏿‍♂️",
    ":man_construction_worker::skin-tone-5:": "👷🏿‍♂️",
    ":man_cook:": "👨‍🍳",
    ":man_cook_tone1:": "👨🏻‍🍳",
    ":man_cook_light_skin_tone:": "👨🏻‍🍳",
    ":man_cook::skin-tone-1:": "👨🏻‍🍳",
    ":man_cook_tone2:": "👨🏼‍🍳",
    ":man_cook_medium_light_skin_tone:": "👨🏼‍🍳",
    ":man_cook::skin-tone-2:": "👨🏼‍🍳",
    ":man_cook_tone3:": "👨🏽‍🍳",
    ":man_cook_medium_skin_tone:": "👨🏽‍🍳",
    ":man_cook::skin-tone-3:": "👨🏽‍🍳",
    ":man_cook_tone4:": "👨🏾‍🍳",
    ":man_cook_medium_dark_skin_tone:": "👨🏾‍🍳",
    ":man_cook::skin-tone-4:": "👨🏾‍🍳",
    ":man_cook_tone5:": "👨🏿‍🍳",
    ":man_cook_dark_skin_tone:": "👨🏿‍🍳",
    ":man_cook::skin-tone-5:": "👨🏿‍🍳",
    ":man_curly_haired:": "👨‍🦱",
    ":man_curly_haired_tone1:": "👨🏻‍🦱",
    ":man_curly_haired_light_skin_tone:": "👨🏻‍🦱",
    ":man_curly_haired::skin-tone-1:": "👨🏻‍🦱",
    ":man_curly_haired_tone2:": "👨🏼‍🦱",
    ":man_curly_haired_medium_light_skin_tone:": "👨🏼‍🦱",
    ":man_curly_haired::skin-tone-2:": "👨🏼‍🦱",
    ":man_curly_haired_tone3:": "👨🏽‍🦱",
    ":man_curly_haired_medium_skin_tone:": "👨🏽‍🦱",
    ":man_curly_haired::skin-tone-3:": "👨🏽‍🦱",
    ":man_curly_haired_tone4:": "👨🏾‍🦱",
    ":man_curly_haired_medium_dark_skin_tone:": "👨🏾‍🦱",
    ":man_curly_haired::skin-tone-4:": "👨🏾‍🦱",
    ":man_curly_haired_tone5:": "👨🏿‍🦱",
    ":man_curly_haired_dark_skin_tone:": "👨🏿‍🦱",
    ":man_curly_haired::skin-tone-5:": "👨🏿‍🦱",
    ":man_dancing:": "🕺",
    ":male_dancer:": "🕺",
    ":man_dancing_tone1:": "🕺🏻",
    ":male_dancer_tone1:": "🕺🏻",
    ":man_dancing::skin-tone-1:": "🕺🏻",
    ":male_dancer::skin-tone-1:": "🕺🏻",
    ":man_dancing_tone2:": "🕺🏼",
    ":male_dancer_tone2:": "🕺🏼",
    ":man_dancing::skin-tone-2:": "🕺🏼",
    ":male_dancer::skin-tone-2:": "🕺🏼",
    ":man_dancing_tone3:": "🕺🏽",
    ":male_dancer_tone3:": "🕺🏽",
    ":man_dancing::skin-tone-3:": "🕺🏽",
    ":male_dancer::skin-tone-3:": "🕺🏽",
    ":man_dancing_tone4:": "🕺🏾",
    ":male_dancer_tone4:": "🕺🏾",
    ":man_dancing::skin-tone-4:": "🕺🏾",
    ":male_dancer::skin-tone-4:": "🕺🏾",
    ":man_dancing_tone5:": "🕺🏿",
    ":male_dancer_tone5:": "🕺🏿",
    ":man_dancing::skin-tone-5:": "🕺🏿",
    ":male_dancer::skin-tone-5:": "🕺🏿",
    ":man_detective:": "🕵️‍♂️",
    ":man_detective_tone1:": "🕵🏻‍♂️",
    ":man_detective_light_skin_tone:": "🕵🏻‍♂️",
    ":man_detective::skin-tone-1:": "🕵🏻‍♂️",
    ":man_detective_tone2:": "🕵🏼‍♂️",
    ":man_detective_medium_light_skin_tone:": "🕵🏼‍♂️",
    ":man_detective::skin-tone-2:": "🕵🏼‍♂️",
    ":man_detective_tone3:": "🕵🏽‍♂️",
    ":man_detective_medium_skin_tone:": "🕵🏽‍♂️",
    ":man_detective::skin-tone-3:": "🕵🏽‍♂️",
    ":man_detective_tone4:": "🕵🏾‍♂️",
    ":man_detective_medium_dark_skin_tone:": "🕵🏾‍♂️",
    ":man_detective::skin-tone-4:": "🕵🏾‍♂️",
    ":man_detective_tone5:": "🕵🏿‍♂️",
    ":man_detective_dark_skin_tone:": "🕵🏿‍♂️",
    ":man_detective::skin-tone-5:": "🕵🏿‍♂️",
    ":man_elf:": "🧝‍♂️",
    ":man_elf_tone1:": "🧝🏻‍♂️",
    ":man_elf_light_skin_tone:": "🧝🏻‍♂️",
    ":man_elf::skin-tone-1:": "🧝🏻‍♂️",
    ":man_elf_tone2:": "🧝🏼‍♂️",
    ":man_elf_medium_light_skin_tone:": "🧝🏼‍♂️",
    ":man_elf::skin-tone-2:": "🧝🏼‍♂️",
    ":man_elf_tone3:": "🧝🏽‍♂️",
    ":man_elf_medium_skin_tone:": "🧝🏽‍♂️",
    ":man_elf::skin-tone-3:": "🧝🏽‍♂️",
    ":man_elf_tone4:": "🧝🏾‍♂️",
    ":man_elf_medium_dark_skin_tone:": "🧝🏾‍♂️",
    ":man_elf::skin-tone-4:": "🧝🏾‍♂️",
    ":man_elf_tone5:": "🧝🏿‍♂️",
    ":man_elf_dark_skin_tone:": "🧝🏿‍♂️",
    ":man_elf::skin-tone-5:": "🧝🏿‍♂️",
    ":man_facepalming:": "🤦‍♂️",
    ":man_facepalming_tone1:": "🤦🏻‍♂️",
    ":man_facepalming_light_skin_tone:": "🤦🏻‍♂️",
    ":man_facepalming::skin-tone-1:": "🤦🏻‍♂️",
    ":man_facepalming_tone2:": "🤦🏼‍♂️",
    ":man_facepalming_medium_light_skin_tone:": "🤦🏼‍♂️",
    ":man_facepalming::skin-tone-2:": "🤦🏼‍♂️",
    ":man_facepalming_tone3:": "🤦🏽‍♂️",
    ":man_facepalming_medium_skin_tone:": "🤦🏽‍♂️",
    ":man_facepalming::skin-tone-3:": "🤦🏽‍♂️",
    ":man_facepalming_tone4:": "🤦🏾‍♂️",
    ":man_facepalming_medium_dark_skin_tone:": "🤦🏾‍♂️",
    ":man_facepalming::skin-tone-4:": "🤦🏾‍♂️",
    ":man_facepalming_tone5:": "🤦🏿‍♂️",
    ":man_facepalming_dark_skin_tone:": "🤦🏿‍♂️",
    ":man_facepalming::skin-tone-5:": "🤦🏿‍♂️",
    ":man_factory_worker:": "👨‍🏭",
    ":man_factory_worker_tone1:": "👨🏻‍🏭",
    ":man_factory_worker_light_skin_tone:": "👨🏻‍🏭",
    ":man_factory_worker::skin-tone-1:": "👨🏻‍🏭",
    ":man_factory_worker_tone2:": "👨🏼‍🏭",
    ":man_factory_worker_medium_light_skin_tone:": "👨🏼‍🏭",
    ":man_factory_worker::skin-tone-2:": "👨🏼‍🏭",
    ":man_factory_worker_tone3:": "👨🏽‍🏭",
    ":man_factory_worker_medium_skin_tone:": "👨🏽‍🏭",
    ":man_factory_worker::skin-tone-3:": "👨🏽‍🏭",
    ":man_factory_worker_tone4:": "👨🏾‍🏭",
    ":man_factory_worker_medium_dark_skin_tone:": "👨🏾‍🏭",
    ":man_factory_worker::skin-tone-4:": "👨🏾‍🏭",
    ":man_factory_worker_tone5:": "👨🏿‍🏭",
    ":man_factory_worker_dark_skin_tone:": "👨🏿‍🏭",
    ":man_factory_worker::skin-tone-5:": "👨🏿‍🏭",
    ":man_fairy:": "🧚‍♂️",
    ":man_fairy_tone1:": "🧚🏻‍♂️",
    ":man_fairy_light_skin_tone:": "🧚🏻‍♂️",
    ":man_fairy::skin-tone-1:": "🧚🏻‍♂️",
    ":man_fairy_tone2:": "🧚🏼‍♂️",
    ":man_fairy_medium_light_skin_tone:": "🧚🏼‍♂️",
    ":man_fairy::skin-tone-2:": "🧚🏼‍♂️",
    ":man_fairy_tone3:": "🧚🏽‍♂️",
    ":man_fairy_medium_skin_tone:": "🧚🏽‍♂️",
    ":man_fairy::skin-tone-3:": "🧚🏽‍♂️",
    ":man_fairy_tone4:": "🧚🏾‍♂️",
    ":man_fairy_medium_dark_skin_tone:": "🧚🏾‍♂️",
    ":man_fairy::skin-tone-4:": "🧚🏾‍♂️",
    ":man_fairy_tone5:": "🧚🏿‍♂️",
    ":man_fairy_dark_skin_tone:": "🧚🏿‍♂️",
    ":man_fairy::skin-tone-5:": "🧚🏿‍♂️",
    ":man_farmer:": "👨‍🌾",
    ":man_farmer_tone1:": "👨🏻‍🌾",
    ":man_farmer_light_skin_tone:": "👨🏻‍🌾",
    ":man_farmer::skin-tone-1:": "👨🏻‍🌾",
    ":man_farmer_tone2:": "👨🏼‍🌾",
    ":man_farmer_medium_light_skin_tone:": "👨🏼‍🌾",
    ":man_farmer::skin-tone-2:": "👨🏼‍🌾",
    ":man_farmer_tone3:": "👨🏽‍🌾",
    ":man_farmer_medium_skin_tone:": "👨🏽‍🌾",
    ":man_farmer::skin-tone-3:": "👨🏽‍🌾",
    ":man_farmer_tone4:": "👨🏾‍🌾",
    ":man_farmer_medium_dark_skin_tone:": "👨🏾‍🌾",
    ":man_farmer::skin-tone-4:": "👨🏾‍🌾",
    ":man_farmer_tone5:": "👨🏿‍🌾",
    ":man_farmer_dark_skin_tone:": "👨🏿‍🌾",
    ":man_farmer::skin-tone-5:": "👨🏿‍🌾",
    ":man_feeding_baby:": "👨‍🍼",
    ":man_feeding_baby_tone1:": "👨🏻‍🍼",
    ":man_feeding_baby_light_skin_tone:": "👨🏻‍🍼",
    ":man_feeding_baby::skin-tone-1:": "👨🏻‍🍼",
    ":man_feeding_baby_tone2:": "👨🏼‍🍼",
    ":man_feeding_baby_medium_light_skin_tone:": "👨🏼‍🍼",
    ":man_feeding_baby::skin-tone-2:": "👨🏼‍🍼",
    ":man_feeding_baby_tone3:": "👨🏽‍🍼",
    ":man_feeding_baby_medium_skin_tone:": "👨🏽‍🍼",
    ":man_feeding_baby::skin-tone-3:": "👨🏽‍🍼",
    ":man_feeding_baby_tone4:": "👨🏾‍🍼",
    ":man_feeding_baby_medium_dark_skin_tone:": "👨🏾‍🍼",
    ":man_feeding_baby::skin-tone-4:": "👨🏾‍🍼",
    ":man_feeding_baby_tone5:": "👨🏿‍🍼",
    ":man_feeding_baby_dark_skin_tone:": "👨🏿‍🍼",
    ":man_feeding_baby::skin-tone-5:": "👨🏿‍🍼",
    ":man_firefighter:": "👨‍🚒",
    ":man_firefighter_tone1:": "👨🏻‍🚒",
    ":man_firefighter_light_skin_tone:": "👨🏻‍🚒",
    ":man_firefighter::skin-tone-1:": "👨🏻‍🚒",
    ":man_firefighter_tone2:": "👨🏼‍🚒",
    ":man_firefighter_medium_light_skin_tone:": "👨🏼‍🚒",
    ":man_firefighter::skin-tone-2:": "👨🏼‍🚒",
    ":man_firefighter_tone3:": "👨🏽‍🚒",
    ":man_firefighter_medium_skin_tone:": "👨🏽‍🚒",
    ":man_firefighter::skin-tone-3:": "👨🏽‍🚒",
    ":man_firefighter_tone4:": "👨🏾‍🚒",
    ":man_firefighter_medium_dark_skin_tone:": "👨🏾‍🚒",
    ":man_firefighter::skin-tone-4:": "👨🏾‍🚒",
    ":man_firefighter_tone5:": "👨🏿‍🚒",
    ":man_firefighter_dark_skin_tone:": "👨🏿‍🚒",
    ":man_firefighter::skin-tone-5:": "👨🏿‍🚒",
    ":man_frowning:": "🙍‍♂️",
    ":man_frowning_tone1:": "🙍🏻‍♂️",
    ":man_frowning_light_skin_tone:": "🙍🏻‍♂️",
    ":man_frowning::skin-tone-1:": "🙍🏻‍♂️",
    ":man_frowning_tone2:": "🙍🏼‍♂️",
    ":man_frowning_medium_light_skin_tone:": "🙍🏼‍♂️",
    ":man_frowning::skin-tone-2:": "🙍🏼‍♂️",
    ":man_frowning_tone3:": "🙍🏽‍♂️",
    ":man_frowning_medium_skin_tone:": "🙍🏽‍♂️",
    ":man_frowning::skin-tone-3:": "🙍🏽‍♂️",
    ":man_frowning_tone4:": "🙍🏾‍♂️",
    ":man_frowning_medium_dark_skin_tone:": "🙍🏾‍♂️",
    ":man_frowning::skin-tone-4:": "🙍🏾‍♂️",
    ":man_frowning_tone5:": "🙍🏿‍♂️",
    ":man_frowning_dark_skin_tone:": "🙍🏿‍♂️",
    ":man_frowning::skin-tone-5:": "🙍🏿‍♂️",
    ":man_genie:": "🧞‍♂️",
    ":man_gesturing_no:": "🙅‍♂️",
    ":man_gesturing_no_tone1:": "🙅🏻‍♂️",
    ":man_gesturing_no_light_skin_tone:": "🙅🏻‍♂️",
    ":man_gesturing_no::skin-tone-1:": "🙅🏻‍♂️",
    ":man_gesturing_no_tone2:": "🙅🏼‍♂️",
    ":man_gesturing_no_medium_light_skin_tone:": "🙅🏼‍♂️",
    ":man_gesturing_no::skin-tone-2:": "🙅🏼‍♂️",
    ":man_gesturing_no_tone3:": "🙅🏽‍♂️",
    ":man_gesturing_no_medium_skin_tone:": "🙅🏽‍♂️",
    ":man_gesturing_no::skin-tone-3:": "🙅🏽‍♂️",
    ":man_gesturing_no_tone4:": "🙅🏾‍♂️",
    ":man_gesturing_no_medium_dark_skin_tone:": "🙅🏾‍♂️",
    ":man_gesturing_no::skin-tone-4:": "🙅🏾‍♂️",
    ":man_gesturing_no_tone5:": "🙅🏿‍♂️",
    ":man_gesturing_no_dark_skin_tone:": "🙅🏿‍♂️",
    ":man_gesturing_no::skin-tone-5:": "🙅🏿‍♂️",
    ":man_gesturing_ok:": "🙆‍♂️",
    ":man_gesturing_ok_tone1:": "🙆🏻‍♂️",
    ":man_gesturing_ok_light_skin_tone:": "🙆🏻‍♂️",
    ":man_gesturing_ok::skin-tone-1:": "🙆🏻‍♂️",
    ":man_gesturing_ok_tone2:": "🙆🏼‍♂️",
    ":man_gesturing_ok_medium_light_skin_tone:": "🙆🏼‍♂️",
    ":man_gesturing_ok::skin-tone-2:": "🙆🏼‍♂️",
    ":man_gesturing_ok_tone3:": "🙆🏽‍♂️",
    ":man_gesturing_ok_medium_skin_tone:": "🙆🏽‍♂️",
    ":man_gesturing_ok::skin-tone-3:": "🙆🏽‍♂️",
    ":man_gesturing_ok_tone4:": "🙆🏾‍♂️",
    ":man_gesturing_ok_medium_dark_skin_tone:": "🙆🏾‍♂️",
    ":man_gesturing_ok::skin-tone-4:": "🙆🏾‍♂️",
    ":man_gesturing_ok_tone5:": "🙆🏿‍♂️",
    ":man_gesturing_ok_dark_skin_tone:": "🙆🏿‍♂️",
    ":man_gesturing_ok::skin-tone-5:": "🙆🏿‍♂️",
    ":man_getting_face_massage:": "💆‍♂️",
    ":man_getting_face_massage_tone1:": "💆🏻‍♂️",
    ":man_getting_face_massage_light_skin_tone:": "💆🏻‍♂️",
    ":man_getting_face_massage::skin-tone-1:": "💆🏻‍♂️",
    ":man_getting_face_massage_tone2:": "💆🏼‍♂️",
    ":man_getting_face_massage_medium_light_skin_tone:": "💆🏼‍♂️",
    ":man_getting_face_massage::skin-tone-2:": "💆🏼‍♂️",
    ":man_getting_face_massage_tone3:": "💆🏽‍♂️",
    ":man_getting_face_massage_medium_skin_tone:": "💆🏽‍♂️",
    ":man_getting_face_massage::skin-tone-3:": "💆🏽‍♂️",
    ":man_getting_face_massage_tone4:": "💆🏾‍♂️",
    ":man_getting_face_massage_medium_dark_skin_tone:": "💆🏾‍♂️",
    ":man_getting_face_massage::skin-tone-4:": "💆🏾‍♂️",
    ":man_getting_face_massage_tone5:": "💆🏿‍♂️",
    ":man_getting_face_massage_dark_skin_tone:": "💆🏿‍♂️",
    ":man_getting_face_massage::skin-tone-5:": "💆🏿‍♂️",
    ":man_getting_haircut:": "💇‍♂️",
    ":man_getting_haircut_tone1:": "💇🏻‍♂️",
    ":man_getting_haircut_light_skin_tone:": "💇🏻‍♂️",
    ":man_getting_haircut::skin-tone-1:": "💇🏻‍♂️",
    ":man_getting_haircut_tone2:": "💇🏼‍♂️",
    ":man_getting_haircut_medium_light_skin_tone:": "💇🏼‍♂️",
    ":man_getting_haircut::skin-tone-2:": "💇🏼‍♂️",
    ":man_getting_haircut_tone3:": "💇🏽‍♂️",
    ":man_getting_haircut_medium_skin_tone:": "💇🏽‍♂️",
    ":man_getting_haircut::skin-tone-3:": "💇🏽‍♂️",
    ":man_getting_haircut_tone4:": "💇🏾‍♂️",
    ":man_getting_haircut_medium_dark_skin_tone:": "💇🏾‍♂️",
    ":man_getting_haircut::skin-tone-4:": "💇🏾‍♂️",
    ":man_getting_haircut_tone5:": "💇🏿‍♂️",
    ":man_getting_haircut_dark_skin_tone:": "💇🏿‍♂️",
    ":man_getting_haircut::skin-tone-5:": "💇🏿‍♂️",
    ":man_golfing:": "🏌️‍♂️",
    ":man_golfing_tone1:": "🏌🏻‍♂️",
    ":man_golfing_light_skin_tone:": "🏌🏻‍♂️",
    ":man_golfing::skin-tone-1:": "🏌🏻‍♂️",
    ":man_golfing_tone2:": "🏌🏼‍♂️",
    ":man_golfing_medium_light_skin_tone:": "🏌🏼‍♂️",
    ":man_golfing::skin-tone-2:": "🏌🏼‍♂️",
    ":man_golfing_tone3:": "🏌🏽‍♂️",
    ":man_golfing_medium_skin_tone:": "🏌🏽‍♂️",
    ":man_golfing::skin-tone-3:": "🏌🏽‍♂️",
    ":man_golfing_tone4:": "🏌🏾‍♂️",
    ":man_golfing_medium_dark_skin_tone:": "🏌🏾‍♂️",
    ":man_golfing::skin-tone-4:": "🏌🏾‍♂️",
    ":man_golfing_tone5:": "🏌🏿‍♂️",
    ":man_golfing_dark_skin_tone:": "🏌🏿‍♂️",
    ":man_golfing::skin-tone-5:": "🏌🏿‍♂️",
    ":man_guard:": "💂‍♂️",
    ":man_guard_tone1:": "💂🏻‍♂️",
    ":man_guard_light_skin_tone:": "💂🏻‍♂️",
    ":man_guard::skin-tone-1:": "💂🏻‍♂️",
    ":man_guard_tone2:": "💂🏼‍♂️",
    ":man_guard_medium_light_skin_tone:": "💂🏼‍♂️",
    ":man_guard::skin-tone-2:": "💂🏼‍♂️",
    ":man_guard_tone3:": "💂🏽‍♂️",
    ":man_guard_medium_skin_tone:": "💂🏽‍♂️",
    ":man_guard::skin-tone-3:": "💂🏽‍♂️",
    ":man_guard_tone4:": "💂🏾‍♂️",
    ":man_guard_medium_dark_skin_tone:": "💂🏾‍♂️",
    ":man_guard::skin-tone-4:": "💂🏾‍♂️",
    ":man_guard_tone5:": "💂🏿‍♂️",
    ":man_guard_dark_skin_tone:": "💂🏿‍♂️",
    ":man_guard::skin-tone-5:": "💂🏿‍♂️",
    ":man_health_worker:": "👨‍⚕️",
    ":man_health_worker_tone1:": "👨🏻‍⚕️",
    ":man_health_worker_light_skin_tone:": "👨🏻‍⚕️",
    ":man_health_worker::skin-tone-1:": "👨🏻‍⚕️",
    ":man_health_worker_tone2:": "👨🏼‍⚕️",
    ":man_health_worker_medium_light_skin_tone:": "👨🏼‍⚕️",
    ":man_health_worker::skin-tone-2:": "👨🏼‍⚕️",
    ":man_health_worker_tone3:": "👨🏽‍⚕️",
    ":man_health_worker_medium_skin_tone:": "👨🏽‍⚕️",
    ":man_health_worker::skin-tone-3:": "👨🏽‍⚕️",
    ":man_health_worker_tone4:": "👨🏾‍⚕️",
    ":man_health_worker_medium_dark_skin_tone:": "👨🏾‍⚕️",
    ":man_health_worker::skin-tone-4:": "👨🏾‍⚕️",
    ":man_health_worker_tone5:": "👨🏿‍⚕️",
    ":man_health_worker_dark_skin_tone:": "👨🏿‍⚕️",
    ":man_health_worker::skin-tone-5:": "👨🏿‍⚕️",
    ":man_in_lotus_position:": "🧘‍♂️",
    ":man_in_lotus_position_tone1:": "🧘🏻‍♂️",
    ":man_in_lotus_position_light_skin_tone:": "🧘🏻‍♂️",
    ":man_in_lotus_position::skin-tone-1:": "🧘🏻‍♂️",
    ":man_in_lotus_position_tone2:": "🧘🏼‍♂️",
    ":man_in_lotus_position_medium_light_skin_tone:": "🧘🏼‍♂️",
    ":man_in_lotus_position::skin-tone-2:": "🧘🏼‍♂️",
    ":man_in_lotus_position_tone3:": "🧘🏽‍♂️",
    ":man_in_lotus_position_medium_skin_tone:": "🧘🏽‍♂️",
    ":man_in_lotus_position::skin-tone-3:": "🧘🏽‍♂️",
    ":man_in_lotus_position_tone4:": "🧘🏾‍♂️",
    ":man_in_lotus_position_medium_dark_skin_tone:": "🧘🏾‍♂️",
    ":man_in_lotus_position::skin-tone-4:": "🧘🏾‍♂️",
    ":man_in_lotus_position_tone5:": "🧘🏿‍♂️",
    ":man_in_lotus_position_dark_skin_tone:": "🧘🏿‍♂️",
    ":man_in_lotus_position::skin-tone-5:": "🧘🏿‍♂️",
    ":man_in_manual_wheelchair:": "👨‍🦽",
    ":man_in_manual_wheelchair_tone1:": "👨🏻‍🦽",
    ":man_in_manual_wheelchair_light_skin_tone:": "👨🏻‍🦽",
    ":man_in_manual_wheelchair::skin-tone-1:": "👨🏻‍🦽",
    ":man_in_manual_wheelchair_tone2:": "👨🏼‍🦽",
    ":man_in_manual_wheelchair_medium_light_skin_tone:": "👨🏼‍🦽",
    ":man_in_manual_wheelchair::skin-tone-2:": "👨🏼‍🦽",
    ":man_in_manual_wheelchair_tone3:": "👨🏽‍🦽",
    ":man_in_manual_wheelchair_medium_skin_tone:": "👨🏽‍🦽",
    ":man_in_manual_wheelchair::skin-tone-3:": "👨🏽‍🦽",
    ":man_in_manual_wheelchair_tone4:": "👨🏾‍🦽",
    ":man_in_manual_wheelchair_medium_dark_skin_tone:": "👨🏾‍🦽",
    ":man_in_manual_wheelchair::skin-tone-4:": "👨🏾‍🦽",
    ":man_in_manual_wheelchair_tone5:": "👨🏿‍🦽",
    ":man_in_manual_wheelchair_dark_skin_tone:": "👨🏿‍🦽",
    ":man_in_manual_wheelchair::skin-tone-5:": "👨🏿‍🦽",
    ":man_in_motorized_wheelchair:": "👨‍🦼",
    ":man_in_motorized_wheelchair_tone1:": "👨🏻‍🦼",
    ":man_in_motorized_wheelchair_light_skin_tone:": "👨🏻‍🦼",
    ":man_in_motorized_wheelchair::skin-tone-1:": "👨🏻‍🦼",
    ":man_in_motorized_wheelchair_tone2:": "👨🏼‍🦼",
    ":man_in_motorized_wheelchair_medium_light_skin_tone:": "👨🏼‍🦼",
    ":man_in_motorized_wheelchair::skin-tone-2:": "👨🏼‍🦼",
    ":man_in_motorized_wheelchair_tone3:": "👨🏽‍🦼",
    ":man_in_motorized_wheelchair_medium_skin_tone:": "👨🏽‍🦼",
    ":man_in_motorized_wheelchair::skin-tone-3:": "👨🏽‍🦼",
    ":man_in_motorized_wheelchair_tone4:": "👨🏾‍🦼",
    ":man_in_motorized_wheelchair_medium_dark_skin_tone:": "👨🏾‍🦼",
    ":man_in_motorized_wheelchair::skin-tone-4:": "👨🏾‍🦼",
    ":man_in_motorized_wheelchair_tone5:": "👨🏿‍🦼",
    ":man_in_motorized_wheelchair_dark_skin_tone:": "👨🏿‍🦼",
    ":man_in_motorized_wheelchair::skin-tone-5:": "👨🏿‍🦼",
    ":man_in_steamy_room:": "🧖‍♂️",
    ":man_in_steamy_room_tone1:": "🧖🏻‍♂️",
    ":man_in_steamy_room_light_skin_tone:": "🧖🏻‍♂️",
    ":man_in_steamy_room::skin-tone-1:": "🧖🏻‍♂️",
    ":man_in_steamy_room_tone2:": "🧖🏼‍♂️",
    ":man_in_steamy_room_medium_light_skin_tone:": "🧖🏼‍♂️",
    ":man_in_steamy_room::skin-tone-2:": "🧖🏼‍♂️",
    ":man_in_steamy_room_tone3:": "🧖🏽‍♂️",
    ":man_in_steamy_room_medium_skin_tone:": "🧖🏽‍♂️",
    ":man_in_steamy_room::skin-tone-3:": "🧖🏽‍♂️",
    ":man_in_steamy_room_tone4:": "🧖🏾‍♂️",
    ":man_in_steamy_room_medium_dark_skin_tone:": "🧖🏾‍♂️",
    ":man_in_steamy_room::skin-tone-4:": "🧖🏾‍♂️",
    ":man_in_steamy_room_tone5:": "🧖🏿‍♂️",
    ":man_in_steamy_room_dark_skin_tone:": "🧖🏿‍♂️",
    ":man_in_steamy_room::skin-tone-5:": "🧖🏿‍♂️",
    ":man_in_tuxedo:": "🤵‍♂️",
    ":man_in_tuxedo_tone1:": "🤵🏻‍♂️",
    ":man_in_tuxedo_light_skin_tone:": "🤵🏻‍♂️",
    ":man_in_tuxedo::skin-tone-1:": "🤵🏻‍♂️",
    ":man_in_tuxedo_tone2:": "🤵🏼‍♂️",
    ":man_in_tuxedo_medium_light_skin_tone:": "🤵🏼‍♂️",
    ":man_in_tuxedo::skin-tone-2:": "🤵🏼‍♂️",
    ":man_in_tuxedo_tone3:": "🤵🏽‍♂️",
    ":man_in_tuxedo_medium_skin_tone:": "🤵🏽‍♂️",
    ":man_in_tuxedo::skin-tone-3:": "🤵🏽‍♂️",
    ":man_in_tuxedo_tone4:": "🤵🏾‍♂️",
    ":man_in_tuxedo_medium_dark_skin_tone:": "🤵🏾‍♂️",
    ":man_in_tuxedo::skin-tone-4:": "🤵🏾‍♂️",
    ":man_in_tuxedo_tone5:": "🤵🏿‍♂️",
    ":man_in_tuxedo_dark_skin_tone:": "🤵🏿‍♂️",
    ":man_in_tuxedo::skin-tone-5:": "🤵🏿‍♂️",
    ":man_judge:": "👨‍⚖️",
    ":man_judge_tone1:": "👨🏻‍⚖️",
    ":man_judge_light_skin_tone:": "👨🏻‍⚖️",
    ":man_judge::skin-tone-1:": "👨🏻‍⚖️",
    ":man_judge_tone2:": "👨🏼‍⚖️",
    ":man_judge_medium_light_skin_tone:": "👨🏼‍⚖️",
    ":man_judge::skin-tone-2:": "👨🏼‍⚖️",
    ":man_judge_tone3:": "👨🏽‍⚖️",
    ":man_judge_medium_skin_tone:": "👨🏽‍⚖️",
    ":man_judge::skin-tone-3:": "👨🏽‍⚖️",
    ":man_judge_tone4:": "👨🏾‍⚖️",
    ":man_judge_medium_dark_skin_tone:": "👨🏾‍⚖️",
    ":man_judge::skin-tone-4:": "👨🏾‍⚖️",
    ":man_judge_tone5:": "👨🏿‍⚖️",
    ":man_judge_dark_skin_tone:": "👨🏿‍⚖️",
    ":man_judge::skin-tone-5:": "👨🏿‍⚖️",
    ":man_juggling:": "🤹‍♂️",
    ":man_juggling_tone1:": "🤹🏻‍♂️",
    ":man_juggling_light_skin_tone:": "🤹🏻‍♂️",
    ":man_juggling::skin-tone-1:": "🤹🏻‍♂️",
    ":man_juggling_tone2:": "🤹🏼‍♂️",
    ":man_juggling_medium_light_skin_tone:": "🤹🏼‍♂️",
    ":man_juggling::skin-tone-2:": "🤹🏼‍♂️",
    ":man_juggling_tone3:": "🤹🏽‍♂️",
    ":man_juggling_medium_skin_tone:": "🤹🏽‍♂️",
    ":man_juggling::skin-tone-3:": "🤹🏽‍♂️",
    ":man_juggling_tone4:": "🤹🏾‍♂️",
    ":man_juggling_medium_dark_skin_tone:": "🤹🏾‍♂️",
    ":man_juggling::skin-tone-4:": "🤹🏾‍♂️",
    ":man_juggling_tone5:": "🤹🏿‍♂️",
    ":man_juggling_dark_skin_tone:": "🤹🏿‍♂️",
    ":man_juggling::skin-tone-5:": "🤹🏿‍♂️",
    ":man_kneeling:": "🧎‍♂️",
    ":man_kneeling_tone1:": "🧎🏻‍♂️",
    ":man_kneeling_light_skin_tone:": "🧎🏻‍♂️",
    ":man_kneeling::skin-tone-1:": "🧎🏻‍♂️",
    ":man_kneeling_tone2:": "🧎🏼‍♂️",
    ":man_kneeling_medium_light_skin_tone:": "🧎🏼‍♂️",
    ":man_kneeling::skin-tone-2:": "🧎🏼‍♂️",
    ":man_kneeling_tone3:": "🧎🏽‍♂️",
    ":man_kneeling_medium_skin_tone:": "🧎🏽‍♂️",
    ":man_kneeling::skin-tone-3:": "🧎🏽‍♂️",
    ":man_kneeling_tone4:": "🧎🏾‍♂️",
    ":man_kneeling_medium_dark_skin_tone:": "🧎🏾‍♂️",
    ":man_kneeling::skin-tone-4:": "🧎🏾‍♂️",
    ":man_kneeling_tone5:": "🧎🏿‍♂️",
    ":man_kneeling_dark_skin_tone:": "🧎🏿‍♂️",
    ":man_kneeling::skin-tone-5:": "🧎🏿‍♂️",
    ":man_lifting_weights:": "🏋️‍♂️",
    ":man_lifting_weights_tone1:": "🏋🏻‍♂️",
    ":man_lifting_weights_light_skin_tone:": "🏋🏻‍♂️",
    ":man_lifting_weights::skin-tone-1:": "🏋🏻‍♂️",
    ":man_lifting_weights_tone2:": "🏋🏼‍♂️",
    ":man_lifting_weights_medium_light_skin_tone:": "🏋🏼‍♂️",
    ":man_lifting_weights::skin-tone-2:": "🏋🏼‍♂️",
    ":man_lifting_weights_tone3:": "🏋🏽‍♂️",
    ":man_lifting_weights_medium_skin_tone:": "🏋🏽‍♂️",
    ":man_lifting_weights::skin-tone-3:": "🏋🏽‍♂️",
    ":man_lifting_weights_tone4:": "🏋🏾‍♂️",
    ":man_lifting_weights_medium_dark_skin_tone:": "🏋🏾‍♂️",
    ":man_lifting_weights::skin-tone-4:": "🏋🏾‍♂️",
    ":man_lifting_weights_tone5:": "🏋🏿‍♂️",
    ":man_lifting_weights_dark_skin_tone:": "🏋🏿‍♂️",
    ":man_lifting_weights::skin-tone-5:": "🏋🏿‍♂️",
    ":man_mage:": "🧙‍♂️",
    ":man_mage_tone1:": "🧙🏻‍♂️",
    ":man_mage_light_skin_tone:": "🧙🏻‍♂️",
    ":man_mage::skin-tone-1:": "🧙🏻‍♂️",
    ":man_mage_tone2:": "🧙🏼‍♂️",
    ":man_mage_medium_light_skin_tone:": "🧙🏼‍♂️",
    ":man_mage::skin-tone-2:": "🧙🏼‍♂️",
    ":man_mage_tone3:": "🧙🏽‍♂️",
    ":man_mage_medium_skin_tone:": "🧙🏽‍♂️",
    ":man_mage::skin-tone-3:": "🧙🏽‍♂️",
    ":man_mage_tone4:": "🧙🏾‍♂️",
    ":man_mage_medium_dark_skin_tone:": "🧙🏾‍♂️",
    ":man_mage::skin-tone-4:": "🧙🏾‍♂️",
    ":man_mage_tone5:": "🧙🏿‍♂️",
    ":man_mage_dark_skin_tone:": "🧙🏿‍♂️",
    ":man_mage::skin-tone-5:": "🧙🏿‍♂️",
    ":man_mechanic:": "👨‍🔧",
    ":man_mechanic_tone1:": "👨🏻‍🔧",
    ":man_mechanic_light_skin_tone:": "👨🏻‍🔧",
    ":man_mechanic::skin-tone-1:": "👨🏻‍🔧",
    ":man_mechanic_tone2:": "👨🏼‍🔧",
    ":man_mechanic_medium_light_skin_tone:": "👨🏼‍🔧",
    ":man_mechanic::skin-tone-2:": "👨🏼‍🔧",
    ":man_mechanic_tone3:": "👨🏽‍🔧",
    ":man_mechanic_medium_skin_tone:": "👨🏽‍🔧",
    ":man_mechanic::skin-tone-3:": "👨🏽‍🔧",
    ":man_mechanic_tone4:": "👨🏾‍🔧",
    ":man_mechanic_medium_dark_skin_tone:": "👨🏾‍🔧",
    ":man_mechanic::skin-tone-4:": "👨🏾‍🔧",
    ":man_mechanic_tone5:": "👨🏿‍🔧",
    ":man_mechanic_dark_skin_tone:": "👨🏿‍🔧",
    ":man_mechanic::skin-tone-5:": "👨🏿‍🔧",
    ":man_mountain_biking:": "🚵‍♂️",
    ":man_mountain_biking_tone1:": "🚵🏻‍♂️",
    ":man_mountain_biking_light_skin_tone:": "🚵🏻‍♂️",
    ":man_mountain_biking::skin-tone-1:": "🚵🏻‍♂️",
    ":man_mountain_biking_tone2:": "🚵🏼‍♂️",
    ":man_mountain_biking_medium_light_skin_tone:": "🚵🏼‍♂️",
    ":man_mountain_biking::skin-tone-2:": "🚵🏼‍♂️",
    ":man_mountain_biking_tone3:": "🚵🏽‍♂️",
    ":man_mountain_biking_medium_skin_tone:": "🚵🏽‍♂️",
    ":man_mountain_biking::skin-tone-3:": "🚵🏽‍♂️",
    ":man_mountain_biking_tone4:": "🚵🏾‍♂️",
    ":man_mountain_biking_medium_dark_skin_tone:": "🚵🏾‍♂️",
    ":man_mountain_biking::skin-tone-4:": "🚵🏾‍♂️",
    ":man_mountain_biking_tone5:": "🚵🏿‍♂️",
    ":man_mountain_biking_dark_skin_tone:": "🚵🏿‍♂️",
    ":man_mountain_biking::skin-tone-5:": "🚵🏿‍♂️",
    ":man_office_worker:": "👨‍💼",
    ":man_office_worker_tone1:": "👨🏻‍💼",
    ":man_office_worker_light_skin_tone:": "👨🏻‍💼",
    ":man_office_worker::skin-tone-1:": "👨🏻‍💼",
    ":man_office_worker_tone2:": "👨🏼‍💼",
    ":man_office_worker_medium_light_skin_tone:": "👨🏼‍💼",
    ":man_office_worker::skin-tone-2:": "👨🏼‍💼",
    ":man_office_worker_tone3:": "👨🏽‍💼",
    ":man_office_worker_medium_skin_tone:": "👨🏽‍💼",
    ":man_office_worker::skin-tone-3:": "👨🏽‍💼",
    ":man_office_worker_tone4:": "👨🏾‍💼",
    ":man_office_worker_medium_dark_skin_tone:": "👨🏾‍💼",
    ":man_office_worker::skin-tone-4:": "👨🏾‍💼",
    ":man_office_worker_tone5:": "👨🏿‍💼",
    ":man_office_worker_dark_skin_tone:": "👨🏿‍💼",
    ":man_office_worker::skin-tone-5:": "👨🏿‍💼",
    ":man_pilot:": "👨‍✈️",
    ":man_pilot_tone1:": "👨🏻‍✈️",
    ":man_pilot_light_skin_tone:": "👨🏻‍✈️",
    ":man_pilot::skin-tone-1:": "👨🏻‍✈️",
    ":man_pilot_tone2:": "👨🏼‍✈️",
    ":man_pilot_medium_light_skin_tone:": "👨🏼‍✈️",
    ":man_pilot::skin-tone-2:": "👨🏼‍✈️",
    ":man_pilot_tone3:": "👨🏽‍✈️",
    ":man_pilot_medium_skin_tone:": "👨🏽‍✈️",
    ":man_pilot::skin-tone-3:": "👨🏽‍✈️",
    ":man_pilot_tone4:": "👨🏾‍✈️",
    ":man_pilot_medium_dark_skin_tone:": "👨🏾‍✈️",
    ":man_pilot::skin-tone-4:": "👨🏾‍✈️",
    ":man_pilot_tone5:": "👨🏿‍✈️",
    ":man_pilot_dark_skin_tone:": "👨🏿‍✈️",
    ":man_pilot::skin-tone-5:": "👨🏿‍✈️",
    ":man_playing_handball:": "🤾‍♂️",
    ":man_playing_handball_tone1:": "🤾🏻‍♂️",
    ":man_playing_handball_light_skin_tone:": "🤾🏻‍♂️",
    ":man_playing_handball::skin-tone-1:": "🤾🏻‍♂️",
    ":man_playing_handball_tone2:": "🤾🏼‍♂️",
    ":man_playing_handball_medium_light_skin_tone:": "🤾🏼‍♂️",
    ":man_playing_handball::skin-tone-2:": "🤾🏼‍♂️",
    ":man_playing_handball_tone3:": "🤾🏽‍♂️",
    ":man_playing_handball_medium_skin_tone:": "🤾🏽‍♂️",
    ":man_playing_handball::skin-tone-3:": "🤾🏽‍♂️",
    ":man_playing_handball_tone4:": "🤾🏾‍♂️",
    ":man_playing_handball_medium_dark_skin_tone:": "🤾🏾‍♂️",
    ":man_playing_handball::skin-tone-4:": "🤾🏾‍♂️",
    ":man_playing_handball_tone5:": "🤾🏿‍♂️",
    ":man_playing_handball_dark_skin_tone:": "🤾🏿‍♂️",
    ":man_playing_handball::skin-tone-5:": "🤾🏿‍♂️",
    ":man_playing_water_polo:": "🤽‍♂️",
    ":man_playing_water_polo_tone1:": "🤽🏻‍♂️",
    ":man_playing_water_polo_light_skin_tone:": "🤽🏻‍♂️",
    ":man_playing_water_polo::skin-tone-1:": "🤽🏻‍♂️",
    ":man_playing_water_polo_tone2:": "🤽🏼‍♂️",
    ":man_playing_water_polo_medium_light_skin_tone:": "🤽🏼‍♂️",
    ":man_playing_water_polo::skin-tone-2:": "🤽🏼‍♂️",
    ":man_playing_water_polo_tone3:": "🤽🏽‍♂️",
    ":man_playing_water_polo_medium_skin_tone:": "🤽🏽‍♂️",
    ":man_playing_water_polo::skin-tone-3:": "🤽🏽‍♂️",
    ":man_playing_water_polo_tone4:": "🤽🏾‍♂️",
    ":man_playing_water_polo_medium_dark_skin_tone:": "🤽🏾‍♂️",
    ":man_playing_water_polo::skin-tone-4:": "🤽🏾‍♂️",
    ":man_playing_water_polo_tone5:": "🤽🏿‍♂️",
    ":man_playing_water_polo_dark_skin_tone:": "🤽🏿‍♂️",
    ":man_playing_water_polo::skin-tone-5:": "🤽🏿‍♂️",
    ":man_police_officer:": "👮‍♂️",
    ":man_police_officer_tone1:": "👮🏻‍♂️",
    ":man_police_officer_light_skin_tone:": "👮🏻‍♂️",
    ":man_police_officer::skin-tone-1:": "👮🏻‍♂️",
    ":man_police_officer_tone2:": "👮🏼‍♂️",
    ":man_police_officer_medium_light_skin_tone:": "👮🏼‍♂️",
    ":man_police_officer::skin-tone-2:": "👮🏼‍♂️",
    ":man_police_officer_tone3:": "👮🏽‍♂️",
    ":man_police_officer_medium_skin_tone:": "👮🏽‍♂️",
    ":man_police_officer::skin-tone-3:": "👮🏽‍♂️",
    ":man_police_officer_tone4:": "👮🏾‍♂️",
    ":man_police_officer_medium_dark_skin_tone:": "👮🏾‍♂️",
    ":man_police_officer::skin-tone-4:": "👮🏾‍♂️",
    ":man_police_officer_tone5:": "👮🏿‍♂️",
    ":man_police_officer_dark_skin_tone:": "👮🏿‍♂️",
    ":man_police_officer::skin-tone-5:": "👮🏿‍♂️",
    ":man_pouting:": "🙎‍♂️",
    ":man_pouting_tone1:": "🙎🏻‍♂️",
    ":man_pouting_light_skin_tone:": "🙎🏻‍♂️",
    ":man_pouting::skin-tone-1:": "🙎🏻‍♂️",
    ":man_pouting_tone2:": "🙎🏼‍♂️",
    ":man_pouting_medium_light_skin_tone:": "🙎🏼‍♂️",
    ":man_pouting::skin-tone-2:": "🙎🏼‍♂️",
    ":man_pouting_tone3:": "🙎🏽‍♂️",
    ":man_pouting_medium_skin_tone:": "🙎🏽‍♂️",
    ":man_pouting::skin-tone-3:": "🙎🏽‍♂️",
    ":man_pouting_tone4:": "🙎🏾‍♂️",
    ":man_pouting_medium_dark_skin_tone:": "🙎🏾‍♂️",
    ":man_pouting::skin-tone-4:": "🙎🏾‍♂️",
    ":man_pouting_tone5:": "🙎🏿‍♂️",
    ":man_pouting_dark_skin_tone:": "🙎🏿‍♂️",
    ":man_pouting::skin-tone-5:": "🙎🏿‍♂️",
    ":man_raising_hand:": "🙋‍♂️",
    ":man_raising_hand_tone1:": "🙋🏻‍♂️",
    ":man_raising_hand_light_skin_tone:": "🙋🏻‍♂️",
    ":man_raising_hand::skin-tone-1:": "🙋🏻‍♂️",
    ":man_raising_hand_tone2:": "🙋🏼‍♂️",
    ":man_raising_hand_medium_light_skin_tone:": "🙋🏼‍♂️",
    ":man_raising_hand::skin-tone-2:": "🙋🏼‍♂️",
    ":man_raising_hand_tone3:": "🙋🏽‍♂️",
    ":man_raising_hand_medium_skin_tone:": "🙋🏽‍♂️",
    ":man_raising_hand::skin-tone-3:": "🙋🏽‍♂️",
    ":man_raising_hand_tone4:": "🙋🏾‍♂️",
    ":man_raising_hand_medium_dark_skin_tone:": "🙋🏾‍♂️",
    ":man_raising_hand::skin-tone-4:": "🙋🏾‍♂️",
    ":man_raising_hand_tone5:": "🙋🏿‍♂️",
    ":man_raising_hand_dark_skin_tone:": "🙋🏿‍♂️",
    ":man_raising_hand::skin-tone-5:": "🙋🏿‍♂️",
    ":man_red_haired:": "👨‍🦰",
    ":man_red_haired_tone1:": "👨🏻‍🦰",
    ":man_red_haired_light_skin_tone:": "👨🏻‍🦰",
    ":man_red_haired::skin-tone-1:": "👨🏻‍🦰",
    ":man_red_haired_tone2:": "👨🏼‍🦰",
    ":man_red_haired_medium_light_skin_tone:": "👨🏼‍🦰",
    ":man_red_haired::skin-tone-2:": "👨🏼‍🦰",
    ":man_red_haired_tone3:": "👨🏽‍🦰",
    ":man_red_haired_medium_skin_tone:": "👨🏽‍🦰",
    ":man_red_haired::skin-tone-3:": "👨🏽‍🦰",
    ":man_red_haired_tone4:": "👨🏾‍🦰",
    ":man_red_haired_medium_dark_skin_tone:": "👨🏾‍🦰",
    ":man_red_haired::skin-tone-4:": "👨🏾‍🦰",
    ":man_red_haired_tone5:": "👨🏿‍🦰",
    ":man_red_haired_dark_skin_tone:": "👨🏿‍🦰",
    ":man_red_haired::skin-tone-5:": "👨🏿‍🦰",
    ":man_rowing_boat:": "🚣‍♂️",
    ":man_rowing_boat_tone1:": "🚣🏻‍♂️",
    ":man_rowing_boat_light_skin_tone:": "🚣🏻‍♂️",
    ":man_rowing_boat::skin-tone-1:": "🚣🏻‍♂️",
    ":man_rowing_boat_tone2:": "🚣🏼‍♂️",
    ":man_rowing_boat_medium_light_skin_tone:": "🚣🏼‍♂️",
    ":man_rowing_boat::skin-tone-2:": "🚣🏼‍♂️",
    ":man_rowing_boat_tone3:": "🚣🏽‍♂️",
    ":man_rowing_boat_medium_skin_tone:": "🚣🏽‍♂️",
    ":man_rowing_boat::skin-tone-3:": "🚣🏽‍♂️",
    ":man_rowing_boat_tone4:": "🚣🏾‍♂️",
    ":man_rowing_boat_medium_dark_skin_tone:": "🚣🏾‍♂️",
    ":man_rowing_boat::skin-tone-4:": "🚣🏾‍♂️",
    ":man_rowing_boat_tone5:": "🚣🏿‍♂️",
    ":man_rowing_boat_dark_skin_tone:": "🚣🏿‍♂️",
    ":man_rowing_boat::skin-tone-5:": "🚣🏿‍♂️",
    ":man_running:": "🏃‍♂️",
    ":man_running_tone1:": "🏃🏻‍♂️",
    ":man_running_light_skin_tone:": "🏃🏻‍♂️",
    ":man_running::skin-tone-1:": "🏃🏻‍♂️",
    ":man_running_tone2:": "🏃🏼‍♂️",
    ":man_running_medium_light_skin_tone:": "🏃🏼‍♂️",
    ":man_running::skin-tone-2:": "🏃🏼‍♂️",
    ":man_running_tone3:": "🏃🏽‍♂️",
    ":man_running_medium_skin_tone:": "🏃🏽‍♂️",
    ":man_running::skin-tone-3:": "🏃🏽‍♂️",
    ":man_running_tone4:": "🏃🏾‍♂️",
    ":man_running_medium_dark_skin_tone:": "🏃🏾‍♂️",
    ":man_running::skin-tone-4:": "🏃🏾‍♂️",
    ":man_running_tone5:": "🏃🏿‍♂️",
    ":man_running_dark_skin_tone:": "🏃🏿‍♂️",
    ":man_running::skin-tone-5:": "🏃🏿‍♂️",
    ":man_scientist:": "👨‍🔬",
    ":man_scientist_tone1:": "👨🏻‍🔬",
    ":man_scientist_light_skin_tone:": "👨🏻‍🔬",
    ":man_scientist::skin-tone-1:": "👨🏻‍🔬",
    ":man_scientist_tone2:": "👨🏼‍🔬",
    ":man_scientist_medium_light_skin_tone:": "👨🏼‍🔬",
    ":man_scientist::skin-tone-2:": "👨🏼‍🔬",
    ":man_scientist_tone3:": "👨🏽‍🔬",
    ":man_scientist_medium_skin_tone:": "👨🏽‍🔬",
    ":man_scientist::skin-tone-3:": "👨🏽‍🔬",
    ":man_scientist_tone4:": "👨🏾‍🔬",
    ":man_scientist_medium_dark_skin_tone:": "👨🏾‍🔬",
    ":man_scientist::skin-tone-4:": "👨🏾‍🔬",
    ":man_scientist_tone5:": "👨🏿‍🔬",
    ":man_scientist_dark_skin_tone:": "👨🏿‍🔬",
    ":man_scientist::skin-tone-5:": "👨🏿‍🔬",
    ":man_shrugging:": "🤷‍♂️",
    ":man_shrugging_tone1:": "🤷🏻‍♂️",
    ":man_shrugging_light_skin_tone:": "🤷🏻‍♂️",
    ":man_shrugging::skin-tone-1:": "🤷🏻‍♂️",
    ":man_shrugging_tone2:": "🤷🏼‍♂️",
    ":man_shrugging_medium_light_skin_tone:": "🤷🏼‍♂️",
    ":man_shrugging::skin-tone-2:": "🤷🏼‍♂️",
    ":man_shrugging_tone3:": "🤷🏽‍♂️",
    ":man_shrugging_medium_skin_tone:": "🤷🏽‍♂️",
    ":man_shrugging::skin-tone-3:": "🤷🏽‍♂️",
    ":man_shrugging_tone4:": "🤷🏾‍♂️",
    ":man_shrugging_medium_dark_skin_tone:": "🤷🏾‍♂️",
    ":man_shrugging::skin-tone-4:": "🤷🏾‍♂️",
    ":man_shrugging_tone5:": "🤷🏿‍♂️",
    ":man_shrugging_dark_skin_tone:": "🤷🏿‍♂️",
    ":man_shrugging::skin-tone-5:": "🤷🏿‍♂️",
    ":man_singer:": "👨‍🎤",
    ":man_singer_tone1:": "👨🏻‍🎤",
    ":man_singer_light_skin_tone:": "👨🏻‍🎤",
    ":man_singer::skin-tone-1:": "👨🏻‍🎤",
    ":man_singer_tone2:": "👨🏼‍🎤",
    ":man_singer_medium_light_skin_tone:": "👨🏼‍🎤",
    ":man_singer::skin-tone-2:": "👨🏼‍🎤",
    ":man_singer_tone3:": "👨🏽‍🎤",
    ":man_singer_medium_skin_tone:": "👨🏽‍🎤",
    ":man_singer::skin-tone-3:": "👨🏽‍🎤",
    ":man_singer_tone4:": "👨🏾‍🎤",
    ":man_singer_medium_dark_skin_tone:": "👨🏾‍🎤",
    ":man_singer::skin-tone-4:": "👨🏾‍🎤",
    ":man_singer_tone5:": "👨🏿‍🎤",
    ":man_singer_dark_skin_tone:": "👨🏿‍🎤",
    ":man_singer::skin-tone-5:": "👨🏿‍🎤",
    ":man_standing:": "🧍‍♂️",
    ":man_standing_tone1:": "🧍🏻‍♂️",
    ":man_standing_light_skin_tone:": "🧍🏻‍♂️",
    ":man_standing::skin-tone-1:": "🧍🏻‍♂️",
    ":man_standing_tone2:": "🧍🏼‍♂️",
    ":man_standing_medium_light_skin_tone:": "🧍🏼‍♂️",
    ":man_standing::skin-tone-2:": "🧍🏼‍♂️",
    ":man_standing_tone3:": "🧍🏽‍♂️",
    ":man_standing_medium_skin_tone:": "🧍🏽‍♂️",
    ":man_standing::skin-tone-3:": "🧍🏽‍♂️",
    ":man_standing_tone4:": "🧍🏾‍♂️",
    ":man_standing_medium_dark_skin_tone:": "🧍🏾‍♂️",
    ":man_standing::skin-tone-4:": "🧍🏾‍♂️",
    ":man_standing_tone5:": "🧍🏿‍♂️",
    ":man_standing_dark_skin_tone:": "🧍🏿‍♂️",
    ":man_standing::skin-tone-5:": "🧍🏿‍♂️",
    ":man_student:": "👨‍🎓",
    ":man_student_tone1:": "👨🏻‍🎓",
    ":man_student_light_skin_tone:": "👨🏻‍🎓",
    ":man_student::skin-tone-1:": "👨🏻‍🎓",
    ":man_student_tone2:": "👨🏼‍🎓",
    ":man_student_medium_light_skin_tone:": "👨🏼‍🎓",
    ":man_student::skin-tone-2:": "👨🏼‍🎓",
    ":man_student_tone3:": "👨🏽‍🎓",
    ":man_student_medium_skin_tone:": "👨🏽‍🎓",
    ":man_student::skin-tone-3:": "👨🏽‍🎓",
    ":man_student_tone4:": "👨🏾‍🎓",
    ":man_student_medium_dark_skin_tone:": "👨🏾‍🎓",
    ":man_student::skin-tone-4:": "👨🏾‍🎓",
    ":man_student_tone5:": "👨🏿‍🎓",
    ":man_student_dark_skin_tone:": "👨🏿‍🎓",
    ":man_student::skin-tone-5:": "👨🏿‍🎓",
    ":man_superhero:": "🦸‍♂️",
    ":man_superhero_tone1:": "🦸🏻‍♂️",
    ":man_superhero_light_skin_tone:": "🦸🏻‍♂️",
    ":man_superhero::skin-tone-1:": "🦸🏻‍♂️",
    ":man_superhero_tone2:": "🦸🏼‍♂️",
    ":man_superhero_medium_light_skin_tone:": "🦸🏼‍♂️",
    ":man_superhero::skin-tone-2:": "🦸🏼‍♂️",
    ":man_superhero_tone3:": "🦸🏽‍♂️",
    ":man_superhero_medium_skin_tone:": "🦸🏽‍♂️",
    ":man_superhero::skin-tone-3:": "🦸🏽‍♂️",
    ":man_superhero_tone4:": "🦸🏾‍♂️",
    ":man_superhero_medium_dark_skin_tone:": "🦸🏾‍♂️",
    ":man_superhero::skin-tone-4:": "🦸🏾‍♂️",
    ":man_superhero_tone5:": "🦸🏿‍♂️",
    ":man_superhero_dark_skin_tone:": "🦸🏿‍♂️",
    ":man_superhero::skin-tone-5:": "🦸🏿‍♂️",
    ":man_supervillain:": "🦹‍♂️",
    ":man_supervillain_tone1:": "🦹🏻‍♂️",
    ":man_supervillain_light_skin_tone:": "🦹🏻‍♂️",
    ":man_supervillain::skin-tone-1:": "🦹🏻‍♂️",
    ":man_supervillain_tone2:": "🦹🏼‍♂️",
    ":man_supervillain_medium_light_skin_tone:": "🦹🏼‍♂️",
    ":man_supervillain::skin-tone-2:": "🦹🏼‍♂️",
    ":man_supervillain_tone3:": "🦹🏽‍♂️",
    ":man_supervillain_medium_skin_tone:": "🦹🏽‍♂️",
    ":man_supervillain::skin-tone-3:": "🦹🏽‍♂️",
    ":man_supervillain_tone4:": "🦹🏾‍♂️",
    ":man_supervillain_medium_dark_skin_tone:": "🦹🏾‍♂️",
    ":man_supervillain::skin-tone-4:": "🦹🏾‍♂️",
    ":man_supervillain_tone5:": "🦹🏿‍♂️",
    ":man_supervillain_dark_skin_tone:": "🦹🏿‍♂️",
    ":man_supervillain::skin-tone-5:": "🦹🏿‍♂️",
    ":man_surfing:": "🏄‍♂️",
    ":man_surfing_tone1:": "🏄🏻‍♂️",
    ":man_surfing_light_skin_tone:": "🏄🏻‍♂️",
    ":man_surfing::skin-tone-1:": "🏄🏻‍♂️",
    ":man_surfing_tone2:": "🏄🏼‍♂️",
    ":man_surfing_medium_light_skin_tone:": "🏄🏼‍♂️",
    ":man_surfing::skin-tone-2:": "🏄🏼‍♂️",
    ":man_surfing_tone3:": "🏄🏽‍♂️",
    ":man_surfing_medium_skin_tone:": "🏄🏽‍♂️",
    ":man_surfing::skin-tone-3:": "🏄🏽‍♂️",
    ":man_surfing_tone4:": "🏄🏾‍♂️",
    ":man_surfing_medium_dark_skin_tone:": "🏄🏾‍♂️",
    ":man_surfing::skin-tone-4:": "🏄🏾‍♂️",
    ":man_surfing_tone5:": "🏄🏿‍♂️",
    ":man_surfing_dark_skin_tone:": "🏄🏿‍♂️",
    ":man_surfing::skin-tone-5:": "🏄🏿‍♂️",
    ":man_swimming:": "🏊‍♂️",
    ":man_swimming_tone1:": "🏊🏻‍♂️",
    ":man_swimming_light_skin_tone:": "🏊🏻‍♂️",
    ":man_swimming::skin-tone-1:": "🏊🏻‍♂️",
    ":man_swimming_tone2:": "🏊🏼‍♂️",
    ":man_swimming_medium_light_skin_tone:": "🏊🏼‍♂️",
    ":man_swimming::skin-tone-2:": "🏊🏼‍♂️",
    ":man_swimming_tone3:": "🏊🏽‍♂️",
    ":man_swimming_medium_skin_tone:": "🏊🏽‍♂️",
    ":man_swimming::skin-tone-3:": "🏊🏽‍♂️",
    ":man_swimming_tone4:": "🏊🏾‍♂️",
    ":man_swimming_medium_dark_skin_tone:": "🏊🏾‍♂️",
    ":man_swimming::skin-tone-4:": "🏊🏾‍♂️",
    ":man_swimming_tone5:": "🏊🏿‍♂️",
    ":man_swimming_dark_skin_tone:": "🏊🏿‍♂️",
    ":man_swimming::skin-tone-5:": "🏊🏿‍♂️",
    ":man_teacher:": "👨‍🏫",
    ":man_teacher_tone1:": "👨🏻‍🏫",
    ":man_teacher_light_skin_tone:": "👨🏻‍🏫",
    ":man_teacher::skin-tone-1:": "👨🏻‍🏫",
    ":man_teacher_tone2:": "👨🏼‍🏫",
    ":man_teacher_medium_light_skin_tone:": "👨🏼‍🏫",
    ":man_teacher::skin-tone-2:": "👨🏼‍🏫",
    ":man_teacher_tone3:": "👨🏽‍🏫",
    ":man_teacher_medium_skin_tone:": "👨🏽‍🏫",
    ":man_teacher::skin-tone-3:": "👨🏽‍🏫",
    ":man_teacher_tone4:": "👨🏾‍🏫",
    ":man_teacher_medium_dark_skin_tone:": "👨🏾‍🏫",
    ":man_teacher::skin-tone-4:": "👨🏾‍🏫",
    ":man_teacher_tone5:": "👨🏿‍🏫",
    ":man_teacher_dark_skin_tone:": "👨🏿‍🏫",
    ":man_teacher::skin-tone-5:": "👨🏿‍🏫",
    ":man_technologist:": "👨‍💻",
    ":man_technologist_tone1:": "👨🏻‍💻",
    ":man_technologist_light_skin_tone:": "👨🏻‍💻",
    ":man_technologist::skin-tone-1:": "👨🏻‍💻",
    ":man_technologist_tone2:": "👨🏼‍💻",
    ":man_technologist_medium_light_skin_tone:": "👨🏼‍💻",
    ":man_technologist::skin-tone-2:": "👨🏼‍💻",
    ":man_technologist_tone3:": "👨🏽‍💻",
    ":man_technologist_medium_skin_tone:": "👨🏽‍💻",
    ":man_technologist::skin-tone-3:": "👨🏽‍💻",
    ":man_technologist_tone4:": "👨🏾‍💻",
    ":man_technologist_medium_dark_skin_tone:": "👨🏾‍💻",
    ":man_technologist::skin-tone-4:": "👨🏾‍💻",
    ":man_technologist_tone5:": "👨🏿‍💻",
    ":man_technologist_dark_skin_tone:": "👨🏿‍💻",
    ":man_technologist::skin-tone-5:": "👨🏿‍💻",
    ":man_tipping_hand:": "💁‍♂️",
    ":man_tipping_hand_tone1:": "💁🏻‍♂️",
    ":man_tipping_hand_light_skin_tone:": "💁🏻‍♂️",
    ":man_tipping_hand::skin-tone-1:": "💁🏻‍♂️",
    ":man_tipping_hand_tone2:": "💁🏼‍♂️",
    ":man_tipping_hand_medium_light_skin_tone:": "💁🏼‍♂️",
    ":man_tipping_hand::skin-tone-2:": "💁🏼‍♂️",
    ":man_tipping_hand_tone3:": "💁🏽‍♂️",
    ":man_tipping_hand_medium_skin_tone:": "💁🏽‍♂️",
    ":man_tipping_hand::skin-tone-3:": "💁🏽‍♂️",
    ":man_tipping_hand_tone4:": "💁🏾‍♂️",
    ":man_tipping_hand_medium_dark_skin_tone:": "💁🏾‍♂️",
    ":man_tipping_hand::skin-tone-4:": "💁🏾‍♂️",
    ":man_tipping_hand_tone5:": "💁🏿‍♂️",
    ":man_tipping_hand_dark_skin_tone:": "💁🏿‍♂️",
    ":man_tipping_hand::skin-tone-5:": "💁🏿‍♂️",
    ":man_tone1:": "👨🏻",
    ":man::skin-tone-1:": "👨🏻",
    ":man_tone2:": "👨🏼",
    ":man::skin-tone-2:": "👨🏼",
    ":man_tone3:": "👨🏽",
    ":man::skin-tone-3:": "👨🏽",
    ":man_tone4:": "👨🏾",
    ":man::skin-tone-4:": "👨🏾",
    ":man_tone5:": "👨🏿",
    ":man::skin-tone-5:": "👨🏿",
    ":man_vampire:": "🧛‍♂️",
    ":man_vampire_tone1:": "🧛🏻‍♂️",
    ":man_vampire_light_skin_tone:": "🧛🏻‍♂️",
    ":man_vampire::skin-tone-1:": "🧛🏻‍♂️",
    ":man_vampire_tone2:": "🧛🏼‍♂️",
    ":man_vampire_medium_light_skin_tone:": "🧛🏼‍♂️",
    ":man_vampire::skin-tone-2:": "🧛🏼‍♂️",
    ":man_vampire_tone3:": "🧛🏽‍♂️",
    ":man_vampire_medium_skin_tone:": "🧛🏽‍♂️",
    ":man_vampire::skin-tone-3:": "🧛🏽‍♂️",
    ":man_vampire_tone4:": "🧛🏾‍♂️",
    ":man_vampire_medium_dark_skin_tone:": "🧛🏾‍♂️",
    ":man_vampire::skin-tone-4:": "🧛🏾‍♂️",
    ":man_vampire_tone5:": "🧛🏿‍♂️",
    ":man_vampire_dark_skin_tone:": "🧛🏿‍♂️",
    ":man_vampire::skin-tone-5:": "🧛🏿‍♂️",
    ":man_walking:": "🚶‍♂️",
    ":man_walking_tone1:": "🚶🏻‍♂️",
    ":man_walking_light_skin_tone:": "🚶🏻‍♂️",
    ":man_walking::skin-tone-1:": "🚶🏻‍♂️",
    ":man_walking_tone2:": "🚶🏼‍♂️",
    ":man_walking_medium_light_skin_tone:": "🚶🏼‍♂️",
    ":man_walking::skin-tone-2:": "🚶🏼‍♂️",
    ":man_walking_tone3:": "🚶🏽‍♂️",
    ":man_walking_medium_skin_tone:": "🚶🏽‍♂️",
    ":man_walking::skin-tone-3:": "🚶🏽‍♂️",
    ":man_walking_tone4:": "🚶🏾‍♂️",
    ":man_walking_medium_dark_skin_tone:": "🚶🏾‍♂️",
    ":man_walking::skin-tone-4:": "🚶🏾‍♂️",
    ":man_walking_tone5:": "🚶🏿‍♂️",
    ":man_walking_dark_skin_tone:": "🚶🏿‍♂️",
    ":man_walking::skin-tone-5:": "🚶🏿‍♂️",
    ":man_wearing_turban:": "👳‍♂️",
    ":man_wearing_turban_tone1:": "👳🏻‍♂️",
    ":man_wearing_turban_light_skin_tone:": "👳🏻‍♂️",
    ":man_wearing_turban::skin-tone-1:": "👳🏻‍♂️",
    ":man_wearing_turban_tone2:": "👳🏼‍♂️",
    ":man_wearing_turban_medium_light_skin_tone:": "👳🏼‍♂️",
    ":man_wearing_turban::skin-tone-2:": "👳🏼‍♂️",
    ":man_wearing_turban_tone3:": "👳🏽‍♂️",
    ":man_wearing_turban_medium_skin_tone:": "👳🏽‍♂️",
    ":man_wearing_turban::skin-tone-3:": "👳🏽‍♂️",
    ":man_wearing_turban_tone4:": "👳🏾‍♂️",
    ":man_wearing_turban_medium_dark_skin_tone:": "👳🏾‍♂️",
    ":man_wearing_turban::skin-tone-4:": "👳🏾‍♂️",
    ":man_wearing_turban_tone5:": "👳🏿‍♂️",
    ":man_wearing_turban_dark_skin_tone:": "👳🏿‍♂️",
    ":man_wearing_turban::skin-tone-5:": "👳🏿‍♂️",
    ":man_white_haired:": "👨‍🦳",
    ":man_white_haired_tone1:": "👨🏻‍🦳",
    ":man_white_haired_light_skin_tone:": "👨🏻‍🦳",
    ":man_white_haired::skin-tone-1:": "👨🏻‍🦳",
    ":man_white_haired_tone2:": "👨🏼‍🦳",
    ":man_white_haired_medium_light_skin_tone:": "👨🏼‍🦳",
    ":man_white_haired::skin-tone-2:": "👨🏼‍🦳",
    ":man_white_haired_tone3:": "👨🏽‍🦳",
    ":man_white_haired_medium_skin_tone:": "👨🏽‍🦳",
    ":man_white_haired::skin-tone-3:": "👨🏽‍🦳",
    ":man_white_haired_tone4:": "👨🏾‍🦳",
    ":man_white_haired_medium_dark_skin_tone:": "👨🏾‍🦳",
    ":man_white_haired::skin-tone-4:": "👨🏾‍🦳",
    ":man_white_haired_tone5:": "👨🏿‍🦳",
    ":man_white_haired_dark_skin_tone:": "👨🏿‍🦳",
    ":man_white_haired::skin-tone-5:": "👨🏿‍🦳",
    ":man_with_chinese_cap:": "👲",
    ":man_with_gua_pi_mao:": "👲",
    ":man_with_chinese_cap_tone1:": "👲🏻",
    ":man_with_gua_pi_mao_tone1:": "👲🏻",
    ":man_with_chinese_cap::skin-tone-1:": "👲🏻",
    ":man_with_gua_pi_mao::skin-tone-1:": "👲🏻",
    ":man_with_chinese_cap_tone2:": "👲🏼",
    ":man_with_gua_pi_mao_tone2:": "👲🏼",
    ":man_with_chinese_cap::skin-tone-2:": "👲🏼",
    ":man_with_gua_pi_mao::skin-tone-2:": "👲🏼",
    ":man_with_chinese_cap_tone3:": "👲🏽",
    ":man_with_gua_pi_mao_tone3:": "👲🏽",
    ":man_with_chinese_cap::skin-tone-3:": "👲🏽",
    ":man_with_gua_pi_mao::skin-tone-3:": "👲🏽",
    ":man_with_chinese_cap_tone4:": "👲🏾",
    ":man_with_gua_pi_mao_tone4:": "👲🏾",
    ":man_with_chinese_cap::skin-tone-4:": "👲🏾",
    ":man_with_gua_pi_mao::skin-tone-4:": "👲🏾",
    ":man_with_chinese_cap_tone5:": "👲🏿",
    ":man_with_gua_pi_mao_tone5:": "👲🏿",
    ":man_with_chinese_cap::skin-tone-5:": "👲🏿",
    ":man_with_gua_pi_mao::skin-tone-5:": "👲🏿",
    ":man_with_probing_cane:": "👨‍🦯",
    ":man_with_probing_cane_tone1:": "👨🏻‍🦯",
    ":man_with_probing_cane_light_skin_tone:": "👨🏻‍🦯",
    ":man_with_probing_cane::skin-tone-1:": "👨🏻‍🦯",
    ":man_with_probing_cane_tone2:": "👨🏼‍🦯",
    ":man_with_probing_cane_medium_light_skin_tone:": "👨🏼‍🦯",
    ":man_with_probing_cane::skin-tone-2:": "👨🏼‍🦯",
    ":man_with_probing_cane_tone3:": "👨🏽‍🦯",
    ":man_with_probing_cane_medium_skin_tone:": "👨🏽‍🦯",
    ":man_with_probing_cane::skin-tone-3:": "👨🏽‍🦯",
    ":man_with_probing_cane_tone4:": "👨🏾‍🦯",
    ":man_with_probing_cane_medium_dark_skin_tone:": "👨🏾‍🦯",
    ":man_with_probing_cane::skin-tone-4:": "👨🏾‍🦯",
    ":man_with_probing_cane_tone5:": "👨🏿‍🦯",
    ":man_with_probing_cane_dark_skin_tone:": "👨🏿‍🦯",
    ":man_with_probing_cane::skin-tone-5:": "👨🏿‍🦯",
    ":man_with_veil:": "👰‍♂️",
    ":man_with_veil_tone1:": "👰🏻‍♂️",
    ":man_with_veil_light_skin_tone:": "👰🏻‍♂️",
    ":man_with_veil::skin-tone-1:": "👰🏻‍♂️",
    ":man_with_veil_tone2:": "👰🏼‍♂️",
    ":man_with_veil_medium_light_skin_tone:": "👰🏼‍♂️",
    ":man_with_veil::skin-tone-2:": "👰🏼‍♂️",
    ":man_with_veil_tone3:": "👰🏽‍♂️",
    ":man_with_veil_medium_skin_tone:": "👰🏽‍♂️",
    ":man_with_veil::skin-tone-3:": "👰🏽‍♂️",
    ":man_with_veil_tone4:": "👰🏾‍♂️",
    ":man_with_veil_medium_dark_skin_tone:": "👰🏾‍♂️",
    ":man_with_veil::skin-tone-4:": "👰🏾‍♂️",
    ":man_with_veil_tone5:": "👰🏿‍♂️",
    ":man_with_veil_dark_skin_tone:": "👰🏿‍♂️",
    ":man_with_veil::skin-tone-5:": "👰🏿‍♂️",
    ":man_zombie:": "🧟‍♂️",
    ":mango:": "🥭",
    ":mans_shoe:": "👞",
    ":manual_wheelchair:": "🦽",
    ":map:": "🗺️",
    ":world_map:": "🗺️",
    ":maple_leaf:": "🍁",
    ":martial_arts_uniform:": "🥋",
    ":karate_uniform:": "🥋",
    ":mask:": "😷",
    ":mate:": "🧉",
    ":meat_on_bone:": "🍖",
    ":mechanic:": "🧑‍🔧",
    ":mechanic_tone1:": "🧑🏻‍🔧",
    ":mechanic_light_skin_tone:": "🧑🏻‍🔧",
    ":mechanic::skin-tone-1:": "🧑🏻‍🔧",
    ":mechanic_tone2:": "🧑🏼‍🔧",
    ":mechanic_medium_light_skin_tone:": "🧑🏼‍🔧",
    ":mechanic::skin-tone-2:": "🧑🏼‍🔧",
    ":mechanic_tone3:": "🧑🏽‍🔧",
    ":mechanic_medium_skin_tone:": "🧑🏽‍🔧",
    ":mechanic::skin-tone-3:": "🧑🏽‍🔧",
    ":mechanic_tone4:": "🧑🏾‍🔧",
    ":mechanic_medium_dark_skin_tone:": "🧑🏾‍🔧",
    ":mechanic::skin-tone-4:": "🧑🏾‍🔧",
    ":mechanic_tone5:": "🧑🏿‍🔧",
    ":mechanic_dark_skin_tone:": "🧑🏿‍🔧",
    ":mechanic::skin-tone-5:": "🧑🏿‍🔧",
    ":mechanical_arm:": "🦾",
    ":mechanical_leg:": "🦿",
    ":medal:": "🏅",
    ":sports_medal:": "🏅",
    ":medical_symbol:": "⚕️",
    ":mega:": "📣",
    ":melon:": "🍈",
    ":men_holding_hands_tone1:": "👬",
    ":men_holding_hands_light_skin_tone:": "👬",
    ":men_holding_hands_tone1_tone2:": "👬",
    ":men_holding_hands_light_skin_tone_medium_light_skin_tone:": "👬",
    ":men_holding_hands_tone1_tone3:": "👬",
    ":men_holding_hands_light_skin_tone_medium_skin_tone:": "👬",
    ":men_holding_hands_tone1_tone4:": "👬",
    ":men_holding_hands_light_skin_tone_medium_dark_skin_tone:": "👬",
    ":men_holding_hands_tone1_tone5:": "👬",
    ":men_holding_hands_light_skin_tone_dark_skin_tone:": "👬",
    ":men_holding_hands_tone2:": "👬",
    ":men_holding_hands_medium_light_skin_tone:": "👬",
    ":men_holding_hands_tone2_tone1:": "👬",
    ":men_holding_hands_medium_light_skin_tone_light_skin_tone:": "👬",
    ":men_holding_hands_tone2_tone3:": "👬",
    ":men_holding_hands_medium_light_skin_tone_medium_skin_tone:": "👬",
    ":men_holding_hands_tone2_tone4:": "👬",
    ":men_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:": "👬",
    ":men_holding_hands_tone2_tone5:": "👬",
    ":men_holding_hands_medium_light_skin_tone_dark_skin_tone:": "👬",
    ":men_holding_hands_tone3:": "👬",
    ":men_holding_hands_medium_skin_tone:": "👬",
    ":men_holding_hands_tone3_tone1:": "👬",
    ":men_holding_hands_medium_skin_tone_light_skin_tone:": "👬",
    ":men_holding_hands_tone3_tone2:": "👬",
    ":men_holding_hands_medium_skin_tone_medium_light_skin_tone:": "👬",
    ":men_holding_hands_tone3_tone4:": "👬",
    ":men_holding_hands_medium_skin_tone_medium_dark_skin_tone:": "👬",
    ":men_holding_hands_tone3_tone5:": "👬",
    ":men_holding_hands_medium_skin_tone_dark_skin_tone:": "👬",
    ":men_holding_hands_tone4:": "👬",
    ":men_holding_hands_medium_dark_skin_tone:": "👬",
    ":men_holding_hands_tone4_tone1:": "👬",
    ":men_holding_hands_medium_dark_skin_tone_light_skin_tone:": "👬",
    ":men_holding_hands_tone4_tone2:": "👬",
    ":men_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:": "👬",
    ":men_holding_hands_tone4_tone3:": "👬",
    ":men_holding_hands_medium_dark_skin_tone_medium_skin_tone:": "👬",
    ":men_holding_hands_tone4_tone5:": "👬",
    ":men_holding_hands_medium_dark_skin_tone_dark_skin_tone:": "👬",
    ":men_holding_hands_tone5:": "👬",
    ":men_holding_hands_dark_skin_tone:": "👬",
    ":men_holding_hands_tone5_tone1:": "👬",
    ":men_holding_hands_dark_skin_tone_light_skin_tone:": "👬",
    ":men_holding_hands_tone5_tone2:": "👬",
    ":men_holding_hands_dark_skin_tone_medium_light_skin_tone:": "👬",
    ":men_holding_hands_tone5_tone3:": "👬",
    ":men_holding_hands_dark_skin_tone_medium_skin_tone:": "👬",
    ":men_holding_hands_tone5_tone4:": "👬",
    ":men_holding_hands_dark_skin_tone_medium_dark_skin_tone:": "👬",
    ":two_men_holding_hands:": "👬",
    ":men_with_bunny_ears_partying:": "👯‍♂️",
    ":men_wrestling:": "🤼‍♂️",
    ":menorah:": "🕎",
    ":mens:": "🚹",
    ":mermaid:": "🧜‍♀️",
    ":mermaid_tone1:": "🧜🏻‍♀️",
    ":mermaid_light_skin_tone:": "🧜🏻‍♀️",
    ":mermaid::skin-tone-1:": "🧜🏻‍♀️",
    ":mermaid_tone2:": "🧜🏼‍♀️",
    ":mermaid_medium_light_skin_tone:": "🧜🏼‍♀️",
    ":mermaid::skin-tone-2:": "🧜🏼‍♀️",
    ":mermaid_tone3:": "🧜🏽‍♀️",
    ":mermaid_medium_skin_tone:": "🧜🏽‍♀️",
    ":mermaid::skin-tone-3:": "🧜🏽‍♀️",
    ":mermaid_tone4:": "🧜🏾‍♀️",
    ":mermaid_medium_dark_skin_tone:": "🧜🏾‍♀️",
    ":mermaid::skin-tone-4:": "🧜🏾‍♀️",
    ":mermaid_tone5:": "🧜🏿‍♀️",
    ":mermaid_dark_skin_tone:": "🧜🏿‍♀️",
    ":mermaid::skin-tone-5:": "🧜🏿‍♀️",
    ":merman:": "🧜‍♂️",
    ":merman_tone1:": "🧜🏻‍♂️",
    ":merman_light_skin_tone:": "🧜🏻‍♂️",
    ":merman::skin-tone-1:": "🧜🏻‍♂️",
    ":merman_tone2:": "🧜🏼‍♂️",
    ":merman_medium_light_skin_tone:": "🧜🏼‍♂️",
    ":merman::skin-tone-2:": "🧜🏼‍♂️",
    ":merman_tone3:": "🧜🏽‍♂️",
    ":merman_medium_skin_tone:": "🧜🏽‍♂️",
    ":merman::skin-tone-3:": "🧜🏽‍♂️",
    ":merman_tone4:": "🧜🏾‍♂️",
    ":merman_medium_dark_skin_tone:": "🧜🏾‍♂️",
    ":merman::skin-tone-4:": "🧜🏾‍♂️",
    ":merman_tone5:": "🧜🏿‍♂️",
    ":merman_dark_skin_tone:": "🧜🏿‍♂️",
    ":merman::skin-tone-5:": "🧜🏿‍♂️",
    ":merperson:": "🧜",
    ":merperson_tone1:": "🧜🏻",
    ":merperson_light_skin_tone:": "🧜🏻",
    ":merperson::skin-tone-1:": "🧜🏻",
    ":merperson_tone2:": "🧜🏼",
    ":merperson_medium_light_skin_tone:": "🧜🏼",
    ":merperson::skin-tone-2:": "🧜🏼",
    ":merperson_tone3:": "🧜🏽",
    ":merperson_medium_skin_tone:": "🧜🏽",
    ":merperson::skin-tone-3:": "🧜🏽",
    ":merperson_tone4:": "🧜🏾",
    ":merperson_medium_dark_skin_tone:": "🧜🏾",
    ":merperson::skin-tone-4:": "🧜🏾",
    ":merperson_tone5:": "🧜🏿",
    ":merperson_dark_skin_tone:": "🧜🏿",
    ":merperson::skin-tone-5:": "🧜🏿",
    ":metal:": "🤘",
    ":sign_of_the_horns:": "🤘",
    ":metal_tone1:": "🤘🏻",
    ":sign_of_the_horns_tone1:": "🤘🏻",
    ":metal::skin-tone-1:": "🤘🏻",
    ":sign_of_the_horns::skin-tone-1:": "🤘🏻",
    ":metal_tone2:": "🤘🏼",
    ":sign_of_the_horns_tone2:": "🤘🏼",
    ":metal::skin-tone-2:": "🤘🏼",
    ":sign_of_the_horns::skin-tone-2:": "🤘🏼",
    ":metal_tone3:": "🤘🏽",
    ":sign_of_the_horns_tone3:": "🤘🏽",
    ":metal::skin-tone-3:": "🤘🏽",
    ":sign_of_the_horns::skin-tone-3:": "🤘🏽",
    ":metal_tone4:": "🤘🏾",
    ":sign_of_the_horns_tone4:": "🤘🏾",
    ":metal::skin-tone-4:": "🤘🏾",
    ":sign_of_the_horns::skin-tone-4:": "🤘🏾",
    ":metal_tone5:": "🤘🏿",
    ":sign_of_the_horns_tone5:": "🤘🏿",
    ":metal::skin-tone-5:": "🤘🏿",
    ":sign_of_the_horns::skin-tone-5:": "🤘🏿",
    ":metro:": "🚇",
    ":microbe:": "🦠",
    ":microphone:": "🎤",
    ":microphone2:": "🎙️",
    ":studio_microphone:": "🎙️",
    ":microscope:": "🔬",
    ":middle_finger:": "🖕",
    ":reversed_hand_with_middle_finger_extended:": "🖕",
    ":middle_finger_tone1:": "🖕🏻",
    ":reversed_hand_with_middle_finger_extended_tone1:": "🖕🏻",
    ":middle_finger::skin-tone-1:": "🖕🏻",
    ":reversed_hand_with_middle_finger_extended::skin-tone-1:": "🖕🏻",
    ":middle_finger_tone2:": "🖕🏼",
    ":reversed_hand_with_middle_finger_extended_tone2:": "🖕🏼",
    ":middle_finger::skin-tone-2:": "🖕🏼",
    ":reversed_hand_with_middle_finger_extended::skin-tone-2:": "🖕🏼",
    ":middle_finger_tone3:": "🖕🏽",
    ":reversed_hand_with_middle_finger_extended_tone3:": "🖕🏽",
    ":middle_finger::skin-tone-3:": "🖕🏽",
    ":reversed_hand_with_middle_finger_extended::skin-tone-3:": "🖕🏽",
    ":middle_finger_tone4:": "🖕🏾",
    ":reversed_hand_with_middle_finger_extended_tone4:": "🖕🏾",
    ":middle_finger::skin-tone-4:": "🖕🏾",
    ":reversed_hand_with_middle_finger_extended::skin-tone-4:": "🖕🏾",
    ":middle_finger_tone5:": "🖕🏿",
    ":reversed_hand_with_middle_finger_extended_tone5:": "🖕🏿",
    ":middle_finger::skin-tone-5:": "🖕🏿",
    ":reversed_hand_with_middle_finger_extended::skin-tone-5:": "🖕🏿",
    ":military_helmet:": "🪖",
    ":military_medal:": "🎖️",
    ":milk:": "🥛",
    ":glass_of_milk:": "🥛",
    ":milky_way:": "🌌",
    ":minibus:": "🚐",
    ":minidisc:": "💽",
    ":mirror:": "🪞",
    ":mobile_phone:": "📱",
    ":iphone:": "📱",
    ":mobile_phone_off:": "📴",
    ":money_mouth:": "🤑",
    ":money_mouth_face:": "🤑",
    ":money_with_wings:": "💸",
    ":moneybag:": "💰",
    ":monkey:": "🐒",
    ":monkey_face:": "🐵",
    ":monorail:": "🚝",
    ":moon_cake:": "🥮",
    ":mortar_board:": "🎓",
    ":mosque:": "🕌",
    ":mosquito:": "🦟",
    ":motor_scooter:": "🛵",
    ":motorbike:": "🛵",
    ":motorboat:": "🛥️",
    ":motorcycle:": "🏍️",
    ":racing_motorcycle:": "🏍️",
    ":motorized_wheelchair:": "🦼",
    ":motorway:": "🛣️",
    ":mount_fuji:": "🗻",
    ":mountain:": "⛰️",
    ":mountain_cableway:": "🚠",
    ":mountain_railway:": "🚞",
    ":mountain_snow:": "🏔️",
    ":snow_capped_mountain:": "🏔️",
    ":mouse:": "🐭",
    ":mouse_three_button:": "🖱️",
    ":three_button_mouse:": "🖱️",
    ":mouse_trap:": "🪤",
    ":mouse2:": "🐁",
    ":movie_camera:": "🎥",
    ":moyai:": "🗿",
    ":mrs_claus:": "🤶",
    ":mother_christmas:": "🤶",
    ":mrs_claus_tone1:": "🤶🏻",
    ":mother_christmas_tone1:": "🤶🏻",
    ":mrs_claus::skin-tone-1:": "🤶🏻",
    ":mother_christmas::skin-tone-1:": "🤶🏻",
    ":mrs_claus_tone2:": "🤶🏼",
    ":mother_christmas_tone2:": "🤶🏼",
    ":mrs_claus::skin-tone-2:": "🤶🏼",
    ":mother_christmas::skin-tone-2:": "🤶🏼",
    ":mrs_claus_tone3:": "🤶🏽",
    ":mother_christmas_tone3:": "🤶🏽",
    ":mrs_claus::skin-tone-3:": "🤶🏽",
    ":mother_christmas::skin-tone-3:": "🤶🏽",
    ":mrs_claus_tone4:": "🤶🏾",
    ":mother_christmas_tone4:": "🤶🏾",
    ":mrs_claus::skin-tone-4:": "🤶🏾",
    ":mother_christmas::skin-tone-4:": "🤶🏾",
    ":mrs_claus_tone5:": "🤶🏿",
    ":mother_christmas_tone5:": "🤶🏿",
    ":mrs_claus::skin-tone-5:": "🤶🏿",
    ":mother_christmas::skin-tone-5:": "🤶🏿",
    ":muscle:": "💪",
    ":muscle_tone1:": "💪🏻",
    ":muscle::skin-tone-1:": "💪🏻",
    ":muscle_tone2:": "💪🏼",
    ":muscle::skin-tone-2:": "💪🏼",
    ":muscle_tone3:": "💪🏽",
    ":muscle::skin-tone-3:": "💪🏽",
    ":muscle_tone4:": "💪🏾",
    ":muscle::skin-tone-4:": "💪🏾",
    ":muscle_tone5:": "💪🏿",
    ":muscle::skin-tone-5:": "💪🏿",
    ":mushroom:": "🍄",
    ":musical_keyboard:": "🎹",
    ":musical_note:": "🎵",
    ":musical_score:": "🎼",
    ":mute:": "🔇",
    ":mx_claus:": "🧑‍🎄",
    ":mx_claus_tone1:": "🧑🏻‍🎄",
    ":mx_claus_light_skin_tone:": "🧑🏻‍🎄",
    ":mx_claus::skin-tone-1:": "🧑🏻‍🎄",
    ":mx_claus_tone2:": "🧑🏼‍🎄",
    ":mx_claus_medium_light_skin_tone:": "🧑🏼‍🎄",
    ":mx_claus::skin-tone-2:": "🧑🏼‍🎄",
    ":mx_claus_tone3:": "🧑🏽‍🎄",
    ":mx_claus_medium_skin_tone:": "🧑🏽‍🎄",
    ":mx_claus::skin-tone-3:": "🧑🏽‍🎄",
    ":mx_claus_tone4:": "🧑🏾‍🎄",
    ":mx_claus_medium_dark_skin_tone:": "🧑🏾‍🎄",
    ":mx_claus::skin-tone-4:": "🧑🏾‍🎄",
    ":mx_claus_tone5:": "🧑🏿‍🎄",
    ":mx_claus_dark_skin_tone:": "🧑🏿‍🎄",
    ":mx_claus::skin-tone-5:": "🧑🏿‍🎄",
    ":nail_care:": "💅",
    ":nail_care_tone1:": "💅🏻",
    ":nail_care::skin-tone-1:": "💅🏻",
    ":nail_care_tone2:": "💅🏼",
    ":nail_care::skin-tone-2:": "💅🏼",
    ":nail_care_tone3:": "💅🏽",
    ":nail_care::skin-tone-3:": "💅🏽",
    ":nail_care_tone4:": "💅🏾",
    ":nail_care::skin-tone-4:": "💅🏾",
    ":nail_care_tone5:": "💅🏿",
    ":nail_care::skin-tone-5:": "💅🏿",
    ":name_badge:": "📛",
    ":nauseated_face:": "🤢",
    ":sick:": "🤢",
    ":nazar_amulet:": "🧿",
    ":necktie:": "👔",
    ":negative_squared_cross_mark:": "❎",
    ":nerd:": "🤓",
    ":nerd_face:": "🤓",
    ":nesting_dolls:": "🪆",
    ":neutral_face:": "😐",
    ":|": "😐",
    ":-|": "😐",
    "|": "😐",
    "-|": "😐",
    ":new:": "🆕",
    ":new_moon:": "🌑",
    ":new_moon_with_face:": "🌚",
    ":newspaper:": "📰",
    ":newspaper2:": "🗞️",
    ":rolled_up_newspaper:": "🗞️",
    ":ng:": "🆖",
    ":night_with_stars:": "🌃",
    ":nine:": "9️⃣",
    ":ninja:": "🥷",
    ":ninja_tone1:": "🥷🏻",
    ":ninja_light_skin_tone:": "🥷🏻",
    ":ninja::skin-tone-1:": "🥷🏻",
    ":ninja_tone2:": "🥷🏼",
    ":ninja_medium_light_skin_tone:": "🥷🏼",
    ":ninja::skin-tone-2:": "🥷🏼",
    ":ninja_tone3:": "🥷🏽",
    ":ninja_medium_skin_tone:": "🥷🏽",
    ":ninja::skin-tone-3:": "🥷🏽",
    ":ninja_tone4:": "🥷🏾",
    ":ninja_medium_dark_skin_tone:": "🥷🏾",
    ":ninja::skin-tone-4:": "🥷🏾",
    ":ninja_tone5:": "🥷🏿",
    ":ninja_dark_skin_tone:": "🥷🏿",
    ":ninja::skin-tone-5:": "🥷🏿",
    ":no_bell:": "🔕",
    ":no_bicycles:": "🚳",
    ":no_entry:": "⛔",
    ":no_entry_sign:": "🚫",
    ":no_mobile_phones:": "📵",
    ":no_mouth:": "😶",
    ":no_pedestrians:": "🚷",
    ":no_smoking:": "🚭",
    ":non_potable_water:": "🚱",
    ":nose:": "👃",
    ":nose_tone1:": "👃🏻",
    ":nose::skin-tone-1:": "👃🏻",
    ":nose_tone2:": "👃🏼",
    ":nose::skin-tone-2:": "👃🏼",
    ":nose_tone3:": "👃🏽",
    ":nose::skin-tone-3:": "👃🏽",
    ":nose_tone4:": "👃🏾",
    ":nose::skin-tone-4:": "👃🏾",
    ":nose_tone5:": "👃🏿",
    ":nose::skin-tone-5:": "👃🏿",
    ":notebook:": "📓",
    ":notebook_with_decorative_cover:": "📔",
    ":notepad_spiral:": "🗒️",
    ":spiral_note_pad:": "🗒️",
    ":notes:": "🎶",
    ":nut_and_bolt:": "🔩",
    ":o:": "⭕",
    ":o2:": "🅾️",
    ":ocean:": "🌊",
    ":octagonal_sign:": "🛑",
    ":stop_sign:": "🛑",
    ":octopus:": "🐙",
    ":oden:": "🍢",
    ":office:": "🏢",
    ":office_worker:": "🧑‍💼",
    ":office_worker_tone1:": "🧑🏻‍💼",
    ":office_worker_light_skin_tone:": "🧑🏻‍💼",
    ":office_worker::skin-tone-1:": "🧑🏻‍💼",
    ":office_worker_tone2:": "🧑🏼‍💼",
    ":office_worker_medium_light_skin_tone:": "🧑🏼‍💼",
    ":office_worker::skin-tone-2:": "🧑🏼‍💼",
    ":office_worker_tone3:": "🧑🏽‍💼",
    ":office_worker_medium_skin_tone:": "🧑🏽‍💼",
    ":office_worker::skin-tone-3:": "🧑🏽‍💼",
    ":office_worker_tone4:": "🧑🏾‍💼",
    ":office_worker_medium_dark_skin_tone:": "🧑🏾‍💼",
    ":office_worker::skin-tone-4:": "🧑🏾‍💼",
    ":office_worker_tone5:": "🧑🏿‍💼",
    ":office_worker_dark_skin_tone:": "🧑🏿‍💼",
    ":office_worker::skin-tone-5:": "🧑🏿‍💼",
    ":oil:": "🛢️",
    ":oil_drum:": "🛢️",
    ":ok:": "🆗",
    ":ok_hand:": "👌",
    ":ok_hand_tone1:": "👌🏻",
    ":ok_hand::skin-tone-1:": "👌🏻",
    ":ok_hand_tone2:": "👌🏼",
    ":ok_hand::skin-tone-2:": "👌🏼",
    ":ok_hand_tone3:": "👌🏽",
    ":ok_hand::skin-tone-3:": "👌🏽",
    ":ok_hand_tone4:": "👌🏾",
    ":ok_hand::skin-tone-4:": "👌🏾",
    ":ok_hand_tone5:": "👌🏿",
    ":ok_hand::skin-tone-5:": "👌🏿",
    ":older_adult:": "🧓",
    ":older_adult_tone1:": "🧓🏻",
    ":older_adult_light_skin_tone:": "🧓🏻",
    ":older_adult::skin-tone-1:": "🧓🏻",
    ":older_adult_tone2:": "🧓🏼",
    ":older_adult_medium_light_skin_tone:": "🧓🏼",
    ":older_adult::skin-tone-2:": "🧓🏼",
    ":older_adult_tone3:": "🧓🏽",
    ":older_adult_medium_skin_tone:": "🧓🏽",
    ":older_adult::skin-tone-3:": "🧓🏽",
    ":older_adult_tone4:": "🧓🏾",
    ":older_adult_medium_dark_skin_tone:": "🧓🏾",
    ":older_adult::skin-tone-4:": "🧓🏾",
    ":older_adult_tone5:": "🧓🏿",
    ":older_adult_dark_skin_tone:": "🧓🏿",
    ":older_adult::skin-tone-5:": "🧓🏿",
    ":older_man:": "👴",
    ":older_man_tone1:": "👴🏻",
    ":older_man::skin-tone-1:": "👴🏻",
    ":older_man_tone2:": "👴🏼",
    ":older_man::skin-tone-2:": "👴🏼",
    ":older_man_tone3:": "👴🏽",
    ":older_man::skin-tone-3:": "👴🏽",
    ":older_man_tone4:": "👴🏾",
    ":older_man::skin-tone-4:": "👴🏾",
    ":older_man_tone5:": "👴🏿",
    ":older_man::skin-tone-5:": "👴🏿",
    ":older_woman:": "👵",
    ":grandma:": "👵",
    ":older_woman_tone1:": "👵🏻",
    ":grandma_tone1:": "👵🏻",
    ":older_woman::skin-tone-1:": "👵🏻",
    ":grandma::skin-tone-1:": "👵🏻",
    ":older_woman_tone2:": "👵🏼",
    ":grandma_tone2:": "👵🏼",
    ":older_woman::skin-tone-2:": "👵🏼",
    ":grandma::skin-tone-2:": "👵🏼",
    ":older_woman_tone3:": "👵🏽",
    ":grandma_tone3:": "👵🏽",
    ":older_woman::skin-tone-3:": "👵🏽",
    ":grandma::skin-tone-3:": "👵🏽",
    ":older_woman_tone4:": "👵🏾",
    ":grandma_tone4:": "👵🏾",
    ":older_woman::skin-tone-4:": "👵🏾",
    ":grandma::skin-tone-4:": "👵🏾",
    ":older_woman_tone5:": "👵🏿",
    ":grandma_tone5:": "👵🏿",
    ":older_woman::skin-tone-5:": "👵🏿",
    ":grandma::skin-tone-5:": "👵🏿",
    ":olive:": "🫒",
    ":om_symbol:": "🕉️",
    ":on:": "🔛",
    ":oncoming_automobile:": "🚘",
    ":oncoming_bus:": "🚍",
    ":oncoming_police_car:": "🚔",
    ":oncoming_taxi:": "🚖",
    ":one:": "1️⃣",
    ":one_piece_swimsuit:": "🩱",
    ":onion:": "🧅",
    ":open_file_folder:": "📂",
    ":open_hands:": "👐",
    ":open_hands_tone1:": "👐🏻",
    ":open_hands::skin-tone-1:": "👐🏻",
    ":open_hands_tone2:": "👐🏼",
    ":open_hands::skin-tone-2:": "👐🏼",
    ":open_hands_tone3:": "👐🏽",
    ":open_hands::skin-tone-3:": "👐🏽",
    ":open_hands_tone4:": "👐🏾",
    ":open_hands::skin-tone-4:": "👐🏾",
    ":open_hands_tone5:": "👐🏿",
    ":open_hands::skin-tone-5:": "👐🏿",
    ":open_mouth:": "😮",
    ":o": "😮",
    ":-o": "😮",
    ":O": "😮",
    ":-O": "😮",
    "o": "😮",
    "-o": "😮",
    "O": "😮",
    "-O": "😮",
    ":ophiuchus:": "⛎",
    ":orange_book:": "📙",
    ":orange_circle:": "🟠",
    ":orange_heart:": "🧡",
    ":orange_square:": "🟧",
    ":orangutan:": "🦧",
    ":orthodox_cross:": "☦️",
    ":otter:": "🦦",
    ":outbox_tray:": "📤",
    ":owl:": "🦉",
    ":ox:": "🐂",
    ":oyster:": "🦪",
    ":package:": "📦",
    ":page_facing_up:": "📄",
    ":page_with_curl:": "📃",
    ":pager:": "📟",
    ":paintbrush:": "🖌️",
    ":lower_left_paintbrush:": "🖌️",
    ":palm_tree:": "🌴",
    ":palms_up_together:": "🤲",
    ":palms_up_together_tone1:": "🤲🏻",
    ":palms_up_together_light_skin_tone:": "🤲🏻",
    ":palms_up_together::skin-tone-1:": "🤲🏻",
    ":palms_up_together_tone2:": "🤲🏼",
    ":palms_up_together_medium_light_skin_tone:": "🤲🏼",
    ":palms_up_together::skin-tone-2:": "🤲🏼",
    ":palms_up_together_tone3:": "🤲🏽",
    ":palms_up_together_medium_skin_tone:": "🤲🏽",
    ":palms_up_together::skin-tone-3:": "🤲🏽",
    ":palms_up_together_tone4:": "🤲🏾",
    ":palms_up_together_medium_dark_skin_tone:": "🤲🏾",
    ":palms_up_together::skin-tone-4:": "🤲🏾",
    ":palms_up_together_tone5:": "🤲🏿",
    ":palms_up_together_dark_skin_tone:": "🤲🏿",
    ":palms_up_together::skin-tone-5:": "🤲🏿",
    ":pancakes:": "🥞",
    ":panda_face:": "🐼",
    ":paperclip:": "📎",
    ":paperclips:": "🖇️",
    ":linked_paperclips:": "🖇️",
    ":parachute:": "🪂",
    ":park:": "🏞️",
    ":national_park:": "🏞️",
    ":parking:": "🅿️",
    ":parrot:": "🦜",
    ":part_alternation_mark:": "〽️",
    ":partly_sunny:": "⛅",
    ":partying_face:": "🥳",
    ":passport_control:": "🛂",
    ":pause_button:": "⏸️",
    ":double_vertical_bar:": "⏸️",
    ":peace:": "☮️",
    ":peace_symbol:": "☮️",
    ":peach:": "🍑",
    ":peacock:": "🦚",
    ":peanuts:": "🥜",
    ":shelled_peanut:": "🥜",
    ":pear:": "🍐",
    ":pen_ballpoint:": "🖊️",
    ":lower_left_ballpoint_pen:": "🖊️",
    ":pen_fountain:": "🖋️",
    ":lower_left_fountain_pen:": "🖋️",
    ":pencil:": "📝",
    ":memo:": "📝",
    ":pencil2:": "✏️",
    ":penguin:": "🐧",
    ":pensive:": "😔",
    ":people_holding_hands:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone1:": "🧑‍🤝‍🧑",
    ":people_holding_hands_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone1_tone2:": "🧑‍🤝‍🧑",
    ":people_holding_hands_light_skin_tone_medium_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone1_tone3:": "🧑‍🤝‍🧑",
    ":people_holding_hands_light_skin_tone_medium_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone1_tone4:": "🧑‍🤝‍🧑",
    ":people_holding_hands_light_skin_tone_medium_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone1_tone5:": "🧑‍🤝‍🧑",
    ":people_holding_hands_light_skin_tone_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone2:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone2_tone1:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_light_skin_tone_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone2_tone3:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_light_skin_tone_medium_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone2_tone4:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone2_tone5:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_light_skin_tone_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone3:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone3_tone1:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_skin_tone_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone3_tone2:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_skin_tone_medium_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone3_tone4:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_skin_tone_medium_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone3_tone5:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_skin_tone_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone4:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone4_tone1:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_dark_skin_tone_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone4_tone2:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone4_tone3:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_dark_skin_tone_medium_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone4_tone5:": "🧑‍🤝‍🧑",
    ":people_holding_hands_medium_dark_skin_tone_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone5:": "🧑‍🤝‍🧑",
    ":people_holding_hands_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone5_tone1:": "🧑‍🤝‍🧑",
    ":people_holding_hands_dark_skin_tone_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone5_tone2:": "🧑‍🤝‍🧑",
    ":people_holding_hands_dark_skin_tone_medium_light_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone5_tone3:": "🧑‍🤝‍🧑",
    ":people_holding_hands_dark_skin_tone_medium_skin_tone:": "🧑‍🤝‍🧑",
    ":people_holding_hands_tone5_tone4:": "🧑‍🤝‍🧑",
    ":people_holding_hands_dark_skin_tone_medium_dark_skin_tone:": "🧑‍🤝‍🧑",
    ":people_hugging:": "🫂",
    ":people_with_bunny_ears_partying:": "👯",
    ":dancers:": "👯",
    ":people_wrestling:": "🤼",
    ":wrestlers:": "🤼",
    ":wrestling:": "🤼",
    ":performing_arts:": "🎭",
    ":persevere:": "😣",
    ":person_bald:": "🧑‍🦲",
    ":person_biking:": "🚴",
    ":bicyclist:": "🚴",
    ":person_biking_tone1:": "🚴🏻",
    ":bicyclist_tone1:": "🚴🏻",
    ":person_biking::skin-tone-1:": "🚴🏻",
    ":bicyclist::skin-tone-1:": "🚴🏻",
    ":person_biking_tone2:": "🚴🏼",
    ":bicyclist_tone2:": "🚴🏼",
    ":person_biking::skin-tone-2:": "🚴🏼",
    ":bicyclist::skin-tone-2:": "🚴🏼",
    ":person_biking_tone3:": "🚴🏽",
    ":bicyclist_tone3:": "🚴🏽",
    ":person_biking::skin-tone-3:": "🚴🏽",
    ":bicyclist::skin-tone-3:": "🚴🏽",
    ":person_biking_tone4:": "🚴🏾",
    ":bicyclist_tone4:": "🚴🏾",
    ":person_biking::skin-tone-4:": "🚴🏾",
    ":bicyclist::skin-tone-4:": "🚴🏾",
    ":person_biking_tone5:": "🚴🏿",
    ":bicyclist_tone5:": "🚴🏿",
    ":person_biking::skin-tone-5:": "🚴🏿",
    ":bicyclist::skin-tone-5:": "🚴🏿",
    ":person_bouncing_ball:": "⛹️",
    ":basketball_player:": "⛹️",
    ":person_with_ball:": "⛹️",
    ":person_bouncing_ball_tone1:": "⛹🏻",
    ":basketball_player_tone1:": "⛹🏻",
    ":person_with_ball_tone1:": "⛹🏻",
    ":person_bouncing_ball::skin-tone-1:": "⛹🏻",
    ":basketball_player::skin-tone-1:": "⛹🏻",
    ":person_with_ball::skin-tone-1:": "⛹🏻",
    ":person_bouncing_ball_tone2:": "⛹🏼",
    ":basketball_player_tone2:": "⛹🏼",
    ":person_with_ball_tone2:": "⛹🏼",
    ":person_bouncing_ball::skin-tone-2:": "⛹🏼",
    ":basketball_player::skin-tone-2:": "⛹🏼",
    ":person_with_ball::skin-tone-2:": "⛹🏼",
    ":person_bouncing_ball_tone3:": "⛹🏽",
    ":basketball_player_tone3:": "⛹🏽",
    ":person_with_ball_tone3:": "⛹🏽",
    ":person_bouncing_ball::skin-tone-3:": "⛹🏽",
    ":basketball_player::skin-tone-3:": "⛹🏽",
    ":person_with_ball::skin-tone-3:": "⛹🏽",
    ":person_bouncing_ball_tone4:": "⛹🏾",
    ":basketball_player_tone4:": "⛹🏾",
    ":person_with_ball_tone4:": "⛹🏾",
    ":person_bouncing_ball::skin-tone-4:": "⛹🏾",
    ":basketball_player::skin-tone-4:": "⛹🏾",
    ":person_with_ball::skin-tone-4:": "⛹🏾",
    ":person_bouncing_ball_tone5:": "⛹🏿",
    ":basketball_player_tone5:": "⛹🏿",
    ":person_with_ball_tone5:": "⛹🏿",
    ":person_bouncing_ball::skin-tone-5:": "⛹🏿",
    ":basketball_player::skin-tone-5:": "⛹🏿",
    ":person_with_ball::skin-tone-5:": "⛹🏿",
    ":person_bowing:": "🙇",
    ":bow:": "🙇",
    ":person_bowing_tone1:": "🙇🏻",
    ":bow_tone1:": "🙇🏻",
    ":person_bowing::skin-tone-1:": "🙇🏻",
    ":bow::skin-tone-1:": "🙇🏻",
    ":person_bowing_tone2:": "🙇🏼",
    ":bow_tone2:": "🙇🏼",
    ":person_bowing::skin-tone-2:": "🙇🏼",
    ":bow::skin-tone-2:": "🙇🏼",
    ":person_bowing_tone3:": "🙇🏽",
    ":bow_tone3:": "🙇🏽",
    ":person_bowing::skin-tone-3:": "🙇🏽",
    ":bow::skin-tone-3:": "🙇🏽",
    ":person_bowing_tone4:": "🙇🏾",
    ":bow_tone4:": "🙇🏾",
    ":person_bowing::skin-tone-4:": "🙇🏾",
    ":bow::skin-tone-4:": "🙇🏾",
    ":person_bowing_tone5:": "🙇🏿",
    ":bow_tone5:": "🙇🏿",
    ":person_bowing::skin-tone-5:": "🙇🏿",
    ":bow::skin-tone-5:": "🙇🏿",
    ":person_climbing:": "🧗",
    ":person_climbing_tone1:": "🧗🏻",
    ":person_climbing_light_skin_tone:": "🧗🏻",
    ":person_climbing::skin-tone-1:": "🧗🏻",
    ":person_climbing_tone2:": "🧗🏼",
    ":person_climbing_medium_light_skin_tone:": "🧗🏼",
    ":person_climbing::skin-tone-2:": "🧗🏼",
    ":person_climbing_tone3:": "🧗🏽",
    ":person_climbing_medium_skin_tone:": "🧗🏽",
    ":person_climbing::skin-tone-3:": "🧗🏽",
    ":person_climbing_tone4:": "🧗🏾",
    ":person_climbing_medium_dark_skin_tone:": "🧗🏾",
    ":person_climbing::skin-tone-4:": "🧗🏾",
    ":person_climbing_tone5:": "🧗🏿",
    ":person_climbing_dark_skin_tone:": "🧗🏿",
    ":person_climbing::skin-tone-5:": "🧗🏿",
    ":person_curly_hair:": "🧑‍🦱",
    ":person_doing_cartwheel:": "🤸",
    ":cartwheel:": "🤸",
    ":person_doing_cartwheel_tone1:": "🤸🏻",
    ":cartwheel_tone1:": "🤸🏻",
    ":person_doing_cartwheel::skin-tone-1:": "🤸🏻",
    ":cartwheel::skin-tone-1:": "🤸🏻",
    ":person_doing_cartwheel_tone2:": "🤸🏼",
    ":cartwheel_tone2:": "🤸🏼",
    ":person_doing_cartwheel::skin-tone-2:": "🤸🏼",
    ":cartwheel::skin-tone-2:": "🤸🏼",
    ":person_doing_cartwheel_tone3:": "🤸🏽",
    ":cartwheel_tone3:": "🤸🏽",
    ":person_doing_cartwheel::skin-tone-3:": "🤸🏽",
    ":cartwheel::skin-tone-3:": "🤸🏽",
    ":person_doing_cartwheel_tone4:": "🤸🏾",
    ":cartwheel_tone4:": "🤸🏾",
    ":person_doing_cartwheel::skin-tone-4:": "🤸🏾",
    ":cartwheel::skin-tone-4:": "🤸🏾",
    ":person_doing_cartwheel_tone5:": "🤸🏿",
    ":cartwheel_tone5:": "🤸🏿",
    ":person_doing_cartwheel::skin-tone-5:": "🤸🏿",
    ":cartwheel::skin-tone-5:": "🤸🏿",
    ":person_facepalming:": "🤦",
    ":face_palm:": "🤦",
    ":facepalm:": "🤦",
    ":person_facepalming_tone1:": "🤦🏻",
    ":face_palm_tone1:": "🤦🏻",
    ":facepalm_tone1:": "🤦🏻",
    ":person_facepalming::skin-tone-1:": "🤦🏻",
    ":face_palm::skin-tone-1:": "🤦🏻",
    ":facepalm::skin-tone-1:": "🤦🏻",
    ":person_facepalming_tone2:": "🤦🏼",
    ":face_palm_tone2:": "🤦🏼",
    ":facepalm_tone2:": "🤦🏼",
    ":person_facepalming::skin-tone-2:": "🤦🏼",
    ":face_palm::skin-tone-2:": "🤦🏼",
    ":facepalm::skin-tone-2:": "🤦🏼",
    ":person_facepalming_tone3:": "🤦🏽",
    ":face_palm_tone3:": "🤦🏽",
    ":facepalm_tone3:": "🤦🏽",
    ":person_facepalming::skin-tone-3:": "🤦🏽",
    ":face_palm::skin-tone-3:": "🤦🏽",
    ":facepalm::skin-tone-3:": "🤦🏽",
    ":person_facepalming_tone4:": "🤦🏾",
    ":face_palm_tone4:": "🤦🏾",
    ":facepalm_tone4:": "🤦🏾",
    ":person_facepalming::skin-tone-4:": "🤦🏾",
    ":face_palm::skin-tone-4:": "🤦🏾",
    ":facepalm::skin-tone-4:": "🤦🏾",
    ":person_facepalming_tone5:": "🤦🏿",
    ":face_palm_tone5:": "🤦🏿",
    ":facepalm_tone5:": "🤦🏿",
    ":person_facepalming::skin-tone-5:": "🤦🏿",
    ":face_palm::skin-tone-5:": "🤦🏿",
    ":facepalm::skin-tone-5:": "🤦🏿",
    ":person_feeding_baby:": "🧑‍🍼",
    ":person_feeding_baby_tone1:": "🧑🏻‍🍼",
    ":person_feeding_baby_light_skin_tone:": "🧑🏻‍🍼",
    ":person_feeding_baby::skin-tone-1:": "🧑🏻‍🍼",
    ":person_feeding_baby_tone2:": "🧑🏼‍🍼",
    ":person_feeding_baby_medium_light_skin_tone:": "🧑🏼‍🍼",
    ":person_feeding_baby::skin-tone-2:": "🧑🏼‍🍼",
    ":person_feeding_baby_tone3:": "🧑🏽‍🍼",
    ":person_feeding_baby_medium_skin_tone:": "🧑🏽‍🍼",
    ":person_feeding_baby::skin-tone-3:": "🧑🏽‍🍼",
    ":person_feeding_baby_tone4:": "🧑🏾‍🍼",
    ":person_feeding_baby_medium_dark_skin_tone:": "🧑🏾‍🍼",
    ":person_feeding_baby::skin-tone-4:": "🧑🏾‍🍼",
    ":person_feeding_baby_tone5:": "🧑🏿‍🍼",
    ":person_feeding_baby_dark_skin_tone:": "🧑🏿‍🍼",
    ":person_feeding_baby::skin-tone-5:": "🧑🏿‍🍼",
    ":person_fencing:": "🤺",
    ":fencer:": "🤺",
    ":fencing:": "🤺",
    ":person_frowning:": "🙍",
    ":person_frowning_tone1:": "🙍🏻",
    ":person_frowning::skin-tone-1:": "🙍🏻",
    ":person_frowning_tone2:": "🙍🏼",
    ":person_frowning::skin-tone-2:": "🙍🏼",
    ":person_frowning_tone3:": "🙍🏽",
    ":person_frowning::skin-tone-3:": "🙍🏽",
    ":person_frowning_tone4:": "🙍🏾",
    ":person_frowning::skin-tone-4:": "🙍🏾",
    ":person_frowning_tone5:": "🙍🏿",
    ":person_frowning::skin-tone-5:": "🙍🏿",
    ":person_gesturing_no:": "🙅",
    ":no_good:": "🙅",
    ":person_gesturing_no_tone1:": "🙅🏻",
    ":no_good_tone1:": "🙅🏻",
    ":person_gesturing_no::skin-tone-1:": "🙅🏻",
    ":no_good::skin-tone-1:": "🙅🏻",
    ":person_gesturing_no_tone2:": "🙅🏼",
    ":no_good_tone2:": "🙅🏼",
    ":person_gesturing_no::skin-tone-2:": "🙅🏼",
    ":no_good::skin-tone-2:": "🙅🏼",
    ":person_gesturing_no_tone3:": "🙅🏽",
    ":no_good_tone3:": "🙅🏽",
    ":person_gesturing_no::skin-tone-3:": "🙅🏽",
    ":no_good::skin-tone-3:": "🙅🏽",
    ":person_gesturing_no_tone4:": "🙅🏾",
    ":no_good_tone4:": "🙅🏾",
    ":person_gesturing_no::skin-tone-4:": "🙅🏾",
    ":no_good::skin-tone-4:": "🙅🏾",
    ":person_gesturing_no_tone5:": "🙅🏿",
    ":no_good_tone5:": "🙅🏿",
    ":person_gesturing_no::skin-tone-5:": "🙅🏿",
    ":no_good::skin-tone-5:": "🙅🏿",
    ":person_gesturing_ok:": "🙆",
    ":ok_woman:": "🙆",
    ":person_gesturing_ok_tone1:": "🙆🏻",
    ":ok_woman_tone1:": "🙆🏻",
    ":person_gesturing_ok::skin-tone-1:": "🙆🏻",
    ":ok_woman::skin-tone-1:": "🙆🏻",
    ":person_gesturing_ok_tone2:": "🙆🏼",
    ":ok_woman_tone2:": "🙆🏼",
    ":person_gesturing_ok::skin-tone-2:": "🙆🏼",
    ":ok_woman::skin-tone-2:": "🙆🏼",
    ":person_gesturing_ok_tone3:": "🙆🏽",
    ":ok_woman_tone3:": "🙆🏽",
    ":person_gesturing_ok::skin-tone-3:": "🙆🏽",
    ":ok_woman::skin-tone-3:": "🙆🏽",
    ":person_gesturing_ok_tone4:": "🙆🏾",
    ":ok_woman_tone4:": "🙆🏾",
    ":person_gesturing_ok::skin-tone-4:": "🙆🏾",
    ":ok_woman::skin-tone-4:": "🙆🏾",
    ":person_gesturing_ok_tone5:": "🙆🏿",
    ":ok_woman_tone5:": "🙆🏿",
    ":person_gesturing_ok::skin-tone-5:": "🙆🏿",
    ":ok_woman::skin-tone-5:": "🙆🏿",
    ":person_getting_haircut:": "💇",
    ":haircut:": "💇",
    ":person_getting_haircut_tone1:": "💇🏻",
    ":haircut_tone1:": "💇🏻",
    ":person_getting_haircut::skin-tone-1:": "💇🏻",
    ":haircut::skin-tone-1:": "💇🏻",
    ":person_getting_haircut_tone2:": "💇🏼",
    ":haircut_tone2:": "💇🏼",
    ":person_getting_haircut::skin-tone-2:": "💇🏼",
    ":haircut::skin-tone-2:": "💇🏼",
    ":person_getting_haircut_tone3:": "💇🏽",
    ":haircut_tone3:": "💇🏽",
    ":person_getting_haircut::skin-tone-3:": "💇🏽",
    ":haircut::skin-tone-3:": "💇🏽",
    ":person_getting_haircut_tone4:": "💇🏾",
    ":haircut_tone4:": "💇🏾",
    ":person_getting_haircut::skin-tone-4:": "💇🏾",
    ":haircut::skin-tone-4:": "💇🏾",
    ":person_getting_haircut_tone5:": "💇🏿",
    ":haircut_tone5:": "💇🏿",
    ":person_getting_haircut::skin-tone-5:": "💇🏿",
    ":haircut::skin-tone-5:": "💇🏿",
    ":person_getting_massage:": "💆",
    ":massage:": "💆",
    ":person_getting_massage_tone1:": "💆🏻",
    ":massage_tone1:": "💆🏻",
    ":person_getting_massage::skin-tone-1:": "💆🏻",
    ":massage::skin-tone-1:": "💆🏻",
    ":person_getting_massage_tone2:": "💆🏼",
    ":massage_tone2:": "💆🏼",
    ":person_getting_massage::skin-tone-2:": "💆🏼",
    ":massage::skin-tone-2:": "💆🏼",
    ":person_getting_massage_tone3:": "💆🏽",
    ":massage_tone3:": "💆🏽",
    ":person_getting_massage::skin-tone-3:": "💆🏽",
    ":massage::skin-tone-3:": "💆🏽",
    ":person_getting_massage_tone4:": "💆🏾",
    ":massage_tone4:": "💆🏾",
    ":person_getting_massage::skin-tone-4:": "💆🏾",
    ":massage::skin-tone-4:": "💆🏾",
    ":person_getting_massage_tone5:": "💆🏿",
    ":massage_tone5:": "💆🏿",
    ":person_getting_massage::skin-tone-5:": "💆🏿",
    ":massage::skin-tone-5:": "💆🏿",
    ":person_golfing:": "🏌️",
    ":golfer:": "🏌️",
    ":person_golfing_tone1:": "🏌🏻",
    ":person_golfing_light_skin_tone:": "🏌🏻",
    ":person_golfing::skin-tone-1:": "🏌🏻",
    ":golfer::skin-tone-1:": "🏌🏻",
    ":person_golfing_tone2:": "🏌🏼",
    ":person_golfing_medium_light_skin_tone:": "🏌🏼",
    ":person_golfing::skin-tone-2:": "🏌🏼",
    ":golfer::skin-tone-2:": "🏌🏼",
    ":person_golfing_tone3:": "🏌🏽",
    ":person_golfing_medium_skin_tone:": "🏌🏽",
    ":person_golfing::skin-tone-3:": "🏌🏽",
    ":golfer::skin-tone-3:": "🏌🏽",
    ":person_golfing_tone4:": "🏌🏾",
    ":person_golfing_medium_dark_skin_tone:": "🏌🏾",
    ":person_golfing::skin-tone-4:": "🏌🏾",
    ":golfer::skin-tone-4:": "🏌🏾",
    ":person_golfing_tone5:": "🏌🏿",
    ":person_golfing_dark_skin_tone:": "🏌🏿",
    ":person_golfing::skin-tone-5:": "🏌🏿",
    ":golfer::skin-tone-5:": "🏌🏿",
    ":person_in_bed_tone1:": "🛌🏻",
    ":person_in_bed_light_skin_tone:": "🛌🏻",
    ":sleeping_accommodation::skin-tone-1:": "🛌🏻",
    ":person_in_bed_tone2:": "🛌🏼",
    ":person_in_bed_medium_light_skin_tone:": "🛌🏼",
    ":sleeping_accommodation::skin-tone-2:": "🛌🏼",
    ":person_in_bed_tone3:": "🛌🏽",
    ":person_in_bed_medium_skin_tone:": "🛌🏽",
    ":sleeping_accommodation::skin-tone-3:": "🛌🏽",
    ":person_in_bed_tone4:": "🛌🏾",
    ":person_in_bed_medium_dark_skin_tone:": "🛌🏾",
    ":sleeping_accommodation::skin-tone-4:": "🛌🏾",
    ":person_in_bed_tone5:": "🛌🏿",
    ":person_in_bed_dark_skin_tone:": "🛌🏿",
    ":sleeping_accommodation::skin-tone-5:": "🛌🏿",
    ":person_in_lotus_position:": "🧘",
    ":person_in_lotus_position_tone1:": "🧘🏻",
    ":person_in_lotus_position_light_skin_tone:": "🧘🏻",
    ":person_in_lotus_position::skin-tone-1:": "🧘🏻",
    ":person_in_lotus_position_tone2:": "🧘🏼",
    ":person_in_lotus_position_medium_light_skin_tone:": "🧘🏼",
    ":person_in_lotus_position::skin-tone-2:": "🧘🏼",
    ":person_in_lotus_position_tone3:": "🧘🏽",
    ":person_in_lotus_position_medium_skin_tone:": "🧘🏽",
    ":person_in_lotus_position::skin-tone-3:": "🧘🏽",
    ":person_in_lotus_position_tone4:": "🧘🏾",
    ":person_in_lotus_position_medium_dark_skin_tone:": "🧘🏾",
    ":person_in_lotus_position::skin-tone-4:": "🧘🏾",
    ":person_in_lotus_position_tone5:": "🧘🏿",
    ":person_in_lotus_position_dark_skin_tone:": "🧘🏿",
    ":person_in_lotus_position::skin-tone-5:": "🧘🏿",
    ":person_in_manual_wheelchair:": "🧑‍🦽",
    ":person_in_manual_wheelchair_tone1:": "🧑🏻‍🦽",
    ":person_in_manual_wheelchair_light_skin_tone:": "🧑🏻‍🦽",
    ":person_in_manual_wheelchair::skin-tone-1:": "🧑🏻‍🦽",
    ":person_in_manual_wheelchair_tone2:": "🧑🏼‍🦽",
    ":person_in_manual_wheelchair_medium_light_skin_tone:": "🧑🏼‍🦽",
    ":person_in_manual_wheelchair::skin-tone-2:": "🧑🏼‍🦽",
    ":person_in_manual_wheelchair_tone3:": "🧑🏽‍🦽",
    ":person_in_manual_wheelchair_medium_skin_tone:": "🧑🏽‍🦽",
    ":person_in_manual_wheelchair::skin-tone-3:": "🧑🏽‍🦽",
    ":person_in_manual_wheelchair_tone4:": "🧑🏾‍🦽",
    ":person_in_manual_wheelchair_medium_dark_skin_tone:": "🧑🏾‍🦽",
    ":person_in_manual_wheelchair::skin-tone-4:": "🧑🏾‍🦽",
    ":person_in_manual_wheelchair_tone5:": "🧑🏿‍🦽",
    ":person_in_manual_wheelchair_dark_skin_tone:": "🧑🏿‍🦽",
    ":person_in_manual_wheelchair::skin-tone-5:": "🧑🏿‍🦽",
    ":person_in_motorized_wheelchair:": "🧑‍🦼",
    ":person_in_motorized_wheelchair_tone1:": "🧑🏻‍🦼",
    ":person_in_motorized_wheelchair_light_skin_tone:": "🧑🏻‍🦼",
    ":person_in_motorized_wheelchair::skin-tone-1:": "🧑🏻‍🦼",
    ":person_in_motorized_wheelchair_tone2:": "🧑🏼‍🦼",
    ":person_in_motorized_wheelchair_medium_light_skin_tone:": "🧑🏼‍🦼",
    ":person_in_motorized_wheelchair::skin-tone-2:": "🧑🏼‍🦼",
    ":person_in_motorized_wheelchair_tone3:": "🧑🏽‍🦼",
    ":person_in_motorized_wheelchair_medium_skin_tone:": "🧑🏽‍🦼",
    ":person_in_motorized_wheelchair::skin-tone-3:": "🧑🏽‍🦼",
    ":person_in_motorized_wheelchair_tone4:": "🧑🏾‍🦼",
    ":person_in_motorized_wheelchair_medium_dark_skin_tone:": "🧑🏾‍🦼",
    ":person_in_motorized_wheelchair::skin-tone-4:": "🧑🏾‍🦼",
    ":person_in_motorized_wheelchair_tone5:": "🧑🏿‍🦼",
    ":person_in_motorized_wheelchair_dark_skin_tone:": "🧑🏿‍🦼",
    ":person_in_motorized_wheelchair::skin-tone-5:": "🧑🏿‍🦼",
    ":person_in_steamy_room:": "🧖",
    ":person_in_steamy_room_tone1:": "🧖🏻",
    ":person_in_steamy_room_light_skin_tone:": "🧖🏻",
    ":person_in_steamy_room::skin-tone-1:": "🧖🏻",
    ":person_in_steamy_room_tone2:": "🧖🏼",
    ":person_in_steamy_room_medium_light_skin_tone:": "🧖🏼",
    ":person_in_steamy_room::skin-tone-2:": "🧖🏼",
    ":person_in_steamy_room_tone3:": "🧖🏽",
    ":person_in_steamy_room_medium_skin_tone:": "🧖🏽",
    ":person_in_steamy_room::skin-tone-3:": "🧖🏽",
    ":person_in_steamy_room_tone4:": "🧖🏾",
    ":person_in_steamy_room_medium_dark_skin_tone:": "🧖🏾",
    ":person_in_steamy_room::skin-tone-4:": "🧖🏾",
    ":person_in_steamy_room_tone5:": "🧖🏿",
    ":person_in_steamy_room_dark_skin_tone:": "🧖🏿",
    ":person_in_steamy_room::skin-tone-5:": "🧖🏿",
    ":person_in_tuxedo:": "🤵",
    ":person_in_tuxedo_tone1:": "🤵🏻",
    ":tuxedo_tone1:": "🤵🏻",
    ":person_in_tuxedo::skin-tone-1:": "🤵🏻",
    ":person_in_tuxedo_tone2:": "🤵🏼",
    ":tuxedo_tone2:": "🤵🏼",
    ":person_in_tuxedo::skin-tone-2:": "🤵🏼",
    ":person_in_tuxedo_tone3:": "🤵🏽",
    ":tuxedo_tone3:": "🤵🏽",
    ":person_in_tuxedo::skin-tone-3:": "🤵🏽",
    ":person_in_tuxedo_tone4:": "🤵🏾",
    ":tuxedo_tone4:": "🤵🏾",
    ":person_in_tuxedo::skin-tone-4:": "🤵🏾",
    ":person_in_tuxedo_tone5:": "🤵🏿",
    ":tuxedo_tone5:": "🤵🏿",
    ":person_in_tuxedo::skin-tone-5:": "🤵🏿",
    ":person_juggling:": "🤹",
    ":juggling:": "🤹",
    ":juggler:": "🤹",
    ":person_juggling_tone1:": "🤹🏻",
    ":juggling_tone1:": "🤹🏻",
    ":juggler_tone1:": "🤹🏻",
    ":person_juggling::skin-tone-1:": "🤹🏻",
    ":juggling::skin-tone-1:": "🤹🏻",
    ":juggler::skin-tone-1:": "🤹🏻",
    ":person_juggling_tone2:": "🤹🏼",
    ":juggling_tone2:": "🤹🏼",
    ":juggler_tone2:": "🤹🏼",
    ":person_juggling::skin-tone-2:": "🤹🏼",
    ":juggling::skin-tone-2:": "🤹🏼",
    ":juggler::skin-tone-2:": "🤹🏼",
    ":person_juggling_tone3:": "🤹🏽",
    ":juggling_tone3:": "🤹🏽",
    ":juggler_tone3:": "🤹🏽",
    ":person_juggling::skin-tone-3:": "🤹🏽",
    ":juggling::skin-tone-3:": "🤹🏽",
    ":juggler::skin-tone-3:": "🤹🏽",
    ":person_juggling_tone4:": "🤹🏾",
    ":juggling_tone4:": "🤹🏾",
    ":juggler_tone4:": "🤹🏾",
    ":person_juggling::skin-tone-4:": "🤹🏾",
    ":juggling::skin-tone-4:": "🤹🏾",
    ":juggler::skin-tone-4:": "🤹🏾",
    ":person_juggling_tone5:": "🤹🏿",
    ":juggling_tone5:": "🤹🏿",
    ":juggler_tone5:": "🤹🏿",
    ":person_juggling::skin-tone-5:": "🤹🏿",
    ":juggling::skin-tone-5:": "🤹🏿",
    ":juggler::skin-tone-5:": "🤹🏿",
    ":person_kneeling:": "🧎",
    ":person_kneeling_tone1:": "🧎🏻",
    ":person_kneeling_light_skin_tone:": "🧎🏻",
    ":person_kneeling::skin-tone-1:": "🧎🏻",
    ":person_kneeling_tone2:": "🧎🏼",
    ":person_kneeling_medium_light_skin_tone:": "🧎🏼",
    ":person_kneeling::skin-tone-2:": "🧎🏼",
    ":person_kneeling_tone3:": "🧎🏽",
    ":person_kneeling_medium_skin_tone:": "🧎🏽",
    ":person_kneeling::skin-tone-3:": "🧎🏽",
    ":person_kneeling_tone4:": "🧎🏾",
    ":person_kneeling_medium_dark_skin_tone:": "🧎🏾",
    ":person_kneeling::skin-tone-4:": "🧎🏾",
    ":person_kneeling_tone5:": "🧎🏿",
    ":person_kneeling_dark_skin_tone:": "🧎🏿",
    ":person_kneeling::skin-tone-5:": "🧎🏿",
    ":person_lifting_weights:": "🏋️",
    ":lifter:": "🏋️",
    ":weight_lifter:": "🏋️",
    ":person_lifting_weights_tone1:": "🏋🏻",
    ":lifter_tone1:": "🏋🏻",
    ":weight_lifter_tone1:": "🏋🏻",
    ":person_lifting_weights::skin-tone-1:": "🏋🏻",
    ":lifter::skin-tone-1:": "🏋🏻",
    ":weight_lifter::skin-tone-1:": "🏋🏻",
    ":person_lifting_weights_tone2:": "🏋🏼",
    ":lifter_tone2:": "🏋🏼",
    ":weight_lifter_tone2:": "🏋🏼",
    ":person_lifting_weights::skin-tone-2:": "🏋🏼",
    ":lifter::skin-tone-2:": "🏋🏼",
    ":weight_lifter::skin-tone-2:": "🏋🏼",
    ":person_lifting_weights_tone3:": "🏋🏽",
    ":lifter_tone3:": "🏋🏽",
    ":weight_lifter_tone3:": "🏋🏽",
    ":person_lifting_weights::skin-tone-3:": "🏋🏽",
    ":lifter::skin-tone-3:": "🏋🏽",
    ":weight_lifter::skin-tone-3:": "🏋🏽",
    ":person_lifting_weights_tone4:": "🏋🏾",
    ":lifter_tone4:": "🏋🏾",
    ":weight_lifter_tone4:": "🏋🏾",
    ":person_lifting_weights::skin-tone-4:": "🏋🏾",
    ":lifter::skin-tone-4:": "🏋🏾",
    ":weight_lifter::skin-tone-4:": "🏋🏾",
    ":person_lifting_weights_tone5:": "🏋🏿",
    ":lifter_tone5:": "🏋🏿",
    ":weight_lifter_tone5:": "🏋🏿",
    ":person_lifting_weights::skin-tone-5:": "🏋🏿",
    ":lifter::skin-tone-5:": "🏋🏿",
    ":weight_lifter::skin-tone-5:": "🏋🏿",
    ":person_mountain_biking:": "🚵",
    ":mountain_bicyclist:": "🚵",
    ":person_mountain_biking_tone1:": "🚵🏻",
    ":mountain_bicyclist_tone1:": "🚵🏻",
    ":person_mountain_biking::skin-tone-1:": "🚵🏻",
    ":mountain_bicyclist::skin-tone-1:": "🚵🏻",
    ":person_mountain_biking_tone2:": "🚵🏼",
    ":mountain_bicyclist_tone2:": "🚵🏼",
    ":person_mountain_biking::skin-tone-2:": "🚵🏼",
    ":mountain_bicyclist::skin-tone-2:": "🚵🏼",
    ":person_mountain_biking_tone3:": "🚵🏽",
    ":mountain_bicyclist_tone3:": "🚵🏽",
    ":person_mountain_biking::skin-tone-3:": "🚵🏽",
    ":mountain_bicyclist::skin-tone-3:": "🚵🏽",
    ":person_mountain_biking_tone4:": "🚵🏾",
    ":mountain_bicyclist_tone4:": "🚵🏾",
    ":person_mountain_biking::skin-tone-4:": "🚵🏾",
    ":mountain_bicyclist::skin-tone-4:": "🚵🏾",
    ":person_mountain_biking_tone5:": "🚵🏿",
    ":mountain_bicyclist_tone5:": "🚵🏿",
    ":person_mountain_biking::skin-tone-5:": "🚵🏿",
    ":mountain_bicyclist::skin-tone-5:": "🚵🏿",
    ":person_playing_handball:": "🤾",
    ":handball:": "🤾",
    ":person_playing_handball_tone1:": "🤾🏻",
    ":handball_tone1:": "🤾🏻",
    ":person_playing_handball::skin-tone-1:": "🤾🏻",
    ":handball::skin-tone-1:": "🤾🏻",
    ":person_playing_handball_tone2:": "🤾🏼",
    ":handball_tone2:": "🤾🏼",
    ":person_playing_handball::skin-tone-2:": "🤾🏼",
    ":handball::skin-tone-2:": "🤾🏼",
    ":person_playing_handball_tone3:": "🤾🏽",
    ":handball_tone3:": "🤾🏽",
    ":person_playing_handball::skin-tone-3:": "🤾🏽",
    ":handball::skin-tone-3:": "🤾🏽",
    ":person_playing_handball_tone4:": "🤾🏾",
    ":handball_tone4:": "🤾🏾",
    ":person_playing_handball::skin-tone-4:": "🤾🏾",
    ":handball::skin-tone-4:": "🤾🏾",
    ":person_playing_handball_tone5:": "🤾🏿",
    ":handball_tone5:": "🤾🏿",
    ":person_playing_handball::skin-tone-5:": "🤾🏿",
    ":handball::skin-tone-5:": "🤾🏿",
    ":person_playing_water_polo:": "🤽",
    ":water_polo:": "🤽",
    ":person_playing_water_polo_tone1:": "🤽🏻",
    ":water_polo_tone1:": "🤽🏻",
    ":person_playing_water_polo::skin-tone-1:": "🤽🏻",
    ":water_polo::skin-tone-1:": "🤽🏻",
    ":person_playing_water_polo_tone2:": "🤽🏼",
    ":water_polo_tone2:": "🤽🏼",
    ":person_playing_water_polo::skin-tone-2:": "🤽🏼",
    ":water_polo::skin-tone-2:": "🤽🏼",
    ":person_playing_water_polo_tone3:": "🤽🏽",
    ":water_polo_tone3:": "🤽🏽",
    ":person_playing_water_polo::skin-tone-3:": "🤽🏽",
    ":water_polo::skin-tone-3:": "🤽🏽",
    ":person_playing_water_polo_tone4:": "🤽🏾",
    ":water_polo_tone4:": "🤽🏾",
    ":person_playing_water_polo::skin-tone-4:": "🤽🏾",
    ":water_polo::skin-tone-4:": "🤽🏾",
    ":person_playing_water_polo_tone5:": "🤽🏿",
    ":water_polo_tone5:": "🤽🏿",
    ":person_playing_water_polo::skin-tone-5:": "🤽🏿",
    ":water_polo::skin-tone-5:": "🤽🏿",
    ":person_pouting:": "🙎",
    ":person_with_pouting_face:": "🙎",
    ":person_pouting_tone1:": "🙎🏻",
    ":person_with_pouting_face_tone1:": "🙎🏻",
    ":person_pouting::skin-tone-1:": "🙎🏻",
    ":person_with_pouting_face::skin-tone-1:": "🙎🏻",
    ":person_pouting_tone2:": "🙎🏼",
    ":person_with_pouting_face_tone2:": "🙎🏼",
    ":person_pouting::skin-tone-2:": "🙎🏼",
    ":person_with_pouting_face::skin-tone-2:": "🙎🏼",
    ":person_pouting_tone3:": "🙎🏽",
    ":person_with_pouting_face_tone3:": "🙎🏽",
    ":person_pouting::skin-tone-3:": "🙎🏽",
    ":person_with_pouting_face::skin-tone-3:": "🙎🏽",
    ":person_pouting_tone4:": "🙎🏾",
    ":person_with_pouting_face_tone4:": "🙎🏾",
    ":person_pouting::skin-tone-4:": "🙎🏾",
    ":person_with_pouting_face::skin-tone-4:": "🙎🏾",
    ":person_pouting_tone5:": "🙎🏿",
    ":person_with_pouting_face_tone5:": "🙎🏿",
    ":person_pouting::skin-tone-5:": "🙎🏿",
    ":person_with_pouting_face::skin-tone-5:": "🙎🏿",
    ":person_raising_hand:": "🙋",
    ":raising_hand:": "🙋",
    ":person_raising_hand_tone1:": "🙋🏻",
    ":raising_hand_tone1:": "🙋🏻",
    ":person_raising_hand::skin-tone-1:": "🙋🏻",
    ":raising_hand::skin-tone-1:": "🙋🏻",
    ":person_raising_hand_tone2:": "🙋🏼",
    ":raising_hand_tone2:": "🙋🏼",
    ":person_raising_hand::skin-tone-2:": "🙋🏼",
    ":raising_hand::skin-tone-2:": "🙋🏼",
    ":person_raising_hand_tone3:": "🙋🏽",
    ":raising_hand_tone3:": "🙋🏽",
    ":person_raising_hand::skin-tone-3:": "🙋🏽",
    ":raising_hand::skin-tone-3:": "🙋🏽",
    ":person_raising_hand_tone4:": "🙋🏾",
    ":raising_hand_tone4:": "🙋🏾",
    ":person_raising_hand::skin-tone-4:": "🙋🏾",
    ":raising_hand::skin-tone-4:": "🙋🏾",
    ":person_raising_hand_tone5:": "🙋🏿",
    ":raising_hand_tone5:": "🙋🏿",
    ":person_raising_hand::skin-tone-5:": "🙋🏿",
    ":raising_hand::skin-tone-5:": "🙋🏿",
    ":person_red_hair:": "🧑‍🦰",
    ":person_rowing_boat:": "🚣",
    ":rowboat:": "🚣",
    ":person_rowing_boat_tone1:": "🚣🏻",
    ":rowboat_tone1:": "🚣🏻",
    ":person_rowing_boat::skin-tone-1:": "🚣🏻",
    ":rowboat::skin-tone-1:": "🚣🏻",
    ":person_rowing_boat_tone2:": "🚣🏼",
    ":rowboat_tone2:": "🚣🏼",
    ":person_rowing_boat::skin-tone-2:": "🚣🏼",
    ":rowboat::skin-tone-2:": "🚣🏼",
    ":person_rowing_boat_tone3:": "🚣🏽",
    ":rowboat_tone3:": "🚣🏽",
    ":person_rowing_boat::skin-tone-3:": "🚣🏽",
    ":rowboat::skin-tone-3:": "🚣🏽",
    ":person_rowing_boat_tone4:": "🚣🏾",
    ":rowboat_tone4:": "🚣🏾",
    ":person_rowing_boat::skin-tone-4:": "🚣🏾",
    ":rowboat::skin-tone-4:": "🚣🏾",
    ":person_rowing_boat_tone5:": "🚣🏿",
    ":rowboat_tone5:": "🚣🏿",
    ":person_rowing_boat::skin-tone-5:": "🚣🏿",
    ":rowboat::skin-tone-5:": "🚣🏿",
    ":person_running:": "🏃",
    ":runner:": "🏃",
    ":person_running_tone1:": "🏃🏻",
    ":runner_tone1:": "🏃🏻",
    ":person_running::skin-tone-1:": "🏃🏻",
    ":runner::skin-tone-1:": "🏃🏻",
    ":person_running_tone2:": "🏃🏼",
    ":runner_tone2:": "🏃🏼",
    ":person_running::skin-tone-2:": "🏃🏼",
    ":runner::skin-tone-2:": "🏃🏼",
    ":person_running_tone3:": "🏃🏽",
    ":runner_tone3:": "🏃🏽",
    ":person_running::skin-tone-3:": "🏃🏽",
    ":runner::skin-tone-3:": "🏃🏽",
    ":person_running_tone4:": "🏃🏾",
    ":runner_tone4:": "🏃🏾",
    ":person_running::skin-tone-4:": "🏃🏾",
    ":runner::skin-tone-4:": "🏃🏾",
    ":person_running_tone5:": "🏃🏿",
    ":runner_tone5:": "🏃🏿",
    ":person_running::skin-tone-5:": "🏃🏿",
    ":runner::skin-tone-5:": "🏃🏿",
    ":person_shrugging:": "🤷",
    ":shrug:": "🤷",
    ":person_shrugging_tone1:": "🤷🏻",
    ":shrug_tone1:": "🤷🏻",
    ":person_shrugging::skin-tone-1:": "🤷🏻",
    ":shrug::skin-tone-1:": "🤷🏻",
    ":person_shrugging_tone2:": "🤷🏼",
    ":shrug_tone2:": "🤷🏼",
    ":person_shrugging::skin-tone-2:": "🤷🏼",
    ":shrug::skin-tone-2:": "🤷🏼",
    ":person_shrugging_tone3:": "🤷🏽",
    ":shrug_tone3:": "🤷🏽",
    ":person_shrugging::skin-tone-3:": "🤷🏽",
    ":shrug::skin-tone-3:": "🤷🏽",
    ":person_shrugging_tone4:": "🤷🏾",
    ":shrug_tone4:": "🤷🏾",
    ":person_shrugging::skin-tone-4:": "🤷🏾",
    ":shrug::skin-tone-4:": "🤷🏾",
    ":person_shrugging_tone5:": "🤷🏿",
    ":shrug_tone5:": "🤷🏿",
    ":person_shrugging::skin-tone-5:": "🤷🏿",
    ":shrug::skin-tone-5:": "🤷🏿",
    ":person_standing:": "🧍",
    ":person_standing_tone1:": "🧍🏻",
    ":person_standing_light_skin_tone:": "🧍🏻",
    ":person_standing::skin-tone-1:": "🧍🏻",
    ":person_standing_tone2:": "🧍🏼",
    ":person_standing_medium_light_skin_tone:": "🧍🏼",
    ":person_standing::skin-tone-2:": "🧍🏼",
    ":person_standing_tone3:": "🧍🏽",
    ":person_standing_medium_skin_tone:": "🧍🏽",
    ":person_standing::skin-tone-3:": "🧍🏽",
    ":person_standing_tone4:": "🧍🏾",
    ":person_standing_medium_dark_skin_tone:": "🧍🏾",
    ":person_standing::skin-tone-4:": "🧍🏾",
    ":person_standing_tone5:": "🧍🏿",
    ":person_standing_dark_skin_tone:": "🧍🏿",
    ":person_standing::skin-tone-5:": "🧍🏿",
    ":person_surfing:": "🏄",
    ":surfer:": "🏄",
    ":person_surfing_tone1:": "🏄🏻",
    ":surfer_tone1:": "🏄🏻",
    ":person_surfing::skin-tone-1:": "🏄🏻",
    ":surfer::skin-tone-1:": "🏄🏻",
    ":person_surfing_tone2:": "🏄🏼",
    ":surfer_tone2:": "🏄🏼",
    ":person_surfing::skin-tone-2:": "🏄🏼",
    ":surfer::skin-tone-2:": "🏄🏼",
    ":person_surfing_tone3:": "🏄🏽",
    ":surfer_tone3:": "🏄🏽",
    ":person_surfing::skin-tone-3:": "🏄🏽",
    ":surfer::skin-tone-3:": "🏄🏽",
    ":person_surfing_tone4:": "🏄🏾",
    ":surfer_tone4:": "🏄🏾",
    ":person_surfing::skin-tone-4:": "🏄🏾",
    ":surfer::skin-tone-4:": "🏄🏾",
    ":person_surfing_tone5:": "🏄🏿",
    ":surfer_tone5:": "🏄🏿",
    ":person_surfing::skin-tone-5:": "🏄🏿",
    ":surfer::skin-tone-5:": "🏄🏿",
    ":person_swimming:": "🏊",
    ":swimmer:": "🏊",
    ":person_swimming_tone1:": "🏊🏻",
    ":swimmer_tone1:": "🏊🏻",
    ":person_swimming::skin-tone-1:": "🏊🏻",
    ":swimmer::skin-tone-1:": "🏊🏻",
    ":person_swimming_tone2:": "🏊🏼",
    ":swimmer_tone2:": "🏊🏼",
    ":person_swimming::skin-tone-2:": "🏊🏼",
    ":swimmer::skin-tone-2:": "🏊🏼",
    ":person_swimming_tone3:": "🏊🏽",
    ":swimmer_tone3:": "🏊🏽",
    ":person_swimming::skin-tone-3:": "🏊🏽",
    ":swimmer::skin-tone-3:": "🏊🏽",
    ":person_swimming_tone4:": "🏊🏾",
    ":swimmer_tone4:": "🏊🏾",
    ":person_swimming::skin-tone-4:": "🏊🏾",
    ":swimmer::skin-tone-4:": "🏊🏾",
    ":person_swimming_tone5:": "🏊🏿",
    ":swimmer_tone5:": "🏊🏿",
    ":person_swimming::skin-tone-5:": "🏊🏿",
    ":swimmer::skin-tone-5:": "🏊🏿",
    ":person_tipping_hand:": "💁",
    ":information_desk_person:": "💁",
    ":person_tipping_hand_tone1:": "💁🏻",
    ":information_desk_person_tone1:": "💁🏻",
    ":person_tipping_hand::skin-tone-1:": "💁🏻",
    ":information_desk_person::skin-tone-1:": "💁🏻",
    ":person_tipping_hand_tone2:": "💁🏼",
    ":information_desk_person_tone2:": "💁🏼",
    ":person_tipping_hand::skin-tone-2:": "💁🏼",
    ":information_desk_person::skin-tone-2:": "💁🏼",
    ":person_tipping_hand_tone3:": "💁🏽",
    ":information_desk_person_tone3:": "💁🏽",
    ":person_tipping_hand::skin-tone-3:": "💁🏽",
    ":information_desk_person::skin-tone-3:": "💁🏽",
    ":person_tipping_hand_tone4:": "💁🏾",
    ":information_desk_person_tone4:": "💁🏾",
    ":person_tipping_hand::skin-tone-4:": "💁🏾",
    ":information_desk_person::skin-tone-4:": "💁🏾",
    ":person_tipping_hand_tone5:": "💁🏿",
    ":information_desk_person_tone5:": "💁🏿",
    ":person_tipping_hand::skin-tone-5:": "💁🏿",
    ":information_desk_person::skin-tone-5:": "💁🏿",
    ":person_tone1_bald:": "🧑🏻‍🦲",
    ":person_light_skin_tone_bald:": "🧑🏻‍🦲",
    ":person_bald::skin-tone-1:": "🧑🏻‍🦲",
    ":person_tone1_curly_hair:": "🧑🏻‍🦱",
    ":person_light_skin_tone_curly_hair:": "🧑🏻‍🦱",
    ":person_curly_hair::skin-tone-1:": "🧑🏻‍🦱",
    ":person_tone1_red_hair:": "🧑🏻‍🦰",
    ":person_light_skin_tone_red_hair:": "🧑🏻‍🦰",
    ":person_red_hair::skin-tone-1:": "🧑🏻‍🦰",
    ":person_tone1_white_hair:": "🧑🏻‍🦳",
    ":person_light_skin_tone_white_hair:": "🧑🏻‍🦳",
    ":person_white_hair::skin-tone-1:": "🧑🏻‍🦳",
    ":person_tone2_bald:": "🧑🏼‍🦲",
    ":person_medium_light_skin_tone_bald:": "🧑🏼‍🦲",
    ":person_bald::skin-tone-2:": "🧑🏼‍🦲",
    ":person_tone2_curly_hair:": "🧑🏼‍🦱",
    ":person_medium_light_skin_tone_curly_hair:": "🧑🏼‍🦱",
    ":person_curly_hair::skin-tone-2:": "🧑🏼‍🦱",
    ":person_tone2_red_hair:": "🧑🏼‍🦰",
    ":person_medium_light_skin_tone_red_hair:": "🧑🏼‍🦰",
    ":person_red_hair::skin-tone-2:": "🧑🏼‍🦰",
    ":person_tone2_white_hair:": "🧑🏼‍🦳",
    ":person_medium_light_skin_tone_white_hair:": "🧑🏼‍🦳",
    ":person_white_hair::skin-tone-2:": "🧑🏼‍🦳",
    ":person_tone3_bald:": "🧑🏽‍🦲",
    ":person_medium_skin_tone_bald:": "🧑🏽‍🦲",
    ":person_bald::skin-tone-3:": "🧑🏽‍🦲",
    ":person_tone3_curly_hair:": "🧑🏽‍🦱",
    ":person_medium_skin_tone_curly_hair:": "🧑🏽‍🦱",
    ":person_curly_hair::skin-tone-3:": "🧑🏽‍🦱",
    ":person_tone3_red_hair:": "🧑🏽‍🦰",
    ":person_medium_skin_tone_red_hair:": "🧑🏽‍🦰",
    ":person_red_hair::skin-tone-3:": "🧑🏽‍🦰",
    ":person_tone3_white_hair:": "🧑🏽‍🦳",
    ":person_medium_skin_tone_white_hair:": "🧑🏽‍🦳",
    ":person_white_hair::skin-tone-3:": "🧑🏽‍🦳",
    ":person_tone4_bald:": "🧑🏾‍🦲",
    ":person_medium_dark_skin_tone_bald:": "🧑🏾‍🦲",
    ":person_bald::skin-tone-4:": "🧑🏾‍🦲",
    ":person_tone4_curly_hair:": "🧑🏾‍🦱",
    ":person_medium_dark_skin_tone_curly_hair:": "🧑🏾‍🦱",
    ":person_curly_hair::skin-tone-4:": "🧑🏾‍🦱",
    ":person_tone4_red_hair:": "🧑🏾‍🦰",
    ":person_medium_dark_skin_tone_red_hair:": "🧑🏾‍🦰",
    ":person_red_hair::skin-tone-4:": "🧑🏾‍🦰",
    ":person_tone4_white_hair:": "🧑🏾‍🦳",
    ":person_medium_dark_skin_tone_white_hair:": "🧑🏾‍🦳",
    ":person_white_hair::skin-tone-4:": "🧑🏾‍🦳",
    ":person_tone5_bald:": "🧑🏿‍🦲",
    ":person_dark_skin_tone_bald:": "🧑🏿‍🦲",
    ":person_bald::skin-tone-5:": "🧑🏿‍🦲",
    ":person_tone5_curly_hair:": "🧑🏿‍🦱",
    ":person_dark_skin_tone_curly_hair:": "🧑🏿‍🦱",
    ":person_curly_hair::skin-tone-5:": "🧑🏿‍🦱",
    ":person_tone5_red_hair:": "🧑🏿‍🦰",
    ":person_dark_skin_tone_red_hair:": "🧑🏿‍🦰",
    ":person_red_hair::skin-tone-5:": "🧑🏿‍🦰",
    ":person_tone5_white_hair:": "🧑🏿‍🦳",
    ":person_dark_skin_tone_white_hair:": "🧑🏿‍🦳",
    ":person_white_hair::skin-tone-5:": "🧑🏿‍🦳",
    ":person_walking:": "🚶",
    ":walking:": "🚶",
    ":person_walking_tone1:": "🚶🏻",
    ":walking_tone1:": "🚶🏻",
    ":person_walking::skin-tone-1:": "🚶🏻",
    ":walking::skin-tone-1:": "🚶🏻",
    ":person_walking_tone2:": "🚶🏼",
    ":walking_tone2:": "🚶🏼",
    ":person_walking::skin-tone-2:": "🚶🏼",
    ":walking::skin-tone-2:": "🚶🏼",
    ":person_walking_tone3:": "🚶🏽",
    ":walking_tone3:": "🚶🏽",
    ":person_walking::skin-tone-3:": "🚶🏽",
    ":walking::skin-tone-3:": "🚶🏽",
    ":person_walking_tone4:": "🚶🏾",
    ":walking_tone4:": "🚶🏾",
    ":person_walking::skin-tone-4:": "🚶🏾",
    ":walking::skin-tone-4:": "🚶🏾",
    ":person_walking_tone5:": "🚶🏿",
    ":walking_tone5:": "🚶🏿",
    ":person_walking::skin-tone-5:": "🚶🏿",
    ":walking::skin-tone-5:": "🚶🏿",
    ":person_wearing_turban:": "👳",
    ":man_with_turban:": "👳",
    ":person_wearing_turban_tone1:": "👳🏻",
    ":man_with_turban_tone1:": "👳🏻",
    ":person_wearing_turban::skin-tone-1:": "👳🏻",
    ":man_with_turban::skin-tone-1:": "👳🏻",
    ":person_wearing_turban_tone2:": "👳🏼",
    ":man_with_turban_tone2:": "👳🏼",
    ":person_wearing_turban::skin-tone-2:": "👳🏼",
    ":man_with_turban::skin-tone-2:": "👳🏼",
    ":person_wearing_turban_tone3:": "👳🏽",
    ":man_with_turban_tone3:": "👳🏽",
    ":person_wearing_turban::skin-tone-3:": "👳🏽",
    ":man_with_turban::skin-tone-3:": "👳🏽",
    ":person_wearing_turban_tone4:": "👳🏾",
    ":man_with_turban_tone4:": "👳🏾",
    ":person_wearing_turban::skin-tone-4:": "👳🏾",
    ":man_with_turban::skin-tone-4:": "👳🏾",
    ":person_wearing_turban_tone5:": "👳🏿",
    ":man_with_turban_tone5:": "👳🏿",
    ":person_wearing_turban::skin-tone-5:": "👳🏿",
    ":man_with_turban::skin-tone-5:": "👳🏿",
    ":person_white_hair:": "🧑‍🦳",
    ":person_with_probing_cane:": "🧑‍🦯",
    ":person_with_probing_cane_tone1:": "🧑🏻‍🦯",
    ":person_with_probing_cane_light_skin_tone:": "🧑🏻‍🦯",
    ":person_with_probing_cane::skin-tone-1:": "🧑🏻‍🦯",
    ":person_with_probing_cane_tone2:": "🧑🏼‍🦯",
    ":person_with_probing_cane_medium_light_skin_tone:": "🧑🏼‍🦯",
    ":person_with_probing_cane::skin-tone-2:": "🧑🏼‍🦯",
    ":person_with_probing_cane_tone3:": "🧑🏽‍🦯",
    ":person_with_probing_cane_medium_skin_tone:": "🧑🏽‍🦯",
    ":person_with_probing_cane::skin-tone-3:": "🧑🏽‍🦯",
    ":person_with_probing_cane_tone4:": "🧑🏾‍🦯",
    ":person_with_probing_cane_medium_dark_skin_tone:": "🧑🏾‍🦯",
    ":person_with_probing_cane::skin-tone-4:": "🧑🏾‍🦯",
    ":person_with_probing_cane_tone5:": "🧑🏿‍🦯",
    ":person_with_probing_cane_dark_skin_tone:": "🧑🏿‍🦯",
    ":person_with_probing_cane::skin-tone-5:": "🧑🏿‍🦯",
    ":person_with_veil:": "👰",
    ":person_with_veil_tone1:": "👰🏻",
    ":person_with_veil::skin-tone-1:": "👰🏻",
    ":person_with_veil_tone2:": "👰🏼",
    ":person_with_veil::skin-tone-2:": "👰🏼",
    ":person_with_veil_tone3:": "👰🏽",
    ":person_with_veil::skin-tone-3:": "👰🏽",
    ":person_with_veil_tone4:": "👰🏾",
    ":person_with_veil::skin-tone-4:": "👰🏾",
    ":person_with_veil_tone5:": "👰🏿",
    ":person_with_veil::skin-tone-5:": "👰🏿",
    ":petri_dish:": "🧫",
    ":pick:": "⛏️",
    ":pickup_truck:": "🛻",
    ":pie:": "🥧",
    ":pig:": "🐷",
    ":pig_nose:": "🐽",
    ":pig2:": "🐖",
    ":pill:": "💊",
    ":pilot:": "🧑‍✈️",
    ":pilot_tone1:": "🧑🏻‍✈️",
    ":pilot_light_skin_tone:": "🧑🏻‍✈️",
    ":pilot::skin-tone-1:": "🧑🏻‍✈️",
    ":pilot_tone2:": "🧑🏼‍✈️",
    ":pilot_medium_light_skin_tone:": "🧑🏼‍✈️",
    ":pilot::skin-tone-2:": "🧑🏼‍✈️",
    ":pilot_tone3:": "🧑🏽‍✈️",
    ":pilot_medium_skin_tone:": "🧑🏽‍✈️",
    ":pilot::skin-tone-3:": "🧑🏽‍✈️",
    ":pilot_tone4:": "🧑🏾‍✈️",
    ":pilot_medium_dark_skin_tone:": "🧑🏾‍✈️",
    ":pilot::skin-tone-4:": "🧑🏾‍✈️",
    ":pilot_tone5:": "🧑🏿‍✈️",
    ":pilot_dark_skin_tone:": "🧑🏿‍✈️",
    ":pilot::skin-tone-5:": "🧑🏿‍✈️",
    ":piñata:": "🪅",
    ":pinched_fingers:": "🤌",
    ":pinched_fingers_tone1:": "🤌🏻",
    ":pinched_fingers_light_skin_tone:": "🤌🏻",
    ":pinched_fingers::skin-tone-1:": "🤌🏻",
    ":pinched_fingers_tone2:": "🤌🏼",
    ":pinched_fingers_medium_light_skin_tone:": "🤌🏼",
    ":pinched_fingers::skin-tone-2:": "🤌🏼",
    ":pinched_fingers_tone3:": "🤌🏽",
    ":pinched_fingers_medium_skin_tone:": "🤌🏽",
    ":pinched_fingers::skin-tone-3:": "🤌🏽",
    ":pinched_fingers_tone4:": "🤌🏾",
    ":pinched_fingers_medium_dark_skin_tone:": "🤌🏾",
    ":pinched_fingers::skin-tone-4:": "🤌🏾",
    ":pinched_fingers_tone5:": "🤌🏿",
    ":pinched_fingers_dark_skin_tone:": "🤌🏿",
    ":pinched_fingers::skin-tone-5:": "🤌🏿",
    ":pinching_hand:": "🤏",
    ":pinching_hand_tone1:": "🤏🏻",
    ":pinching_hand_light_skin_tone:": "🤏🏻",
    ":pinching_hand::skin-tone-1:": "🤏🏻",
    ":pinching_hand_tone2:": "🤏🏼",
    ":pinching_hand_medium_light_skin_tone:": "🤏🏼",
    ":pinching_hand::skin-tone-2:": "🤏🏼",
    ":pinching_hand_tone3:": "🤏🏽",
    ":pinching_hand_medium_skin_tone:": "🤏🏽",
    ":pinching_hand::skin-tone-3:": "🤏🏽",
    ":pinching_hand_tone4:": "🤏🏾",
    ":pinching_hand_medium_dark_skin_tone:": "🤏🏾",
    ":pinching_hand::skin-tone-4:": "🤏🏾",
    ":pinching_hand_tone5:": "🤏🏿",
    ":pinching_hand_dark_skin_tone:": "🤏🏿",
    ":pinching_hand::skin-tone-5:": "🤏🏿",
    ":pineapple:": "🍍",
    ":ping_pong:": "🏓",
    ":table_tennis:": "🏓",
    ":pirate_flag:": "🏴‍☠️",
    ":pisces:": "♓",
    ":pizza:": "🍕",
    ":placard:": "🪧",
    ":place_of_worship:": "🛐",
    ":worship_symbol:": "🛐",
    ":play_pause:": "⏯️",
    ":pleading_face:": "🥺",
    ":plunger:": "🪠",
    ":point_down:": "👇",
    ":point_down_tone1:": "👇🏻",
    ":point_down::skin-tone-1:": "👇🏻",
    ":point_down_tone2:": "👇🏼",
    ":point_down::skin-tone-2:": "👇🏼",
    ":point_down_tone3:": "👇🏽",
    ":point_down::skin-tone-3:": "👇🏽",
    ":point_down_tone4:": "👇🏾",
    ":point_down::skin-tone-4:": "👇🏾",
    ":point_down_tone5:": "👇🏿",
    ":point_down::skin-tone-5:": "👇🏿",
    ":point_left:": "👈",
    ":point_left_tone1:": "👈🏻",
    ":point_left::skin-tone-1:": "👈🏻",
    ":point_left_tone2:": "👈🏼",
    ":point_left::skin-tone-2:": "👈🏼",
    ":point_left_tone3:": "👈🏽",
    ":point_left::skin-tone-3:": "👈🏽",
    ":point_left_tone4:": "👈🏾",
    ":point_left::skin-tone-4:": "👈🏾",
    ":point_left_tone5:": "👈🏿",
    ":point_left::skin-tone-5:": "👈🏿",
    ":point_right:": "👉",
    ":point_right_tone1:": "👉🏻",
    ":point_right::skin-tone-1:": "👉🏻",
    ":point_right_tone2:": "👉🏼",
    ":point_right::skin-tone-2:": "👉🏼",
    ":point_right_tone3:": "👉🏽",
    ":point_right::skin-tone-3:": "👉🏽",
    ":point_right_tone4:": "👉🏾",
    ":point_right::skin-tone-4:": "👉🏾",
    ":point_right_tone5:": "👉🏿",
    ":point_right::skin-tone-5:": "👉🏿",
    ":point_up:": "☝️",
    ":point_up_2:": "👆",
    ":point_up_2_tone1:": "👆🏻",
    ":point_up_2::skin-tone-1:": "👆🏻",
    ":point_up_2_tone2:": "👆🏼",
    ":point_up_2::skin-tone-2:": "👆🏼",
    ":point_up_2_tone3:": "👆🏽",
    ":point_up_2::skin-tone-3:": "👆🏽",
    ":point_up_2_tone4:": "👆🏾",
    ":point_up_2::skin-tone-4:": "👆🏾",
    ":point_up_2_tone5:": "👆🏿",
    ":point_up_2::skin-tone-5:": "👆🏿",
    ":point_up_tone1:": "☝🏻",
    ":point_up::skin-tone-1:": "☝🏻",
    ":point_up_tone2:": "☝🏼",
    ":point_up::skin-tone-2:": "☝🏼",
    ":point_up_tone3:": "☝🏽",
    ":point_up::skin-tone-3:": "☝🏽",
    ":point_up_tone4:": "☝🏾",
    ":point_up::skin-tone-4:": "☝🏾",
    ":point_up_tone5:": "☝🏿",
    ":point_up::skin-tone-5:": "☝🏿",
    ":polar_bear:": "🐻‍❄️",
    ":police_car:": "🚓",
    ":police_officer:": "👮",
    ":cop:": "👮",
    ":police_officer_tone1:": "👮🏻",
    ":cop_tone1:": "👮🏻",
    ":police_officer::skin-tone-1:": "👮🏻",
    ":cop::skin-tone-1:": "👮🏻",
    ":police_officer_tone2:": "👮🏼",
    ":cop_tone2:": "👮🏼",
    ":police_officer::skin-tone-2:": "👮🏼",
    ":cop::skin-tone-2:": "👮🏼",
    ":police_officer_tone3:": "👮🏽",
    ":cop_tone3:": "👮🏽",
    ":police_officer::skin-tone-3:": "👮🏽",
    ":cop::skin-tone-3:": "👮🏽",
    ":police_officer_tone4:": "👮🏾",
    ":cop_tone4:": "👮🏾",
    ":police_officer::skin-tone-4:": "👮🏾",
    ":cop::skin-tone-4:": "👮🏾",
    ":police_officer_tone5:": "👮🏿",
    ":cop_tone5:": "👮🏿",
    ":police_officer::skin-tone-5:": "👮🏿",
    ":cop::skin-tone-5:": "👮🏿",
    ":poodle:": "🐩",
    ":poop:": "💩",
    ":shit:": "💩",
    ":hankey:": "💩",
    ":poo:": "💩",
    ":popcorn:": "🍿",
    ":post_office:": "🏣",
    ":postal_horn:": "📯",
    ":postbox:": "📮",
    ":potable_water:": "🚰",
    ":potato:": "🥔",
    ":potted_plant:": "🪴",
    ":pouch:": "👝",
    ":poultry_leg:": "🍗",
    ":pound:": "💷",
    ":pouting_cat:": "😾",
    ":pray:": "🙏",
    ":pray_tone1:": "🙏🏻",
    ":pray::skin-tone-1:": "🙏🏻",
    ":pray_tone2:": "🙏🏼",
    ":pray::skin-tone-2:": "🙏🏼",
    ":pray_tone3:": "🙏🏽",
    ":pray::skin-tone-3:": "🙏🏽",
    ":pray_tone4:": "🙏🏾",
    ":pray::skin-tone-4:": "🙏🏾",
    ":pray_tone5:": "🙏🏿",
    ":pray::skin-tone-5:": "🙏🏿",
    ":prayer_beads:": "📿",
    ":pregnant_woman:": "🤰",
    ":expecting_woman:": "🤰",
    ":pregnant_woman_tone1:": "🤰🏻",
    ":expecting_woman_tone1:": "🤰🏻",
    ":pregnant_woman::skin-tone-1:": "🤰🏻",
    ":expecting_woman::skin-tone-1:": "🤰🏻",
    ":pregnant_woman_tone2:": "🤰🏼",
    ":expecting_woman_tone2:": "🤰🏼",
    ":pregnant_woman::skin-tone-2:": "🤰🏼",
    ":expecting_woman::skin-tone-2:": "🤰🏼",
    ":pregnant_woman_tone3:": "🤰🏽",
    ":expecting_woman_tone3:": "🤰🏽",
    ":pregnant_woman::skin-tone-3:": "🤰🏽",
    ":expecting_woman::skin-tone-3:": "🤰🏽",
    ":pregnant_woman_tone4:": "🤰🏾",
    ":expecting_woman_tone4:": "🤰🏾",
    ":pregnant_woman::skin-tone-4:": "🤰🏾",
    ":expecting_woman::skin-tone-4:": "🤰🏾",
    ":pregnant_woman_tone5:": "🤰🏿",
    ":expecting_woman_tone5:": "🤰🏿",
    ":pregnant_woman::skin-tone-5:": "🤰🏿",
    ":expecting_woman::skin-tone-5:": "🤰🏿",
    ":pretzel:": "🥨",
    ":prince:": "🤴",
    ":prince_tone1:": "🤴🏻",
    ":prince::skin-tone-1:": "🤴🏻",
    ":prince_tone2:": "🤴🏼",
    ":prince::skin-tone-2:": "🤴🏼",
    ":prince_tone3:": "🤴🏽",
    ":prince::skin-tone-3:": "🤴🏽",
    ":prince_tone4:": "🤴🏾",
    ":prince::skin-tone-4:": "🤴🏾",
    ":prince_tone5:": "🤴🏿",
    ":prince::skin-tone-5:": "🤴🏿",
    ":princess:": "👸",
    ":princess_tone1:": "👸🏻",
    ":princess::skin-tone-1:": "👸🏻",
    ":princess_tone2:": "👸🏼",
    ":princess::skin-tone-2:": "👸🏼",
    ":princess_tone3:": "👸🏽",
    ":princess::skin-tone-3:": "👸🏽",
    ":princess_tone4:": "👸🏾",
    ":princess::skin-tone-4:": "👸🏾",
    ":princess_tone5:": "👸🏿",
    ":princess::skin-tone-5:": "👸🏿",
    ":printer:": "🖨️",
    ":probing_cane:": "🦯",
    ":projector:": "📽️",
    ":film_projector:": "📽️",
    ":punch:": "👊",
    ":punch_tone1:": "👊🏻",
    ":punch::skin-tone-1:": "👊🏻",
    ":punch_tone2:": "👊🏼",
    ":punch::skin-tone-2:": "👊🏼",
    ":punch_tone3:": "👊🏽",
    ":punch::skin-tone-3:": "👊🏽",
    ":punch_tone4:": "👊🏾",
    ":punch::skin-tone-4:": "👊🏾",
    ":punch_tone5:": "👊🏿",
    ":punch::skin-tone-5:": "👊🏿",
    ":purple_circle:": "🟣",
    ":purple_heart:": "💜",
    ":purple_square:": "🟪",
    ":purse:": "👛",
    ":pushpin:": "📌",
    ":put_litter_in_its_place:": "🚮",
    ":question:": "❓",
    ":rabbit:": "🐰",
    ":rabbit2:": "🐇",
    ":raccoon:": "🦝",
    ":race_car:": "🏎️",
    ":racing_car:": "🏎️",
    ":racehorse:": "🐎",
    ":radio:": "📻",
    ":radio_button:": "🔘",
    ":radioactive:": "☢️",
    ":radioactive_sign:": "☢️",
    ":rage:": "😡",
    ":@": "😡",
    ":-@": "😡",
    "@": "😡",
    "-@": "😡",
    ":railway_car:": "🚃",
    ":railway_track:": "🛤️",
    ":railroad_track:": "🛤️",
    ":rainbow:": "🌈",
    ":rainbow_flag:": "🏳️‍🌈",
    ":gay_pride_flag:": "🏳️‍🌈",
    ":raised_back_of_hand:": "🤚",
    ":back_of_hand:": "🤚",
    ":raised_back_of_hand_tone1:": "🤚🏻",
    ":back_of_hand_tone1:": "🤚🏻",
    ":raised_back_of_hand::skin-tone-1:": "🤚🏻",
    ":back_of_hand::skin-tone-1:": "🤚🏻",
    ":raised_back_of_hand_tone2:": "🤚🏼",
    ":back_of_hand_tone2:": "🤚🏼",
    ":raised_back_of_hand::skin-tone-2:": "🤚🏼",
    ":back_of_hand::skin-tone-2:": "🤚🏼",
    ":raised_back_of_hand_tone3:": "🤚🏽",
    ":back_of_hand_tone3:": "🤚🏽",
    ":raised_back_of_hand::skin-tone-3:": "🤚🏽",
    ":back_of_hand::skin-tone-3:": "🤚🏽",
    ":raised_back_of_hand_tone4:": "🤚🏾",
    ":back_of_hand_tone4:": "🤚🏾",
    ":raised_back_of_hand::skin-tone-4:": "🤚🏾",
    ":back_of_hand::skin-tone-4:": "🤚🏾",
    ":raised_back_of_hand_tone5:": "🤚🏿",
    ":back_of_hand_tone5:": "🤚🏿",
    ":raised_back_of_hand::skin-tone-5:": "🤚🏿",
    ":back_of_hand::skin-tone-5:": "🤚🏿",
    ":raised_hand:": "✋",
    ":raised_hand_tone1:": "✋🏻",
    ":raised_hand::skin-tone-1:": "✋🏻",
    ":raised_hand_tone2:": "✋🏼",
    ":raised_hand::skin-tone-2:": "✋🏼",
    ":raised_hand_tone3:": "✋🏽",
    ":raised_hand::skin-tone-3:": "✋🏽",
    ":raised_hand_tone4:": "✋🏾",
    ":raised_hand::skin-tone-4:": "✋🏾",
    ":raised_hand_tone5:": "✋🏿",
    ":raised_hand::skin-tone-5:": "✋🏿",
    ":raised_hands:": "🙌",
    ":raised_hands_tone1:": "🙌🏻",
    ":raised_hands::skin-tone-1:": "🙌🏻",
    ":raised_hands_tone2:": "🙌🏼",
    ":raised_hands::skin-tone-2:": "🙌🏼",
    ":raised_hands_tone3:": "🙌🏽",
    ":raised_hands::skin-tone-3:": "🙌🏽",
    ":raised_hands_tone4:": "🙌🏾",
    ":raised_hands::skin-tone-4:": "🙌🏾",
    ":raised_hands_tone5:": "🙌🏿",
    ":raised_hands::skin-tone-5:": "🙌🏿",
    ":ram:": "🐏",
    ":ramen:": "🍜",
    ":rat:": "🐀",
    ":razor:": "🪒",
    ":receipt:": "🧾",
    ":record_button:": "⏺️",
    ":recycle:": "♻️",
    ":red_car:": "🚗",
    ":red_circle:": "🔴",
    ":red_envelope:": "🧧",
    ":red_square:": "🟥",
    ":regional_indicator_a:": "🇦",
    ":regional_indicator_b:": "🇧",
    ":regional_indicator_c:": "🇨",
    ":regional_indicator_d:": "🇩",
    ":regional_indicator_e:": "🇪",
    ":regional_indicator_f:": "🇫",
    ":regional_indicator_g:": "🇬",
    ":regional_indicator_h:": "🇭",
    ":regional_indicator_i:": "🇮",
    ":regional_indicator_j:": "🇯",
    ":regional_indicator_k:": "🇰",
    ":regional_indicator_l:": "🇱",
    ":regional_indicator_m:": "🇲",
    ":regional_indicator_n:": "🇳",
    ":regional_indicator_o:": "🇴",
    ":regional_indicator_p:": "🇵",
    ":regional_indicator_q:": "🇶",
    ":regional_indicator_r:": "🇷",
    ":regional_indicator_s:": "🇸",
    ":regional_indicator_t:": "🇹",
    ":regional_indicator_u:": "🇺",
    ":regional_indicator_v:": "🇻",
    ":regional_indicator_w:": "🇼",
    ":regional_indicator_x:": "🇽",
    ":regional_indicator_y:": "🇾",
    ":regional_indicator_z:": "🇿",
    ":registered:": "®️",
    ":relaxed:": "☺️",
    ":relieved:": "😌",
    ":reminder_ribbon:": "🎗️",
    ":repeat:": "🔁",
    ":repeat_one:": "🔂",
    ":restroom:": "🚻",
    ":revolving_hearts:": "💞",
    ":rewind:": "⏪",
    ":rhino:": "🦏",
    ":rhinoceros:": "🦏",
    ":ribbon:": "🎀",
    ":rice:": "🍚",
    ":rice_ball:": "🍙",
    ":rice_cracker:": "🍘",
    ":rice_scene:": "🎑",
    ":right_facing_fist:": "🤜",
    ":right_fist:": "🤜",
    ":right_facing_fist_tone1:": "🤜🏻",
    ":right_fist_tone1:": "🤜🏻",
    ":right_facing_fist::skin-tone-1:": "🤜🏻",
    ":right_fist::skin-tone-1:": "🤜🏻",
    ":right_facing_fist_tone2:": "🤜🏼",
    ":right_fist_tone2:": "🤜🏼",
    ":right_facing_fist::skin-tone-2:": "🤜🏼",
    ":right_fist::skin-tone-2:": "🤜🏼",
    ":right_facing_fist_tone3:": "🤜🏽",
    ":right_fist_tone3:": "🤜🏽",
    ":right_facing_fist::skin-tone-3:": "🤜🏽",
    ":right_fist::skin-tone-3:": "🤜🏽",
    ":right_facing_fist_tone4:": "🤜🏾",
    ":right_fist_tone4:": "🤜🏾",
    ":right_facing_fist::skin-tone-4:": "🤜🏾",
    ":right_fist::skin-tone-4:": "🤜🏾",
    ":right_facing_fist_tone5:": "🤜🏿",
    ":right_fist_tone5:": "🤜🏿",
    ":right_facing_fist::skin-tone-5:": "🤜🏿",
    ":right_fist::skin-tone-5:": "🤜🏿",
    ":ring:": "💍",
    ":ringed_planet:": "🪐",
    ":robot:": "🤖",
    ":robot_face:": "🤖",
    ":rock:": "🪨",
    ":rocket:": "🚀",
    ":rofl:": "🤣",
    ":rolling_on_the_floor_laughing:": "🤣",
    ":roll_of_paper:": "🧻",
    ":roller_coaster:": "🎢",
    ":roller_skate:": "🛼",
    ":rolling_eyes:": "🙄",
    ":face_with_rolling_eyes:": "🙄",
    ":rooster:": "🐓",
    ":rose:": "🌹",
    ":rosette:": "🏵️",
    ":rotating_light:": "🚨",
    ":round_pushpin:": "📍",
    ":rugby_football:": "🏉",
    ":running_shirt_with_sash:": "🎽",
    ":sa:": "🈂️",
    ":safety_pin:": "🧷",
    ":safety_vest:": "🦺",
    ":sagittarius:": "♐",
    ":sailboat:": "⛵",
    ":sake:": "🍶",
    ":salad:": "🥗",
    ":green_salad:": "🥗",
    ":salt:": "🧂",
    ":sandal:": "👡",
    ":sandwich:": "🥪",
    ":santa:": "🎅",
    ":santa_tone1:": "🎅🏻",
    ":santa::skin-tone-1:": "🎅🏻",
    ":santa_tone2:": "🎅🏼",
    ":santa::skin-tone-2:": "🎅🏼",
    ":santa_tone3:": "🎅🏽",
    ":santa::skin-tone-3:": "🎅🏽",
    ":santa_tone4:": "🎅🏾",
    ":santa::skin-tone-4:": "🎅🏾",
    ":santa_tone5:": "🎅🏿",
    ":santa::skin-tone-5:": "🎅🏿",
    ":sari:": "🥻",
    ":satellite:": "📡",
    ":satellite_orbital:": "🛰️",
    ":sauropod:": "🦕",
    ":saxophone:": "🎷",
    ":scales:": "⚖️",
    ":scarf:": "🧣",
    ":school:": "🏫",
    ":school_satchel:": "🎒",
    ":scientist:": "🧑‍🔬",
    ":scientist_tone1:": "🧑🏻‍🔬",
    ":scientist_light_skin_tone:": "🧑🏻‍🔬",
    ":scientist::skin-tone-1:": "🧑🏻‍🔬",
    ":scientist_tone2:": "🧑🏼‍🔬",
    ":scientist_medium_light_skin_tone:": "🧑🏼‍🔬",
    ":scientist::skin-tone-2:": "🧑🏼‍🔬",
    ":scientist_tone3:": "🧑🏽‍🔬",
    ":scientist_medium_skin_tone:": "🧑🏽‍🔬",
    ":scientist::skin-tone-3:": "🧑🏽‍🔬",
    ":scientist_tone4:": "🧑🏾‍🔬",
    ":scientist_medium_dark_skin_tone:": "🧑🏾‍🔬",
    ":scientist::skin-tone-4:": "🧑🏾‍🔬",
    ":scientist_tone5:": "🧑🏿‍🔬",
    ":scientist_dark_skin_tone:": "🧑🏿‍🔬",
    ":scientist::skin-tone-5:": "🧑🏿‍🔬",
    ":scissors:": "✂️",
    ":scooter:": "🛴",
    ":scorpion:": "🦂",
    ":scorpius:": "♏",
    ":scotland:": "🏴󠁧󠁢󠁳󠁣󠁴󠁿",
    ":scream:": "😱",
    ":scream_cat:": "🙀",
    ":screwdriver:": "🪛",
    ":scroll:": "📜",
    ":seal:": "🦭",
    ":seat:": "💺",
    ":second_place:": "🥈",
    ":second_place_medal:": "🥈",
    ":secret:": "㊙️",
    ":see_no_evil:": "🙈",
    ":seedling:": "🌱",
    ":selfie:": "🤳",
    ":selfie_tone1:": "🤳🏻",
    ":selfie::skin-tone-1:": "🤳🏻",
    ":selfie_tone2:": "🤳🏼",
    ":selfie::skin-tone-2:": "🤳🏼",
    ":selfie_tone3:": "🤳🏽",
    ":selfie::skin-tone-3:": "🤳🏽",
    ":selfie_tone4:": "🤳🏾",
    ":selfie::skin-tone-4:": "🤳🏾",
    ":selfie_tone5:": "🤳🏿",
    ":selfie::skin-tone-5:": "🤳🏿",
    ":service_dog:": "🐕‍🦺",
    ":seven:": "7️⃣",
    ":sewing_needle:": "🪡",
    ":shallow_pan_of_food:": "🥘",
    ":paella:": "🥘",
    ":shamrock:": "☘️",
    ":shark:": "🦈",
    ":shaved_ice:": "🍧",
    ":sheep:": "🐑",
    ":shell:": "🐚",
    ":shield:": "🛡️",
    ":shinto_shrine:": "⛩️",
    ":ship:": "🚢",
    ":shirt:": "👕",
    ":shopping_bags:": "🛍️",
    ":shopping_cart:": "🛒",
    ":shopping_trolley:": "🛒",
    ":shorts:": "🩳",
    ":shower:": "🚿",
    ":shrimp:": "🦐",
    ":shushing_face:": "🤫",
    ":signal_strength:": "📶",
    ":singer:": "🧑‍🎤",
    ":singer_tone1:": "🧑🏻‍🎤",
    ":singer_light_skin_tone:": "🧑🏻‍🎤",
    ":singer::skin-tone-1:": "🧑🏻‍🎤",
    ":singer_tone2:": "🧑🏼‍🎤",
    ":singer_medium_light_skin_tone:": "🧑🏼‍🎤",
    ":singer::skin-tone-2:": "🧑🏼‍🎤",
    ":singer_tone3:": "🧑🏽‍🎤",
    ":singer_medium_skin_tone:": "🧑🏽‍🎤",
    ":singer::skin-tone-3:": "🧑🏽‍🎤",
    ":singer_tone4:": "🧑🏾‍🎤",
    ":singer_medium_dark_skin_tone:": "🧑🏾‍🎤",
    ":singer::skin-tone-4:": "🧑🏾‍🎤",
    ":singer_tone5:": "🧑🏿‍🎤",
    ":singer_dark_skin_tone:": "🧑🏿‍🎤",
    ":singer::skin-tone-5:": "🧑🏿‍🎤",
    ":six:": "6️⃣",
    ":six_pointed_star:": "🔯",
    ":skateboard:": "🛹",
    ":ski:": "🎿",
    ":skier:": "⛷️",
    ":skull:": "💀",
    ":skeleton:": "💀",
    ":skull_crossbones:": "☠️",
    ":skull_and_crossbones:": "☠️",
    ":skunk:": "🦨",
    ":sled:": "🛷",
    ":sleeping:": "😴",
    ":sleeping_accommodation:": "🛌",
    ":sleepy:": "😪",
    ":slight_frown:": "🙁",
    ":slightly_frowning_face:": "🙁",
    ":slight_smile:": "🙂",
    ":slightly_smiling_face:": "🙂",
    ":slot_machine:": "🎰",
    ":sloth:": "🦥",
    ":small_blue_diamond:": "🔹",
    ":small_orange_diamond:": "🔸",
    ":small_red_triangle:": "🔺",
    ":small_red_triangle_down:": "🔻",
    ":smile:": "😄",
    ":smile_cat:": "😸",
    ":smiley:": "😃",
    ":smiley_cat:": "😺",
    ":smiling_face_with_3_hearts:": "🥰",
    ":smiling_face_with_tear:": "🥲",
    ":smiling_imp:": "😈",
    ":smirk:": "😏",
    ":smirk_cat:": "😼",
    ":smoking:": "🚬",
    ":snail:": "🐌",
    ":snake:": "🐍",
    ":sneezing_face:": "🤧",
    ":sneeze:": "🤧",
    ":snowboarder:": "🏂",
    ":snowboarder_tone1:": "🏂🏻",
    ":snowboarder_light_skin_tone:": "🏂🏻",
    ":snowboarder::skin-tone-1:": "🏂🏻",
    ":snowboarder_tone2:": "🏂🏼",
    ":snowboarder_medium_light_skin_tone:": "🏂🏼",
    ":snowboarder::skin-tone-2:": "🏂🏼",
    ":snowboarder_tone3:": "🏂🏽",
    ":snowboarder_medium_skin_tone:": "🏂🏽",
    ":snowboarder::skin-tone-3:": "🏂🏽",
    ":snowboarder_tone4:": "🏂🏾",
    ":snowboarder_medium_dark_skin_tone:": "🏂🏾",
    ":snowboarder::skin-tone-4:": "🏂🏾",
    ":snowboarder_tone5:": "🏂🏿",
    ":snowboarder_dark_skin_tone:": "🏂🏿",
    ":snowboarder::skin-tone-5:": "🏂🏿",
    ":snowflake:": "❄️",
    ":snowman:": "⛄",
    ":snowman2:": "☃️",
    ":soap:": "🧼",
    ":sob:": "😭",
    ";(": "😭",
    ";-(": "😭",
    ":soccer:": "⚽",
    ":socks:": "🧦",
    ":softball:": "🥎",
    ":soon:": "🔜",
    ":sos:": "🆘",
    ":sound:": "🔉",
    ":space_invader:": "👾",
    ":spades:": "♠️",
    ":spaghetti:": "🍝",
    ":sparkle:": "❇️",
    ":sparkler:": "🎇",
    ":sparkles:": "✨",
    ":sparkling_heart:": "💖",
    ":speak_no_evil:": "🙊",
    ":speaker:": "🔈",
    ":speaking_head:": "🗣️",
    ":speaking_head_in_silhouette:": "🗣️",
    ":speech_balloon:": "💬",
    ":speech_left:": "🗨️",
    ":left_speech_bubble:": "🗨️",
    ":speedboat:": "🚤",
    ":spider:": "🕷️",
    ":spider_web:": "🕸️",
    ":sponge:": "🧽",
    ":spoon:": "🥄",
    ":squeeze_bottle:": "🧴",
    ":squid:": "🦑",
    ":stadium:": "🏟️",
    ":star:": "⭐",
    ":star_and_crescent:": "☪️",
    ":star_of_david:": "✡️",
    ":star_struck:": "🤩",
    ":star2:": "🌟",
    ":stars:": "🌠",
    ":station:": "🚉",
    ":statue_of_liberty:": "🗽",
    ":steam_locomotive:": "🚂",
    ":stethoscope:": "🩺",
    ":stew:": "🍲",
    ":stop_button:": "⏹️",
    ":stopwatch:": "⏱️",
    ":straight_ruler:": "📏",
    ":strawberry:": "🍓",
    ":stuck_out_tongue:": "😛",
    ":P": "😛",
    ":-P": "😛",
    "P": "😛",
    "-P": "😛",
    ":stuck_out_tongue_closed_eyes:": "😝",
    ":stuck_out_tongue_winking_eye:": "😜",
    ":student:": "🧑‍🎓",
    ":student_tone1:": "🧑🏻‍🎓",
    ":student_light_skin_tone:": "🧑🏻‍🎓",
    ":student::skin-tone-1:": "🧑🏻‍🎓",
    ":student_tone2:": "🧑🏼‍🎓",
    ":student_medium_light_skin_tone:": "🧑🏼‍🎓",
    ":student::skin-tone-2:": "🧑🏼‍🎓",
    ":student_tone3:": "🧑🏽‍🎓",
    ":student_medium_skin_tone:": "🧑🏽‍🎓",
    ":student::skin-tone-3:": "🧑🏽‍🎓",
    ":student_tone4:": "🧑🏾‍🎓",
    ":student_medium_dark_skin_tone:": "🧑🏾‍🎓",
    ":student::skin-tone-4:": "🧑🏾‍🎓",
    ":student_tone5:": "🧑🏿‍🎓",
    ":student_dark_skin_tone:": "🧑🏿‍🎓",
    ":student::skin-tone-5:": "🧑🏿‍🎓",
    ":stuffed_flatbread:": "🥙",
    ":stuffed_pita:": "🥙",
    ":sun_with_face:": "🌞",
    ":sunflower:": "🌻",
    ":sunglasses:": "😎",
    "8-)": "😎",
    "B-)": "😎",
    ":sunny:": "☀️",
    ":sunrise:": "🌅",
    ":sunrise_over_mountains:": "🌄",
    ":superhero:": "🦸",
    ":superhero_tone1:": "🦸🏻",
    ":superhero_light_skin_tone:": "🦸🏻",
    ":superhero::skin-tone-1:": "🦸🏻",
    ":superhero_tone2:": "🦸🏼",
    ":superhero_medium_light_skin_tone:": "🦸🏼",
    ":superhero::skin-tone-2:": "🦸🏼",
    ":superhero_tone3:": "🦸🏽",
    ":superhero_medium_skin_tone:": "🦸🏽",
    ":superhero::skin-tone-3:": "🦸🏽",
    ":superhero_tone4:": "🦸🏾",
    ":superhero_medium_dark_skin_tone:": "🦸🏾",
    ":superhero::skin-tone-4:": "🦸🏾",
    ":superhero_tone5:": "🦸🏿",
    ":superhero_dark_skin_tone:": "🦸🏿",
    ":superhero::skin-tone-5:": "🦸🏿",
    ":supervillain:": "🦹",
    ":supervillain_tone1:": "🦹🏻",
    ":supervillain_light_skin_tone:": "🦹🏻",
    ":supervillain::skin-tone-1:": "🦹🏻",
    ":supervillain_tone2:": "🦹🏼",
    ":supervillain_medium_light_skin_tone:": "🦹🏼",
    ":supervillain::skin-tone-2:": "🦹🏼",
    ":supervillain_tone3:": "🦹🏽",
    ":supervillain_medium_skin_tone:": "🦹🏽",
    ":supervillain::skin-tone-3:": "🦹🏽",
    ":supervillain_tone4:": "🦹🏾",
    ":supervillain_medium_dark_skin_tone:": "🦹🏾",
    ":supervillain::skin-tone-4:": "🦹🏾",
    ":supervillain_tone5:": "🦹🏿",
    ":supervillain_dark_skin_tone:": "🦹🏿",
    ":supervillain::skin-tone-5:": "🦹🏿",
    ":sushi:": "🍣",
    ":suspension_railway:": "🚟",
    ":swan:": "🦢",
    ":sweat:": "😓",
    ":sweat_drops:": "💦",
    ":sweat_smile:": "😅",
    ":sweet_potato:": "🍠",
    ":symbols:": "🔣",
    ":synagogue:": "🕍",
    ":syringe:": "💉",
    ":t_rex:": "🦖",
    ":taco:": "🌮",
    ":tada:": "🎉",
    ":takeout_box:": "🥡",
    ":tamale:": "🫔",
    ":tanabata_tree:": "🎋",
    ":tangerine:": "🍊",
    ":taurus:": "♉",
    ":taxi:": "🚕",
    ":tea:": "🍵",
    ":teacher:": "🧑‍🏫",
    ":teacher_tone1:": "🧑🏻‍🏫",
    ":teacher_light_skin_tone:": "🧑🏻‍🏫",
    ":teacher::skin-tone-1:": "🧑🏻‍🏫",
    ":teacher_tone2:": "🧑🏼‍🏫",
    ":teacher_medium_light_skin_tone:": "🧑🏼‍🏫",
    ":teacher::skin-tone-2:": "🧑🏼‍🏫",
    ":teacher_tone3:": "🧑🏽‍🏫",
    ":teacher_medium_skin_tone:": "🧑🏽‍🏫",
    ":teacher::skin-tone-3:": "🧑🏽‍🏫",
    ":teacher_tone4:": "🧑🏾‍🏫",
    ":teacher_medium_dark_skin_tone:": "🧑🏾‍🏫",
    ":teacher::skin-tone-4:": "🧑🏾‍🏫",
    ":teacher_tone5:": "🧑🏿‍🏫",
    ":teacher_dark_skin_tone:": "🧑🏿‍🏫",
    ":teacher::skin-tone-5:": "🧑🏿‍🏫",
    ":teapot:": "🫖",
    ":technologist:": "🧑‍💻",
    ":technologist_tone1:": "🧑🏻‍💻",
    ":technologist_light_skin_tone:": "🧑🏻‍💻",
    ":technologist::skin-tone-1:": "🧑🏻‍💻",
    ":technologist_tone2:": "🧑🏼‍💻",
    ":technologist_medium_light_skin_tone:": "🧑🏼‍💻",
    ":technologist::skin-tone-2:": "🧑🏼‍💻",
    ":technologist_tone3:": "🧑🏽‍💻",
    ":technologist_medium_skin_tone:": "🧑🏽‍💻",
    ":technologist::skin-tone-3:": "🧑🏽‍💻",
    ":technologist_tone4:": "🧑🏾‍💻",
    ":technologist_medium_dark_skin_tone:": "🧑🏾‍💻",
    ":technologist::skin-tone-4:": "🧑🏾‍💻",
    ":technologist_tone5:": "🧑🏿‍💻",
    ":technologist_dark_skin_tone:": "🧑🏿‍💻",
    ":technologist::skin-tone-5:": "🧑🏿‍💻",
    ":teddy_bear:": "🧸",
    ":telephone:": "☎️",
    ":telephone_receiver:": "📞",
    ":telescope:": "🔭",
    ":tennis:": "🎾",
    ":tent:": "⛺",
    ":test_tube:": "🧪",
    ":thermometer:": "🌡️",
    ":thermometer_face:": "🤒",
    ":face_with_thermometer:": "🤒",
    ":thinking:": "🤔",
    ":thinking_face:": "🤔",
    ":third_place:": "🥉",
    ":third_place_medal:": "🥉",
    ":thong_sandal:": "🩴",
    ":thought_balloon:": "💭",
    ":thread:": "🧵",
    ":three:": "3️⃣",
    ":thumbsdown:": "👎",
    ":-1:": "👎",
    ":thumbdown:": "👎",
    ":thumbsdown_tone1:": "👎🏻",
    ":_1_tone1:": "👎🏻",
    ":thumbdown_tone1:": "👎🏻",
    ":thumbsdown::skin-tone-1:": "👎🏻",
    ":-1::skin-tone-1:": "👎🏻",
    ":thumbdown::skin-tone-1:": "👎🏻",
    ":thumbsdown_tone2:": "👎🏼",
    ":_1_tone2:": "👎🏼",
    ":thumbdown_tone2:": "👎🏼",
    ":thumbsdown::skin-tone-2:": "👎🏼",
    ":-1::skin-tone-2:": "👎🏼",
    ":thumbdown::skin-tone-2:": "👎🏼",
    ":thumbsdown_tone3:": "👎🏽",
    ":_1_tone3:": "👎🏽",
    ":thumbdown_tone3:": "👎🏽",
    ":thumbsdown::skin-tone-3:": "👎🏽",
    ":-1::skin-tone-3:": "👎🏽",
    ":thumbdown::skin-tone-3:": "👎🏽",
    ":thumbsdown_tone4:": "👎🏾",
    ":_1_tone4:": "👎🏾",
    ":thumbdown_tone4:": "👎🏾",
    ":thumbsdown::skin-tone-4:": "👎🏾",
    ":-1::skin-tone-4:": "👎🏾",
    ":thumbdown::skin-tone-4:": "👎🏾",
    ":thumbsdown_tone5:": "👎🏿",
    ":_1_tone5:": "👎🏿",
    ":thumbdown_tone5:": "👎🏿",
    ":thumbsdown::skin-tone-5:": "👎🏿",
    ":-1::skin-tone-5:": "👎🏿",
    ":thumbdown::skin-tone-5:": "👎🏿",
    ":thumbsup:": "👍",
    ":+1:": "👍",
    ":thumbup:": "👍",
    ":thumbsup_tone1:": "👍🏻",
    ":+1_tone1:": "👍🏻",
    ":thumbup_tone1:": "👍🏻",
    ":thumbsup::skin-tone-1:": "👍🏻",
    ":+1::skin-tone-1:": "👍🏻",
    ":thumbup::skin-tone-1:": "👍🏻",
    ":thumbsup_tone2:": "👍🏼",
    ":+1_tone2:": "👍🏼",
    ":thumbup_tone2:": "👍🏼",
    ":thumbsup::skin-tone-2:": "👍🏼",
    ":+1::skin-tone-2:": "👍🏼",
    ":thumbup::skin-tone-2:": "👍🏼",
    ":thumbsup_tone3:": "👍🏽",
    ":+1_tone3:": "👍🏽",
    ":thumbup_tone3:": "👍🏽",
    ":thumbsup::skin-tone-3:": "👍🏽",
    ":+1::skin-tone-3:": "👍🏽",
    ":thumbup::skin-tone-3:": "👍🏽",
    ":thumbsup_tone4:": "👍🏾",
    ":+1_tone4:": "👍🏾",
    ":thumbup_tone4:": "👍🏾",
    ":thumbsup::skin-tone-4:": "👍🏾",
    ":+1::skin-tone-4:": "👍🏾",
    ":thumbup::skin-tone-4:": "👍🏾",
    ":thumbsup_tone5:": "👍🏿",
    ":+1_tone5:": "👍🏿",
    ":thumbup_tone5:": "👍🏿",
    ":thumbsup::skin-tone-5:": "👍🏿",
    ":+1::skin-tone-5:": "👍🏿",
    ":thumbup::skin-tone-5:": "👍🏿",
    ":thunder_cloud_rain:": "⛈️",
    ":thunder_cloud_and_rain:": "⛈️",
    ":ticket:": "🎫",
    ":tickets:": "🎟️",
    ":admission_tickets:": "🎟️",
    ":tiger:": "🐯",
    ":tiger2:": "🐅",
    ":timer:": "⏲️",
    ":timer_clock:": "⏲️",
    ":tired_face:": "😫",
    ":tm:": "™️",
    ":toilet:": "🚽",
    ":tokyo_tower:": "🗼",
    ":tomato:": "🍅",
    ":tongue:": "👅",
    ":toolbox:": "🧰",
    ":tools:": "🛠️",
    ":hammer_and_wrench:": "🛠️",
    ":tooth:": "🦷",
    ":toothbrush:": "🪥",
    ":top:": "🔝",
    ":tophat:": "🎩",
    ":track_next:": "⏭️",
    ":next_track:": "⏭️",
    ":track_previous:": "⏮️",
    ":previous_track:": "⏮️",
    ":trackball:": "🖲️",
    ":tractor:": "🚜",
    ":traffic_light:": "🚥",
    ":train:": "🚋",
    ":train2:": "🚆",
    ":tram:": "🚊",
    ":transgender_flag:": "🏳️‍⚧️",
    ":transgender_symbol:": "⚧",
    ":triangular_flag_on_post:": "🚩",
    ":triangular_ruler:": "📐",
    ":trident:": "🔱",
    ":triumph:": "😤",
    ":trolleybus:": "🚎",
    ":trophy:": "🏆",
    ":tropical_drink:": "🍹",
    ":tropical_fish:": "🐠",
    ":truck:": "🚚",
    ":trumpet:": "🎺",
    ":tulip:": "🌷",
    ":tumbler_glass:": "🥃",
    ":whisky:": "🥃",
    ":turkey:": "🦃",
    ":turtle:": "🐢",
    ":tv:": "📺",
    ":twisted_rightwards_arrows:": "🔀",
    ":two:": "2️⃣",
    ":two_hearts:": "💕",
    ":two_women_holding_hands:": "👭",
    ":women_holding_hands_tone1:": "👭",
    ":women_holding_hands_light_skin_tone:": "👭",
    ":women_holding_hands_tone1_tone2:": "👭",
    ":women_holding_hands_light_skin_tone_medium_light_skin_tone:": "👭",
    ":women_holding_hands_tone1_tone3:": "👭",
    ":women_holding_hands_light_skin_tone_medium_skin_tone:": "👭",
    ":women_holding_hands_tone1_tone4:": "👭",
    ":women_holding_hands_light_skin_tone_medium_dark_skin_tone:": "👭",
    ":women_holding_hands_tone1_tone5:": "👭",
    ":women_holding_hands_light_skin_tone_dark_skin_tone:": "👭",
    ":women_holding_hands_tone2:": "👭",
    ":women_holding_hands_medium_light_skin_tone:": "👭",
    ":women_holding_hands_tone2_tone1:": "👭",
    ":women_holding_hands_medium_light_skin_tone_light_skin_tone:": "👭",
    ":women_holding_hands_tone2_tone3:": "👭",
    ":women_holding_hands_medium_light_skin_tone_medium_skin_tone:": "👭",
    ":women_holding_hands_tone2_tone4:": "👭",
    ":women_holding_hands_medium_light_skin_tone_medium_dark_skin_tone:": "👭",
    ":women_holding_hands_tone2_tone5:": "👭",
    ":women_holding_hands_medium_light_skin_tone_dark_skin_tone:": "👭",
    ":women_holding_hands_tone3:": "👭",
    ":women_holding_hands_medium_skin_tone:": "👭",
    ":women_holding_hands_tone3_tone1:": "👭",
    ":women_holding_hands_medium_skin_tone_light_skin_tone:": "👭",
    ":women_holding_hands_tone3_tone2:": "👭",
    ":women_holding_hands_medium_skin_tone_medium_light_skin_tone:": "👭",
    ":women_holding_hands_tone3_tone4:": "👭",
    ":women_holding_hands_medium_skin_tone_medium_dark_skin_tone:": "👭",
    ":women_holding_hands_tone3_tone5:": "👭",
    ":women_holding_hands_medium_skin_tone_dark_skin_tone:": "👭",
    ":women_holding_hands_tone4:": "👭",
    ":women_holding_hands_medium_dark_skin_tone:": "👭",
    ":women_holding_hands_tone4_tone1:": "👭",
    ":women_holding_hands_medium_dark_skin_tone_light_skin_tone:": "👭",
    ":women_holding_hands_tone4_tone2:": "👭",
    ":women_holding_hands_medium_dark_skin_tone_medium_light_skin_tone:": "👭",
    ":women_holding_hands_tone4_tone3:": "👭",
    ":women_holding_hands_medium_dark_skin_tone_medium_skin_tone:": "👭",
    ":women_holding_hands_tone4_tone5:": "👭",
    ":women_holding_hands_medium_dark_skin_tone_dark_skin_tone:": "👭",
    ":women_holding_hands_tone5:": "👭",
    ":women_holding_hands_dark_skin_tone:": "👭",
    ":women_holding_hands_tone5_tone1:": "👭",
    ":women_holding_hands_dark_skin_tone_light_skin_tone:": "👭",
    ":women_holding_hands_tone5_tone2:": "👭",
    ":women_holding_hands_dark_skin_tone_medium_light_skin_tone:": "👭",
    ":women_holding_hands_tone5_tone3:": "👭",
    ":women_holding_hands_dark_skin_tone_medium_skin_tone:": "👭",
    ":women_holding_hands_tone5_tone4:": "👭",
    ":women_holding_hands_dark_skin_tone_medium_dark_skin_tone:": "👭",
    ":u5272:": "🈹",
    ":u5408:": "🈴",
    ":u55b6:": "🈺",
    ":u6307:": "🈯",
    ":u6708:": "🈷️",
    ":u6709:": "🈶",
    ":u6e80:": "🈵",
    ":u7121:": "🈚",
    ":u7533:": "🈸",
    ":u7981:": "🈲",
    ":u7a7a:": "🈳",
    ":umbrella:": "☔",
    ":umbrella2:": "☂️",
    ":unamused:": "😒",
    ":s": "😒",
    ":-S": "😒",
    ":z": "😒",
    ":-Z": "😒",
    ":$": "😒",
    ":-$": "😒",
    "s": "😒",
    "-S": "😒",
    "z": "😒",
    "-Z": "😒",
    "$": "😒",
    "-$": "😒",
    ":underage:": "🔞",
    ":unicorn:": "🦄",
    ":unicorn_face:": "🦄",
    ":united_nations:": "🇺🇳",
    ":unlock:": "🔓",
    ":up:": "🆙",
    ":upside_down:": "🙃",
    ":upside_down_face:": "🙃",
    ":urn:": "⚱️",
    ":funeral_urn:": "⚱️",
    ":v:": "✌️",
    ":v_tone1:": "✌🏻",
    ":v::skin-tone-1:": "✌🏻",
    ":v_tone2:": "✌🏼",
    ":v::skin-tone-2:": "✌🏼",
    ":v_tone3:": "✌🏽",
    ":v::skin-tone-3:": "✌🏽",
    ":v_tone4:": "✌🏾",
    ":v::skin-tone-4:": "✌🏾",
    ":v_tone5:": "✌🏿",
    ":v::skin-tone-5:": "✌🏿",
    ":vampire:": "🧛",
    ":vampire_tone1:": "🧛🏻",
    ":vampire_light_skin_tone:": "🧛🏻",
    ":vampire::skin-tone-1:": "🧛🏻",
    ":vampire_tone2:": "🧛🏼",
    ":vampire_medium_light_skin_tone:": "🧛🏼",
    ":vampire::skin-tone-2:": "🧛🏼",
    ":vampire_tone3:": "🧛🏽",
    ":vampire_medium_skin_tone:": "🧛🏽",
    ":vampire::skin-tone-3:": "🧛🏽",
    ":vampire_tone4:": "🧛🏾",
    ":vampire_medium_dark_skin_tone:": "🧛🏾",
    ":vampire::skin-tone-4:": "🧛🏾",
    ":vampire_tone5:": "🧛🏿",
    ":vampire_dark_skin_tone:": "🧛🏿",
    ":vampire::skin-tone-5:": "🧛🏿",
    ":vertical_traffic_light:": "🚦",
    ":vhs:": "📼",
    ":vibration_mode:": "📳",
    ":video_camera:": "📹",
    ":video_game:": "🎮",
    ":violin:": "🎻",
    ":virgo:": "♍",
    ":volcano:": "🌋",
    ":volleyball:": "🏐",
    ":vs:": "🆚",
    ":vulcan:": "🖖",
    ":raised_hand_with_part_between_middle_and_ring_fingers:": "🖖",
    ":vulcan_tone1:": "🖖🏻",
    ":raised_hand_with_part_between_middle_and_ring_fingers_tone1:": "🖖🏻",
    ":vulcan::skin-tone-1:": "🖖🏻",
    ":raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-1:": "🖖🏻",
    ":vulcan_tone2:": "🖖🏼",
    ":raised_hand_with_part_between_middle_and_ring_fingers_tone2:": "🖖🏼",
    ":vulcan::skin-tone-2:": "🖖🏼",
    ":raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-2:": "🖖🏼",
    ":vulcan_tone3:": "🖖🏽",
    ":raised_hand_with_part_between_middle_and_ring_fingers_tone3:": "🖖🏽",
    ":vulcan::skin-tone-3:": "🖖🏽",
    ":raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-3:": "🖖🏽",
    ":vulcan_tone4:": "🖖🏾",
    ":raised_hand_with_part_between_middle_and_ring_fingers_tone4:": "🖖🏾",
    ":vulcan::skin-tone-4:": "🖖🏾",
    ":raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-4:": "🖖🏾",
    ":vulcan_tone5:": "🖖🏿",
    ":raised_hand_with_part_between_middle_and_ring_fingers_tone5:": "🖖🏿",
    ":vulcan::skin-tone-5:": "🖖🏿",
    ":raised_hand_with_part_between_middle_and_ring_fingers::skin-tone-5:": "🖖🏿",
    ":waffle:": "🧇",
    ":wales:": "🏴󠁧󠁢󠁷󠁬󠁳󠁿",
    ":waning_crescent_moon:": "🌘",
    ":waning_gibbous_moon:": "🌖",
    ":warning:": "⚠️",
    ":wastebasket:": "🗑️",
    ":watch:": "⌚",
    ":water_buffalo:": "🐃",
    ":watermelon:": "🍉",
    ":wave:": "👋",
    ":wave_tone1:": "👋🏻",
    ":wave::skin-tone-1:": "👋🏻",
    ":wave_tone2:": "👋🏼",
    ":wave::skin-tone-2:": "👋🏼",
    ":wave_tone3:": "👋🏽",
    ":wave::skin-tone-3:": "👋🏽",
    ":wave_tone4:": "👋🏾",
    ":wave::skin-tone-4:": "👋🏾",
    ":wave_tone5:": "👋🏿",
    ":wave::skin-tone-5:": "👋🏿",
    ":wavy_dash:": "〰️",
    ":waxing_crescent_moon:": "🌒",
    ":waxing_gibbous_moon:": "🌔",
    ":wc:": "🚾",
    ":weary:": "😩",
    ":wedding:": "💒",
    ":whale:": "🐳",
    ":whale2:": "🐋",
    ":wheel_of_dharma:": "☸️",
    ":wheelchair:": "♿",
    ":white_check_mark:": "✅",
    ":white_circle:": "⚪",
    ":white_flower:": "💮",
    ":white_heart:": "🤍",
    ":white_large_square:": "⬜",
    ":white_medium_small_square:": "◽",
    ":white_medium_square:": "◻️",
    ":white_small_square:": "▫️",
    ":white_square_button:": "🔳",
    ":white_sun_cloud:": "🌥️",
    ":white_sun_behind_cloud:": "🌥️",
    ":white_sun_rain_cloud:": "🌦️",
    ":white_sun_behind_cloud_with_rain:": "🌦️",
    ":white_sun_small_cloud:": "🌤️",
    ":white_sun_with_small_cloud:": "🌤️",
    ":wilted_rose:": "🥀",
    ":wilted_flower:": "🥀",
    ":wind_blowing_face:": "🌬️",
    ":wind_chime:": "🎐",
    ":window:": "🪟",
    ":wine_glass:": "🍷",
    ":wink:": "😉",
    ";)": "😉",
    ";-)": "😉",
    ":wolf:": "🐺",
    ":woman:": "👩",
    ":woman_artist:": "👩‍🎨",
    ":woman_artist_tone1:": "👩🏻‍🎨",
    ":woman_artist_light_skin_tone:": "👩🏻‍🎨",
    ":woman_artist::skin-tone-1:": "👩🏻‍🎨",
    ":woman_artist_tone2:": "👩🏼‍🎨",
    ":woman_artist_medium_light_skin_tone:": "👩🏼‍🎨",
    ":woman_artist::skin-tone-2:": "👩🏼‍🎨",
    ":woman_artist_tone3:": "👩🏽‍🎨",
    ":woman_artist_medium_skin_tone:": "👩🏽‍🎨",
    ":woman_artist::skin-tone-3:": "👩🏽‍🎨",
    ":woman_artist_tone4:": "👩🏾‍🎨",
    ":woman_artist_medium_dark_skin_tone:": "👩🏾‍🎨",
    ":woman_artist::skin-tone-4:": "👩🏾‍🎨",
    ":woman_artist_tone5:": "👩🏿‍🎨",
    ":woman_artist_dark_skin_tone:": "👩🏿‍🎨",
    ":woman_artist::skin-tone-5:": "👩🏿‍🎨",
    ":woman_astronaut:": "👩‍🚀",
    ":woman_astronaut_tone1:": "👩🏻‍🚀",
    ":woman_astronaut_light_skin_tone:": "👩🏻‍🚀",
    ":woman_astronaut::skin-tone-1:": "👩🏻‍🚀",
    ":woman_astronaut_tone2:": "👩🏼‍🚀",
    ":woman_astronaut_medium_light_skin_tone:": "👩🏼‍🚀",
    ":woman_astronaut::skin-tone-2:": "👩🏼‍🚀",
    ":woman_astronaut_tone3:": "👩🏽‍🚀",
    ":woman_astronaut_medium_skin_tone:": "👩🏽‍🚀",
    ":woman_astronaut::skin-tone-3:": "👩🏽‍🚀",
    ":woman_astronaut_tone4:": "👩🏾‍🚀",
    ":woman_astronaut_medium_dark_skin_tone:": "👩🏾‍🚀",
    ":woman_astronaut::skin-tone-4:": "👩🏾‍🚀",
    ":woman_astronaut_tone5:": "👩🏿‍🚀",
    ":woman_astronaut_dark_skin_tone:": "👩🏿‍🚀",
    ":woman_astronaut::skin-tone-5:": "👩🏿‍🚀",
    ":woman_bald:": "👩‍🦲",
    ":woman_bald_tone1:": "👩🏻‍🦲",
    ":woman_bald_light_skin_tone:": "👩🏻‍🦲",
    ":woman_bald::skin-tone-1:": "👩🏻‍🦲",
    ":woman_bald_tone2:": "👩🏼‍🦲",
    ":woman_bald_medium_light_skin_tone:": "👩🏼‍🦲",
    ":woman_bald::skin-tone-2:": "👩🏼‍🦲",
    ":woman_bald_tone3:": "👩🏽‍🦲",
    ":woman_bald_medium_skin_tone:": "👩🏽‍🦲",
    ":woman_bald::skin-tone-3:": "👩🏽‍🦲",
    ":woman_bald_tone4:": "👩🏾‍🦲",
    ":woman_bald_medium_dark_skin_tone:": "👩🏾‍🦲",
    ":woman_bald::skin-tone-4:": "👩🏾‍🦲",
    ":woman_bald_tone5:": "👩🏿‍🦲",
    ":woman_bald_dark_skin_tone:": "👩🏿‍🦲",
    ":woman_bald::skin-tone-5:": "👩🏿‍🦲",
    ":woman_biking:": "🚴‍♀️",
    ":woman_biking_tone1:": "🚴🏻‍♀️",
    ":woman_biking_light_skin_tone:": "🚴🏻‍♀️",
    ":woman_biking::skin-tone-1:": "🚴🏻‍♀️",
    ":woman_biking_tone2:": "🚴🏼‍♀️",
    ":woman_biking_medium_light_skin_tone:": "🚴🏼‍♀️",
    ":woman_biking::skin-tone-2:": "🚴🏼‍♀️",
    ":woman_biking_tone3:": "🚴🏽‍♀️",
    ":woman_biking_medium_skin_tone:": "🚴🏽‍♀️",
    ":woman_biking::skin-tone-3:": "🚴🏽‍♀️",
    ":woman_biking_tone4:": "🚴🏾‍♀️",
    ":woman_biking_medium_dark_skin_tone:": "🚴🏾‍♀️",
    ":woman_biking::skin-tone-4:": "🚴🏾‍♀️",
    ":woman_biking_tone5:": "🚴🏿‍♀️",
    ":woman_biking_dark_skin_tone:": "🚴🏿‍♀️",
    ":woman_biking::skin-tone-5:": "🚴🏿‍♀️",
    ":woman_bouncing_ball:": "⛹️‍♀️",
    ":woman_bouncing_ball_tone1:": "⛹🏻‍♀️",
    ":woman_bouncing_ball_light_skin_tone:": "⛹🏻‍♀️",
    ":woman_bouncing_ball::skin-tone-1:": "⛹🏻‍♀️",
    ":woman_bouncing_ball_tone2:": "⛹🏼‍♀️",
    ":woman_bouncing_ball_medium_light_skin_tone:": "⛹🏼‍♀️",
    ":woman_bouncing_ball::skin-tone-2:": "⛹🏼‍♀️",
    ":woman_bouncing_ball_tone3:": "⛹🏽‍♀️",
    ":woman_bouncing_ball_medium_skin_tone:": "⛹🏽‍♀️",
    ":woman_bouncing_ball::skin-tone-3:": "⛹🏽‍♀️",
    ":woman_bouncing_ball_tone4:": "⛹🏾‍♀️",
    ":woman_bouncing_ball_medium_dark_skin_tone:": "⛹🏾‍♀️",
    ":woman_bouncing_ball::skin-tone-4:": "⛹🏾‍♀️",
    ":woman_bouncing_ball_tone5:": "⛹🏿‍♀️",
    ":woman_bouncing_ball_dark_skin_tone:": "⛹🏿‍♀️",
    ":woman_bouncing_ball::skin-tone-5:": "⛹🏿‍♀️",
    ":woman_bowing:": "🙇‍♀️",
    ":woman_bowing_tone1:": "🙇🏻‍♀️",
    ":woman_bowing_light_skin_tone:": "🙇🏻‍♀️",
    ":woman_bowing::skin-tone-1:": "🙇🏻‍♀️",
    ":woman_bowing_tone2:": "🙇🏼‍♀️",
    ":woman_bowing_medium_light_skin_tone:": "🙇🏼‍♀️",
    ":woman_bowing::skin-tone-2:": "🙇🏼‍♀️",
    ":woman_bowing_tone3:": "🙇🏽‍♀️",
    ":woman_bowing_medium_skin_tone:": "🙇🏽‍♀️",
    ":woman_bowing::skin-tone-3:": "🙇🏽‍♀️",
    ":woman_bowing_tone4:": "🙇🏾‍♀️",
    ":woman_bowing_medium_dark_skin_tone:": "🙇🏾‍♀️",
    ":woman_bowing::skin-tone-4:": "🙇🏾‍♀️",
    ":woman_bowing_tone5:": "🙇🏿‍♀️",
    ":woman_bowing_dark_skin_tone:": "🙇🏿‍♀️",
    ":woman_bowing::skin-tone-5:": "🙇🏿‍♀️",
    ":woman_cartwheeling:": "🤸‍♀️",
    ":woman_cartwheeling_tone1:": "🤸🏻‍♀️",
    ":woman_cartwheeling_light_skin_tone:": "🤸🏻‍♀️",
    ":woman_cartwheeling::skin-tone-1:": "🤸🏻‍♀️",
    ":woman_cartwheeling_tone2:": "🤸🏼‍♀️",
    ":woman_cartwheeling_medium_light_skin_tone:": "🤸🏼‍♀️",
    ":woman_cartwheeling::skin-tone-2:": "🤸🏼‍♀️",
    ":woman_cartwheeling_tone3:": "🤸🏽‍♀️",
    ":woman_cartwheeling_medium_skin_tone:": "🤸🏽‍♀️",
    ":woman_cartwheeling::skin-tone-3:": "🤸🏽‍♀️",
    ":woman_cartwheeling_tone4:": "🤸🏾‍♀️",
    ":woman_cartwheeling_medium_dark_skin_tone:": "🤸🏾‍♀️",
    ":woman_cartwheeling::skin-tone-4:": "🤸🏾‍♀️",
    ":woman_cartwheeling_tone5:": "🤸🏿‍♀️",
    ":woman_cartwheeling_dark_skin_tone:": "🤸🏿‍♀️",
    ":woman_cartwheeling::skin-tone-5:": "🤸🏿‍♀️",
    ":woman_climbing:": "🧗‍♀️",
    ":woman_climbing_tone1:": "🧗🏻‍♀️",
    ":woman_climbing_light_skin_tone:": "🧗🏻‍♀️",
    ":woman_climbing::skin-tone-1:": "🧗🏻‍♀️",
    ":woman_climbing_tone2:": "🧗🏼‍♀️",
    ":woman_climbing_medium_light_skin_tone:": "🧗🏼‍♀️",
    ":woman_climbing::skin-tone-2:": "🧗🏼‍♀️",
    ":woman_climbing_tone3:": "🧗🏽‍♀️",
    ":woman_climbing_medium_skin_tone:": "🧗🏽‍♀️",
    ":woman_climbing::skin-tone-3:": "🧗🏽‍♀️",
    ":woman_climbing_tone4:": "🧗🏾‍♀️",
    ":woman_climbing_medium_dark_skin_tone:": "🧗🏾‍♀️",
    ":woman_climbing::skin-tone-4:": "🧗🏾‍♀️",
    ":woman_climbing_tone5:": "🧗🏿‍♀️",
    ":woman_climbing_dark_skin_tone:": "🧗🏿‍♀️",
    ":woman_climbing::skin-tone-5:": "🧗🏿‍♀️",
    ":woman_construction_worker:": "👷‍♀️",
    ":woman_construction_worker_tone1:": "👷🏻‍♀️",
    ":woman_construction_worker_light_skin_tone:": "👷🏻‍♀️",
    ":woman_construction_worker::skin-tone-1:": "👷🏻‍♀️",
    ":woman_construction_worker_tone2:": "👷🏼‍♀️",
    ":woman_construction_worker_medium_light_skin_tone:": "👷🏼‍♀️",
    ":woman_construction_worker::skin-tone-2:": "👷🏼‍♀️",
    ":woman_construction_worker_tone3:": "👷🏽‍♀️",
    ":woman_construction_worker_medium_skin_tone:": "👷🏽‍♀️",
    ":woman_construction_worker::skin-tone-3:": "👷🏽‍♀️",
    ":woman_construction_worker_tone4:": "👷🏾‍♀️",
    ":woman_construction_worker_medium_dark_skin_tone:": "👷🏾‍♀️",
    ":woman_construction_worker::skin-tone-4:": "👷🏾‍♀️",
    ":woman_construction_worker_tone5:": "👷🏿‍♀️",
    ":woman_construction_worker_dark_skin_tone:": "👷🏿‍♀️",
    ":woman_construction_worker::skin-tone-5:": "👷🏿‍♀️",
    ":woman_cook:": "👩‍🍳",
    ":woman_cook_tone1:": "👩🏻‍🍳",
    ":woman_cook_light_skin_tone:": "👩🏻‍🍳",
    ":woman_cook::skin-tone-1:": "👩🏻‍🍳",
    ":woman_cook_tone2:": "👩🏼‍🍳",
    ":woman_cook_medium_light_skin_tone:": "👩🏼‍🍳",
    ":woman_cook::skin-tone-2:": "👩🏼‍🍳",
    ":woman_cook_tone3:": "👩🏽‍🍳",
    ":woman_cook_medium_skin_tone:": "👩🏽‍🍳",
    ":woman_cook::skin-tone-3:": "👩🏽‍🍳",
    ":woman_cook_tone4:": "👩🏾‍🍳",
    ":woman_cook_medium_dark_skin_tone:": "👩🏾‍🍳",
    ":woman_cook::skin-tone-4:": "👩🏾‍🍳",
    ":woman_cook_tone5:": "👩🏿‍🍳",
    ":woman_cook_dark_skin_tone:": "👩🏿‍🍳",
    ":woman_cook::skin-tone-5:": "👩🏿‍🍳",
    ":woman_curly_haired:": "👩‍🦱",
    ":woman_curly_haired_tone1:": "👩🏻‍🦱",
    ":woman_curly_haired_light_skin_tone:": "👩🏻‍🦱",
    ":woman_curly_haired::skin-tone-1:": "👩🏻‍🦱",
    ":woman_curly_haired_tone2:": "👩🏼‍🦱",
    ":woman_curly_haired_medium_light_skin_tone:": "👩🏼‍🦱",
    ":woman_curly_haired::skin-tone-2:": "👩🏼‍🦱",
    ":woman_curly_haired_tone3:": "👩🏽‍🦱",
    ":woman_curly_haired_medium_skin_tone:": "👩🏽‍🦱",
    ":woman_curly_haired::skin-tone-3:": "👩🏽‍🦱",
    ":woman_curly_haired_tone4:": "👩🏾‍🦱",
    ":woman_curly_haired_medium_dark_skin_tone:": "👩🏾‍🦱",
    ":woman_curly_haired::skin-tone-4:": "👩🏾‍🦱",
    ":woman_curly_haired_tone5:": "👩🏿‍🦱",
    ":woman_curly_haired_dark_skin_tone:": "👩🏿‍🦱",
    ":woman_curly_haired::skin-tone-5:": "👩🏿‍🦱",
    ":woman_detective:": "🕵️‍♀️",
    ":woman_detective_tone1:": "🕵🏻‍♀️",
    ":woman_detective_light_skin_tone:": "🕵🏻‍♀️",
    ":woman_detective::skin-tone-1:": "🕵🏻‍♀️",
    ":woman_detective_tone2:": "🕵🏼‍♀️",
    ":woman_detective_medium_light_skin_tone:": "🕵🏼‍♀️",
    ":woman_detective::skin-tone-2:": "🕵🏼‍♀️",
    ":woman_detective_tone3:": "🕵🏽‍♀️",
    ":woman_detective_medium_skin_tone:": "🕵🏽‍♀️",
    ":woman_detective::skin-tone-3:": "🕵🏽‍♀️",
    ":woman_detective_tone4:": "🕵🏾‍♀️",
    ":woman_detective_medium_dark_skin_tone:": "🕵🏾‍♀️",
    ":woman_detective::skin-tone-4:": "🕵🏾‍♀️",
    ":woman_detective_tone5:": "🕵🏿‍♀️",
    ":woman_detective_dark_skin_tone:": "🕵🏿‍♀️",
    ":woman_detective::skin-tone-5:": "🕵🏿‍♀️",
    ":woman_elf:": "🧝‍♀️",
    ":woman_elf_tone1:": "🧝🏻‍♀️",
    ":woman_elf_light_skin_tone:": "🧝🏻‍♀️",
    ":woman_elf::skin-tone-1:": "🧝🏻‍♀️",
    ":woman_elf_tone2:": "🧝🏼‍♀️",
    ":woman_elf_medium_light_skin_tone:": "🧝🏼‍♀️",
    ":woman_elf::skin-tone-2:": "🧝🏼‍♀️",
    ":woman_elf_tone3:": "🧝🏽‍♀️",
    ":woman_elf_medium_skin_tone:": "🧝🏽‍♀️",
    ":woman_elf::skin-tone-3:": "🧝🏽‍♀️",
    ":woman_elf_tone4:": "🧝🏾‍♀️",
    ":woman_elf_medium_dark_skin_tone:": "🧝🏾‍♀️",
    ":woman_elf::skin-tone-4:": "🧝🏾‍♀️",
    ":woman_elf_tone5:": "🧝🏿‍♀️",
    ":woman_elf_dark_skin_tone:": "🧝🏿‍♀️",
    ":woman_elf::skin-tone-5:": "🧝🏿‍♀️",
    ":woman_facepalming:": "🤦‍♀️",
    ":woman_facepalming_tone1:": "🤦🏻‍♀️",
    ":woman_facepalming_light_skin_tone:": "🤦🏻‍♀️",
    ":woman_facepalming::skin-tone-1:": "🤦🏻‍♀️",
    ":woman_facepalming_tone2:": "🤦🏼‍♀️",
    ":woman_facepalming_medium_light_skin_tone:": "🤦🏼‍♀️",
    ":woman_facepalming::skin-tone-2:": "🤦🏼‍♀️",
    ":woman_facepalming_tone3:": "🤦🏽‍♀️",
    ":woman_facepalming_medium_skin_tone:": "🤦🏽‍♀️",
    ":woman_facepalming::skin-tone-3:": "🤦🏽‍♀️",
    ":woman_facepalming_tone4:": "🤦🏾‍♀️",
    ":woman_facepalming_medium_dark_skin_tone:": "🤦🏾‍♀️",
    ":woman_facepalming::skin-tone-4:": "🤦🏾‍♀️",
    ":woman_facepalming_tone5:": "🤦🏿‍♀️",
    ":woman_facepalming_dark_skin_tone:": "🤦🏿‍♀️",
    ":woman_facepalming::skin-tone-5:": "🤦🏿‍♀️",
    ":woman_factory_worker:": "👩‍🏭",
    ":woman_factory_worker_tone1:": "👩🏻‍🏭",
    ":woman_factory_worker_light_skin_tone:": "👩🏻‍🏭",
    ":woman_factory_worker::skin-tone-1:": "👩🏻‍🏭",
    ":woman_factory_worker_tone2:": "👩🏼‍🏭",
    ":woman_factory_worker_medium_light_skin_tone:": "👩🏼‍🏭",
    ":woman_factory_worker::skin-tone-2:": "👩🏼‍🏭",
    ":woman_factory_worker_tone3:": "👩🏽‍🏭",
    ":woman_factory_worker_medium_skin_tone:": "👩🏽‍🏭",
    ":woman_factory_worker::skin-tone-3:": "👩🏽‍🏭",
    ":woman_factory_worker_tone4:": "👩🏾‍🏭",
    ":woman_factory_worker_medium_dark_skin_tone:": "👩🏾‍🏭",
    ":woman_factory_worker::skin-tone-4:": "👩🏾‍🏭",
    ":woman_factory_worker_tone5:": "👩🏿‍🏭",
    ":woman_factory_worker_dark_skin_tone:": "👩🏿‍🏭",
    ":woman_factory_worker::skin-tone-5:": "👩🏿‍🏭",
    ":woman_fairy:": "🧚‍♀️",
    ":woman_fairy_tone1:": "🧚🏻‍♀️",
    ":woman_fairy_light_skin_tone:": "🧚🏻‍♀️",
    ":woman_fairy::skin-tone-1:": "🧚🏻‍♀️",
    ":woman_fairy_tone2:": "🧚🏼‍♀️",
    ":woman_fairy_medium_light_skin_tone:": "🧚🏼‍♀️",
    ":woman_fairy::skin-tone-2:": "🧚🏼‍♀️",
    ":woman_fairy_tone3:": "🧚🏽‍♀️",
    ":woman_fairy_medium_skin_tone:": "🧚🏽‍♀️",
    ":woman_fairy::skin-tone-3:": "🧚🏽‍♀️",
    ":woman_fairy_tone4:": "🧚🏾‍♀️",
    ":woman_fairy_medium_dark_skin_tone:": "🧚🏾‍♀️",
    ":woman_fairy::skin-tone-4:": "🧚🏾‍♀️",
    ":woman_fairy_tone5:": "🧚🏿‍♀️",
    ":woman_fairy_dark_skin_tone:": "🧚🏿‍♀️",
    ":woman_fairy::skin-tone-5:": "🧚🏿‍♀️",
    ":woman_farmer:": "👩‍🌾",
    ":woman_farmer_tone1:": "👩🏻‍🌾",
    ":woman_farmer_light_skin_tone:": "👩🏻‍🌾",
    ":woman_farmer::skin-tone-1:": "👩🏻‍🌾",
    ":woman_farmer_tone2:": "👩🏼‍🌾",
    ":woman_farmer_medium_light_skin_tone:": "👩🏼‍🌾",
    ":woman_farmer::skin-tone-2:": "👩🏼‍🌾",
    ":woman_farmer_tone3:": "👩🏽‍🌾",
    ":woman_farmer_medium_skin_tone:": "👩🏽‍🌾",
    ":woman_farmer::skin-tone-3:": "👩🏽‍🌾",
    ":woman_farmer_tone4:": "👩🏾‍🌾",
    ":woman_farmer_medium_dark_skin_tone:": "👩🏾‍🌾",
    ":woman_farmer::skin-tone-4:": "👩🏾‍🌾",
    ":woman_farmer_tone5:": "👩🏿‍🌾",
    ":woman_farmer_dark_skin_tone:": "👩🏿‍🌾",
    ":woman_farmer::skin-tone-5:": "👩🏿‍🌾",
    ":woman_feeding_baby:": "👩‍🍼",
    ":woman_feeding_baby_tone1:": "👩🏻‍🍼",
    ":woman_feeding_baby_light_skin_tone:": "👩🏻‍🍼",
    ":woman_feeding_baby::skin-tone-1:": "👩🏻‍🍼",
    ":woman_feeding_baby_tone2:": "👩🏼‍🍼",
    ":woman_feeding_baby_medium_light_skin_tone:": "👩🏼‍🍼",
    ":woman_feeding_baby::skin-tone-2:": "👩🏼‍🍼",
    ":woman_feeding_baby_tone3:": "👩🏽‍🍼",
    ":woman_feeding_baby_medium_skin_tone:": "👩🏽‍🍼",
    ":woman_feeding_baby::skin-tone-3:": "👩🏽‍🍼",
    ":woman_feeding_baby_tone4:": "👩🏾‍🍼",
    ":woman_feeding_baby_medium_dark_skin_tone:": "👩🏾‍🍼",
    ":woman_feeding_baby::skin-tone-4:": "👩🏾‍🍼",
    ":woman_feeding_baby_tone5:": "👩🏿‍🍼",
    ":woman_feeding_baby_dark_skin_tone:": "👩🏿‍🍼",
    ":woman_feeding_baby::skin-tone-5:": "👩🏿‍🍼",
    ":woman_firefighter:": "👩‍🚒",
    ":woman_firefighter_tone1:": "👩🏻‍🚒",
    ":woman_firefighter_light_skin_tone:": "👩🏻‍🚒",
    ":woman_firefighter::skin-tone-1:": "👩🏻‍🚒",
    ":woman_firefighter_tone2:": "👩🏼‍🚒",
    ":woman_firefighter_medium_light_skin_tone:": "👩🏼‍🚒",
    ":woman_firefighter::skin-tone-2:": "👩🏼‍🚒",
    ":woman_firefighter_tone3:": "👩🏽‍🚒",
    ":woman_firefighter_medium_skin_tone:": "👩🏽‍🚒",
    ":woman_firefighter::skin-tone-3:": "👩🏽‍🚒",
    ":woman_firefighter_tone4:": "👩🏾‍🚒",
    ":woman_firefighter_medium_dark_skin_tone:": "👩🏾‍🚒",
    ":woman_firefighter::skin-tone-4:": "👩🏾‍🚒",
    ":woman_firefighter_tone5:": "👩🏿‍🚒",
    ":woman_firefighter_dark_skin_tone:": "👩🏿‍🚒",
    ":woman_firefighter::skin-tone-5:": "👩🏿‍🚒",
    ":woman_frowning:": "🙍‍♀️",
    ":woman_frowning_tone1:": "🙍🏻‍♀️",
    ":woman_frowning_light_skin_tone:": "🙍🏻‍♀️",
    ":woman_frowning::skin-tone-1:": "🙍🏻‍♀️",
    ":woman_frowning_tone2:": "🙍🏼‍♀️",
    ":woman_frowning_medium_light_skin_tone:": "🙍🏼‍♀️",
    ":woman_frowning::skin-tone-2:": "🙍🏼‍♀️",
    ":woman_frowning_tone3:": "🙍🏽‍♀️",
    ":woman_frowning_medium_skin_tone:": "🙍🏽‍♀️",
    ":woman_frowning::skin-tone-3:": "🙍🏽‍♀️",
    ":woman_frowning_tone4:": "🙍🏾‍♀️",
    ":woman_frowning_medium_dark_skin_tone:": "🙍🏾‍♀️",
    ":woman_frowning::skin-tone-4:": "🙍🏾‍♀️",
    ":woman_frowning_tone5:": "🙍🏿‍♀️",
    ":woman_frowning_dark_skin_tone:": "🙍🏿‍♀️",
    ":woman_frowning::skin-tone-5:": "🙍🏿‍♀️",
    ":woman_genie:": "🧞‍♀️",
    ":woman_gesturing_no:": "🙅‍♀️",
    ":woman_gesturing_no_tone1:": "🙅🏻‍♀️",
    ":woman_gesturing_no_light_skin_tone:": "🙅🏻‍♀️",
    ":woman_gesturing_no::skin-tone-1:": "🙅🏻‍♀️",
    ":woman_gesturing_no_tone2:": "🙅🏼‍♀️",
    ":woman_gesturing_no_medium_light_skin_tone:": "🙅🏼‍♀️",
    ":woman_gesturing_no::skin-tone-2:": "🙅🏼‍♀️",
    ":woman_gesturing_no_tone3:": "🙅🏽‍♀️",
    ":woman_gesturing_no_medium_skin_tone:": "🙅🏽‍♀️",
    ":woman_gesturing_no::skin-tone-3:": "🙅🏽‍♀️",
    ":woman_gesturing_no_tone4:": "🙅🏾‍♀️",
    ":woman_gesturing_no_medium_dark_skin_tone:": "🙅🏾‍♀️",
    ":woman_gesturing_no::skin-tone-4:": "🙅🏾‍♀️",
    ":woman_gesturing_no_tone5:": "🙅🏿‍♀️",
    ":woman_gesturing_no_dark_skin_tone:": "🙅🏿‍♀️",
    ":woman_gesturing_no::skin-tone-5:": "🙅🏿‍♀️",
    ":woman_gesturing_ok:": "🙆‍♀️",
    ":woman_gesturing_ok_tone1:": "🙆🏻‍♀️",
    ":woman_gesturing_ok_light_skin_tone:": "🙆🏻‍♀️",
    ":woman_gesturing_ok::skin-tone-1:": "🙆🏻‍♀️",
    ":woman_gesturing_ok_tone2:": "🙆🏼‍♀️",
    ":woman_gesturing_ok_medium_light_skin_tone:": "🙆🏼‍♀️",
    ":woman_gesturing_ok::skin-tone-2:": "🙆🏼‍♀️",
    ":woman_gesturing_ok_tone3:": "🙆🏽‍♀️",
    ":woman_gesturing_ok_medium_skin_tone:": "🙆🏽‍♀️",
    ":woman_gesturing_ok::skin-tone-3:": "🙆🏽‍♀️",
    ":woman_gesturing_ok_tone4:": "🙆🏾‍♀️",
    ":woman_gesturing_ok_medium_dark_skin_tone:": "🙆🏾‍♀️",
    ":woman_gesturing_ok::skin-tone-4:": "🙆🏾‍♀️",
    ":woman_gesturing_ok_tone5:": "🙆🏿‍♀️",
    ":woman_gesturing_ok_dark_skin_tone:": "🙆🏿‍♀️",
    ":woman_gesturing_ok::skin-tone-5:": "🙆🏿‍♀️",
    ":woman_getting_face_massage:": "💆‍♀️",
    ":woman_getting_face_massage_tone1:": "💆🏻‍♀️",
    ":woman_getting_face_massage_light_skin_tone:": "💆🏻‍♀️",
    ":woman_getting_face_massage::skin-tone-1:": "💆🏻‍♀️",
    ":woman_getting_face_massage_tone2:": "💆🏼‍♀️",
    ":woman_getting_face_massage_medium_light_skin_tone:": "💆🏼‍♀️",
    ":woman_getting_face_massage::skin-tone-2:": "💆🏼‍♀️",
    ":woman_getting_face_massage_tone3:": "💆🏽‍♀️",
    ":woman_getting_face_massage_medium_skin_tone:": "💆🏽‍♀️",
    ":woman_getting_face_massage::skin-tone-3:": "💆🏽‍♀️",
    ":woman_getting_face_massage_tone4:": "💆🏾‍♀️",
    ":woman_getting_face_massage_medium_dark_skin_tone:": "💆🏾‍♀️",
    ":woman_getting_face_massage::skin-tone-4:": "💆🏾‍♀️",
    ":woman_getting_face_massage_tone5:": "💆🏿‍♀️",
    ":woman_getting_face_massage_dark_skin_tone:": "💆🏿‍♀️",
    ":woman_getting_face_massage::skin-tone-5:": "💆🏿‍♀️",
    ":woman_getting_haircut:": "💇‍♀️",
    ":woman_getting_haircut_tone1:": "💇🏻‍♀️",
    ":woman_getting_haircut_light_skin_tone:": "💇🏻‍♀️",
    ":woman_getting_haircut::skin-tone-1:": "💇🏻‍♀️",
    ":woman_getting_haircut_tone2:": "💇🏼‍♀️",
    ":woman_getting_haircut_medium_light_skin_tone:": "💇🏼‍♀️",
    ":woman_getting_haircut::skin-tone-2:": "💇🏼‍♀️",
    ":woman_getting_haircut_tone3:": "💇🏽‍♀️",
    ":woman_getting_haircut_medium_skin_tone:": "💇🏽‍♀️",
    ":woman_getting_haircut::skin-tone-3:": "💇🏽‍♀️",
    ":woman_getting_haircut_tone4:": "💇🏾‍♀️",
    ":woman_getting_haircut_medium_dark_skin_tone:": "💇🏾‍♀️",
    ":woman_getting_haircut::skin-tone-4:": "💇🏾‍♀️",
    ":woman_getting_haircut_tone5:": "💇🏿‍♀️",
    ":woman_getting_haircut_dark_skin_tone:": "💇🏿‍♀️",
    ":woman_getting_haircut::skin-tone-5:": "💇🏿‍♀️",
    ":woman_golfing:": "🏌️‍♀️",
    ":woman_golfing_tone1:": "🏌🏻‍♀️",
    ":woman_golfing_light_skin_tone:": "🏌🏻‍♀️",
    ":woman_golfing::skin-tone-1:": "🏌🏻‍♀️",
    ":woman_golfing_tone2:": "🏌🏼‍♀️",
    ":woman_golfing_medium_light_skin_tone:": "🏌🏼‍♀️",
    ":woman_golfing::skin-tone-2:": "🏌🏼‍♀️",
    ":woman_golfing_tone3:": "🏌🏽‍♀️",
    ":woman_golfing_medium_skin_tone:": "🏌🏽‍♀️",
    ":woman_golfing::skin-tone-3:": "🏌🏽‍♀️",
    ":woman_golfing_tone4:": "🏌🏾‍♀️",
    ":woman_golfing_medium_dark_skin_tone:": "🏌🏾‍♀️",
    ":woman_golfing::skin-tone-4:": "🏌🏾‍♀️",
    ":woman_golfing_tone5:": "🏌🏿‍♀️",
    ":woman_golfing_dark_skin_tone:": "🏌🏿‍♀️",
    ":woman_golfing::skin-tone-5:": "🏌🏿‍♀️",
    ":woman_guard:": "💂‍♀️",
    ":woman_guard_tone1:": "💂🏻‍♀️",
    ":woman_guard_light_skin_tone:": "💂🏻‍♀️",
    ":woman_guard::skin-tone-1:": "💂🏻‍♀️",
    ":woman_guard_tone2:": "💂🏼‍♀️",
    ":woman_guard_medium_light_skin_tone:": "💂🏼‍♀️",
    ":woman_guard::skin-tone-2:": "💂🏼‍♀️",
    ":woman_guard_tone3:": "💂🏽‍♀️",
    ":woman_guard_medium_skin_tone:": "💂🏽‍♀️",
    ":woman_guard::skin-tone-3:": "💂🏽‍♀️",
    ":woman_guard_tone4:": "💂🏾‍♀️",
    ":woman_guard_medium_dark_skin_tone:": "💂🏾‍♀️",
    ":woman_guard::skin-tone-4:": "💂🏾‍♀️",
    ":woman_guard_tone5:": "💂🏿‍♀️",
    ":woman_guard_dark_skin_tone:": "💂🏿‍♀️",
    ":woman_guard::skin-tone-5:": "💂🏿‍♀️",
    ":woman_health_worker:": "👩‍⚕️",
    ":woman_health_worker_tone1:": "👩🏻‍⚕️",
    ":woman_health_worker_light_skin_tone:": "👩🏻‍⚕️",
    ":woman_health_worker::skin-tone-1:": "👩🏻‍⚕️",
    ":woman_health_worker_tone2:": "👩🏼‍⚕️",
    ":woman_health_worker_medium_light_skin_tone:": "👩🏼‍⚕️",
    ":woman_health_worker::skin-tone-2:": "👩🏼‍⚕️",
    ":woman_health_worker_tone3:": "👩🏽‍⚕️",
    ":woman_health_worker_medium_skin_tone:": "👩🏽‍⚕️",
    ":woman_health_worker::skin-tone-3:": "👩🏽‍⚕️",
    ":woman_health_worker_tone4:": "👩🏾‍⚕️",
    ":woman_health_worker_medium_dark_skin_tone:": "👩🏾‍⚕️",
    ":woman_health_worker::skin-tone-4:": "👩🏾‍⚕️",
    ":woman_health_worker_tone5:": "👩🏿‍⚕️",
    ":woman_health_worker_dark_skin_tone:": "👩🏿‍⚕️",
    ":woman_health_worker::skin-tone-5:": "👩🏿‍⚕️",
    ":woman_in_lotus_position:": "🧘‍♀️",
    ":woman_in_lotus_position_tone1:": "🧘🏻‍♀️",
    ":woman_in_lotus_position_light_skin_tone:": "🧘🏻‍♀️",
    ":woman_in_lotus_position::skin-tone-1:": "🧘🏻‍♀️",
    ":woman_in_lotus_position_tone2:": "🧘🏼‍♀️",
    ":woman_in_lotus_position_medium_light_skin_tone:": "🧘🏼‍♀️",
    ":woman_in_lotus_position::skin-tone-2:": "🧘🏼‍♀️",
    ":woman_in_lotus_position_tone3:": "🧘🏽‍♀️",
    ":woman_in_lotus_position_medium_skin_tone:": "🧘🏽‍♀️",
    ":woman_in_lotus_position::skin-tone-3:": "🧘🏽‍♀️",
    ":woman_in_lotus_position_tone4:": "🧘🏾‍♀️",
    ":woman_in_lotus_position_medium_dark_skin_tone:": "🧘🏾‍♀️",
    ":woman_in_lotus_position::skin-tone-4:": "🧘🏾‍♀️",
    ":woman_in_lotus_position_tone5:": "🧘🏿‍♀️",
    ":woman_in_lotus_position_dark_skin_tone:": "🧘🏿‍♀️",
    ":woman_in_lotus_position::skin-tone-5:": "🧘🏿‍♀️",
    ":woman_in_manual_wheelchair:": "👩‍🦽",
    ":woman_in_manual_wheelchair_tone1:": "👩🏻‍🦽",
    ":woman_in_manual_wheelchair_light_skin_tone:": "👩🏻‍🦽",
    ":woman_in_manual_wheelchair::skin-tone-1:": "👩🏻‍🦽",
    ":woman_in_manual_wheelchair_tone2:": "👩🏼‍🦽",
    ":woman_in_manual_wheelchair_medium_light_skin_tone:": "👩🏼‍🦽",
    ":woman_in_manual_wheelchair::skin-tone-2:": "👩🏼‍🦽",
    ":woman_in_manual_wheelchair_tone3:": "👩🏽‍🦽",
    ":woman_in_manual_wheelchair_medium_skin_tone:": "👩🏽‍🦽",
    ":woman_in_manual_wheelchair::skin-tone-3:": "👩🏽‍🦽",
    ":woman_in_manual_wheelchair_tone4:": "👩🏾‍🦽",
    ":woman_in_manual_wheelchair_medium_dark_skin_tone:": "👩🏾‍🦽",
    ":woman_in_manual_wheelchair::skin-tone-4:": "👩🏾‍🦽",
    ":woman_in_manual_wheelchair_tone5:": "👩🏿‍🦽",
    ":woman_in_manual_wheelchair_dark_skin_tone:": "👩🏿‍🦽",
    ":woman_in_manual_wheelchair::skin-tone-5:": "👩🏿‍🦽",
    ":woman_in_motorized_wheelchair:": "👩‍🦼",
    ":woman_in_motorized_wheelchair_tone1:": "👩🏻‍🦼",
    ":woman_in_motorized_wheelchair_light_skin_tone:": "👩🏻‍🦼",
    ":woman_in_motorized_wheelchair::skin-tone-1:": "👩🏻‍🦼",
    ":woman_in_motorized_wheelchair_tone2:": "👩🏼‍🦼",
    ":woman_in_motorized_wheelchair_medium_light_skin_tone:": "👩🏼‍🦼",
    ":woman_in_motorized_wheelchair::skin-tone-2:": "👩🏼‍🦼",
    ":woman_in_motorized_wheelchair_tone3:": "👩🏽‍🦼",
    ":woman_in_motorized_wheelchair_medium_skin_tone:": "👩🏽‍🦼",
    ":woman_in_motorized_wheelchair::skin-tone-3:": "👩🏽‍🦼",
    ":woman_in_motorized_wheelchair_tone4:": "👩🏾‍🦼",
    ":woman_in_motorized_wheelchair_medium_dark_skin_tone:": "👩🏾‍🦼",
    ":woman_in_motorized_wheelchair::skin-tone-4:": "👩🏾‍🦼",
    ":woman_in_motorized_wheelchair_tone5:": "👩🏿‍🦼",
    ":woman_in_motorized_wheelchair_dark_skin_tone:": "👩🏿‍🦼",
    ":woman_in_motorized_wheelchair::skin-tone-5:": "👩🏿‍🦼",
    ":woman_in_steamy_room:": "🧖‍♀️",
    ":woman_in_steamy_room_tone1:": "🧖🏻‍♀️",
    ":woman_in_steamy_room_light_skin_tone:": "🧖🏻‍♀️",
    ":woman_in_steamy_room::skin-tone-1:": "🧖🏻‍♀️",
    ":woman_in_steamy_room_tone2:": "🧖🏼‍♀️",
    ":woman_in_steamy_room_medium_light_skin_tone:": "🧖🏼‍♀️",
    ":woman_in_steamy_room::skin-tone-2:": "🧖🏼‍♀️",
    ":woman_in_steamy_room_tone3:": "🧖🏽‍♀️",
    ":woman_in_steamy_room_medium_skin_tone:": "🧖🏽‍♀️",
    ":woman_in_steamy_room::skin-tone-3:": "🧖🏽‍♀️",
    ":woman_in_steamy_room_tone4:": "🧖🏾‍♀️",
    ":woman_in_steamy_room_medium_dark_skin_tone:": "🧖🏾‍♀️",
    ":woman_in_steamy_room::skin-tone-4:": "🧖🏾‍♀️",
    ":woman_in_steamy_room_tone5:": "🧖🏿‍♀️",
    ":woman_in_steamy_room_dark_skin_tone:": "🧖🏿‍♀️",
    ":woman_in_steamy_room::skin-tone-5:": "🧖🏿‍♀️",
    ":woman_in_tuxedo:": "🤵‍♀️",
    ":woman_in_tuxedo_tone1:": "🤵🏻‍♀️",
    ":woman_in_tuxedo_light_skin_tone:": "🤵🏻‍♀️",
    ":woman_in_tuxedo::skin-tone-1:": "🤵🏻‍♀️",
    ":woman_in_tuxedo_tone2:": "🤵🏼‍♀️",
    ":woman_in_tuxedo_medium_light_skin_tone:": "🤵🏼‍♀️",
    ":woman_in_tuxedo::skin-tone-2:": "🤵🏼‍♀️",
    ":woman_in_tuxedo_tone3:": "🤵🏽‍♀️",
    ":woman_in_tuxedo_medium_skin_tone:": "🤵🏽‍♀️",
    ":woman_in_tuxedo::skin-tone-3:": "🤵🏽‍♀️",
    ":woman_in_tuxedo_tone4:": "🤵🏾‍♀️",
    ":woman_in_tuxedo_medium_dark_skin_tone:": "🤵🏾‍♀️",
    ":woman_in_tuxedo::skin-tone-4:": "🤵🏾‍♀️",
    ":woman_in_tuxedo_tone5:": "🤵🏿‍♀️",
    ":woman_in_tuxedo_dark_skin_tone:": "🤵🏿‍♀️",
    ":woman_in_tuxedo::skin-tone-5:": "🤵🏿‍♀️",
    ":woman_judge:": "👩‍⚖️",
    ":woman_judge_tone1:": "👩🏻‍⚖️",
    ":woman_judge_light_skin_tone:": "👩🏻‍⚖️",
    ":woman_judge::skin-tone-1:": "👩🏻‍⚖️",
    ":woman_judge_tone2:": "👩🏼‍⚖️",
    ":woman_judge_medium_light_skin_tone:": "👩🏼‍⚖️",
    ":woman_judge::skin-tone-2:": "👩🏼‍⚖️",
    ":woman_judge_tone3:": "👩🏽‍⚖️",
    ":woman_judge_medium_skin_tone:": "👩🏽‍⚖️",
    ":woman_judge::skin-tone-3:": "👩🏽‍⚖️",
    ":woman_judge_tone4:": "👩🏾‍⚖️",
    ":woman_judge_medium_dark_skin_tone:": "👩🏾‍⚖️",
    ":woman_judge::skin-tone-4:": "👩🏾‍⚖️",
    ":woman_judge_tone5:": "👩🏿‍⚖️",
    ":woman_judge_dark_skin_tone:": "👩🏿‍⚖️",
    ":woman_judge::skin-tone-5:": "👩🏿‍⚖️",
    ":woman_juggling:": "🤹‍♀️",
    ":woman_juggling_tone1:": "🤹🏻‍♀️",
    ":woman_juggling_light_skin_tone:": "🤹🏻‍♀️",
    ":woman_juggling::skin-tone-1:": "🤹🏻‍♀️",
    ":woman_juggling_tone2:": "🤹🏼‍♀️",
    ":woman_juggling_medium_light_skin_tone:": "🤹🏼‍♀️",
    ":woman_juggling::skin-tone-2:": "🤹🏼‍♀️",
    ":woman_juggling_tone3:": "🤹🏽‍♀️",
    ":woman_juggling_medium_skin_tone:": "🤹🏽‍♀️",
    ":woman_juggling::skin-tone-3:": "🤹🏽‍♀️",
    ":woman_juggling_tone4:": "🤹🏾‍♀️",
    ":woman_juggling_medium_dark_skin_tone:": "🤹🏾‍♀️",
    ":woman_juggling::skin-tone-4:": "🤹🏾‍♀️",
    ":woman_juggling_tone5:": "🤹🏿‍♀️",
    ":woman_juggling_dark_skin_tone:": "🤹🏿‍♀️",
    ":woman_juggling::skin-tone-5:": "🤹🏿‍♀️",
    ":woman_kneeling:": "🧎‍♀️",
    ":woman_kneeling_tone1:": "🧎🏻‍♀️",
    ":woman_kneeling_light_skin_tone:": "🧎🏻‍♀️",
    ":woman_kneeling::skin-tone-1:": "🧎🏻‍♀️",
    ":woman_kneeling_tone2:": "🧎🏼‍♀️",
    ":woman_kneeling_medium_light_skin_tone:": "🧎🏼‍♀️",
    ":woman_kneeling::skin-tone-2:": "🧎🏼‍♀️",
    ":woman_kneeling_tone3:": "🧎🏽‍♀️",
    ":woman_kneeling_medium_skin_tone:": "🧎🏽‍♀️",
    ":woman_kneeling::skin-tone-3:": "🧎🏽‍♀️",
    ":woman_kneeling_tone4:": "🧎🏾‍♀️",
    ":woman_kneeling_medium_dark_skin_tone:": "🧎🏾‍♀️",
    ":woman_kneeling::skin-tone-4:": "🧎🏾‍♀️",
    ":woman_kneeling_tone5:": "🧎🏿‍♀️",
    ":woman_kneeling_dark_skin_tone:": "🧎🏿‍♀️",
    ":woman_kneeling::skin-tone-5:": "🧎🏿‍♀️",
    ":woman_lifting_weights:": "🏋️‍♀️",
    ":woman_lifting_weights_tone1:": "🏋🏻‍♀️",
    ":woman_lifting_weights_light_skin_tone:": "🏋🏻‍♀️",
    ":woman_lifting_weights::skin-tone-1:": "🏋🏻‍♀️",
    ":woman_lifting_weights_tone2:": "🏋🏼‍♀️",
    ":woman_lifting_weights_medium_light_skin_tone:": "🏋🏼‍♀️",
    ":woman_lifting_weights::skin-tone-2:": "🏋🏼‍♀️",
    ":woman_lifting_weights_tone3:": "🏋🏽‍♀️",
    ":woman_lifting_weights_medium_skin_tone:": "🏋🏽‍♀️",
    ":woman_lifting_weights::skin-tone-3:": "🏋🏽‍♀️",
    ":woman_lifting_weights_tone4:": "🏋🏾‍♀️",
    ":woman_lifting_weights_medium_dark_skin_tone:": "🏋🏾‍♀️",
    ":woman_lifting_weights::skin-tone-4:": "🏋🏾‍♀️",
    ":woman_lifting_weights_tone5:": "🏋🏿‍♀️",
    ":woman_lifting_weights_dark_skin_tone:": "🏋🏿‍♀️",
    ":woman_lifting_weights::skin-tone-5:": "🏋🏿‍♀️",
    ":woman_mage:": "🧙‍♀️",
    ":woman_mage_tone1:": "🧙🏻‍♀️",
    ":woman_mage_light_skin_tone:": "🧙🏻‍♀️",
    ":woman_mage::skin-tone-1:": "🧙🏻‍♀️",
    ":woman_mage_tone2:": "🧙🏼‍♀️",
    ":woman_mage_medium_light_skin_tone:": "🧙🏼‍♀️",
    ":woman_mage::skin-tone-2:": "🧙🏼‍♀️",
    ":woman_mage_tone3:": "🧙🏽‍♀️",
    ":woman_mage_medium_skin_tone:": "🧙🏽‍♀️",
    ":woman_mage::skin-tone-3:": "🧙🏽‍♀️",
    ":woman_mage_tone4:": "🧙🏾‍♀️",
    ":woman_mage_medium_dark_skin_tone:": "🧙🏾‍♀️",
    ":woman_mage::skin-tone-4:": "🧙🏾‍♀️",
    ":woman_mage_tone5:": "🧙🏿‍♀️",
    ":woman_mage_dark_skin_tone:": "🧙🏿‍♀️",
    ":woman_mage::skin-tone-5:": "🧙🏿‍♀️",
    ":woman_mechanic:": "👩‍🔧",
    ":woman_mechanic_tone1:": "👩🏻‍🔧",
    ":woman_mechanic_light_skin_tone:": "👩🏻‍🔧",
    ":woman_mechanic::skin-tone-1:": "👩🏻‍🔧",
    ":woman_mechanic_tone2:": "👩🏼‍🔧",
    ":woman_mechanic_medium_light_skin_tone:": "👩🏼‍🔧",
    ":woman_mechanic::skin-tone-2:": "👩🏼‍🔧",
    ":woman_mechanic_tone3:": "👩🏽‍🔧",
    ":woman_mechanic_medium_skin_tone:": "👩🏽‍🔧",
    ":woman_mechanic::skin-tone-3:": "👩🏽‍🔧",
    ":woman_mechanic_tone4:": "👩🏾‍🔧",
    ":woman_mechanic_medium_dark_skin_tone:": "👩🏾‍🔧",
    ":woman_mechanic::skin-tone-4:": "👩🏾‍🔧",
    ":woman_mechanic_tone5:": "👩🏿‍🔧",
    ":woman_mechanic_dark_skin_tone:": "👩🏿‍🔧",
    ":woman_mechanic::skin-tone-5:": "👩🏿‍🔧",
    ":woman_mountain_biking:": "🚵‍♀️",
    ":woman_mountain_biking_tone1:": "🚵🏻‍♀️",
    ":woman_mountain_biking_light_skin_tone:": "🚵🏻‍♀️",
    ":woman_mountain_biking::skin-tone-1:": "🚵🏻‍♀️",
    ":woman_mountain_biking_tone2:": "🚵🏼‍♀️",
    ":woman_mountain_biking_medium_light_skin_tone:": "🚵🏼‍♀️",
    ":woman_mountain_biking::skin-tone-2:": "🚵🏼‍♀️",
    ":woman_mountain_biking_tone3:": "🚵🏽‍♀️",
    ":woman_mountain_biking_medium_skin_tone:": "🚵🏽‍♀️",
    ":woman_mountain_biking::skin-tone-3:": "🚵🏽‍♀️",
    ":woman_mountain_biking_tone4:": "🚵🏾‍♀️",
    ":woman_mountain_biking_medium_dark_skin_tone:": "🚵🏾‍♀️",
    ":woman_mountain_biking::skin-tone-4:": "🚵🏾‍♀️",
    ":woman_mountain_biking_tone5:": "🚵🏿‍♀️",
    ":woman_mountain_biking_dark_skin_tone:": "🚵🏿‍♀️",
    ":woman_mountain_biking::skin-tone-5:": "🚵🏿‍♀️",
    ":woman_office_worker:": "👩‍💼",
    ":woman_office_worker_tone1:": "👩🏻‍💼",
    ":woman_office_worker_light_skin_tone:": "👩🏻‍💼",
    ":woman_office_worker::skin-tone-1:": "👩🏻‍💼",
    ":woman_office_worker_tone2:": "👩🏼‍💼",
    ":woman_office_worker_medium_light_skin_tone:": "👩🏼‍💼",
    ":woman_office_worker::skin-tone-2:": "👩🏼‍💼",
    ":woman_office_worker_tone3:": "👩🏽‍💼",
    ":woman_office_worker_medium_skin_tone:": "👩🏽‍💼",
    ":woman_office_worker::skin-tone-3:": "👩🏽‍💼",
    ":woman_office_worker_tone4:": "👩🏾‍💼",
    ":woman_office_worker_medium_dark_skin_tone:": "👩🏾‍💼",
    ":woman_office_worker::skin-tone-4:": "👩🏾‍💼",
    ":woman_office_worker_tone5:": "👩🏿‍💼",
    ":woman_office_worker_dark_skin_tone:": "👩🏿‍💼",
    ":woman_office_worker::skin-tone-5:": "👩🏿‍💼",
    ":woman_pilot:": "👩‍✈️",
    ":woman_pilot_tone1:": "👩🏻‍✈️",
    ":woman_pilot_light_skin_tone:": "👩🏻‍✈️",
    ":woman_pilot::skin-tone-1:": "👩🏻‍✈️",
    ":woman_pilot_tone2:": "👩🏼‍✈️",
    ":woman_pilot_medium_light_skin_tone:": "👩🏼‍✈️",
    ":woman_pilot::skin-tone-2:": "👩🏼‍✈️",
    ":woman_pilot_tone3:": "👩🏽‍✈️",
    ":woman_pilot_medium_skin_tone:": "👩🏽‍✈️",
    ":woman_pilot::skin-tone-3:": "👩🏽‍✈️",
    ":woman_pilot_tone4:": "👩🏾‍✈️",
    ":woman_pilot_medium_dark_skin_tone:": "👩🏾‍✈️",
    ":woman_pilot::skin-tone-4:": "👩🏾‍✈️",
    ":woman_pilot_tone5:": "👩🏿‍✈️",
    ":woman_pilot_dark_skin_tone:": "👩🏿‍✈️",
    ":woman_pilot::skin-tone-5:": "👩🏿‍✈️",
    ":woman_playing_handball:": "🤾‍♀️",
    ":woman_playing_handball_tone1:": "🤾🏻‍♀️",
    ":woman_playing_handball_light_skin_tone:": "🤾🏻‍♀️",
    ":woman_playing_handball::skin-tone-1:": "🤾🏻‍♀️",
    ":woman_playing_handball_tone2:": "🤾🏼‍♀️",
    ":woman_playing_handball_medium_light_skin_tone:": "🤾🏼‍♀️",
    ":woman_playing_handball::skin-tone-2:": "🤾🏼‍♀️",
    ":woman_playing_handball_tone3:": "🤾🏽‍♀️",
    ":woman_playing_handball_medium_skin_tone:": "🤾🏽‍♀️",
    ":woman_playing_handball::skin-tone-3:": "🤾🏽‍♀️",
    ":woman_playing_handball_tone4:": "🤾🏾‍♀️",
    ":woman_playing_handball_medium_dark_skin_tone:": "🤾🏾‍♀️",
    ":woman_playing_handball::skin-tone-4:": "🤾🏾‍♀️",
    ":woman_playing_handball_tone5:": "🤾🏿‍♀️",
    ":woman_playing_handball_dark_skin_tone:": "🤾🏿‍♀️",
    ":woman_playing_handball::skin-tone-5:": "🤾🏿‍♀️",
    ":woman_playing_water_polo:": "🤽‍♀️",
    ":woman_playing_water_polo_tone1:": "🤽🏻‍♀️",
    ":woman_playing_water_polo_light_skin_tone:": "🤽🏻‍♀️",
    ":woman_playing_water_polo::skin-tone-1:": "🤽🏻‍♀️",
    ":woman_playing_water_polo_tone2:": "🤽🏼‍♀️",
    ":woman_playing_water_polo_medium_light_skin_tone:": "🤽🏼‍♀️",
    ":woman_playing_water_polo::skin-tone-2:": "🤽🏼‍♀️",
    ":woman_playing_water_polo_tone3:": "🤽🏽‍♀️",
    ":woman_playing_water_polo_medium_skin_tone:": "🤽🏽‍♀️",
    ":woman_playing_water_polo::skin-tone-3:": "🤽🏽‍♀️",
    ":woman_playing_water_polo_tone4:": "🤽🏾‍♀️",
    ":woman_playing_water_polo_medium_dark_skin_tone:": "🤽🏾‍♀️",
    ":woman_playing_water_polo::skin-tone-4:": "🤽🏾‍♀️",
    ":woman_playing_water_polo_tone5:": "🤽🏿‍♀️",
    ":woman_playing_water_polo_dark_skin_tone:": "🤽🏿‍♀️",
    ":woman_playing_water_polo::skin-tone-5:": "🤽🏿‍♀️",
    ":woman_police_officer:": "👮‍♀️",
    ":woman_police_officer_tone1:": "👮🏻‍♀️",
    ":woman_police_officer_light_skin_tone:": "👮🏻‍♀️",
    ":woman_police_officer::skin-tone-1:": "👮🏻‍♀️",
    ":woman_police_officer_tone2:": "👮🏼‍♀️",
    ":woman_police_officer_medium_light_skin_tone:": "👮🏼‍♀️",
    ":woman_police_officer::skin-tone-2:": "👮🏼‍♀️",
    ":woman_police_officer_tone3:": "👮🏽‍♀️",
    ":woman_police_officer_medium_skin_tone:": "👮🏽‍♀️",
    ":woman_police_officer::skin-tone-3:": "👮🏽‍♀️",
    ":woman_police_officer_tone4:": "👮🏾‍♀️",
    ":woman_police_officer_medium_dark_skin_tone:": "👮🏾‍♀️",
    ":woman_police_officer::skin-tone-4:": "👮🏾‍♀️",
    ":woman_police_officer_tone5:": "👮🏿‍♀️",
    ":woman_police_officer_dark_skin_tone:": "👮🏿‍♀️",
    ":woman_police_officer::skin-tone-5:": "👮🏿‍♀️",
    ":woman_pouting:": "🙎‍♀️",
    ":woman_pouting_tone1:": "🙎🏻‍♀️",
    ":woman_pouting_light_skin_tone:": "🙎🏻‍♀️",
    ":woman_pouting::skin-tone-1:": "🙎🏻‍♀️",
    ":woman_pouting_tone2:": "🙎🏼‍♀️",
    ":woman_pouting_medium_light_skin_tone:": "🙎🏼‍♀️",
    ":woman_pouting::skin-tone-2:": "🙎🏼‍♀️",
    ":woman_pouting_tone3:": "🙎🏽‍♀️",
    ":woman_pouting_medium_skin_tone:": "🙎🏽‍♀️",
    ":woman_pouting::skin-tone-3:": "🙎🏽‍♀️",
    ":woman_pouting_tone4:": "🙎🏾‍♀️",
    ":woman_pouting_medium_dark_skin_tone:": "🙎🏾‍♀️",
    ":woman_pouting::skin-tone-4:": "🙎🏾‍♀️",
    ":woman_pouting_tone5:": "🙎🏿‍♀️",
    ":woman_pouting_dark_skin_tone:": "🙎🏿‍♀️",
    ":woman_pouting::skin-tone-5:": "🙎🏿‍♀️",
    ":woman_raising_hand:": "🙋‍♀️",
    ":woman_raising_hand_tone1:": "🙋🏻‍♀️",
    ":woman_raising_hand_light_skin_tone:": "🙋🏻‍♀️",
    ":woman_raising_hand::skin-tone-1:": "🙋🏻‍♀️",
    ":woman_raising_hand_tone2:": "🙋🏼‍♀️",
    ":woman_raising_hand_medium_light_skin_tone:": "🙋🏼‍♀️",
    ":woman_raising_hand::skin-tone-2:": "🙋🏼‍♀️",
    ":woman_raising_hand_tone3:": "🙋🏽‍♀️",
    ":woman_raising_hand_medium_skin_tone:": "🙋🏽‍♀️",
    ":woman_raising_hand::skin-tone-3:": "🙋🏽‍♀️",
    ":woman_raising_hand_tone4:": "🙋🏾‍♀️",
    ":woman_raising_hand_medium_dark_skin_tone:": "🙋🏾‍♀️",
    ":woman_raising_hand::skin-tone-4:": "🙋🏾‍♀️",
    ":woman_raising_hand_tone5:": "🙋🏿‍♀️",
    ":woman_raising_hand_dark_skin_tone:": "🙋🏿‍♀️",
    ":woman_raising_hand::skin-tone-5:": "🙋🏿‍♀️",
    ":woman_red_haired:": "👩‍🦰",
    ":woman_red_haired_tone1:": "👩🏻‍🦰",
    ":woman_red_haired_light_skin_tone:": "👩🏻‍🦰",
    ":woman_red_haired::skin-tone-1:": "👩🏻‍🦰",
    ":woman_red_haired_tone2:": "👩🏼‍🦰",
    ":woman_red_haired_medium_light_skin_tone:": "👩🏼‍🦰",
    ":woman_red_haired::skin-tone-2:": "👩🏼‍🦰",
    ":woman_red_haired_tone3:": "👩🏽‍🦰",
    ":woman_red_haired_medium_skin_tone:": "👩🏽‍🦰",
    ":woman_red_haired::skin-tone-3:": "👩🏽‍🦰",
    ":woman_red_haired_tone4:": "👩🏾‍🦰",
    ":woman_red_haired_medium_dark_skin_tone:": "👩🏾‍🦰",
    ":woman_red_haired::skin-tone-4:": "👩🏾‍🦰",
    ":woman_red_haired_tone5:": "👩🏿‍🦰",
    ":woman_red_haired_dark_skin_tone:": "👩🏿‍🦰",
    ":woman_red_haired::skin-tone-5:": "👩🏿‍🦰",
    ":woman_rowing_boat:": "🚣‍♀️",
    ":woman_rowing_boat_tone1:": "🚣🏻‍♀️",
    ":woman_rowing_boat_light_skin_tone:": "🚣🏻‍♀️",
    ":woman_rowing_boat::skin-tone-1:": "🚣🏻‍♀️",
    ":woman_rowing_boat_tone2:": "🚣🏼‍♀️",
    ":woman_rowing_boat_medium_light_skin_tone:": "🚣🏼‍♀️",
    ":woman_rowing_boat::skin-tone-2:": "🚣🏼‍♀️",
    ":woman_rowing_boat_tone3:": "🚣🏽‍♀️",
    ":woman_rowing_boat_medium_skin_tone:": "🚣🏽‍♀️",
    ":woman_rowing_boat::skin-tone-3:": "🚣🏽‍♀️",
    ":woman_rowing_boat_tone4:": "🚣🏾‍♀️",
    ":woman_rowing_boat_medium_dark_skin_tone:": "🚣🏾‍♀️",
    ":woman_rowing_boat::skin-tone-4:": "🚣🏾‍♀️",
    ":woman_rowing_boat_tone5:": "🚣🏿‍♀️",
    ":woman_rowing_boat_dark_skin_tone:": "🚣🏿‍♀️",
    ":woman_rowing_boat::skin-tone-5:": "🚣🏿‍♀️",
    ":woman_running:": "🏃‍♀️",
    ":woman_running_tone1:": "🏃🏻‍♀️",
    ":woman_running_light_skin_tone:": "🏃🏻‍♀️",
    ":woman_running::skin-tone-1:": "🏃🏻‍♀️",
    ":woman_running_tone2:": "🏃🏼‍♀️",
    ":woman_running_medium_light_skin_tone:": "🏃🏼‍♀️",
    ":woman_running::skin-tone-2:": "🏃🏼‍♀️",
    ":woman_running_tone3:": "🏃🏽‍♀️",
    ":woman_running_medium_skin_tone:": "🏃🏽‍♀️",
    ":woman_running::skin-tone-3:": "🏃🏽‍♀️",
    ":woman_running_tone4:": "🏃🏾‍♀️",
    ":woman_running_medium_dark_skin_tone:": "🏃🏾‍♀️",
    ":woman_running::skin-tone-4:": "🏃🏾‍♀️",
    ":woman_running_tone5:": "🏃🏿‍♀️",
    ":woman_running_dark_skin_tone:": "🏃🏿‍♀️",
    ":woman_running::skin-tone-5:": "🏃🏿‍♀️",
    ":woman_scientist:": "👩‍🔬",
    ":woman_scientist_tone1:": "👩🏻‍🔬",
    ":woman_scientist_light_skin_tone:": "👩🏻‍🔬",
    ":woman_scientist::skin-tone-1:": "👩🏻‍🔬",
    ":woman_scientist_tone2:": "👩🏼‍🔬",
    ":woman_scientist_medium_light_skin_tone:": "👩🏼‍🔬",
    ":woman_scientist::skin-tone-2:": "👩🏼‍🔬",
    ":woman_scientist_tone3:": "👩🏽‍🔬",
    ":woman_scientist_medium_skin_tone:": "👩🏽‍🔬",
    ":woman_scientist::skin-tone-3:": "👩🏽‍🔬",
    ":woman_scientist_tone4:": "👩🏾‍🔬",
    ":woman_scientist_medium_dark_skin_tone:": "👩🏾‍🔬",
    ":woman_scientist::skin-tone-4:": "👩🏾‍🔬",
    ":woman_scientist_tone5:": "👩🏿‍🔬",
    ":woman_scientist_dark_skin_tone:": "👩🏿‍🔬",
    ":woman_scientist::skin-tone-5:": "👩🏿‍🔬",
    ":woman_shrugging:": "🤷‍♀️",
    ":woman_shrugging_tone1:": "🤷🏻‍♀️",
    ":woman_shrugging_light_skin_tone:": "🤷🏻‍♀️",
    ":woman_shrugging::skin-tone-1:": "🤷🏻‍♀️",
    ":woman_shrugging_tone2:": "🤷🏼‍♀️",
    ":woman_shrugging_medium_light_skin_tone:": "🤷🏼‍♀️",
    ":woman_shrugging::skin-tone-2:": "🤷🏼‍♀️",
    ":woman_shrugging_tone3:": "🤷🏽‍♀️",
    ":woman_shrugging_medium_skin_tone:": "🤷🏽‍♀️",
    ":woman_shrugging::skin-tone-3:": "🤷🏽‍♀️",
    ":woman_shrugging_tone4:": "🤷🏾‍♀️",
    ":woman_shrugging_medium_dark_skin_tone:": "🤷🏾‍♀️",
    ":woman_shrugging::skin-tone-4:": "🤷🏾‍♀️",
    ":woman_shrugging_tone5:": "🤷🏿‍♀️",
    ":woman_shrugging_dark_skin_tone:": "🤷🏿‍♀️",
    ":woman_shrugging::skin-tone-5:": "🤷🏿‍♀️",
    ":woman_singer:": "👩‍🎤",
    ":woman_singer_tone1:": "👩🏻‍🎤",
    ":woman_singer_light_skin_tone:": "👩🏻‍🎤",
    ":woman_singer::skin-tone-1:": "👩🏻‍🎤",
    ":woman_singer_tone2:": "👩🏼‍🎤",
    ":woman_singer_medium_light_skin_tone:": "👩🏼‍🎤",
    ":woman_singer::skin-tone-2:": "👩🏼‍🎤",
    ":woman_singer_tone3:": "👩🏽‍🎤",
    ":woman_singer_medium_skin_tone:": "👩🏽‍🎤",
    ":woman_singer::skin-tone-3:": "👩🏽‍🎤",
    ":woman_singer_tone4:": "👩🏾‍🎤",
    ":woman_singer_medium_dark_skin_tone:": "👩🏾‍🎤",
    ":woman_singer::skin-tone-4:": "👩🏾‍🎤",
    ":woman_singer_tone5:": "👩🏿‍🎤",
    ":woman_singer_dark_skin_tone:": "👩🏿‍🎤",
    ":woman_singer::skin-tone-5:": "👩🏿‍🎤",
    ":woman_standing:": "🧍‍♀️",
    ":woman_standing_tone1:": "🧍🏻‍♀️",
    ":woman_standing_light_skin_tone:": "🧍🏻‍♀️",
    ":woman_standing::skin-tone-1:": "🧍🏻‍♀️",
    ":woman_standing_tone2:": "🧍🏼‍♀️",
    ":woman_standing_medium_light_skin_tone:": "🧍🏼‍♀️",
    ":woman_standing::skin-tone-2:": "🧍🏼‍♀️",
    ":woman_standing_tone3:": "🧍🏽‍♀️",
    ":woman_standing_medium_skin_tone:": "🧍🏽‍♀️",
    ":woman_standing::skin-tone-3:": "🧍🏽‍♀️",
    ":woman_standing_tone4:": "🧍🏾‍♀️",
    ":woman_standing_medium_dark_skin_tone:": "🧍🏾‍♀️",
    ":woman_standing::skin-tone-4:": "🧍🏾‍♀️",
    ":woman_standing_tone5:": "🧍🏿‍♀️",
    ":woman_standing_dark_skin_tone:": "🧍🏿‍♀️",
    ":woman_standing::skin-tone-5:": "🧍🏿‍♀️",
    ":woman_student:": "👩‍🎓",
    ":woman_student_tone1:": "👩🏻‍🎓",
    ":woman_student_light_skin_tone:": "👩🏻‍🎓",
    ":woman_student::skin-tone-1:": "👩🏻‍🎓",
    ":woman_student_tone2:": "👩🏼‍🎓",
    ":woman_student_medium_light_skin_tone:": "👩🏼‍🎓",
    ":woman_student::skin-tone-2:": "👩🏼‍🎓",
    ":woman_student_tone3:": "👩🏽‍🎓",
    ":woman_student_medium_skin_tone:": "👩🏽‍🎓",
    ":woman_student::skin-tone-3:": "👩🏽‍🎓",
    ":woman_student_tone4:": "👩🏾‍🎓",
    ":woman_student_medium_dark_skin_tone:": "👩🏾‍🎓",
    ":woman_student::skin-tone-4:": "👩🏾‍🎓",
    ":woman_student_tone5:": "👩🏿‍🎓",
    ":woman_student_dark_skin_tone:": "👩🏿‍🎓",
    ":woman_student::skin-tone-5:": "👩🏿‍🎓",
    ":woman_superhero:": "🦸‍♀️",
    ":woman_superhero_tone1:": "🦸🏻‍♀️",
    ":woman_superhero_light_skin_tone:": "🦸🏻‍♀️",
    ":woman_superhero::skin-tone-1:": "🦸🏻‍♀️",
    ":woman_superhero_tone2:": "🦸🏼‍♀️",
    ":woman_superhero_medium_light_skin_tone:": "🦸🏼‍♀️",
    ":woman_superhero::skin-tone-2:": "🦸🏼‍♀️",
    ":woman_superhero_tone3:": "🦸🏽‍♀️",
    ":woman_superhero_medium_skin_tone:": "🦸🏽‍♀️",
    ":woman_superhero::skin-tone-3:": "🦸🏽‍♀️",
    ":woman_superhero_tone4:": "🦸🏾‍♀️",
    ":woman_superhero_medium_dark_skin_tone:": "🦸🏾‍♀️",
    ":woman_superhero::skin-tone-4:": "🦸🏾‍♀️",
    ":woman_superhero_tone5:": "🦸🏿‍♀️",
    ":woman_superhero_dark_skin_tone:": "🦸🏿‍♀️",
    ":woman_superhero::skin-tone-5:": "🦸🏿‍♀️",
    ":woman_supervillain:": "🦹‍♀️",
    ":woman_supervillain_tone1:": "🦹🏻‍♀️",
    ":woman_supervillain_light_skin_tone:": "🦹🏻‍♀️",
    ":woman_supervillain::skin-tone-1:": "🦹🏻‍♀️",
    ":woman_supervillain_tone2:": "🦹🏼‍♀️",
    ":woman_supervillain_medium_light_skin_tone:": "🦹🏼‍♀️",
    ":woman_supervillain::skin-tone-2:": "🦹🏼‍♀️",
    ":woman_supervillain_tone3:": "🦹🏽‍♀️",
    ":woman_supervillain_medium_skin_tone:": "🦹🏽‍♀️",
    ":woman_supervillain::skin-tone-3:": "🦹🏽‍♀️",
    ":woman_supervillain_tone4:": "🦹🏾‍♀️",
    ":woman_supervillain_medium_dark_skin_tone:": "🦹🏾‍♀️",
    ":woman_supervillain::skin-tone-4:": "🦹🏾‍♀️",
    ":woman_supervillain_tone5:": "🦹🏿‍♀️",
    ":woman_supervillain_dark_skin_tone:": "🦹🏿‍♀️",
    ":woman_supervillain::skin-tone-5:": "🦹🏿‍♀️",
    ":woman_surfing:": "🏄‍♀️",
    ":woman_surfing_tone1:": "🏄🏻‍♀️",
    ":woman_surfing_light_skin_tone:": "🏄🏻‍♀️",
    ":woman_surfing::skin-tone-1:": "🏄🏻‍♀️",
    ":woman_surfing_tone2:": "🏄🏼‍♀️",
    ":woman_surfing_medium_light_skin_tone:": "🏄🏼‍♀️",
    ":woman_surfing::skin-tone-2:": "🏄🏼‍♀️",
    ":woman_surfing_tone3:": "🏄🏽‍♀️",
    ":woman_surfing_medium_skin_tone:": "🏄🏽‍♀️",
    ":woman_surfing::skin-tone-3:": "🏄🏽‍♀️",
    ":woman_surfing_tone4:": "🏄🏾‍♀️",
    ":woman_surfing_medium_dark_skin_tone:": "🏄🏾‍♀️",
    ":woman_surfing::skin-tone-4:": "🏄🏾‍♀️",
    ":woman_surfing_tone5:": "🏄🏿‍♀️",
    ":woman_surfing_dark_skin_tone:": "🏄🏿‍♀️",
    ":woman_surfing::skin-tone-5:": "🏄🏿‍♀️",
    ":woman_swimming:": "🏊‍♀️",
    ":woman_swimming_tone1:": "🏊🏻‍♀️",
    ":woman_swimming_light_skin_tone:": "🏊🏻‍♀️",
    ":woman_swimming::skin-tone-1:": "🏊🏻‍♀️",
    ":woman_swimming_tone2:": "🏊🏼‍♀️",
    ":woman_swimming_medium_light_skin_tone:": "🏊🏼‍♀️",
    ":woman_swimming::skin-tone-2:": "🏊🏼‍♀️",
    ":woman_swimming_tone3:": "🏊🏽‍♀️",
    ":woman_swimming_medium_skin_tone:": "🏊🏽‍♀️",
    ":woman_swimming::skin-tone-3:": "🏊🏽‍♀️",
    ":woman_swimming_tone4:": "🏊🏾‍♀️",
    ":woman_swimming_medium_dark_skin_tone:": "🏊🏾‍♀️",
    ":woman_swimming::skin-tone-4:": "🏊🏾‍♀️",
    ":woman_swimming_tone5:": "🏊🏿‍♀️",
    ":woman_swimming_dark_skin_tone:": "🏊🏿‍♀️",
    ":woman_swimming::skin-tone-5:": "🏊🏿‍♀️",
    ":woman_teacher:": "👩‍🏫",
    ":woman_teacher_tone1:": "👩🏻‍🏫",
    ":woman_teacher_light_skin_tone:": "👩🏻‍🏫",
    ":woman_teacher::skin-tone-1:": "👩🏻‍🏫",
    ":woman_teacher_tone2:": "👩🏼‍🏫",
    ":woman_teacher_medium_light_skin_tone:": "👩🏼‍🏫",
    ":woman_teacher::skin-tone-2:": "👩🏼‍🏫",
    ":woman_teacher_tone3:": "👩🏽‍🏫",
    ":woman_teacher_medium_skin_tone:": "👩🏽‍🏫",
    ":woman_teacher::skin-tone-3:": "👩🏽‍🏫",
    ":woman_teacher_tone4:": "👩🏾‍🏫",
    ":woman_teacher_medium_dark_skin_tone:": "👩🏾‍🏫",
    ":woman_teacher::skin-tone-4:": "👩🏾‍🏫",
    ":woman_teacher_tone5:": "👩🏿‍🏫",
    ":woman_teacher_dark_skin_tone:": "👩🏿‍🏫",
    ":woman_teacher::skin-tone-5:": "👩🏿‍🏫",
    ":woman_technologist:": "👩‍💻",
    ":woman_technologist_tone1:": "👩🏻‍💻",
    ":woman_technologist_light_skin_tone:": "👩🏻‍💻",
    ":woman_technologist::skin-tone-1:": "👩🏻‍💻",
    ":woman_technologist_tone2:": "👩🏼‍💻",
    ":woman_technologist_medium_light_skin_tone:": "👩🏼‍💻",
    ":woman_technologist::skin-tone-2:": "👩🏼‍💻",
    ":woman_technologist_tone3:": "👩🏽‍💻",
    ":woman_technologist_medium_skin_tone:": "👩🏽‍💻",
    ":woman_technologist::skin-tone-3:": "👩🏽‍💻",
    ":woman_technologist_tone4:": "👩🏾‍💻",
    ":woman_technologist_medium_dark_skin_tone:": "👩🏾‍💻",
    ":woman_technologist::skin-tone-4:": "👩🏾‍💻",
    ":woman_technologist_tone5:": "👩🏿‍💻",
    ":woman_technologist_dark_skin_tone:": "👩🏿‍💻",
    ":woman_technologist::skin-tone-5:": "👩🏿‍💻",
    ":woman_tipping_hand:": "💁‍♀️",
    ":woman_tipping_hand_tone1:": "💁🏻‍♀️",
    ":woman_tipping_hand_light_skin_tone:": "💁🏻‍♀️",
    ":woman_tipping_hand::skin-tone-1:": "💁🏻‍♀️",
    ":woman_tipping_hand_tone2:": "💁🏼‍♀️",
    ":woman_tipping_hand_medium_light_skin_tone:": "💁🏼‍♀️",
    ":woman_tipping_hand::skin-tone-2:": "💁🏼‍♀️",
    ":woman_tipping_hand_tone3:": "💁🏽‍♀️",
    ":woman_tipping_hand_medium_skin_tone:": "💁🏽‍♀️",
    ":woman_tipping_hand::skin-tone-3:": "💁🏽‍♀️",
    ":woman_tipping_hand_tone4:": "💁🏾‍♀️",
    ":woman_tipping_hand_medium_dark_skin_tone:": "💁🏾‍♀️",
    ":woman_tipping_hand::skin-tone-4:": "💁🏾‍♀️",
    ":woman_tipping_hand_tone5:": "💁🏿‍♀️",
    ":woman_tipping_hand_dark_skin_tone:": "💁🏿‍♀️",
    ":woman_tipping_hand::skin-tone-5:": "💁🏿‍♀️",
    ":woman_tone1:": "👩🏻",
    ":woman::skin-tone-1:": "👩🏻",
    ":woman_tone2:": "👩🏼",
    ":woman::skin-tone-2:": "👩🏼",
    ":woman_tone3:": "👩🏽",
    ":woman::skin-tone-3:": "👩🏽",
    ":woman_tone4:": "👩🏾",
    ":woman::skin-tone-4:": "👩🏾",
    ":woman_tone5:": "👩🏿",
    ":woman::skin-tone-5:": "👩🏿",
    ":woman_vampire:": "🧛‍♀️",
    ":woman_vampire_tone1:": "🧛🏻‍♀️",
    ":woman_vampire_light_skin_tone:": "🧛🏻‍♀️",
    ":woman_vampire::skin-tone-1:": "🧛🏻‍♀️",
    ":woman_vampire_tone2:": "🧛🏼‍♀️",
    ":woman_vampire_medium_light_skin_tone:": "🧛🏼‍♀️",
    ":woman_vampire::skin-tone-2:": "🧛🏼‍♀️",
    ":woman_vampire_tone3:": "🧛🏽‍♀️",
    ":woman_vampire_medium_skin_tone:": "🧛🏽‍♀️",
    ":woman_vampire::skin-tone-3:": "🧛🏽‍♀️",
    ":woman_vampire_tone4:": "🧛🏾‍♀️",
    ":woman_vampire_medium_dark_skin_tone:": "🧛🏾‍♀️",
    ":woman_vampire::skin-tone-4:": "🧛🏾‍♀️",
    ":woman_vampire_tone5:": "🧛🏿‍♀️",
    ":woman_vampire_dark_skin_tone:": "🧛🏿‍♀️",
    ":woman_vampire::skin-tone-5:": "🧛🏿‍♀️",
    ":woman_walking:": "🚶‍♀️",
    ":woman_walking_tone1:": "🚶🏻‍♀️",
    ":woman_walking_light_skin_tone:": "🚶🏻‍♀️",
    ":woman_walking::skin-tone-1:": "🚶🏻‍♀️",
    ":woman_walking_tone2:": "🚶🏼‍♀️",
    ":woman_walking_medium_light_skin_tone:": "🚶🏼‍♀️",
    ":woman_walking::skin-tone-2:": "🚶🏼‍♀️",
    ":woman_walking_tone3:": "🚶🏽‍♀️",
    ":woman_walking_medium_skin_tone:": "🚶🏽‍♀️",
    ":woman_walking::skin-tone-3:": "🚶🏽‍♀️",
    ":woman_walking_tone4:": "🚶🏾‍♀️",
    ":woman_walking_medium_dark_skin_tone:": "🚶🏾‍♀️",
    ":woman_walking::skin-tone-4:": "🚶🏾‍♀️",
    ":woman_walking_tone5:": "🚶🏿‍♀️",
    ":woman_walking_dark_skin_tone:": "🚶🏿‍♀️",
    ":woman_walking::skin-tone-5:": "🚶🏿‍♀️",
    ":woman_wearing_turban:": "👳‍♀️",
    ":woman_wearing_turban_tone1:": "👳🏻‍♀️",
    ":woman_wearing_turban_light_skin_tone:": "👳🏻‍♀️",
    ":woman_wearing_turban::skin-tone-1:": "👳🏻‍♀️",
    ":woman_wearing_turban_tone2:": "👳🏼‍♀️",
    ":woman_wearing_turban_medium_light_skin_tone:": "👳🏼‍♀️",
    ":woman_wearing_turban::skin-tone-2:": "👳🏼‍♀️",
    ":woman_wearing_turban_tone3:": "👳🏽‍♀️",
    ":woman_wearing_turban_medium_skin_tone:": "👳🏽‍♀️",
    ":woman_wearing_turban::skin-tone-3:": "👳🏽‍♀️",
    ":woman_wearing_turban_tone4:": "👳🏾‍♀️",
    ":woman_wearing_turban_medium_dark_skin_tone:": "👳🏾‍♀️",
    ":woman_wearing_turban::skin-tone-4:": "👳🏾‍♀️",
    ":woman_wearing_turban_tone5:": "👳🏿‍♀️",
    ":woman_wearing_turban_dark_skin_tone:": "👳🏿‍♀️",
    ":woman_wearing_turban::skin-tone-5:": "👳🏿‍♀️",
    ":woman_white_haired:": "👩‍🦳",
    ":woman_white_haired_tone1:": "👩🏻‍🦳",
    ":woman_white_haired_light_skin_tone:": "👩🏻‍🦳",
    ":woman_white_haired::skin-tone-1:": "👩🏻‍🦳",
    ":woman_white_haired_tone2:": "👩🏼‍🦳",
    ":woman_white_haired_medium_light_skin_tone:": "👩🏼‍🦳",
    ":woman_white_haired::skin-tone-2:": "👩🏼‍🦳",
    ":woman_white_haired_tone3:": "👩🏽‍🦳",
    ":woman_white_haired_medium_skin_tone:": "👩🏽‍🦳",
    ":woman_white_haired::skin-tone-3:": "👩🏽‍🦳",
    ":woman_white_haired_tone4:": "👩🏾‍🦳",
    ":woman_white_haired_medium_dark_skin_tone:": "👩🏾‍🦳",
    ":woman_white_haired::skin-tone-4:": "👩🏾‍🦳",
    ":woman_white_haired_tone5:": "👩🏿‍🦳",
    ":woman_white_haired_dark_skin_tone:": "👩🏿‍🦳",
    ":woman_white_haired::skin-tone-5:": "👩🏿‍🦳",
    ":woman_with_headscarf:": "🧕",
    ":woman_with_headscarf_tone1:": "🧕🏻",
    ":woman_with_headscarf_light_skin_tone:": "🧕🏻",
    ":woman_with_headscarf::skin-tone-1:": "🧕🏻",
    ":woman_with_headscarf_tone2:": "🧕🏼",
    ":woman_with_headscarf_medium_light_skin_tone:": "🧕🏼",
    ":woman_with_headscarf::skin-tone-2:": "🧕🏼",
    ":woman_with_headscarf_tone3:": "🧕🏽",
    ":woman_with_headscarf_medium_skin_tone:": "🧕🏽",
    ":woman_with_headscarf::skin-tone-3:": "🧕🏽",
    ":woman_with_headscarf_tone4:": "🧕🏾",
    ":woman_with_headscarf_medium_dark_skin_tone:": "🧕🏾",
    ":woman_with_headscarf::skin-tone-4:": "🧕🏾",
    ":woman_with_headscarf_tone5:": "🧕🏿",
    ":woman_with_headscarf_dark_skin_tone:": "🧕🏿",
    ":woman_with_headscarf::skin-tone-5:": "🧕🏿",
    ":woman_with_probing_cane:": "👩‍🦯",
    ":woman_with_probing_cane_tone1:": "👩🏻‍🦯",
    ":woman_with_probing_cane_light_skin_tone:": "👩🏻‍🦯",
    ":woman_with_probing_cane::skin-tone-1:": "👩🏻‍🦯",
    ":woman_with_probing_cane_tone2:": "👩🏼‍🦯",
    ":woman_with_probing_cane_medium_light_skin_tone:": "👩🏼‍🦯",
    ":woman_with_probing_cane::skin-tone-2:": "👩🏼‍🦯",
    ":woman_with_probing_cane_tone3:": "👩🏽‍🦯",
    ":woman_with_probing_cane_medium_skin_tone:": "👩🏽‍🦯",
    ":woman_with_probing_cane::skin-tone-3:": "👩🏽‍🦯",
    ":woman_with_probing_cane_tone4:": "👩🏾‍🦯",
    ":woman_with_probing_cane_medium_dark_skin_tone:": "👩🏾‍🦯",
    ":woman_with_probing_cane::skin-tone-4:": "👩🏾‍🦯",
    ":woman_with_probing_cane_tone5:": "👩🏿‍🦯",
    ":woman_with_probing_cane_dark_skin_tone:": "👩🏿‍🦯",
    ":woman_with_probing_cane::skin-tone-5:": "👩🏿‍🦯",
    ":woman_with_veil:": "👰‍♀️",
    ":bride_with_veil:": "👰‍♀️",
    ":woman_with_veil_tone1:": "👰🏻‍♀️",
    ":woman_with_veil_light_skin_tone:": "👰🏻‍♀️",
    ":woman_with_veil::skin-tone-1:": "👰🏻‍♀️",
    ":bride_with_veil::skin-tone-1:": "👰🏻‍♀️",
    ":woman_with_veil_tone2:": "👰🏼‍♀️",
    ":woman_with_veil_medium_light_skin_tone:": "👰🏼‍♀️",
    ":woman_with_veil::skin-tone-2:": "👰🏼‍♀️",
    ":bride_with_veil::skin-tone-2:": "👰🏼‍♀️",
    ":woman_with_veil_tone3:": "👰🏽‍♀️",
    ":woman_with_veil_medium_skin_tone:": "👰🏽‍♀️",
    ":woman_with_veil::skin-tone-3:": "👰🏽‍♀️",
    ":bride_with_veil::skin-tone-3:": "👰🏽‍♀️",
    ":woman_with_veil_tone4:": "👰🏾‍♀️",
    ":woman_with_veil_medium_dark_skin_tone:": "👰🏾‍♀️",
    ":woman_with_veil::skin-tone-4:": "👰🏾‍♀️",
    ":bride_with_veil::skin-tone-4:": "👰🏾‍♀️",
    ":woman_with_veil_tone5:": "👰🏿‍♀️",
    ":woman_with_veil_dark_skin_tone:": "👰🏿‍♀️",
    ":woman_with_veil::skin-tone-5:": "👰🏿‍♀️",
    ":bride_with_veil::skin-tone-5:": "👰🏿‍♀️",
    ":woman_zombie:": "🧟‍♀️",
    ":womans_clothes:": "👚",
    ":womans_flat_shoe:": "🥿",
    ":womans_hat:": "👒",
    ":women_with_bunny_ears_partying:": "👯‍♀️",
    ":women_wrestling:": "🤼‍♀️",
    ":womens:": "🚺",
    ":wood:": "🪵",
    ":woozy_face:": "🥴",
    ":worm:": "🪱",
    ":worried:": "😟",
    ":wrench:": "🔧",
    ":writing_hand:": "✍️",
    ":writing_hand_tone1:": "✍🏻",
    ":writing_hand::skin-tone-1:": "✍🏻",
    ":writing_hand_tone2:": "✍🏼",
    ":writing_hand::skin-tone-2:": "✍🏼",
    ":writing_hand_tone3:": "✍🏽",
    ":writing_hand::skin-tone-3:": "✍🏽",
    ":writing_hand_tone4:": "✍🏾",
    ":writing_hand::skin-tone-4:": "✍🏾",
    ":writing_hand_tone5:": "✍🏿",
    ":writing_hand::skin-tone-5:": "✍🏿",
    ":x:": "❌",
    ":yarn:": "🧶",
    ":yawning_face:": "🥱",
    ":yellow_circle:": "🟡",
    ":yellow_heart:": "💛",
    ":yellow_square:": "🟨",
    ":yen:": "💴",
    ":yin_yang:": "☯️",
    ":yo_yo:": "🪀",
    ":yum:": "😋",
    ":zany_face:": "🤪",
    ":zap:": "⚡",
    ":zebra:": "🦓",
    ":zero:": "0️⃣",
    ":zipper_mouth:": "🤐",
    ":zipper_mouth_face:": "🤐",
    ":zombie:": "🧟",
    ":zzz:": "💤"
}